<?php
namespace Topalis\ActiveSyncFilter\Apache;

use Topalis\ActiveSyncFilter\Apache\ModConfig;

/**
 * Class to manage the Apache filter module fory the ActiveSync Filter.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ASFApacheMod
{
    const CONFIG = "activesyncfilter";
    const MODULE = "modules/mod_activesync.so";
    const NAME = "ACTIVESYNC";

    /**
     * @var \Topalis\Apache\Apache
     */
    private $apache;

    private $configFilename;

    private $libwbxmlFilename;

    public function __construct($configFilename = null)
    {
        if ($configFilename === null) {
            $configFilename = self::CONFIG;
        }
        $this->configFilename = $configFilename;
    }

    /**
     * @return \Topalis\Apache\Apache
     */
    public function getApache()
    {
        return $this->apache;
    }

    /**
     * @param \Topalis\Apache\Apache $apache
     */
    public function setApache(\Topalis\Apache\Apache $apache)
    {
        $this->apache = $apache;
    }

    public function getConfigFilename()
    {
        return $this->configFilename;
    }

    public function getLibwbxmlFilename()
    {
        return $this->libwbxmlFilename;
    }

    public function setLibwbxmlFilename($libWbxmlFilename)
    {
        $this->libwbxmlFilename = $libWbxmlFilename;
    }

    /**
     * Disables the ActiveSync Apache filter module. The Apache config file
     * contains only the module config line afterwards.
     * @param ModConfig $asfModConfig The module configuration to be saved
     *   even though its not used. If null, the configuration stays as it is.
     * @throws \Exception If an error occur when writing in the Apache
     *   configuration file ({@link $this->configFilename})
     */
    public function disable(ModConfig $asfModConfig = null)
    {
        $apache = $this->getApache();
        $oldsum = $apache->getConfigHash($this->getConfigFilename());
        if ($asfModConfig === null) {
            $asfModConfig = self::getConfig();
        }

        if (!$apache->writeConfig($this->getConfigFilename(), "#" . $asfModConfig->toApacheConfigLine() . "\n")) {
            throw new \Exception("Failed to write configuration for the ActiveSync Filter Apache module");
        }

        if ($apache->getConfigHash($this->getConfigFilename()) != $oldsum) {
            $this->getApache()->enableReload();
        }
    }

    /**
     * Enables the ActiveSync Apache filter module.
     * @param ModConfig $asfModConfig The module configuration to use. If null,
     *     the configuration stays as it is.
     * @throws \Exception If an error occur when writing in the Apache
     *   configuration file ({@link $this->getConfigFilename()})
     */
    public function enable(ModConfig $asfModConfig = null)
    {
        $apache = $this->getApache();
        $oldsum = $apache->getConfigHash($this->getConfigFilename());
        if ($asfModConfig === null) {
            $asfModConfig = self::getConfig();
        }
        if (!$apache->writeConfig($this->getConfigFilename(),
            "<IfModule mod_activesync.c>\nSetOutputFilter INFLATE;ACTIVESYNC;DEFLATE\n"
            . $asfModConfig->toApacheConfigLine() . "\n</IfModule>\n")) {
            throw new \Exception("Failed to write configuration for the ActiveSync Filter Apache module");
        }
        if ($apache->getConfigHash($this->getConfigFilename()) != $oldsum) {
            $this->getApache()->enableReload();
        }
    }

    /**
     * @return ModConfig The {@link ModConfig} object representing
     *   the configuration of the ActiveSync Apache filter module. If there's no
     *   configuration a default object is returned. To store config changes you
     *   have to call enable() or disable() with the configuration object as
     *   parameter.
     */
    public function getConfig()
    {
        $apacheConfigLine = $this->getApache()->getConfigLine($this->getConfigFilename(), ModConfig::NAME);
        if ($apacheConfigLine !== null) {
            return ModConfig::fromApacheConfigLine($apacheConfigLine);
        }
        return new ModConfig();
    }

    /**
     * @return boolean true if the ActiveSync Apache filter module is enabled,
     *   false otherwise.
     */
    public function isEnabled()
    {
        return $this->getApache()->isOutputFilterEnabled($this->getConfigFilename(), self::NAME);
    }
}
