<?php
namespace Topalis\ActiveSyncFilter\Apache;

use Topalis\Configuration\ApacheConfigLine;

/**
 * This class encapsulates the configuration of the ActiveSync Apache
 * filter module.
 *
 * @author Dominic Tubach
 */
class ModConfig
{
    const NAME = 'ActiveSyncOptions';
    const FILTER_HOST = 'FilterHost';
    const FILTER_PORT = 'FilterPort';
    const SOCKET_TIMEOUT = 'SocketTimeout';
    const MIN_WBXML_LEN = 'MinWbxmlLen';
    const HALT_ON_ERROR = 'HaltOnError';

    private $filterHost = 'localhost';
    private $filterPort = 7777;
    private $socketTimeout = 60000;
    private $minWbxmlLen = 10240;
    private $haltOnError = '0';

    /**
     * @return string The host of the Filter Server.
     */
    public function getFilterHost()
    {
        return $this->filterHost;
    }

    /**
     * @return int The port of the Filter Server.
     */
    public function getFilterPort()
    {
        return $this->filterPort;
    }

    /**
     * @return int The minimal number of bytes above which ActiveSync responses
     *   will be handled.
     */
    public function getMinWbxmlLen()
    {
        return $this->minWbxmlLen;
    }

    /**
     * @return int The socket timeout for connections to the Filter Server.
     */
    public function getSocketTimeout()
    {
        return $this->socketTimeout;
    }

    /**
     * @return string 1 if halt on error is enabled, 0 otherwise. If enabled
     *   the ActiveSync client will get an error response if an error occurs.
     */
    public function isHaltOnError()
    {
        return $this->haltOnError;
    }

    /**
     * Sets the host of the Filter Server
     * @param string $filterHost The host to be set.
     */
    public function setFilterHost($filterHost)
    {
        $this->filterHost = trim($filterHost);
    }

    /**
     * Sets the port of the Filter Server
     * @param int|string $filterPort The port to be set.
     */
    public function setFilterPort($filterPort)
    {
        $this->filterPort = $filterPort;
    }

    /**
     * Enables or disables halt on error. If enabled the ActiveSync client will
     * get an error response if an error occurs.
     *
     * @param boolean|string $haltOnError true, '1' or 'true' to enable. false,
     *   '0' or 'false' to disable.
     */
    public function setHaltOnError($haltOnError)
    {
        if (empty($haltOnError)) {
            $this->haltOnError = null;
        } else if ($haltOnError && strtolower($haltOnError) != 'false') {
            $this->haltOnError = '1';
        } else {
            $this->haltOnError = '0';
        }
    }

    /**
     * Sets the minimal number of bytes above which ActiveSync responses will
     *   be handled.
     * @param string|int $minWbxmlLen The number of bytes to be set.
     */
    public function setMinWbxmlLen($minWbxmlLen)
    {
        $this->minWbxmlLen = (int) $minWbxmlLen;
    }

    /**
     * Sets the socket timeout for connections to the Filter Server.
     * @param string|int $socketTimeout The socket timeout to be set.
     */
    public function setSocketTimeout($socketTimeout)
    {
        $this->socketTimeout = (int) $socketTimeout;
    }

    /**
     * @return ApacheConfigLine representing this configuration.
     */
    public function toApacheConfigLine()
    {
        $configLine = new ApacheConfigLine();
        $configLine->setName(self::NAME);
        if (!empty($this->filterHost)) {
            $configLine->setOption(self::FILTER_HOST, $this->filterHost);
        }
        if (!empty($this->filterPort)) {
            $configLine->setOption(self::FILTER_PORT, $this->filterPort);
        }
        if (!empty($this->haltOnError)) {
            $configLine->setOption(self::HALT_ON_ERROR, $this->haltOnError);
        }
        if (!empty($this->minWbxmlLen)) {
            $configLine->setOption(self::MIN_WBXML_LEN, $this->minWbxmlLen);
        }
        if (!empty($this->socketTimeout)) {
            $configLine->setOption(self::SOCKET_TIMEOUT, $this->socketTimeout);
        }

        return $configLine;
    }

    /**
     * Creates a new ModConfig based on the specified {@link \ApacheConfigLine}.
     *
     * @param ApacheConfigLine $apacheConfigLine The object containing the configuration.
     *
     * @throws \Exception
     * @return ModConfig The created config object.
     */
    public static function fromApacheConfigLine(ApacheConfigLine $apacheConfigLine)
    {
        if ($apacheConfigLine->getName() != self::NAME) {
            throw new \Exception(
                'Invalid mod_activesync config line: starts
                with ' . $apacheConfigLine->getName() . ', but must start
                with '. self::NAME
            );
        }

        $config = new ModConfig();
        foreach ($apacheConfigLine->getOptions() as $key => $value) {
            switch ($key) {
                case self::FILTER_HOST:
                    $config->setFilterHost($value);
                    break;
                case self::FILTER_PORT:
                    $config->setFilterPort($value);
                    break;
                case self::HALT_ON_ERROR:
                    $config->setHaltOnError($value);
                    break;
                case self::MIN_WBXML_LEN:
                    $config->setMinWbxmlLen($value);
                    break;
                case self::SOCKET_TIMEOUT:
                    $config->setSocketTimeout($value);
                    break;
            }
        }

        return $config;
    }
}
