<?php
namespace Topalis\ActiveSyncFilter\Config;

use Topalis\Util\StringUtil;
use Topalis\ActiveSyncFilter\Util\MimeType;
use Topalis\Configuration\Configuration;

/**
 * This class provides access to the file remove configuration of the Filter
 * Server.
 *
 * @author Dominic Tubach
 */
class FileRemoveConfig
{
    private $generalConfig;
    private $config;

    /**
     * Constructor.
     *
     * @param Configuration $config the configuration which holds the mime types of files to
     *         remove.
     * @param GeneralConfig $generalConfig the general config is used to access the state of
     *         {@link isRemoveUnfiltered()}.
     */
    public function __construct(Configuration $config, GeneralConfig $generalConfig)
    {
        $this->generalConfig = $generalConfig;
        $this->config = $config;
    }

    /**
     * Clears all remove settings.
     */
    public function clearRemoveSettings()
    {
        $this->config->clear();
    }

    /**
     * @return boolean true if all files for which no filter applies shall be removed.
     */
    public function isRemoveUnfiltered()
    {
        return $this->generalConfig->isRemoveUnfiltered();
    }

    /**
     * Enables or disables the removal of unfiltered files.
     *
     * @param boolean $enabled true to enable the removal of unfiltered files,
     *         false otherwise.
     */
    public function setRemoveUnfiltered($enabled)
    {
        $this->generalConfig->setRemoveUnfiltered($enabled);
    }

    /**
     * @return boolean[string] An associative array mapping mime types to
     *         booleans that indicate if a file shall be removed (true) or
     *         shall not be removed (false).
     */
    public function getRemoveSettings()
    {
        $rv = array();
        foreach ($this->config->getProperties() as $type => $enabled) {
            $rv[$type] = StringUtil::strtobool($enabled);
        }
        return $rv;
    }

    /**
     * Enables or disables the removal of files with the specified mime types.
     *
     * @param boolean[string] $removeSettings An associative array mapping mime
     *         types to booleans that indicate if a file shall be removed (true)
     *         or shall not be removed (false).
     */
    public function setRemoveSettings(array $removeSettings)
    {
        $this->config->clear();
        foreach ($removeSettings as $type => $enabled) {
            $this->config->setProperty((string) $type, StringUtil::booltostr($enabled));
        }
    }

    /**
     * @param MimeType $mimeType
     * @return boolean|null true if files with the specified mime type shall be
     *         removed, false otherwise, or null if no value is set for the
     *         specified mime type.
     */
    public function getRemoveSetting(MimeType $mimeType)
    {
        $remove = $this->config->getProperty((string) $mimeType);
        if ($remove !== null) {
            return StringUtil::strtobool($remove);
        }

        return null;
    }

    /**
     * Enables or disables the removal of files with the specified mime type.
     *
     * @param MimeType $mimeType The mime type to configure.
     * @param boolean $enabled true to enable the removal, false to disable it.
     */
    public function setRemoveSetting(MimeType $mimeType, $enabled)
    {
        $this->config->setProperty((string) $mimeType, StringUtil::booltostr($enabled));
    }

    /**
     * Removes the remove setting for the specified mime type.
     *
     * @param MimeType $mimeType The mime type to remove the remove setting for.
     */
    public function removeRemoveSetting(MimeType $mimeType)
    {
        $this->config->removeProperty((string) $mimeType);
    }
}
