<?php
namespace Topalis\ActiveSyncFilter\Config;

use Topalis\Util\StringUtil;
use Topalis\Configuration\Configuration;

/**
 * This class provides access to the general configuration of the Filter Server.
 *
 * @author Dominic Tubach
 */
class GeneralConfig
{
    private $config;

    /**
     * Constructor.
     *
     * @param Configuration $config The configuration which provides access to
     *         the confguration values.
     */
    public function __construct(Configuration $config)
    {
        $this->config = $config;
    }

    public function clear()
    {
        $this->config->clear();
    }

    /**
     * @return string The error behavior.
     * @see ErrorBehavior
     */
    public function getErrorBehavior()
    {
        return $this->config->getProperty("ErrorBehavior", "IGNORE");
    }

    /**
     * Sets the error behavior.
     *
     * @param string $errorBehvior The error behavior to set.
     * @throws \Exception If the specified error behavior is invalid.
     * @see ErrorBehavior
     */
    public function setErrorBehavior($errorBehvior)
    {
        if (!ErrorBehavior::isValid($errorBehvior)) {
            throw new \Exception("The specified error behavior (".$errorBehvior .") is invalid.");
        }
        $this->config->setProperty("ErrorBehavior", $errorBehvior);
    }

    /**
     * @return boolean true if the ActiveSync data of non-roaming users should be
     *     handled.
     * @see UserRoamingConfig
     */
    public function isHandleNonRoaming()
    {
        return StringUtil::strtobool($this->config->getProperty("HandleNonRoaming", "true"));
    }

    /**
     * Enables or disables the handling of ActiveSync data for non-roaming users.
     *
     * @param boolean $enabled true to enables the handling of ActiveSync data
     *         for non-roaming users, false to disable it.
     * @see UserRoamingConfig
     */
    public function setHandleNonRoaming($enabled)
    {
        $this->config->setProperty("HandleNonRoaming", StringUtil::booltostr($enabled));
    }

    /**
     * @return true if files for which no filter is available shall be removed,
     *         false otherwise.
     */
    public function isRemoveUnfiltered()
    {
        return StringUtil::strtobool($this->config->getProperty("RemoveUnfiltered", "false"));
    }

    /**
     * Enables or disables the removal of files for which no filter is available.
     *
     * @param boolean $enabled true to enable the removal of files for which no
     *         filter is available, false to disable it.
     */
    public function setRemoveUnfiltered($enabled)
    {
        $this->config->setProperty("RemoveUnfiltered", StringUtil::booltostr($enabled));
    }
}
