<?php
namespace Topalis\ActiveSyncFilter\Config;

use Topalis\Configuration\Ini;

/**
 * This class allows access to the Filter Server configuration defined in the
 * configuration file.
 *
 * @author Dominic Tubach
 */
class ServerConfig
{
    /**
     * @var string Path to the folder containing the Filer Server jar.
     *   Required if the plugin folder defined in the configuration is relative.
     * @see getRealPluginFolder()
     */
    private static $ASF_JAR_FOLDER = "/usr/share/java/activesyncfilter";

    private static $DEFAULT_PLUGIN_FOLDER = "/usr/share/java/activesyncfilter/plugins";

    private $ini;

    /**
     * Constructor.
     *
     * @param string $filename
     */
    public function __construct($filename = null)
    {
        if ($filename === null) {
            $this->ini = new Ini();
        } else {
            $this->ini = Ini::load($filename, true);
        }
    }

    /**
     * @return string|NULL The JDBC data source class.
     */
    public function getJdbcDataSource()
    {
        return $this->ini->getProperty("jdbc.datasource");
    }

    /**
     * @return string|NULL The JDBC driver class.
     */
    public function getJdbcDriver()
    {
        return $this->ini->getProperty("jdbc.driver");
    }

    /**
     * @return string|NULL The JDBC URL.
     */
    public function getJdbcUrl()
    {
        return $this->ini->getProperty("jdbc.url");
    }

    /**
     * @return string|NULL The JDBC username.
     */
    public function getJdbcUser()
    {
        return $this->ini->getProperty("jdbc.user");
    }

    /**
     * @return string|NULL The JDBC password.
     */
    public function getJdbcPassword()
    {
        return $this->ini->getProperty("jdbc.password");
    }

    /**
     * @return string The database table prefix. Returns "asf_" if not specified.
     */
    public function getJdbcTablePrefix()
    {
        return $this->ini->getProperty("jdbc.tableprefix", "asf_");
    }

    /**
     * @return string The host for which the Filter Server listens for
     *   connections. Returns "localhost" if not specified.
     */
    public function getListenHost()
    {
        return $this->ini->getProperty("server.host", "localhost");
    }

    /**
     * @return integer The port on which the Filter Server listens for
     *   connections. Returns 7777 if not specified.
     */
    public function getPort()
    {
        return intval($this->ini->getProperty("server.port", 7777));
    }

    /**
     * @return string|NULL The socket timeout.
     */
    public function getSocketTimeout()
    {
        return $this->ini->getProperty("server.sockettimeout");
    }

    /**
     * @return string|NULL The path to the plugin folder.
     * @see getRealPluginFolder()
     */
    public function getPluginFolder()
    {
        return $this->ini->getProperty("general.pluginfolder");
    }

    /**
     * Returns the path to the plugin folder. In opposite to
     *   {@link getPluginFolder()} this method always returns a path. If the
     *   defined path is abolute, it is returned as it is. If it is relative
     *   {@link ASF_JAR_FOLDER}/<plugin_path> is returned. If no plugin folder
     *   is set, {@link DEFAULT_PLUGIN_FOLDER} is returned.
     *
     * @return string The path to the plugin folder.
     */
    public function getRealPluginFolder()
    {
        $folder = $this->getPluginFolder();
        if (empty($folder)) {
            return self::$DEFAULT_PLUGIN_FOLDER;
        }
        $absolute = preg_match("/^(?:\\/|\\\\|\w\\:\\\\).*$/", $folder);
        if (!$absolute) {
            $folder = self::$ASF_JAR_FOLDER . "/" . $folder;
        }

        return $folder;
    }

    /**
     * Stores the configuration in the specified file.
     *
     * @param string $filename The name of the file to store the
     *   configuration in.
     */
    public function save($filename)
    {
        $this->ini->save($filename, true);
    }

    /**
     * Sets the JDBC data source class.
     *
     * @param string $jdbcDataSource the JDBC data source class to be set.
     */
    public function setJdbcDataSource($jdbcDataSource)
    {
        $this->ini->setSectionValue("jdbc", "datasource", $jdbcDataSource);
    }

    /**
     * Sets the JDBC driver class.
     *
     * @param string $jdbcDriver The JDBC driver class to be set.
     */
    public function setJdbcDriver($jdbcDriver)
    {
        $this->ini->setSectionValue("jdbc", "driver", $jdbcDriver);
    }

    /**
     * Sets the JDBC URL.
     *
     * @param string $jdbcUrl The JDBC URL to be set.
     */
    public function setJdbcUrl($jdbcUrl)
    {
        $this->ini->setSectionValue("jdbc", "url", $jdbcUrl);
    }

    /**
     * Sets the JDBC username.
     *
     * @param string $jdbcUser The JDBC username to be set.
     */
    public function setJdbcUser($jdbcUser)
    {
        $this->ini->setSectionValue("jdbc", "user", $jdbcUser);
    }

    /**
     * Sets the JDBC password.
     *
     * @param string $jdbcPassword The JDBC password to be set.
     */
    public function setJdbcPassword($jdbcPassword)
    {
        $this->ini->setSectionValue("jdbc", "password", $jdbcPassword);
    }

    /**
     * Sets the database table prefix.
     *
     * @param string $tablePrefix
     */
    public function setJdbcTablePrefix($tablePrefix)
    {
        $this->ini->setSectionValue("jdbc", "tableprefix", $tablePrefix);
    }

    /**
     * Sets host for which the Filter Server listens for connecitons.
     *
     * @param string $host the host to be set.
     */
    public function setListenHost($host)
    {
        $this->ini->setSectionValue("server", "host", $host);
    }

    /**
     * Sets the port on which the Filter Server listens for connections.
     *
     * @param string|int $port The port to be set.
     */
    public function setPort($port)
    {
        $this->ini->setSectionValue("server", "port", (string) $port);
    }

    /**
     * Sets the socket timeout.
     *
     * @param string|int $socketTimeout The socket timeout to be set.
     */
    public function setSocketTimeout($socketTimeout)
    {
        $this->ini->setSectionValue("server", "sockettimeout", (string) $socketTimeout);
    }

    /**
     * Sets the path to the plugin folder.
     *
     * @param string $pluginFolder the path to be set.
     */
    public function setPluginFolder($pluginFolder)
    {
        $this->ini->setSectionValue("general", "pluginfolder", $pluginFolder);
    }
}
