<?php
namespace Topalis\ActiveSyncFilter;

use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Util\Process;
use Topalis\ActiveSyncFilter\Log\TrafficStat;
use Topalis\ActiveSyncFilter\Log\ActiveSyncFilterLog;
use Topalis\ActiveSyncFilter\Config\FileRemoveConfig;
use Topalis\ActiveSyncFilter\Plugin\PluginManager;
use Topalis\ActiveSyncFilter\Config\GeneralConfig;
use Topalis\ActiveSyncFilter\Config\ServerConfig;
use Topalis\Configuration\DatabaseConfiguration;
use Topalis\ActiveSyncFilter\Cache\AsfCacheManager;
use Topalis\ActiveSyncFilter\Entity\AsfCacheEntityManager;
use Topalis\ActiveSyncFilter\Log\MimeTypeStat;
use Topalis\Database\EntityManager;

/**
 * Base class to access the various options of the Filter Server.
 *
 * @author Dominic Tubach
 */
class FilterServer
{
    const SERVICE_COMMAND = '/usr/sbin/activesyncfilterd-service';

    /**
     * @var \Topalis\ActiveSyncFilter\Cache\AsfCacheManager
     */
    private $cacheManager;

    /**
     * @var ServerConfig
     */
    private $config;

    /**
     * @var \Topalis\Database\MySqlDb
     */
    private $db;

    /**
     * @var \Doctrine\ORM\EntityManagerInterface
     */
    private $em;

    /**
     * @var string
     */
    private $language;

    /**
     * @var \Topalis\ActiveSyncFilter\Plugin\PluginManager
     */
    private $pluginManager;

    public function __construct(\Topalis\Database\MySqlDb $db, \Doctrine\ORM\EntityManagerInterface $em)
    {
        $this->db = $db;
        $this->em = $em;

        $this->cacheManager = new AsfCacheManager(new AsfCacheEntityManager($em));
    }

    /**
     * Stops the Filter Server and delete its init script from the system
     * runlevels such that it is not started on boot.
     *
     * @throws \Exception If something failed.
     */
    public function disable()
    {
        $this->stop();
        $this->delFromRunlevels();
    }

    /**
     * Clears the Filter Server configuration for the specified config group.
     *
     * @param ConfigGroup $configGroup The config group to delete the Filter
     *     Server configuration.
     */
    public function clearConfig(ConfigGroup $configGroup)
    {
        $this->getFileRemoveConfig($configGroup)->clearRemoveSettings();
        $this->getGeneralConfig($configGroup)->clear();
        $this->getPluginManager()->clear($configGroup);
    }

    /**
     * Starts the Filter Server and adds its init script to the system runlevels
     * such that it is started on boot.
     *
     * @throws \Exception If something failed.
     */
    public function enable()
    {
        $this->start();
        $this->addToRunlevels();
    }

    public function getCacheManager()
    {
        return $this->cacheManager;
    }

    /**
     * @return \Topalis\Database\MySqlDb A {@link MySqlDb} object with the
     *     connection to the database defined in the Filter Server configuration.
     * @throws DatabaseException If database connection fails.
     */
    public function getDatabase()
    {
        return $this->db;
    }

    /**
     * @return \Doctrine\ORM\EntityManagerInterface The entity manager used
     *     in the Filter Server.
     */
    protected function getEntityManager()
    {
        return $this->getDatabase()->getDoctrine()->getManager("asf");
    }

    /**
     * @param ConfigGroup $config The id of the configuration to access.
     * @return FileRemoveConfig The file remove configuration.
     */
    public function getFileRemoveConfig(ConfigGroup $config)
    {
        $dbConfig = new DatabaseConfiguration(
            $this->getDatabase(),
            $this->getServerConfig()->getJdbcTablePrefix() . "conf_remove",
            "contenttype",
            "enabled"
        );
        $dbConfig->addRequiredValue("configid", $config->getId());
        return new FileRemoveConfig($dbConfig, $this->getGeneralConfig($config));
    }

    /**
     * @param ConfigGroup $config The id of the configuration to access.
     * @return GeneralConfig The general configuration.
     */
    public function getGeneralConfig(ConfigGroup $config)
    {
        $dbConfig = new DatabaseConfiguration(
            $this->getDatabase(),
            $this->getServerConfig()->getJdbcTablePrefix() . "conf",
            "key",
            "value"
        );
        $dbConfig->addRequiredValue("configid", $config->getId());
        return new GeneralConfig($dbConfig);
    }

    /**
     * @return ServerConfig The configuration of this filter server set through
     *     setServerConfig().
     */
    public function getServerConfig()
    {
        return $this->config;
    }

    /**
     * Sets the configuration of this filter server.
     *
     * @param ServerConfig $serverConfig The configuration to set.
     */
    public function setServerConfig(ServerConfig $serverConfig)
    {
        $this->config = $serverConfig;
    }

    /**
     * @return TrafficStat Object to access the traffic statistics this Filter Server.
     */
    public function getTrafficStat()
    {
        return new TrafficStat($this->getEntityManager(), $this->getDatabase(),
            $this->getServerConfig()->getJdbcTablePrefix());
    }

    /**
     * @return string The language to use for plugin information.
     */
    public function getLanguage()
    {
        return $this->language;
    }

    /**
     * Sets the language to use for plugin information.
     * @param string $language The language to set.
     */
    public function setLanguage($language)
    {
        $this->language = $language;
    }

    /**
     * @return ActiveSyncFilterLog Object to access the log of this Filter Server.
     */
    public function getLog()
    {
        return new ActiveSyncFilterLog($this->getDatabase(), $this->getServerConfig()->getJdbcTablePrefix());
    }

    public function getMimeTypeStat()
    {
        $em = $this->getEntityManager();
        return new MimeTypeStat(
            new EntityManager($em, "Topalis\ActiveSyncFilter\Entity\AsfStatMimeType"),
            new EntityManager($em, "Topalis\ActiveSyncFilter\Entity\AsfLogFilterApplications")
        );
    }

    /**
     * @return PluginManager The plugin manager of this Filter Server.
     */
    public function getPluginManager()
    {
        if ($this->pluginManager === null || $this->pluginManager->getLanguage() != $this->getLanguage()) {
            $this->pluginManager = new PluginManager($this, $this->getLanguage());
        }
        return $this->pluginManager;
    }

    /**
     * @return boolean true if the Filter Server is reachable, false otherwise.
     */
    public function isReachable()
    {
        $serverConfig = $this->getServerConfig();
        $timeout = 5;
        try {
            $errno = null;
            $errstr = null;
            $sock = fsockopen($serverConfig->getListenHost(), $serverConfig->getPort(), $errno, $errstr, $timeout);
            if ($sock) {
                fclose($sock);
                return true;
            }
        } catch (\Exception $e) {
            // nothing to do
        }
        return false;
    }

    /**
     * Starts the Filter Server.
     *
     * @throws \Exception If starting failed.
     */
    private function start()
    {
        $out = '';
        $rc = Process::exec(self::SERVICE_COMMAND .' start ', $out);
        if ($rc != 0) {
            throw new \Exception('Failed to start ActiveSync Filter Server: ' . $out);
        }
    }

    /**
     * Stops the Filter Server.
     *
     * @throws \Exception If stopping failed.
     */
    private function stop()
    {
        $out = '';
        $rc = Process::exec(self::SERVICE_COMMAND .' stop ', $out);
        if ($rc != 0) {
            throw new \Exception('Failed to stop ActiveSync Filter Server: '. $out);
        }
    }

    /**
     * Restarts the Filter Server.
     *
     * @throws \Exception If restarting failed.
     */
    public function restart()
    {
        $out = '';
        $rc = Process::exec(self::SERVICE_COMMAND .' restart ', $out);
        if ($rc != 0) {
            throw new \Exception('Failed to restart ActiveSync Filter Server: '. $out);
        }
    }

    /**
     * Adds the Filter Server init script to the system runlevels.
     *
     * @throws \Exception If the operation failed.
     */
    private function addToRunlevels()
    {
        $out = '';
        $rc = Process::exec(self::SERVICE_COMMAND .' runlevel-add', $out);
        if ($rc != 0) {
            throw new \Exception("Failed to add ActiveSync Filter Server init script to system runlevels: $out");
        }
    }

    /**
     * Deletes the Filter Server init script from the system runlevels.
     *
     * @throws \Exception If the operation failed.
     */
    private function delFromRunlevels()
    {
        $out = '';
        $rc = Process::exec(self::SERVICE_COMMAND .' runlevel-del', $out);
        if ($rc != 0) {
            throw new \Exception("Failed to delete ActiveSync Filter Server init script from system runlevels: $out");
        }
    }
}
