<?php
namespace Topalis\ActiveSyncFilter\Log;

use Topalis\Database\EntityManager;
use Topalis\ActiveSyncFilter\Entity\AsfStatMimeType;

/**
 * Class to access MIME type statistics
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class MimeTypeStat
{
    /**
     * @var \Topalis\Database\EntityManager
     */
    private $em;

    /**
     * @var \Topalis\Database\EntityManager
     */
    private $logEm;

    /**
     * Constructor.
     *
     * @param EntityManager $em The entity manager to access the aggregated data.
     * @param EntityManager $logEm The entity manager to access the log entries.
     */
    public function __construct(EntityManager $em, EntityManager $logEm)
    {
        $this->em = $em;
        $this->logEm = $logEm;
    }

    /**
     * @return \Topalis\Database\EntityManager
     */
    protected function getEntityManager()
    {
        return $this->em;
    }

    /**
     * @return \Topalis\Database\EntityManager
     */
    protected function getLogEntityManager()
    {
        return $this->logEm;
    }

    /**
     * @return array
     */
    public function getAvailableMimeTypes()
    {
        return $this->getLogEntityManager()->createQueryBuilder("fa")
            ->select("fa.mediaType as mediaType, fa.subType as subType")
            ->orderBy("fa.mediaType")
            ->addOrderBy("fa.subType")
            ->groupBy("fa.mediaType, fa.subType")
            ->getQuery()->getArrayResult()
        ;
    }

    /**
     * @param integer|string $from The date to start from as timestamp or as
     *     string PHP can interpret as date.
     * @param integer|string $to The date to end as timestamp or as string PHP
     *     can interpret as date.
     * @return array of AsfStatMimeType objects
     */
    public function getAll($from = null, $to = null)
    {
        if ($from === null) {
            $from = 0;
        }
        if (is_string($from)) {
            $from = strtotime($from);
        }

        if ($to === null) {
            $to = time();
        } else {
            if (is_string($to)) {
                $to = strtotime($to);
            }
        }

        $res = array();
        // load aggregated data
        $stats = $this->getEntityManager()->createQueryBuilder("s")
            ->where("s.date >= :from")
            ->andWhere("s.date <= :to")
            ->orderBy("s.mediaType")
            ->addOrderBy("s.subType")
            ->addOrderBy("s.lastChange")
            ->setParameter("from", date("Y-m-d", $from))
            ->setParameter("to", date("Y-m-d", $to))
            ->getQuery()->getResult();
        // -1 because we might use it in the call of getStatsFromLog
        $lastChange = strtotime($from) - 1;
        foreach ($stats as $stat) {
            if (isset($res[$stat->getMediaType()][$stat->getSubType()])) {
                $newStat = $stat;
                $stat = $res[$stat->getMediaType()][$stat->getSubType()];
                $stat->setTotal($stat->getTotal() + $newStat->getTotal());
                $stat->setChanged($stat->getChanged() + $newStat->getChanged());
                $stat->setUnchanged($stat->getUnchanged() + $newStat->getUnchanged());
                $stat->setRemoved($stat->getRemoved() + $newStat->getRemoved());
                $stat->setFailed($stat->getFailed() + $newStat->getFailed());
                $stat->setInSize($stat->getInSize() + $newStat->getInSize());
                $stat->setOutSize($stat->getOutSize() + $newStat->getOutSize());
                $stat->setDiff($stat->getDiff() + $newStat->getDiff());
                $stat->setLastChange($newStat->getLastChange());
            } else {
                $stat = clone $stat;
                $res[$stat->getMediaType()][$stat->getSubType()] = $stat;
            }
            $lastChange = max(array($lastChange, $stat->getLastChange()));
        }

        // add non-aggregated data
        $statsFromLog = $this->getStatsFromLog($lastChange + 1, $to * 1000);
        foreach ($statsFromLog as $statFromLog) {
            $mediaType = $statFromLog['mediaType'];
            $subType = $statFromLog['subType'];
            if (isset($res[$mediaType][$subType])) {
                $stat = $res[$mediaType][$subType];
            } else {
                $stat = new AsfStatMimeType();
                $stat->setChanged(0);
                $stat->setFailed(0);
                $stat->setUnchanged(0);
                $stat->setRemoved(0);
                $stat->setMediaType($mediaType);
                $stat->setSubType($subType);
                $res[$mediaType][$subType] = $stat;
            }

            switch ($statFromLog['result']) {
                case "CHANGED":
                    $stat->setChanged($stat->getChanged() + $statFromLog['number']);
                    break;
                case "UNCHANGED":
                    $stat->setUnchanged($stat->getUnchanged() + $statFromLog['number']);
                    break;
                case "REMOVE":
                    $stat->setRemoved($stat->getRemoved() + $statFromLog['number']);
                    break;
                case "ERROR":
                    $stat->setFailed($stat->getFailed() + $statFromLog['number']);
                    break;
            }
            $diff = $statFromLog['inSize'] - $statFromLog['outSize'];
            $stat->setTotal($stat->getTotal() + $statFromLog['number']);
            $stat->setInSize($stat->getInSize() + $statFromLog['inSize']);
            $stat->setOutSize($stat->getOutSize() + $statFromLog['outSize']);
            $stat->setDiff($stat->getDiff() + $diff);
            $stat->setLastChange($statFromLog['startTime']);
        }

        return $res;
    }

    protected function getStatsFromLog($from, $to)
    {
        return $this->getLogEntityManager()->createQueryBuilder("l")
            ->select("l.mediaType as mediaType, l.subType as subType, l.startTime as startTime, l.result as result,
                COUNT(l) as number, SUM(l.inSize) as inSize, SUM(l.outSize) as outSize")
            ->where("l.startTime >= :from")
            ->andWhere("l.startTime <= :to")
            ->groupBy("l.mediaType, l.subType, l.result")
            ->orderBy("l.id")
            ->setParameter("from", $from)
            ->setParameter("to", $to)
            ->getQuery()->getResult()
        ;
    }
}
