<?php
namespace Topalis\ActiveSyncFilter\Log;

use Topalis\ActiveSyncFilter\Entity\AsfTrafficDay;
use Topalis\Database\EntityManager;
use Topalis\Util\Interpolator;

class TrafficStat
{
    /**
     * @var \Topalis\Database\MySqlDb
     */
    private $db;

    /**
     * @var \Topalis\Database\EntityManager
     */
    private $emTrafficDay;

    /**
     * @var \Topalis\Database\EntityManager
     */
    private $emLog;

    private $tableLog;
    private $tableDay;
    private $tableYearMonth;

    /**
     * Constructor.
     *
     * @param \Doctrine\ORM\EntityManagerInterface $em The entity manager to use.
     * @param MySqlDb $db the database containing the log.
     * @param string $tablePrefix The table prefix used in the Filter Server.
     */
    public function __construct(\Doctrine\ORM\EntityManagerInterface $em, \Topalis\Database\MySqlDb $db, $tablePrefix)
    {
        $this->emTrafficDay = new EntityManager($em, "Topalis\ActiveSyncFilter\Entity\AsfTrafficDay");
        $this->emLog = new EntityManager($em, "Topalis\ActiveSyncFilter\Entity\AsfLog");
        $this->db = $db;
        $this->tableLog = $tablePrefix . "log";
        $this->tableDay = $tablePrefix . "traffic_day";
        $this->tableYearMonth = $tablePrefix . "traffic_yearmonth";
    }

    /**
     * @return \Topalis\Database\MySqlDb
     */
    protected function getDB()
    {
        return $this->db;
    }

    /**
     * @return \Topalis\Database\EntityManager
     */
    protected function getTrafficDayEntityManager()
    {
        return $this->emTrafficDay;
    }

    /**
     * @return \Topalis\Database\EntityManager
     */
    protected function getLogEntityManager()
    {
        return $this->emLog;
    }

    protected function getTableDay()
    {
        return $this->tableDay;
    }

    protected function getTableLog()
    {
        return $this->tableLog;
    }

    protected function getTableYearMonth()
    {
        return $this->tableYearMonth;
    }

    /**
     * @return int The compression in the last 24 hours.
     */
    public function getCompressionLast24h()
    {
        $from = (time() - 86400) * 1000;
        return $this->getLogEntityManager()->createQueryBuilder("l")
            ->select("SUM(l.inSize - l.outSize)")
            ->where("l.endTime >= :from")
            ->setParameter("from", $from)
            ->getQuery()->getSingleScalarResult() ?: 0
        ;
    }

    /**
     * @return int The total compression
     */
    public function getTotalCompression()
    {
        $sql = "SELECT SUM(diff) FROM " . $this->getTableYearMonth()
            . " WHERE `date` <= SUBDATE(NOW(), INTERVAL 1 MONTH)";
        $diff = $this->getDB()->queryValue($sql) + 0;
        $days = intval(date("j"));
        $trafficDays = $this->getTrafficDayLast($days);
        foreach ($trafficDays as $trafficDay) {
            $diff += $trafficDay->diff;
        }
        return $diff;
    }

    /**
     * Returns an array of AsfTrafficDay objects for the days in the specified
     * period.
     *
     * @param integer|string $from The date to start from as timestamp or as
     *     string PHP can interpret as date.
     * @param integer|string $to The date to end as timestamp or as string PHP
     *     can interpret as date.
     * @return AsfTrafficDay[]
     */
    public function getTrafficDay($from = null, $to = null)
    {
        if ($from === null) {
            $from = 0;
        }
        if (is_string($from)) {
            $from = strtotime($from);
        }

        if ($to === null) {
            $to = time();
        } else {
            if (is_string($to)) {
                $to = strtotime($to);
            }
        }

        // load aggregated data
        $stats = $this->getTrafficDayEntityManager()->createQueryBuilder("s")
            ->where("s.date >= :from")
            ->andWhere("s.date <= :to")
            ->orderBy("s.date")
            ->setParameter("from", date("Y-m-d", $from))
            ->setParameter("to", date("Y-m-d", $to))
            ->getQuery()->getResult();

        // add not aggredated data
        if ($stats) {
            // start from next day
            $fromLatest = end($stats)->getDate()->format("U") + 86400;
        } else {
            $fromLatest = $from;
        }
        $latestTraffic = $this->getLogEntityManager()->createQueryBuilder("a")
            ->select("DIV(a.endTime, 600000) * 600 AS endTime", "a.inSize AS inSize", "a.outSize AS outSize",
                "SUM(a.inSize - a.outSize) AS diff")
            ->where("DIV(a.endTime, 1000) >= :from")
            ->andWhere("DIV(a.endTime, 1000) <= :to")
            ->groupBy("endTime")
            ->orderBy("a.id")
            ->setParameter("from", $fromLatest)
            ->setParameter("to", $to)
            ->getQuery()->getArrayResult();
        ;

        $timeZone = new \DateTimeZone(date_default_timezone_get());
        foreach ($latestTraffic as $trafficData) {
            $traffic = new AsfTrafficDay();
            $date = \DateTime::createFromFormat("U", $trafficData['endTime']);
            $date->setTimeZone($timeZone);
            $traffic->setDate($date);
            $traffic->setDiff($trafficData['diff']);
            $traffic->setInSize($trafficData['inSize']);
            $traffic->setOutSize($trafficData['outSize']);
            $stats[] = $traffic;
        }

        return $stats;
    }

    /**
     * Returns an array of timestamp-traffic pairs for the specified period.
     *
     * @param integer|string $from The date to start from as timestamp or as
     *     string PHP can interpret as date.
     * @param integer|string $to The date to end as timestamp or as string PHP
     *     can interpret as date.
     * @return array Array of timestamp-traffic pairs. Values at the beginning
     *     and the end are interpolated if possible.
     */
    public function getTrafficDayArray($from = null, $to = null)
    {
        if ($from === null) {
            $from = 0;
        }
        if (is_string($from)) {
            $from = strtotime($from);
        }

        if ($to === null) {
            $to = time();
        } else {
            if (is_string($to)) {
                $to = strtotime($to);
            }
        }

        $rv = array();
        $origStart = $from;
        $origEnd = $to;
        $from -= 86399; // for interpolation
        $to += 10; // for interpolation

        // load aggregated data
        $stats = $this->getTrafficDayEntityManager()->createQueryBuilder("s")
        ->select("s.date AS date", "s.diff AS diff")
        ->where("s.date >= :from")
        ->andWhere("s.date <= :to")
        ->orderBy("s.date")
        ->setParameter("from", date("Y-m-d", $from))
        ->setParameter("to", date("Y-m-d", $to))
        ->getQuery()->getArrayResult();

        $lastTime = $from;
        if ($stats) {
            foreach ($stats as $stat) {
                $time = strtotime($stat['date']);
                for ($betweenTime = $lastTime + 86400; $betweenTime < $time; $betweenTime += 86400) {
                    // no traffic for one or more days
                    $rv[$betweenTime] = 0;
                }
                $rv[$time] = intval($stat['diff']);
                $lastTime = $time;
            }
            $lastTime = $from = $lastTime + 86400; // start from next day
        }

        // add not aggregated data, aggregate in 10 minute steps
        $latestTraffic = $this->getLogEntityManager()->createQueryBuilder("a")
            ->select("DIV(a.endTime, 600000) * 600 + 300 AS endTime", "SUM(a.inSize - a.outSize) AS diff")
            ->where("DIV(a.endTime, 1000) >= :from")
            ->andWhere("DIV(a.endTime, 1000) <= :to")
            ->groupBy("endTime")
            ->orderBy("a.id")
            ->setParameter("from", $from)
            ->setParameter("to", $to)
            ->getQuery()->getArrayResult();
        ;

        foreach ($latestTraffic as $trafficData) {
            $time = $trafficData['endTime'];
            for ($betweenTime = $lastTime + 600; $betweenTime < $time; $betweenTime += 600) {
                // no traffic for more than 10 minutes
                $rv[$betweenTime] = 0;
            }
            $rv[$time] = intval($trafficData['diff']);
            $lastTime = $time;
        }

        // add not aggregated data, aggregate in 1 minute steps
        $from = $lastTime + 300;
        if ($from > $origEnd) {
            $from = $origEnd;
        }
        $latestTraffic = $this->getLogEntityManager()->createQueryBuilder("a")
        ->select("DIV(a.endTime, 60000) * 60 + 30 AS endTime", "SUM(a.inSize - a.outSize) AS diff")
        ->where("DIV(a.endTime, 1000) >= :from")
        ->andWhere("DIV(a.endTime, 1000) <= :to")
        ->groupBy("endTime")
        ->orderBy("a.id")
        ->setParameter("from", $from)
        ->setParameter("to", $to)
        ->getQuery()->getArrayResult();
        ;

        foreach ($latestTraffic as $trafficData) {
            $time = $trafficData['endTime'];
            for ($betweenTime = $lastTime + 60; $betweenTime < $time; $betweenTime += 60) {
                // no traffic for more than 10 minutes
                $rv[$betweenTime] = 0;
            }
            $rv[$time] = intval($trafficData['diff']);
            $lastTime = $time;
        }

        // add last value
        $from = $origEnd;
        $trafficData = $this->getLogEntityManager()->createQueryBuilder("a")
        ->select("DIV(a.endTime, 60000) * 60 AS endTime", "a.inSize - a.outSize AS diff")
        ->where("DIV(a.endTime, 1000) >= :from")
        ->andWhere("DIV(a.endTime, 1000) <= :to")
        ->groupBy("endTime")
        ->orderBy("a.id")
        ->setParameter("from", $from)
        ->setParameter("to", $to)
        ->getQuery()->setMaxResults(1)->getOneOrNullResult();
        ;

        if ($trafficData) {
            $time = $trafficData['endTime'];
            $rv[$time] = intval($trafficData['diff']);
            $lastTime = $time;
        }

        return Interpolator::interpolateArray($rv, $origStart, $origEnd);
    }

    /**
     * Returns the traffic for the last specified number of days. The value 1
     * is the current day.
     *
     * @param int $count The number of days.
     * @return \stdClass[] Array with a stdClass object for each day Each object
     *     has the properties date, insize, outsize, diff. The property date is
     *     a string in the form YYYY-MM-DD.
     */
    public function getTrafficDayLast($count)
    {
        static $sql = "SELECT * FROM (
            SELECT `date`, insize, outsize, diff FROM %s ORDER BY `date` DESC LIMIT %s
            ) tmp ORDER BY tmp.`date` ASC";
        $traffics = $this->getDB()->queryObjects(sprintf($sql, $this->getTableDay(), intval($count) - 1));

        static $sqlDay = "SELECT DATE(FROM_UNIXTIME(starttime/1000)) AS `date`, SUM(insize) AS insize,
            SUM(outsize) AS outsize, SUM(insize) - SUM(outsize) AS diff
            FROM %s GROUP BY `date` HAVING `date` = CURDATE()";
        $trafficToday = $this->getDB()->queryObject(sprintf($sqlDay, $this->getTableLog()));
        if ($trafficToday === null) {
            // no traffic today, yet
            $trafficToday = new \stdClass();
            $trafficToday->insize = 0;
            $trafficToday->outsize = 0;
            $trafficToday->diff = 0;
        }
        $date = new \DateTime();
        $zone = new \DateTimeZone(date_default_timezone_get());
        $trafficToday->date = date("Y-m-d H:i:s", $date->getTimestamp() + $zone->getOffset($date));
        $traffics[] = $trafficToday;

        return $traffics;
    }

    /**
     * Returns the traffic for each month in the specified period.
     * Note: the returned objects will always contain the values for a full month,
     * meaning that e.g. $from is 2013-01-22, you will get the complete traffic
     * in January 2013.
     *
     * @param array|string|int $from The beginning of the requested period.
     *     Might be an array with the entries 'year' and 'month', a date string,
     *     that can be parsed by strtotime(), or a UNIX timestamp.
     * @param array|string|int $to The end of the requested period.
     *     Might be an array with the entries 'year' and 'month', a date string,
     *     that can be parsed by strtotime(), or a UNIX timestamp.
     * @return \stdClass[] Array with a stdClass object for each month. Each object has the
     *     properties date, insize, outsize, diff. The property date is a string
     *     in the form YYYY-MM-01. The entries are sorted descending by date.
     */
    public function getTrafficMonthEach($from, $to = null)
    {
        static $sql = "SELECT `date`, insize, outsize, diff FROM %s WHERE `date` >= :from AND `date` <= :to";
        $from = $this->getFirstOfMonth($from);
        if ($to === null) {
            $to = $from;
        } else {
            $to = $this->getFirstOfMonth($to);
        }
        if ($to == $this->getFirstOfMonth(time())) {
            // use current values for current month
            $trafficMonth = new \stdClass();
            $trafficMonth->date = $to;
            $trafficMonth->insize = 0;
            $trafficMonth->outsize = 0;
            $trafficMonth->diff = 0;
            $days = intval(date("j"));
            $trafficDays = $this->getTrafficDayLast($days);
            foreach ($trafficDays as $trafficDay) {
                $trafficMonth->insize += $trafficDay->insize;
                $trafficMonth->outsize += $trafficDay->outsize;
                $trafficMonth->diff += $trafficDay->diff;
            }
            $to = $this->getFirstOfMonth(strtotime($to) - 1);
            $trafficMonths = $this->getDB()->queryObjects(
                sprintf($sql, $this->getTableYearMonth()), array("from" => $from, "to" => $to)
            );
            return array_merge(array($trafficMonth), $trafficMonths);
        } else {
            return $this->getDB()->queryObjects(
                sprintf($sql, $this->getTableYearMonth()), array("from" => $from, "to" => $to));
        }
    }

    /**
     * Returns the total traffic for the specified year.
     *
     * @param int|string $year The year to get the traffic for.
     * @return \stdClass Object containing the properties year, insize, outsize and diff.
     */
    public function getTrafficYear($year)
    {
        static $sql = "SELECT :year AS year, SUM(insize) AS insize,
            SUM(outsize) AS outsize, SUM(diff) AS diff
            FROM %s WHERE `date` >= :from AND `date` <= :to";
        $params = array("year" => $year, "from" => "$year-01-01", "to" => "$year-31-01");
        return $this->getDB()->queryObject(sprintf($sql, $this->getTableYearMonth()), $params);
    }

    /**
     * Returns a string in the form YYYY-MM-01 for the specified variable.
     *
     * @param array|string|int $time Might be an array with the entries 'year'
     *     and 'month', a date string, that can be parsed by strtotime(), or a
     *     UNIX timestamp.
     * @return string A string in the form YYYY-MM-01.
     */
    protected function getFirstOfMonth($time)
    {
        if (is_array($time)) {
            if (strlen($time['month']) == 1) {
                return $time['year']."-0".$time['month']."-01";
            }
            return $time['year']."-".$time['month']."-01";
        }
        if (is_string($time)) {
            $time = strtotime($time);
        }
        return date("Y-m-01", $time);
    }
}
