<?php
namespace Topalis\ActiveSyncFilter\Plugin;

use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Configuration\Ini;
use Topalis\Util\ErrorCodeTranslator;

/**
 * This class allows access to the meta data of a plugin and its configuration.
 *
 * @author Dominic Tubach
 */
class Plugin
{
    private $filepath;
    private $pluginManager;
    private $key;
    private $name;
    private $nameTrKey;
    private $desc;
    private $descTrKey;
    private $configMeta;
    private $translation = array();

    /**
     * Constructor.
     *
     * @param string $filepath The path to the plugin file.
     * @param PluginManager $pluginManager the plugin manager that manages this
     *   plugin.
     */
    public function __construct($filepath, PluginManager $pluginManager)
    {
        $this->filepath = $filepath;
        $this->pluginManager = $pluginManager;
        $this->load($filepath);
    }

    /**
     * @param \SimpleXMLElement $xml The {@link \SimpleXMLElement} containing
     *   the translation key.
     * @return string|NULL The translation key defined in the specified
     *   {@link \SimpleXMLElement}, or null if it does not exist.
     */
    public static function getTrKey(\SimpleXMLElement $xml)
    {
        $attributes = $xml->attributes();
        foreach ($attributes as $key => $value) {
            if ($key == "trKey") {
                return (string) $value;
            }
        }

        return null;
    }

    /**
     * @return string The path to the plugin file.
     */
    public function getFilepath()
    {
        return $this->filepath;
    }

    /**
     * @return string The file name exluding the path of the plugin file.
     */
    public function getFilename()
    {
        return substr(strrchr($this->getFilepath(), "/"), 1);
    }

    /**
     * @return PluginManager The plugin manager that manages this plugin.
     */
    public function getPluginManager()
    {
        return $this->pluginManager;
    }

    /**
     * @return string The plugin key.
     */
    public function getKey()
    {
        return $this->key;
    }

    /**
     * @param boolean $localized Specifies if the returned name shall be
     *   localized if possible.
     * @return string The plugin name.
     */
    public function getName($localized = true)
    {
        if ($localized) {
            return $this->translate($this->nameTrKey, $this->name);
        }
        return $this->name;
    }

    /**
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $config The configuration to access.
     *
     * @return PluginConfig The plugin configuration.
     */
    public function getConfig(ConfigGroup $config)
    {
        return new PluginConfig($this->configMeta, $config);
    }

    /**
     * @param boolean $localized Specifies if the returned description shall be
     *   localized if possible.
     * @return string The plugin description.
     */
    public function getDescription($localized = true)
    {
        if ($localized) {
            return $this->translate($this->descTrKey, $this->desc);
        }
        return $this->desc;
    }

    /**
     * Returns the translation for the specified key using the browser language.
     * @param string $key the key of the string to translate.
     * @param string $default the default string if no translation with the
     *   specified key exists.
     * @return string The translated string or $default if no translation exists.
     */
    public function translate($key, $default = null)
    {
        if (array_key_exists($key, $this->translation)) {
            return $this->translation[$key];
        }

        return $default === null ? $key : $default;
    }

    /**
     * Loads the plugin.
     *
     * @param string $filepath The path to the plugin file.
     * @throws \Exception If the plugin jar cannot be opened.
     */
    private function load($filepath)
    {
        $jar = new \ZipArchive();
        $res = $jar->open($filepath);
        if ($res !== true) {
            throw new \Exception("failed to open '" . $filepath . "': ". ErrorCodeTranslator::zipError($res));
        }

        $xml = $jar->getFromName("plugin.xml");

        $lang = $this->getPluginManager()->getLanguage();
        if ($lang !== null) {
            $baselang = strstr($lang, "_", true);
            if ($baselang) {
                $this->loadTranslation($baselang, $jar);
            }
            $this->loadTranslation($lang, $jar);
        }

        $jar->close();

        $this->parsePluginXml($xml);
    }

    /**
     * Loads translation file for the specified language. If a previous
     * translated string exists it will be overwritten.
     *
     * @param string $lang The language to load.
     * @param \ZipArchive $jar The jar containing the translation file.
     */
    private function loadTranslation($lang, \ZipArchive $jar)
    {
        $trBundle = $jar->getFromName($lang . ".properties");
        if (!$trBundle) {
            return;
        }
        try {
            $trBundleIni = Ini::parse($trBundle, false, true);
        } catch (\Exception $e) {
            // Do nothing if we can't parse the translation bundle.
            return;
        }
        foreach ($trBundleIni->getProperties() as $key => $string) {
            $this->translation[$key] = $string;
        }
    }

    /**
     * Parses the given XML document that specifies the plugin.
     *
     * @param string $xml XML document that specifies the plugin.
     */
    private function parsePluginXml($xml)
    {
        $plugin = new \SimpleXMLElement($xml);
        $this->key = (string) $plugin->key;
        $this->name = (string) $plugin->name;
        $this->nameTrKey = self::getTrKey($plugin->name);
        $this->desc = (string) $plugin->desc;
        $this->descTrKey = self::getTrKey($plugin->desc);
        $this->configMeta = new PluginConfigMeta($plugin->config, $this);
    }
}
