<?php
namespace Topalis\ActiveSyncFilter\Plugin;

use Topalis\Util\StringUtil;

/**
 * Implementation of {@link PluginConfigEntryMetaInterface}.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class PluginConfigEntryMeta implements PluginConfigEntryMetaInterface
{
    const MAX_LENGTH = 255;
    private static $MIN_INT;
    const MAX_INT = PHP_INT_MAX;

    private $name;
    private $nameTrKey;
    private $key;
    private $default;
    private $type;
    private $desc;
    private $descTrKey;
    private $min;
    private $max;
    private $options;
    private $disables = array();
    private $requiredBy = array();
    private $plugin;

    /**
     * Constructor.
     *
     * @param \SimpleXmlElement $entryXml The {@link \SimpleXMLElement}
     *         containing the description of the configuration entry.
     * @param Plugin $plugin The plugin the entry belongs to.
     */
    public function __construct(\SimpleXMLElement $entryXml, Plugin $plugin)
    {
        self::$MIN_INT = (PHP_INT_MAX * -1)-1;
        $this->plugin = $plugin;
        $this->name = (string) $entryXml->name;
        $this->nameTrKey = Plugin::getTrKey($entryXml->name);
        $this->key = (string) $entryXml->key;
        $this->default = (string) $entryXml->default;
        $this->type = (string) $entryXml->type;
        $desc = $entryXml->desc->asXML();
        // remove whitespaces
        $desc = preg_replace('/\s\s+/', ' ', $desc);
        // remove outer tags
        $desc = substr($desc, strpos($desc, '>') + 1, - (strlen("desc") + 4));
        $this->desc = trim($desc);
        $this->descTrKey = Plugin::getTrKey($entryXml->desc);
        if ($entryXml->min !== null && is_numeric((string) $entryXml->min)) {
            $this->min = intval($entryXml->min);
        } else if ($this->isInt()) {
            $this->min = self::$MIN_INT;
        } else {
            $this->min = 0;
        }
        if ($entryXml->max !== null && is_numeric((string) $entryXml->max)) {
            $this->max = intval($entryXml->max);
        } else if ($this->isInt()) {
            $this->max = self::MAX_INT;
        } else {
            $this->max = self::MAX_LENGTH;
        }

        $this->detectDependencies($entryXml);

        if ($this->isSelect()) {
            $this->initSelect($entryXml);
        }
    }

    /**
     * Detects the dependencies of the config entry.
     *
     * @param \SimpleXmlElement $entryXml The {@link \SimpleXMLElement}
     *         containing the description of the configuration entry.
     */
    protected function detectDependencies(\SimpleXMLElement $entryXml)
    {
        $disables = $entryXml->disables;
        if ($disables !== null && $disables->count() > 0) {
            foreach ($disables->children() as $child) {
                if ($child->getName() != "key") {
                    continue;
                }
                $this->disables[] = (string) $child;
            }
        }

        $requiredBy = $entryXml->requiredby;
        if ($requiredBy !== null && $requiredBy) {
            foreach ($requiredBy->children() as $child) {
                if ($child->getName() != "key") {
                    continue;
                }
                $this->requiredBy[] = (string) $child;
            }
        }
    }

    /**
     * Initializes the options of a config entry of type "select".
     * @param \SimpleXmlElement $entryXml The {@link \SimpleXMLElement}
     *         containing the description of the configuration entry.
     */
    protected function initSelect(\SimpleXMLElement $entryXml)
    {
        $this->max = 0;
        $this->min = self::MAX_LENGTH;
        // initialize options
        $options = $entryXml->options;
        foreach ($options->children() as $child) {
            if ($child->getName() != "option") {
                continue;
            }
            $option = (string) $child;
            // use node text as value by default
            $value = $option;
            foreach ($child->attributes() as $key => $val) {
                if ($key == "value") {
                    /*
                     * attribute "value" exists, so use this attribute
                    * instead of node text
                    */
                    $value = (string) $val;
                } else if ($key == "trKey") {
                    $option = $this->getPlugin()->translate((string) $val, $option);
                }
            }
            $this->max = max($this->max, strlen($value));
            $this->min = min($this->min, strlen($value));
            $this->options[$option] = $value;
        }
    }

    /**
     * @param boolean $localized Specifies if the returned name shall be
     *   localized if possible.
     * @return string The name of the configuration entry.
     */
    public function getName($localized = true)
    {
        if ($localized) {
            return $this->plugin->translate($this->nameTrKey, $this->name);
        }
        return $this->name;
    }

    /**
     * @return string The key of the configuration entry.
     */
    public function getKey()
    {
        return $this->key;
    }

    /**
     * @return string The default value.
     */
    public function getDefault()
    {
        return $this->default;
    }

    /**
     * @return string The type of the configuration entry.
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * Returns the localized options for entries of type select.
     *
     * @return string[string] An associative array mapping options to values.
     */
    public function getOptions()
    {
        return $this->options;
    }

    /**
     * @return integer Returns the maximal allowed value for number entries.
     *         For all other types of entries it returns the maximal length of
     *         allowed values. If no value is set PHP_INT_MAX is returned
     *         for number entries and {@link MAX_INT} for all other entries.
     */
    public function getMax()
    {
        return $this->max;
    }

    /**
     * @return integer Returns the minimal allowed value for number entries.
     *         For all other types of entries it returns the minimal length of
     *         allowed values. If no value is set (PHP_INT_MAX * -1)-1 is
     *         returned for numbers and 0 for all other entries.
     */
    public function getMin()
    {
        return $this->min;
    }

    /**
     * @return integer The maximal string length of allowed values. Returns the
     *         same value as {@link getMax()} for non-number entries. For number
     *         entries the maximal length is calculated based on the values
     *         returned by {@link getMax()} and {@link getMin()}.
     */
    public function getMaxLength()
    {
        if ($this->getMax() === null) {
            return self::MAX_LENGTH;
        }
        if (!$this->isInt()) {
            return $this->getMax();
        }

        if ($this->getMin() === null) {
            // allow large negative numbers if no minimum is defined
            return self::MAX_LENGTH;
        }

        $minLength = strlen((string) $this->getMin());
        $maxLength = strlen((string) $this->getMax());
        return max(array($minLength, $maxLength));
    }

    /**
     * @return integer The minimal string length of allowed values. Returns the
     *         same value as {@link getMin()} for non-number entries. For number
     *         entries the minimal length is calculated based on the values
     *         returned by {@link getMax()} and {@link getMin()}.
     */
    public function getMinLength()
    {
        if (!$this->isInt()) {
            return $this->getMin();
        }

        $min = $this->getMin();
        $max = $this->getMax();
        if ($min < 0 && $max < 0) {
            return strlen((string) $max);
        }

        if ($min > 0 && $max > 0) {
            return strlen((string) $min);
        }

        return 1;
    }

    /**
     * @param boolean $localized Specifies if the returned description shall be
     *   localized if possible.
     * @return string The description of the configuration entry.
     */
    public function getDesc($localized = true)
    {
        if ($localized) {
            return $this->plugin->translate($this->descTrKey, $this->desc);
        }
        return $this->desc;
    }

    /**
     * @return string[] Array of keys of entries that have to be disabled if
     *         this entry is enabled. This is only relevant for entries of type
     *         bool.
     */
    public function getDisablesKeys()
    {
        return $this->disables;
    }

    /**
     * @return string[] Array of keys of entries that require this entry to be
     *         enabled. This is only relevant for entries of type boolean.
     */
    public function getRequiredByKeys()
    {
        return $this->requiredBy;
    }

    /**
     * @return Plugin The plugin the configuration entry belongs to.
     */
    public function getPlugin()
    {
        return $this->plugin;
    }

    /**
     * @return boolean true if this entry stores a boolean, false otherwise.
     */
    public function isBool()
    {
        return $this->getType() == "boolean";
    }

    /**
     * @return boolean true if this entry stores an integer, false otherwise.
     */
    public function isInt()
    {
        return $this->getType() == "integer";
    }

    /**
     * @return boolean true if this entry is a select (e.g. only the predefined
     *         values are allowed), false otherwise.
     * @see getOptions()
     */
    public function isSelect()
    {
        return $this->getType() == "select";
    }

    /**
     * @return boolean true if this entry stores a string, false otherwise.
     */
    public function isString()
    {
        return $this->getType() == "string";
    }

    /**
     * Converts the specified value to a string that can be stored.
     *
     * @param string|integer|boolean $value The value to be converted.
     * @return string The converted value.
     */
    public function toPropertyStr($value)
    {
        if ($this->isBool() && is_bool($value)) {
            // returns either "true" or "false"
            return StringUtil::booltostr($value);
        }

        return (string) $value;
    }

    /**
     * Checks if the specified value is a valid value for the configuration
     * entry.
     *
     * @param string $value The value to be checked. For entries of type integer
     *         the value might be an integer. For entries of type boolean the
     *         value might be a boolean.
     * @return boolean true if the value is valid, false otherwise.
     */
    public function validateValue($value)
    {
        if ($this->isBool()) {
            if (is_bool($value)) {
                return true;
            }
            $bool = StringUtil::strtobool($value, true);
            return $bool !== null;
        }

        if ($this->isString()) {
            if ($this->getMinLength() !== null && strlen($value) < $this->getMinLength()) {
                return false;
            }
            if ($this->getMaxLength() !== null && strlen($value) > $this->getMaxLength()) {
                return false;
            }
            return true;
        }

        if ($this->isInt()) {
            return $this->validateInt($value);
        }

        if ($this->isSelect()) {
            foreach ($this->getOptions() as $val) {
                if ($value == $val) {
                    return true;
                }
            }
            return false;
        }

        return true;
    }

    protected function validateInt($value)
    {
        if (is_int($value)) {
            $int = $value;
        } else if (preg_match('/^(0|(-{0,1}[1-9]\d*))$/', trim($value))) {
            $int = intval($value);
        } else {
            return false;
        }
        if ($this->getMin() !== null && $int < $this->getMin()) {
            return false;
        }
        if ($this->getMax() !== null && $int > $this->getMax()) {
            return false;
        }
        return true;
    }

}
