<?php
namespace Topalis\ActiveSyncFilter\Plugin;

/**
 * A PluginConfigEntryMeta gives access to all data and methods that are
 * independent of the actual setting, but depends on the options defined in the
 * plugin description.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
interface PluginConfigEntryMetaInterface
{
    /**
     * @param boolean $localized Specifies if the returned name shall be
     *   localized if possible.
     * @return string The name of the configuration entry.
     */
    public function getName($localized = true);

    /**
     * @return string The key of the configuration entry.
     */
    public function getKey();

    /**
     * @return string The default value.
     */
    public function getDefault();

    /**
     * @return string The type of the configuration entry.
     */
    public function getType();

    /**
     * Returns the localized options for entries of type select.
     *
     * @return string[string] An associative array mapping options to values.
     */
    public function getOptions();

    /**
     * @return integer Returns the maximal allowed value for number entries.
     *         For all other types of entries it returns the maximal length of
     *         allowed values. If no value is set PHP_INT_MAX is returned
     *         for number entries and {@link MAX_INT} for all other entries.
     */
    public function getMax();

    /**
     * @return integer Returns the minimal allowed value for number entries.
     *         For all other types of entries it returns the minimal length of
     *         allowed values. If no value is set (PHP_INT_MAX * -1)-1 is
     *         returned for numbers and 0 for all other entries.
     */
    public function getMin();

    /**
     * @return integer The maximal string length of allowed values. Returns the
     *         same value as {@link getMax()} for non-number entries. For number
     *         entries the maximal length is calculated based on the values
     *         returned by {@link getMax()} and {@link getMin()}.
     */
    public function getMaxLength();

    /**
     * @return integer The minimal string length of allowed values. Returns the
     *         same value as {@link getMin()} for non-number entries. For number
     *         entries the minimal length is calculated based on the values
     *         returned by {@link getMax()} and {@link getMin()}.
     */
    public function getMinLength();

    /**
     * @param boolean $localized Specifies if the returned description shall be
     *   localized if possible.
     * @return string The description of the configuration entry.
     */
    public function getDesc($localized = true);

    /**
     * @return string[] Array of keys of entries that have to be disabled if
     *         this entry is enabled. This is only relevant for entries of type
     *         bool.
     */
    public function getDisablesKeys();

    /**
     * @return string[] Array of keys of entries that require this entry to be
     *         enabled. This is only relevant for entries of type boolean.
     */
    public function getRequiredByKeys();

    /**
     * @return Plugin The plugin the configuration entry belongs to.
     */
    public function getPlugin();

    /**
     * @return boolean true if this entry stores a boolean, false otherwise.
     */
    public function isBool();

    /**
     * @return boolean true if this entry stores an integer, false otherwise.
     */
    public function isInt();

    /**
     * @return boolean true if this entry is a select (e.g. only the predefined
     *         values are allowed), false otherwise.
     * @see getOptions()
     */
    public function isSelect();

    /**
     * @return boolean true if this entry stores a string, false otherwise.
     */
    public function isString();

    /**
     * Checks if the specified value is a valid value for the configuration
     * entry.
     *
     * @param string $value The value to be checked. For entries of type integer
     *         the value might be an integer. For entries of type boolean the
     *         value might be a boolean.
     * @return boolean true if the value is valid, false otherwise.
     */
    public function validateValue($value);

    /**
     * Converts the specified value to a string that can be stored.
     *
     * @param string|integer|boolean $value The value to be converted.
     * @return string The converted value.
     */
    public function toPropertyStr($value);
}
