<?php
namespace Topalis\ActiveSyncFilter\Plugin;

use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Util\StringUtil;
use Topalis\ActiveSyncFilter\FilterServer;

/**
 * This class allows to access plugins.
 *
 * @author Dominic Tubach
 */
class PluginManager
{
    /**
     * @var \Topalis\ActiveSyncFilter\FilterServer
     */
    private $filterServer;

    /**
     * @var string
     */
    private $language;

    /**
     * @var Plugin[]
     */
    private $plugins;

    /**
     * Constructor.
     *
     * @param FilterServer $filterServer The filter server for which the plugins shall be managed.
     * @param string $language The language to use.
     */
    public function __construct(FilterServer $filterServer, $language = null)
    {
        $this->filterServer = $filterServer;
        $this->language = $language;
        $this->loadPlugins();
    }

    /**
     * Clears all plugin related configuration for the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config group to
     *                                                             clear the plugin related configuration.
     */
    public function clear(ConfigGroup $configGroup)
    {
        foreach ($this->getPlugins() as $plugin) {
            $plugin->getConfig($configGroup)->clear();
        }
        $config = new PluginsConfig($this->getFilterServer(), $configGroup);
        $config->clear();
    }

    /**
     * @param ConfigGroup $config The configuration to access.
     * @param string $key Plugin key.
     * @return boolean true if the plugin with the specified key is enabled,
     *   otherwise false.
     */
    public function isEnabled(ConfigGroup $config, $key)
    {
        $config = new PluginsConfig($this->getFilterServer(), $config);
        return $config->isEnabled($key);
    }

    /**
     * @return FilterServer The {@link FilterServer} of this PluginManager.
     */
    public function getFilterServer()
    {
        return $this->filterServer;
    }

    /**
     * @return string the language to use.
     */
    public function getLanguage()
    {
        return $this->language;
    }

    /**
     * @return Plugin[string] Associative array of plugins with the
     * plugin's key as key.
     */
    public function getPlugins()
    {
        return $this->plugins;
    }

    /**
     * @param string $key Plugin key
     * @return Plugin|NULL The plugin with the given key,
     *   or null if no such plugin exists.
     */
    public function getPlugin($key)
    {
        if (array_key_exists($key, $this->plugins)) {
            return $this->plugins[$key];
        }
        return null;
    }

    /**
     * Enables or disables the plugin with the specified key.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config
     *                                                             group of the configuration to access.
     * @param string                                  $key         The key of the plugin to enable or disable
     * @param boolean                                 $enabled     true to enable, false to disable the plugin.
     *
     * @return bool
     */
    public function setEnabled(ConfigGroup $configGroup, $key, $enabled)
    {
        $config = new PluginsConfig($this->getFilterServer(), $configGroup);
        return $config->setEnabled($key, $enabled);
    }

    /**
     * Enables or disables plugins based on the boolean for the correspondig
     * plugin key.
     *
     * @param ConfigGroup $config The configuration to access.
     * @param boolean[string] $pluginsStatus Associative array with key as
     *   plugin's key and a boolean as value. true to enable the plugin and
     *   false to disable it.
     * @return boolean true if the status of at least one plugin was changed,
     *     false otherwise.
     */
    public function setPluginsStatus(ConfigGroup $config, array $pluginsStatus)
    {
        $config = new PluginsConfig($this->getFilterServer(), $config);
        $changed = false;
        foreach ($pluginsStatus as $key => $enabled) {
            if ($config->setEnabled($key, $enabled)) {
                $changed = true;
            }
        }
        return $changed;
    }

    /**
     * @return string[] The file names of the existing plugins (including path).
     */
    private function getPluginFilenames()
    {
        $filenames = array();
        $pluginPath = $this->getPluginFolder();
        $pluginsDir = dir($pluginPath);
        while (($filename = $pluginsDir->read()) != false) {
            if (StringUtil::endsWith($filename, ".jar") && is_file($pluginPath . '/' . $filename)) {
                $filenames[] = $this->getPluginFolder() . '/' . $filename;
            }
        }

        return $filenames;
    }

    /**
     * @return string The path to the plugin folder.
     */
    private function getPluginFolder()
    {
        return $this->getFilterServer()->getServerConfig()->getRealPluginFolder();
    }

    /**
     * Loads all available plugins. Creates a {@link Plugin} object for every
     * plugin.
     */
    private function loadPlugins()
    {
        $this->plugins = array();
        foreach ($this->getPluginFilenames() as $filename) {
            $plugin = new Plugin($filename, $this);
            if (!array_key_exists($plugin->getKey(), $this->plugins)) {
                $this->plugins[$plugin->getKey()] = $plugin;
            }
        }
        ksort($this->plugins);
    }
}
