<?php
namespace Topalis\Admin;

use Topalis\Admin\Entity\Admin;
use Topalis\Apache\Apache;
use Topalis\Util\PasswordUtil;
use Topalis\Admin\Entity\AdminEntityManager;

class AdminManager
{
    /**
     * @var \Topalis\Database\MySqlDb
     */
    protected $db;

    /**
     * @var \Topalis\Admin\Entity\AdminEntityManager
     */
    private $entityManager;

    /**
     * @var array
     */
    private $listeners = array();

    public function __construct(\Topalis\Database\MySqlDb $db, AdminEntityManager $entityManager)
    {
        $this->db = $db;
        $this->entityManager = $entityManager;
    }

    /**
     * @param AdminManagerListener $listener
     */
    public function addListener(AdminManagerListenerInterface $listener)
    {
        $this->listeners[] = $listener;
    }

    /**
     * @return array
     */
    public function getListeners()
    {
        return $this->listeners;
    }

    /**
     * Creates a new administrative user with the specified values.
     *
     * @param array $values The values to use.
     * @return the id of the new admin.
     * @throws \Exception On error.
     */
    public function createAdmin(array $values)
    {
        $admin = $this->createAdminObject($values);
        $this->db->insert('admins', $admin, 'id');
        foreach ($this->getListeners() as $listener) {
            $listener->adminAdded($admin->id);
        }

        return $admin->id;
    }

    /**
     * Saves the specified adminsitrative user.
     *
     * @param Admin $admin The administrative user to save.
     */
    public function saveAdmin(Admin $admin)
    {
        $new = $admin->getId() === null;
        $this->getEntityManager()->save($admin);
        if ($new) {
            foreach ($this->getListeners() as $listener) {
                $listener->adminAdded($admin->getId());
            }
        } else {
            foreach ($this->getListeners() as $listener) {
                $listener->adminUpdated($admin->getId());
            }
        }
    }

    /**
     * Updates the administrative user with the specified id using the specified
     * values.
     *
     * @param int $id The id of the administrative user to update.
     * @param array $values The values to use.
     * @throws \Exception On error.
     */
    public function updateAdmin($id, array $values)
    {
        $admin = $this->createAdminObject($values);
        $admin->id = $id;
        if ($this->db->update('admins', $admin)) {
            foreach ($this->getListeners() as $listener) {
                $listener->adminUpdated($admin->id);
            }
        }
    }

    /**
     * @param string $name login name.
     * @return boolean true if an administrative user with the specified login
     *     name exists, false otherwise.
     */
    public function existsByName($name)
    {
        return $this->db->exists("admins", array("login" => $name));
    }

    /**
     * @return boolean true if the initial admin exists (i.e. the administrative
     *     user with the name 'admin'), false otherwise.
     */
    public function existsInitialAdmin()
    {
        return $this->existsByName("admin");
    }

    /**
     * @param string $name The name of the admin to get.
     * @return Admin|NULL The admin with the specified name, or null if no such
     *     admin exists.
     */
    public function getAdminByName($name)
    {
        return $this->getEntityManager()->findOneBy(array("login" => $name));
    }

    /**
     * Returns the admins with the specified columns in the specified order.
     *
     * @param string $sortField The column to sort the admins by.
     * @param string $sortDir The sort direction ("ASC" or "DESC").
     * @return stdClass[] the admins with the specified columns in the specified
     *     order.
     */
    public function getAdmins($sortField = null, $sortDir = "ASC")
    {
        $sortColumns = array();
        if ($sortField !== null) {
            $sortColumns[] = $sortField . " " . $sortDir;
        }
        return $this->db->fetchObjects("admins", array("*"), array("id" => array(">", "0")), $sortColumns);
    }

    /**
     * Retrieve the full name of given administrator.
     *
     * @param string $uname The login name to look up. Defaults to the current user.
     * @return string|null The full name of the given user, if found. Otherwise null.
     */
    public function getFullName($uname = "")
    {
        if ($uname === "") {
            if (!isset($_SERVER['PHP_AUTH_USER'])) {
                return null;
            }
            $uname = $_SERVER['PHP_AUTH_USER'];
        }
        $user = $this->db->fetchObject("admins", array("*"), array("login" => $uname));
        if ($user === null) {
            return null;
        }
        $fullName = null;
        if (strlen($user->realname) > 0) {
            $fullName = $user->realname;
        }
        return $fullName;
    }

    /**
     * Delete an admin user
     *
     * @param int $id The ID of the record to delete.
     * @return boolean TRUE on success, FALSE otherwise.
     * @throws DatabaseException On database error.
     */
    public function deleteAdminUser($id)
    {
        if ($this->db->removeRows("admins", array("id" => $id))) {
            foreach ($this->getListeners() as $listener) {
                $listener->adminDeleted($id);
            }
            return true;
        }

        return false;
    }

    /**
     * Deletes the administrative user with the specified name.
     *
     * @param string $name The name of the administrative user to delete.
     */
    public function deleteAdminByName($name)
    {
        $this->getEntityManager()->deleteBy(array("login" => $name));
    }

    /**
     * Delete an administrative user.
     *
     * @param Admin $admin The administrative user to delete.
     * @return boolean TRUE on success, FALSE otherwise.
     * @throws DatabaseException On database error.
     */
    public function deleteAdmin(Admin $admin)
    {
        return $this->deleteAdminUser($admin->getId());
    }

    protected static function createAdminObject(array $values)
    {
        $adminobj = new \stdClass;
        $adminobj->login = $values['login'];
        $adminobj->level = $values['level'];
        $adminobj->realname = $values['realName'];
        if (isset($values['password'])) {
            $password = $values['password'];
            $adminobj->pwhash = PasswordUtil::createSaltedPassword($password);
            $adminobj->davhash = Apache::genHtpasswdHash($password);
        }

        return $adminobj;
    }

    /**
     * @return \Topalis\Admin\Entity\AdminEntityManager
     */
    protected function getEntityManager()
    {
        return $this->entityManager;
    }
}
