<?php
namespace Topalis\Aggregate\Command;

use Symfony\Component\Console\Command\Command;
use Doctrine\DBAL\Connection;

/**
 * This class aggregates device access log by year and month.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class AggregateAccessStatYearMonthCommand extends Command
{
    /**
     * @var \Symfony\Bridge\Doctrine\RegistryInterface
     */
    private $doctrine;

    /**
     *
     * @var \Topalis\Kernel\TimeProviderInterface
     */
    private $timeProvider;

    /**
     *
     * @var string
     */
    private $table = "access_stat_year_month";

    /**
     * @return \Symfony\Bridge\Doctrine\RegistryInterface
     */
    private function getDoctrine()
    {
        return $this->doctrine;
    }

    /**
     *
     * @param \Symfony\Bridge\Doctrine\RegistryInterface $doctrine
     */
    public function setDoctrine(\Symfony\Bridge\Doctrine\RegistryInterface $doctrine)
    {
        $this->doctrine = $doctrine;
    }

    /**
     * @return \Topalis\Kernel\TimeProviderInterface
     */
    private function getTimeProvider()
    {
        return $this->timeProvider;
    }

    public function setTimeProvider(\Topalis\Kernel\TimeProviderInterface $timeProvider)
    {
        $this->timeProvider = $timeProvider;
    }

    protected function configure()
    {
        $this
        ->setName("aggregate:access_stat_year_month")
        ->setDescription("Aggregates device access log by month.")
        ->setHelp("Aggregates device access log by month.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        $conn = $this->getDoctrine()->getConnection();
        $conn->beginTransaction();
        try {
            $this->aggregate();
            $conn->commit();
        } catch (\Exception $e) {
            $conn->rollBack();
            if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                $output = $output->getErrorOutput();
            }
            $output->writeln("Error updating table " . $this->table . ":");
            $output->writeln((string) $e);
            return -1;
        }
    }

    /**
     * Updates contents in table access_stat_year_month.
     */
    public function aggregate()
    {
        $conn = $this->getDoctrine()->getConnection();

        $lastDate = $this->getLastDate($conn);
        if ($lastDate !== null) {
            $conn->executeUpdate("DELETE FROM " . $this->table . " WHERE date = :date", array("date" => $lastDate));
        } else {
            $lastDate = $this->getMinStartTime($conn);
            if ($lastDate === null) {
                // no log entries, nothing to do
                return;
            }
        }

        $curTime = $this->getTimeProvider()->getTime();
        $curYear = date("Y", $curTime);
        $curMonth = date("n", $curTime);
        $lastYear = date("Y", strtotime($lastDate));
        $lastMonth = date("n", strtotime($lastDate));
        $monthsElapsed = ($curYear - $lastYear) * 12 + ($curMonth - $lastMonth);

        $year = $lastYear;
        $month = $lastMonth;
        $firstDayOfMonth = $year . "-" . $month . "-01";
        for ($i = 0; $i <= $monthsElapsed; $i++) {
            $month++;
            if ($month > 12) {
                $month = 1;
                $year++;
            }
            $firstDayOfNextMonth = $year . "-" . $month . "-01";
            $sql = "INSERT INTO " . $this->table . " (`date`, traffic, access_count, distinct_ips, distinct_devices)
                SELECT :firstDayOfMonth, IFNULL(SUM(traffic), 0), IFNULL(COUNT(id), 0), IFNULL(COUNT(DISTINCT ip), 0),
                IFNULL(COUNT(DISTINCT device_id), 0) from accesslog
                WHERE DATE(tstamp) >= :firstDayOfMonth and DATE(tstamp) < :firstDayOfNextMonth";
            $conn->executeUpdate($sql, array(
                "firstDayOfMonth" => $firstDayOfMonth,
                "firstDayOfNextMonth" => $firstDayOfNextMonth
            ));

            $firstDayOfMonth = $firstDayOfNextMonth;
        }
    }

    private function getLastDate(Connection $conn)
    {
        $rs = $conn->fetchArray("SELECT MAX(`date`) FROM " . $this->table);
        return isset($rs[0]) ? $rs[0] : null;
    }

    private function getMinStartTime(Connection $conn)
    {
        $rs = $conn->fetchArray("SELECT DATE(MIN(tstamp)) FROM " . $this->getLogTable());
        return isset($rs[0]) ? $rs[0] : null;
    }

    private function getLogTable()
    {
        return "accesslog";
    }
}
