<?php
namespace Topalis\Apache;

use Topalis\ReverseProxy\Entity\ReverseProxyUrl;
use Topalis\Util\Process;
use Topalis\Util\StringUtil;
use Topalis\Configuration\ApacheConfigLine;
use Mockery\Matcher\Not;

/**
 * Helper class for managing the Apache configuration.
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class Apache
{
    const ACTION_NONE = 0;
    const ACTION_RELOAD = 1;
    const ACTION_RESTART = 2;

    const ALLOWFROM_CONFIG = "allowfrom";
    const CLIENTCA_CONFIG = "clientca";
    const CLIENTCRL_CONFIG = "clientcrl";
    const DAV_AUTH_USER_CONFIG = "davaccess";
    const DEFINE_CONFIG = "defines";
    const LISTEN_CONFIG = "listen";
    const PROXYSETUP_CONFIG = "proxysetup";
    const RELOAD_COOKIE_CONFIG = "reloadcookie";
    const REVERSE_PROXY_CONFIG = "reverse_proxy";
    const REVERSE_PROXY_DIR = "reverse_proxy.d";
    const SERVERCERT_CONFIG = "servercert";
    const SERVERNAME_CONFIG = "hostname";

    /**
     * @var string
     */
    private $auralisConfigDir;

    /**
     * @var string
     */
    private $configDirBase;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * Flag: Perform a reload.
     * @var boolean
     */
    private $reload = false;

    /**
     * @var boolean $restart
     * Flag: Perform a restart in addition to a reload.
     */
    private $restart = false;

    /**
     * @var boolean indicates whether the shutdown function to reload/restart
     *     the Apache HTTPD was registered.
     * @see registerShutdownFunction()
     */
    private $shutdownFunctionRegistered = false;

    /**
     * Parses the specified "Define" line.
     *
     * @param string $line The line to parse.
     * @return string[]|null Array where the first entry is the variable name,
     *     and the second entry is the value. null if the specified line is not
     *     the line doesn't start with "Define ".
     */
    protected static function parseDefineLine($line)
    {
        $line = ltrim($line);
        if (!StringUtil::startsWith($line, "Define ")) {
            return null;
        }

        $define = trim(substr($line, 7));
        $define = explode(" ", $define, 2);
        $define[1] = str_replace("\\\"", "", trim($define[1], "\""));
        return $define;
    }

    /**
     * Constructor.
     *
     * @param \Psr\Log\LoggerInterface $logger The
     *     logger to use. Uses \Psr\Log\NullLogger by default.
     */
    public function __construct(\Psr\Log\LoggerInterface $logger = null)
    {
        if ($logger === null) {
            $logger = new \Psr\Log\NullLogger();
        }
        $this->logger = $logger;
    }

    /**
     * @return string The Apache HTTPD dir to store auralis related
     *     settings.
     */
    public function getAuralisConfigDir()
    {
        return $this->auralisConfigDir;
    }

    /**
     * @param string $auralisConfigDir The Apache HTTPD dir to store auralis
     *     related settings.
     */
    public function setAuralisConfigDir($auralisConfigDir)
    {
        $this->auralisConfigDir = $auralisConfigDir;
    }

    /**
     * @return string The base Apache HTTPD configuration directory (usually
     *     /etc/httpd).
     */
    public function getConfigDirBase()
    {
        return $this->configDirBase;
    }

    /**
     * @param string $configDirBase The base Apache HTTPD configuration
     *     directory (usually /etc/httpd).
     */
    public function setConfigDirBase($configDirBase)
    {
        $this->configDirBase = $configDirBase;
    }

    /**
     * @return \Psr\Log\LoggerInterface The logger.
     */
    public function getLogger()
    {
        return $this->logger;
    }

    /**
     * Sets the logger to use.
     *
     * @param \Psr\Log\LoggerInterface $logger The
     *     logger to set.
     */
    public function setLogger(\Psr\Log\LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    /**
     * Registers the shutdown function to reload/restart the Apache HTTPD.
     * @see enableReload()
     * @see enableRestart()
     * @see reload()
     */
    private function registerShutdownFunction()
    {
        if (!$this->shutdownFunctionRegistered) {
            $this->getLogger()->info("Register shutdown funtion to reload or restart Apache HTTPD");
            register_shutdown_function(array($this, 'reload'));
            $this->shutdownFunctionRegistered = true;
        }
    }

    /**
     * @return boolean true if reload or restart is enabled, false otherwise.
     */
    public function isReloadEnabled()
    {
        return $this->shutdownFunctionRegistered && $this->reload;
    }

    /**
     * Disables reload upon termination of PHP script.
     * Useful for tests.
     */
    public function disableReload()
    {
        $this->reload = false;
        $this->restart = false;
    }

    /**
     * Activate reload upon termination of PHP script.
     */
    public function enableReload()
    {
        $this->reload = true;
        $this->registerShutdownFunction();
    }

    /**
     * Activate restart upon termination of PHP script.
     */
    public function enableRestart()
    {
        $this->reload = true;
        $this->restart = true;
        $this->registerShutdownFunction();
    }

    /**
     * Callback, invoked by PHP upon termination of PHP script.
     * Performs reload and (optionally) a restart of apache.
     *
     * @return \Topalis\Util\Exception\Process\ExecException|null Exception if
     *     executing areload or arestart fails, null otherwise. Throwing an
     *     exception on script termination causes a fatal error.
     * @see registerShutdownFunction()
     */
    public function reload()
    {
        if (!$this->reload && !$this->restart) {
            return null;
        }

        try {
            if (PHP_SAPI != 'cli') {
                sleep(1); // prg delay doesn't get rendered sometimes
                $cookie = uniqid('reload_');
                $cfg = 'SetEnv RELOAD_COOKIE ' . $cookie;
                $this->writeConfig(self::RELOAD_COOKIE_CONFIG, $cfg);
                session_commit();
                // ob_get_level() never returns 0 on some Windows configurations, so if
                // the level is the same two times in a row, the loop should be stopped.
                $previous = null;
                $obStatus = ob_get_status(1);
                while (($level = ob_get_level()) > 0 && $level !== $previous) {
                    $previous = $level;
                    if ($obStatus[$level - 1] && isset($obStatus[$level - 1]['del']) && $obStatus[$level - 1]['del']) {
                        $this->getLogger()->debug("ob_end_flush", array("ob_status" => $obStatus[$level - 1]));
                        ob_end_flush();
                    }
                }
                // This flush is commented out because it stops executing here,
                // at least on Debian wheezy. In which case do we need it?
                // flush();
            }
            $out = '';
            $cmd = '/usr/sbin/areload';
            $this->logger->info("Reload Apache HTTPD");
            Process::execThrow($cmd, $out);
            if (PHP_SAPI != 'cli') {
                $port = $this->getDefine("auralis_secondary_port");
                $chkurl = "http" . (($_SERVER['HTTPS'] == "on") ? "s" : "") . "://localhost:"
                    . $port . "/admin/reload_status";
                $tmp = file_get_contents($chkurl);
                if ($tmp != $cookie) {
                    $this->logger->error('Reload timeout.');
                    // don't restart
                    return;
                }
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Reloading Apache HTTPD failed", array("exception" => $e));
            return $e;
        }

        try {
            if ($this->restart) {
                $cmd = '/usr/sbin/arestart';
                $this->logger->info("Restart Apache HTTPD");
                Process::execThrow($cmd, $out);
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Restarting Apache HTTPD failed", array("exception" => $e));
            return $e;
        }
    }

    /**
     * @param string $configFilename
     * @return string|NULL The content of the file, or null if the file doesn't
     *    exist.
     */
    public function readConfig($configFilename)
    {
        $filename = $this->getConfigPath($configFilename);
        if (is_file($filename)) {
            return file_get_contents($filename);
        }

        return null;
    }

    /**
     *
     * @param string $configFilename
     * @param int $flags Flags for file().
     * @return array The file in an array. Each element of the array corresponds
     *    to a line in the file, with the newline still attached. An empty
     *    array is returned if the file doesn't exist.
     */
    public function readConfigArray($configFilename, $flags = null)
    {
        $filename = $this->getConfigPath($configFilename);
        if (is_file($filename)) {
            return file($filename, $flags);
        }

        return array();
    }

    public function writeConfig($configFilename, $data)
    {
        $filename = $this->getConfigPath($configFilename);
        if (!is_file($filename)) {
            $dir = substr($filename, 0, strrpos($filename, DIRECTORY_SEPARATOR));
            if (!is_dir($dir)) {
                mkdir($dir, 0755, true);
            }
        }
        return file_put_contents($filename, $data, LOCK_EX);
    }

    public function getConfigHash($configFilename)
    {
        $filename = $this->getConfigPath($configFilename);
        return is_file($filename) ? md5_file($filename) : null;
    }

    /**
     * Returns the config line in the specified file with the specified name.
     * @param string $configFilename The configuration file name.
     * @param string $name The configuration name.
     * @return ApacheConfigLine The configuration line, or null if no matching
     *     line was found.
     */
    public function getConfigLine($configFilename, $name)
    {
        $configLines = $this->readConfigArray($configFilename, FILE_SKIP_EMPTY_LINES);
        $disabledLines = array();
        foreach ($configLines as $line) {
            $line = trim($line);
            if (StringUtil::startsWith($line, $name)) {
                return ApacheConfigLine::parse($line);
            }
            if (StringUtil::startsWith($line, "#")) {
                $line = trim(substr($line, 1));
                if (StringUtil::startsWith($line, $name)) {
                    $disabledLines[] = $line;
                }
            }
        }
        if (!empty($disabledLines)) {
            return ApacheConfigLine::parse($disabledLines[0]);
        }
        return null;
    }

    /**
     * Parses the file with defines and returns the value for the specified
     * variable name.
     *
     * @param string $name The variable name.
     * @return string|null The value for the specified variable name, or null
     *     if not defined.
     */
    public function getDefine($name)
    {
        $configLines = $this->readConfigArray(self::DEFINE_CONFIG, FILE_SKIP_EMPTY_LINES);
        foreach ($configLines as $line) {
            $define = self::parseDefineLine($line);
            if ($define === null) {
                continue;
            }

            if ($define[0] != $name) {
                continue;
            }

            return $define[1];
        }

        return null;
    }

    /**
     * Sets the value for the specified variable name in the file with the
     * defines.
     *
     * @param string $name The variable name.
     * @param scalar $value The value to set. null to remove the variable.
     * @param integer $action The action to perform on change. ACTION_NONE,
     *     ACTION_RELOAD, or ACTION_RESTART. (Default: ACTION_RELOAD)
     * @throws \InvalidArgumentException If the specified value is not a scalar.
     * @return boolean true if the value has been changed, false otherwise.
     */
    public function setDefine($name, $value, $action = self::ACTION_RELOAD)
    {
        if (null !== $value && !is_scalar($value)) {
            throw new \InvalidArgumentException("value must be a scalar or null");
        }

        if (null !== $value) {
            $value = (string) $value;
            if (strlen($value) == 0) {
                $value = '""';
            } else if (preg_match("/\s/", $value) || strpos("\"", $value) !== false) {
                // ensure that a value containing spaces is framed by double quotes
                if (!StringUtil::startsWith("\"", $value)) {
                    $value = "\"" . str_replace("\"", "\\\"", $value) . "\"";
                }
            }
            $defineStr = sprintf("Define %s %s\n", $name, $value);
        }

        $configChanged = false;
        $configLines = $this->readConfigArray(self::DEFINE_CONFIG, FILE_SKIP_EMPTY_LINES);
        foreach ($configLines as $key => $line) {
            $define = self::parseDefineLine($line);
            if ($define === null) {
                continue;
            }

            if ($define[0] != $name) {
                continue;
            }

            if ($define[1] === $value) {
                // value not changed
                return false;
            }

            if (null === $value) {
                $this->getLogger()->debug("Remove " . $name);
                unset($configLines[$key]);
            } else {
                $this->getLogger()->debug("Set " . $name . " to " . $value);
                $configLines[$key] = $defineStr;
            }
            $configChanged = true;
            break;
        }

        if (!$configChanged) {
            // variable was not defined
            if (null === $value) {
                return false;
            }

            $configLines[] = $defineStr;
        }

        $this->writeConfig(self::DEFINE_CONFIG, $configLines);
        switch ($action) {
            case self::ACTION_RELOAD:
                $this->enableReload();
                break;
            case self::ACTION_RESTART:
                $this->enableRestart();
                break;
        }

        return true;
    }

    /**
     * Checks if the filter with the specified name is configured as output
     * filter in the specified file. Looks only for the SetOutputFilter directive.
     * @param string $configFilename The configuration file name.
     * @param string $name The filter name.
     * @return boolean true if the specified filter is configured as output
     *     filter, false otherwise.
     */
    public function isOutputFilterEnabled($configFilename, $name)
    {
        $configLines = $this->readConfigArray($configFilename, FILE_SKIP_EMPTY_LINES);
        foreach ($configLines as $line) {
            $line = trim($line);
            if (StringUtil::startsWith($line, "SetOutputFilter ")) {
                if (strpos($line, $name)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Set the hostname of the apache instance.
     * Triggers a reload on termination of the PHP script.
     * @param string $newhost The new hostname.
     */
    public function setHost($newhost)
    {
        $oldsum = $this->getConfigHash(self::SERVERNAME_CONFIG);
        if (empty($newhost)) {
            $this->writeConfig(self::SERVERNAME_CONFIG, "\n");
        } else {
            $this->writeConfig(self::SERVERNAME_CONFIG, "ServerName " . $newhost . "\n");
        }
        if ($this->getConfigHash(self::SERVERNAME_CONFIG) != $oldsum) {
            $this->enableReload();
        }
    }

    /**
     * Set the proxy parameters of the apache instance.
     * Triggers a reload on termination of the PHP script.
     * @param boolean $ssl Flag: true, if ActiveSync backend should be contacted via SSL.
     * @param string $newhost The new hostname.
     */
    public function setProxy($ssl, $newhost)
    {
        $oldsum = $this->getConfigHash(self::PROXYSETUP_CONFIG);
        if (empty($newhost)) {
            $this->writeConfig(self::PROXYSETUP_CONFIG, "\n");
        } else {
            $proto = ($ssl) ? 'https://' : 'http://';
            $this->writeConfig(self::PROXYSETUP_CONFIG,
                "ProxyPassMatch " . $proto . $newhost . "/Microsoft-Server-ActiveSync$1 keepalive=on\n");
        }
        if ($this->getConfigHash(self::PROXYSETUP_CONFIG) != $oldsum) {
            $this->enableReload();
        }
    }

    /**
     * Enables/disables the reverse proxy.
     * Triggers a reload on termination of the PHP script.
     *
     * @param boolean $enabled true to enable, false to disable.
     */
    public function setReverseProxyEnabled($enabled)
    {
        $oldsum = $this->getConfigHash(self::REVERSE_PROXY_CONFIG);
        if ($enabled) {
            $reverseProxyDirRelative = $this->getRelativeConfigPath(self::REVERSE_PROXY_DIR);
            $includeDirective = "Include " . $reverseProxyDirRelative . "/*\n";
            $this->writeConfig(self::REVERSE_PROXY_CONFIG, $includeDirective);
        } else {
            $this->writeConfig(self::REVERSE_PROXY_CONFIG, "\n");
        }
        if ($this->getConfigHash(self::REVERSE_PROXY_CONFIG) != $oldsum) {
            $this->enableReload();
        }
    }

    /**
     * Enables/disables listening on the specified port in the auralis listen
     * config. Triggers a reload on termination of the PHP script.
     *
     * @param int $port Listen port
     * @param boolean $enabled true to enable listening, false to disable
     *     listening on the specified port.
     */
    public function setPortEnabled($port, $enabled)
    {
        $listenStr = "Listen $port";
        $config = $this->readConfigArray(self::LISTEN_CONFIG);
        $configChanged = false;
        $count = count($config);
        for ($i = 0; $i < $count; $i++) {
            $line = trim($config[$i]);
            if (strpos($line, $listenStr) !== false) {
                if ($enabled) {
                    $config[$i] = $listenStr . "\n";
                } else {
                    unset($config[$i]);
                }
                $configChanged = true;
                break;
            }
        }
        if (!$configChanged && $enabled) {
            $config[] = $listenStr . "\n";
            $configChanged = true;
        }
        if ($configChanged) {
            $this->writeConfig(self::LISTEN_CONFIG, $config);
            $this->enableReload();
        }
    }

    /**
     * Set client CA of the apache instance.
     * Triggers a reload on termination of the PHP script.
     * @param string $caCertificateFile The client CA filename.
     * @param string $rootCaCertificateFile The root CA filename (single
     *     certifcate, no chain).
     * @param string $caDnRequestFile The CA DN request file. Ignored if null.
     *    This file contains the CA certificates that are sent as acceptable CAs
     *    to the client in the SSL handshake.
     * @see https://httpd.apache.org/docs/2.2/mod/mod_ssl.html#sslcadnrequestfile
     */
    public function setClientCA($caCertificateFile, $rootCaCertificateFile, $caDnRequestFile = null)
    {
        $caCertificateFile = realpath($caCertificateFile);
        if (!is_file($caCertificateFile)) {
            throw new \InvalidArgumentException($caCertificateFile . " does not exist");
        }

        if (!is_file($rootCaCertificateFile)) {
            throw new \InvalidArgumentException($rootCaCertificateFile . " does not exist");
        }

        $output = "Alias /cacert.pem " . $rootCaCertificateFile
            . "\nSSLCACertificateFile " . $caCertificateFile . "\n";
        if ($caDnRequestFile !== null) {
            $caDnRequestFile = realpath($caDnRequestFile);
            if (!is_file($caDnRequestFile)) {
                $this->logger->warn('CA DN "'.$caDnRequestFile.'" is not a file');
            } else {
                $output .= "SSLCADNRequestFile " . $caDnRequestFile . "\n";
            }
        }

        $this->writeConfig(self::CLIENTCA_CONFIG, $output);
        // Always restart, because cert file content might have changed.
        // Restart after certificate change is required; reload is not enough.
        $this->enableRestart();
    }

    /**
     * Set client CRL of the apache instance.
     * Triggers a reload on termination of the PHP script.
     * @param string $newpath The new client CRL directory.
     */
    public function setClientCRL($newpath)
    {
        $newpath = realpath($newpath);
        if (!is_dir($newpath)) {
            // If not a directory disable CRL
            $this->logger->warn("'" . $newpath ."' is not a directory, disabling CRL");
            return;
        }

        $oldpath = 'x'.$newpath;
        $tmp = $this->readConfig(self::CLIENTCRL_CONFIG);
        if ($tmp) {
            $matches = "";
            if (preg_match("/SSLCARevocationPath\s+(.*)/", $tmp, $matches)) {
                $oldpath = $matches[1];
            }
        }
        if ($oldpath != $newpath) {
            $this->writeConfig(self::CLIENTCRL_CONFIG,
                empty($newpath) ? '' : 'SSLCARevocationPath '.$newpath."\n");
        }
        // Always reload, because CRL path content might have changed.
        $this->enableReload();
    }

    /**
     * Set server certs, keys and cert-chain of the apache instance.
     * Triggers a reload on termination of the PHP script.
     *
     * @param string $newdir The new directory where the cert files are to be created.
     *
     * @throws \InvalidArgumentException
     */
    public function setServerCerts($newdir)
    {
        $newdir = realpath($newdir);
        if (!is_dir($newdir)) {
            throw new \InvalidArgumentException('Invalid server cert directory');
        }

        static $files = array('servercert.pem', 'private/serverkey.pem', 'serverchain.pem');
        foreach ($files as $f) {
            $path = $newdir. "/" . $f;
            if (!is_file($path)) {
                throw new \InvalidArgumentException($f . " is not a file");
            }
        }

        $cfg = "Alias /servercert.pem " . $newdir . "/servercert.pem\n";
        $cfg .= "SSLCertificateFile " . $newdir . "/servercert.pem\n";
        $cfg .= "SSLCertificateKeyFile " . $newdir . "/private/serverkey.pem\n";
        $cfg .= "#SSLCertificateChainFile " . $newdir . "/serverchain.pem\n";
        $this->writeConfig(self::SERVERCERT_CONFIG, $cfg);

        // Restart after certificate change is required; reload is not enough.
        $this->enableRestart();
    }

    /**
     * Set allowed IP-ranges for client access.
     * Triggers a reload on termination of the PHP script.
     * @param string $allowed The new allowd IP sources, separated by comma.
     */
    public function setAllowFrom($allowed)
    {
        $a = preg_split('/\s*,\s*/', trim($allowed), null, PREG_SPLIT_NO_EMPTY);
        $count = count($a);
        if ($count == 0) {
            // None => Allow anyone
            $a = array('all');
        } else {
            foreach ($a as $ip) {
                if (StringUtil::endsWith($ip, "/0")) {
                    $a = array('all');
                    break;
                }
            }
        }
        $oldsum = $this->getConfigHash(self::ALLOWFROM_CONFIG);
        $this->writeConfig(self::ALLOWFROM_CONFIG, 'Allow from '.implode("\nAllow from ", $a)."\n");
        if ($this->getConfigHash(self::ALLOWFROM_CONFIG) != $oldsum) {
            $this->enableReload();
        }
    }

    /**
     * Generate an apache-style htpasswd hash.
     * Based on (broken) sample code at http://lt.php.net/manual/de/function.crypt.php
     *
     * @param string $pass The cleartext password to use.
     * @return string A string containing the hashed password-entry, suitable for using in a htpasswd file.
     */
    public static function genHtpasswdHash($pass)
    {
        $tmp = '';
        $apr1 = '$apr1$';
        $trto = './0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz';
        $salt = substr(str_shuffle($trto), 0, 8);
        $len = strlen($pass);
        $text = $pass.$apr1.$salt;
        $bin = pack('H32', md5($pass.$salt.$pass));
        for ($i = $len; $i > 0; $i -= 16) {
            $text .= substr($bin, 0, min(16, $i));
        }
        for ($i = $len; $i > 0; $i >>= 1) {
            $text .= ($i & 1) ? chr(0) : $pass{0};
        }
        $bin = pack('H32', md5($text));
        for ($i = 0; $i < 1000; $i++) {
            $new = ($i & 1) ? $pass : $bin;
            if ($i % 3) {
                $new .= $salt;
            }
            if ($i % 7) {
                $new .= $pass;
            }
            $new .= ($i & 1) ? $bin : $pass;
            $bin = pack('H32', md5($new));
        }
        for ($i = 0; $i < 5; $i++) {
            $k = $i + 6;
            $j = $i + 12;
            if ($j == 16) {
                $j = 5;
            }
            $tmp = $bin[$i].$bin[$k].$bin[$j].$tmp;
        }
        $tmp = chr(0).chr(0).$bin[11].$tmp;
        $tmp = strtr(
            strrev(substr(base64_encode($tmp), 2)),
            'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/',
            $trto
        );
        return $apr1.$salt.'$'.$tmp;
    }

    /**
     * Disable DAV access by clearing the AuthUserFile.
     *
     * @throws \Exception On error.
     */
    public function disableDavAccess()
    {
        $this->writeConfig(self::DAV_AUTH_USER_CONFIG, "\n");
    }

    /**
     * Enable DAV access by writing the specifed credentials into the AuthUserFile.
     *
     * @param string $credentials A string containing the content in htaccess format.
     * @throws \Exception On error.
     */
    public function enableDavAccess($credentials)
    {
        $this->writeConfig(self::DAV_AUTH_USER_CONFIG, $credentials);
    }

    /**
     * Creates the configuration for the specified URL.
     * Triggers a reload on termination of the PHP script.
     * @param ReverseProxyUrl $url
     */
    public function writeReverseProxy(ReverseProxyUrl $url)
    {
        /*
         * https://httpd.apache.org/docs/2.4/mod/mod_proxy.html
         * https://httpd.apache.org/docs/2.4/mod/mod_proxy_html.html
         * http://www.apachetutor.org/admin/reverseproxies
         */
        $externalPath = "/proxy/" . $url->getExternalPath();
        $internalUrl = $url->getInternalPath();
        $internalPath = parse_url($internalUrl, PHP_URL_PATH);
        if (strlen($internalPath) == 0) {
            // the path may not be empty
            $internalPath = "/";
            $internalUrl .= "/";
        }
        $fileName = self::REVERSE_PROXY_DIR . "/proxy_" . $url->getId() . ".conf";

        $content = array(
            "ProxyPass " . $externalPath . " " . $internalUrl . " keepalive=on",
            "<Location " . $externalPath . ">",
            "    ProxyPassReverse " . $internalPath,
            "    ProxyPassReverseCookiePath / " . $externalPath,
        );
        if (strlen($internalPath) > 0) {
            $content[] = "    ProxyHTMLURLMap  " . $internalPath . " " . $externalPath;
            $content[] = "    ProxyHTMLURLMap  " . $internalUrl . " " . $externalPath;
            if ($internalPath != "/" && StringUtil::endsWith($internalPath, "/")) {
                // if the path points to a folder we have to map it without final "/", too
                $content[] = "    ProxyHTMLURLMap  " . substr($internalPath, 0, -1) . " " . $externalPath;
                $content[] = "    ProxyHTMLURLMap  " . substr($internalUrl, 0, -1) . " " . $externalPath;
            }
        }
        $content[] = "</Location>";
        if ($url->isUseRemoteProxy() && $this->getDefine("auralis_remote_proxy_server")) {
            $content[] = "ProxyRemote " . $internalUrl . " \${auralis_remote_proxy_server}";
        }

        $content = implode("\n", $content);

        $oldsum = $this->getConfigHash($fileName);
        $this->writeConfig($fileName, $content);
        if ($this->getConfigHash($fileName) != $oldsum) {
            $this->enableReload();
        }
    }

    /**
     * Removes the configuration for the URL with the specified id.
     * Triggers a reload on termination of the PHP script.
     * @param integer|string $urlId
     */
    public function removeReverseProxy($urlId)
    {
        $fileName = $this->getConfigPath(self::REVERSE_PROXY_DIR) . "/proxy_" . $urlId . ".conf";
        if (file_exists($fileName)) {
            unlink($fileName);
            $this->enableReload();
        }
    }

    public function getConfigPath($base)
    {
        return $this->getAuralisConfigDir() . "/" . $base;
    }

    public function getRelativeConfigPath($base)
    {
        return str_replace($this->getConfigDirBase() . "/", "", $this->getConfigPath($base));
    }
}
