<?php
namespace Topalis\Apns;

/**
 * This class extends ApnsPHP_Push_Server to exit the application if we receive
 * a TERM, QUIT or INT signal.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ApnsPushServer extends \ApnsPHP_Push_Server
{
    /**
     * The maximum number of processes. If it is increased, the size of the
     * shared memory to store the connection states has to be increased, too.
     * @var integer
     */
    const MAX_PROCESS_COUNT = 24;

    private $_nConnectedProcesses;
    private $_hConnectedSem;
    private $_hConnectedShm;

    /**
     * Constructor.
     *
     * @param integer $nEnvironment Environment.
     * @param string $sProviderCertificateFile Provider certificate file
     *                                  with key (Bundled PEM).
     *
     * @throws \ApnsPHP_Push_Server_Exception if is unable to
     *         get Shared Memory Segment or Semaphore ID.
     */
    public function __construct($nEnvironment, $sProviderCertificateFile)
    {
        parent::__construct($nEnvironment, $sProviderCertificateFile);

        $this->_hConnectedShm = shm_attach(mt_rand(), 1024);
        if ($this->_hConnectedShm === false) {
            throw new \ApnsPHP_Push_Server_Exception(
                'Unable to get shared memory segment'
            );
        }

        $this->_hConnectedSem = sem_get(mt_rand());
        if ($this->_hConnectedSem === false) {
            throw new \ApnsPHP_Push_Server_Exception(
                'Unable to get semaphore id'
            );
        }

        pcntl_signal(SIGUSR1, array($this, 'onConnected'));
        pcntl_signal(SIGUSR2, array($this, 'onDisconnected'));
    }

    /**
     * Adds a message to the inter-process message queue.
     *
     * Messages are added to the queues in a round-robin fashion starting from the
     * first process to the last.
     *
     * @param \ApnsPHP_Message $message The message.
     */
    public function add(\ApnsPHP_Message $message)
    {
        static $n = 0;
        if ($n >= $this->_nProcesses) {
            $n = 0;
        }

        while (!isset($this->_aPids[$n]) && $this->isConnected()) {
            // find number of running process if there is at least one process
            // running.
            $n++;
        }

        sem_acquire($this->_hSem);
        $aQueue = $this->_getQueue(self::SHM_MESSAGES_QUEUE_KEY_START, $n);
        $aQueue[] = $message;
        $this->_setQueue(self::SHM_MESSAGES_QUEUE_KEY_START, $n, $aQueue);
        sem_release($this->_hSem);
        $n++;
    }

    public function isConnected($pid = null)
    {
        pcntl_signal_dispatch();

        if ($pid === null) {
            return $this->_nConnectedProcesses > 0;
        }

        $connected = false;
        sem_acquire($this->_hConnectedSem);
        try {
            if (shm_has_var($this->_hConnectedShm, $pid)) {
                $connected = shm_get_var($this->_hConnectedShm, $pid);
            }
        } catch (\Exception $e) {
            sem_release($this->_hConnectedSem);
            throw $e;
        }
        sem_release($this->_hConnectedSem);

        return $connected;
    }

    /**
     * Waits until a forked process has exited and decreases the current running
     * process number.
     */
    public function onChildExited()
    {
        while (($pid = pcntl_waitpid(-1, $nStatus, WNOHANG)) > 0) {
            $this->_nRunningProcesses--;
            $this->_setConnected($pid, null);

            if (($nProcess = array_search($pid, $this->_aPids)) !== false) {
                unset($this->_aPids[$nProcess]);

                if ($this->isConnected()) {
                    // move remaining messages to other queues
                    sem_acquire($this->_hSem);
                    $aQueue = $this->_getQueue(self::SHM_MESSAGES_QUEUE_KEY_START, $nProcess);
                    $this->_setQueue(self::SHM_MESSAGES_QUEUE_KEY_START, $nProcess);
                    sem_release($this->_hSem);
                    foreach ($aQueue as $message) {
                        $this->add($message);
                    }
                }
            }
        }
    }

    /**
     * This method is called when a child process gets connected.
     */
    public function onConnected()
    {
        $this->_nConnectedProcesses++;
    }

    /**
     * This method is called when a child process gets disconnected.
     */
    public function onDisconnected()
    {
        $this->_nConnectedProcesses--;
    }

    public function onShutdown()
    {
        parent::onShutdown();
        if (posix_getpid() == $this->_nParentPid) {
            // avoid warning about silencing errors
            // @codingStandardsIgnoreStart
            @shm_remove($this->_hConnectedShm) && @shm_detach($this->_hConnectedShm);
            @sem_remove($this->_hConnectedSem);
            // @codingStandardsIgnoreEnd
        }
    }

    public function onSignal($nSignal)
    {
        parent::onSignal($nSignal);
        if (posix_getpid() != $this->_nParentPid) {
            return;
        }

        switch ($nSignal) {
            case SIGTERM:
            case SIGQUIT:
            case SIGINT:
                // kill child processes
                $aPids = $this->_aPids;
                foreach ($aPids as $pid) {
                    try {
                        posix_kill($pid, 15);
                    } catch (\Exception $e) {
                        $this->getLogger()->log("ERROR: " . $e->getMessage());
                    }
                }
                $aPids = $this->_aPids;
                if (!empty($aPids)) {
                    sleep(2);
                }
                foreach ($aPids as $pid) {
                    try {
                        posix_kill($pid, 9);
                    } catch (\Exception $e) {
                        $this->getLogger()->log("ERROR: " . $e->getMessage());
                    }
                }
                // exit
                exit(0);
        }
    }

    /**
     * Starts the server forking all processes and return immediately.
     *
     * Every forked process is connected to Apple Push Notification Service on start
     * and enter on the main loop.
     */
    public function start()
    {
        if (!is_readable($this->_sProviderCertificateFile)) {
            throw new \ApnsPHP_Exception(
                "Unable to read certificate file '{$this->_sProviderCertificateFile}'"
            );
        }

        for ($i = 0; $i < $this->_nProcesses; $i++) {
            $this->_nCurrentProcess = $i;
            $this->_aPids[$i] = $nPid = pcntl_fork();
            if ($nPid == -1) {
                $this->_log('WARNING: Could not fork');
            } else if ($nPid > 0) {
                // Parent process
                $this->_log("INFO: Forked process PID {$nPid}");
                $this->_nRunningProcesses++;
            } else {
                // Child process
                try {
                    parent::connect();
                    $this->_setConnected(posix_getpid(), true);
                    $this->_mainLoop();
                    parent::disconnect();
                    exit(0);
                } catch (ApnsPHP_Exception $e) {
                    $this->_log('ERROR: ' . $e->getMessage() . ', exiting...');
                    exit(1);
                }
            }
        }
    }

    // ignore method name
    // @codingStandardsIgnoreStart
    protected function _setConnected($pid, $connected)
    {
        $wasConnected = $this->isConnected($pid);

        sem_acquire($this->_hConnectedSem);
        try {
            if ($connected !== null) {
                shm_put_var($this->_hConnectedShm, $pid, $connected);
            } else if (shm_has_var($this->_hConnectedShm, $pid)) {
                shm_remove_var($this->_hConnectedShm, $pid);
            }
        } catch (\Exception $e) {
            sem_release($this->_hConnectedSem);
            throw $e;
        }
        sem_release($this->_hConnectedSem);

        if ($wasConnected != $connected) {
            if ($connected) {
                if (posix_getpid() == $this->_nParentPid) {
                    $this->onConnected();
                } else {
                    posix_kill($this->_nParentPid, SIGUSR1);
                }
            } else {
                if (posix_getpid() == $this->_nParentPid) {
                    $this->onDisconnected();
                } else {
                    posix_kill($this->_nParentPid, SIGUSR2);
                }
            }
        }
    }
    // @codingStandardsIgnoreEnd
}
