<?php
namespace Topalis\Apns\Command;

use Topalis\Apns\ApnsCertificateLoadException;
use Topalis\Kernel\Command\LogrotateAwareCommand;

class ApnsdCommand extends LogrotateAwareCommand
{
    /**
     *
     * @return \Doctrine\ORM\EntityManager
     */
    private function getEntityManager()
    {
        return $this->getDoctrine()->getManager();
    }

    /**
     * @return \Symfony\Bridge\Doctrine\RegistryInterface
     */
    private function getDoctrine()
    {
        return $this->getContainer()->get("doctrine");
    }

    /**
     * @return \Topalis\Apns\AuralisApnsPushServer
     */
    private function getPushServer()
    {
        return $this->getContainer()->get("apns.push_server");
    }

    protected function configure()
    {
        parent::configure();
        $this->setName("apnsd:run")
            ->setDescription("Daemon that pushes messages to the APNS.")
            ->setHelp("Daemon that pushes messages to the APNS.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        try {
            $this->initApnsd();
        } catch (\Exception $e) {
            if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                $output = $output->getErrorOutput();
            }
            $output->writeln("Error initializing apnsd:");
            $output->writeln((string) $e);
            return 1;
        }
        try {
            return $this->runApnsd();
        } catch (\Exception $e) {
            if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                $output = $output->getErrorOutput();
            }
            $output->writeln("Unexpected error while running apnsd:");
            $output->writeln((string) $e);
            return 2;
        }
    }

    protected function initApnsd()
    {
        // Set the Root Certificate Autority to verify the Apple remote peer
        $this->getPushServer()->setRootCertificationAuthority(__DIR__ . "/../Resources/entrust_2048_ca.cer");
        $this->getPushServer()->init();
    }

    protected function runApnsd()
    {
        $this->getContainer()->get("config.auralis")->setCacheEnabled(false);
        $em = $this->getEntityManager();
        $server = $this->getPushServer();

        $lastRunError = false;
        while (true) {
            try {
                $server->start();
            } catch (ApnsCertificateLoadException $e) {
                // Exit if an APNS certificate could not be loaded
                $server->getPsrLogger()->error($e->getMessage());
                return 1;
            }

            while ($server->run() && !$server->isConnected()) {
                // wait until we have a connection or all child processes are
                // exited
                sleep(5);
            }

            while ($server->run()) {
                pcntl_signal_dispatch(); // ensure possible lograte is noticed
                try {
                    $server->pushMessages();
                    $lastRunError = false;
                } catch (\Exception $e) {
                    if ($lastRunError === false) {
                        // Only report error if the previous run was successful to avoid
                        // too much error messages.
                        $server->getPsrLogger()->error($e->getMessage(), array("exception" => $e));
                    }
                    $lastRunError = true;

                    // db connection might be lost, check and reconnect
                    $connection = $this->getDoctrine()->getConnection();
                    try {
                        $connection->query('select 1;')->fetch();
                    } catch (\Exception $e) {
                        // db server might be restarted so wait some time and try to reconnect
                        try {
                            sleep(15);
                            $connection->close();
                            $connection->connect();
                            $connection->query('select 1;')->fetch();
                            $server->getPsrLogger()->info("Reconnected to database");
                        } catch (\Exception $e2) {
                            $server->getPsrLogger()->critical("Cannot access database: " . $e2->getMessage(),
                                array("exception" => $e2));
                            return 2;
                        }
                    }
                }

                // avoid memory leaks
                $em->flush();
                $em->clear();
                gc_collect_cycles();
                // sleep a little
                sleep(10);
            }

            // all connections have been lost; try to start again in next iteration
        }
    }

}
