<?php
namespace Topalis\App\Command;

use Doctrine\ORM\Query\Expr\Join;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Topalis\User\Entity\User;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class RefreshVppCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName("app:ios:vpp-sync")
            ->addOption("incremental", null, InputOption::VALUE_NONE, "perform an incremental sync")
        ;
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        if ($this->getConfig()->isVppSyncDisabled()) {
            $msg = "VPP sync disabled";
            $this->getLogger()->debug($msg);
            if ($output->getVerbosity() >= OutputInterface::VERBOSITY_VERBOSE) {
                $output->writeln($msg);
            }
            return;
        }

        if (!$this->getVppUserService()->isTokenConfigured()) {
            $msg = "No VPP token configured";
            $this->getLogger()->debug($msg);
            if ($output->getVerbosity() >= OutputInterface::VERBOSITY_VERBOSE) {
                $output->writeln($msg);
            }
            return;
        }

        $message = "";
        if (!$this->getVppUserService()->isAvailable($message)) {
            $msg = "VPP service is not available: " . $message;
            $output->writeln($msg);
            $this->getLogger()->notice($msg);
            return 1;
        }

        $incremental = $input->getOption("incremental");
        // find users without vpp id
        $iosId = $this->getContainer()->get("device.type.factory")->getType("ios")->getId();
        $users = $this->getUserEntityManager()->createQueryBuilder("u")
            ->join("u.devices", "d", Join::WITH, "d.os = :ios")
            ->where("u.vppStatus = :vppStatusUnregistered")
            ->setParameter("ios", $iosId)
            ->setParameter("vppStatusUnregistered", User::VPP_STATUS_UNREGISTERED)
            ->getQuery()->getResult();
        ;
        foreach ($users as $user) {
            // only register users with rolled out iOS device
            $hasRolledOutIosDevice = false;
            foreach ($user->getDevices() as $device) {
                if ($device->isActive() && $device->getOs() === $iosId
                    && $device->getType()->isRolloutFinished($device)) {
                    $hasRolledOutIosDevice = true;
                    break;
                }
            }
            if ($hasRolledOutIosDevice) {
                $this->getVppUserService()->registerUser($user);
            }
        }

        // sync registered users
        $this->getVppUserService()->syncUsers($incremental);

        // sync licenses
        $this->getVppLicenseService()->syncLicenses($incremental);

    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    protected function getConfig()
    {
        return $this->getContainer()->get("config.auralis");
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    protected function getLogger()
    {
        return $this->getContainer()->get("logger");
    }

    /**
     * @return \Topalis\User\Entity\UserEntityManager
     */
    protected function getUserEntityManager()
    {
        return $this->getContainer()->get("user.entity.manager.user");
    }

    /**
     * @return \Topalis\App\Ios\VppLicenseService
     */
    protected function getVppLicenseService()
    {
        return $this->getContainer()->get("app.ios.vpp_license");
    }

    /**
     * @return \Topalis\App\Ios\VppUserService
     */
    protected function getVppUserService()
    {
        return $this->getContainer()->get("app.ios.vpp_user");
    }
}
