<?php
namespace Topalis\App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Validator\Mapping\ClassMetadata;
use Topalis\ConfigGroup\Entity\ConfigGroup;

/**
 * NOTE: Check AppEditFormHandlers when adding new attributes that aren't copied
 * in merge().
 *
 * @ORM\MappedSuperclass
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
abstract class App
{
    /**
     * The app is from the app store.
     * @var string
     */
    const TYPE_STORE = "STORE";

    /**
     * The source of the app is unknown.
     * @var string
     */
    const TYPE_UNKNOWN = "UNKNOWN";

    /**
     * The app has been uploaded.
     * @var string
     */
    const TYPE_UPLOADED = "UPLOADED";

    /**
     * @var integer
     * @ORM\Column(type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string
     * @ORM\Column(type="string")
     */
    protected $type;

    /**
     * @var string identifier of the app (e.g. com.apple.Remote)
     * @ORM\Column(type="string", unique=true)
     */
    protected $identifier;

    /**
     * @var string name of the app (e.g. Remote)
     * @ORM\Column(type="string")
     */
    protected $name;

    /**
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    protected $icon;

    /**
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    protected $iconLarge;

    /**
     * @var string
     * @ORM\Column(type="text", nullable=true)
     */
    protected $description;

    /**
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    protected $version;

    /**
     * @var string
     * @ORM\Column(type="string", length=4096, nullable=true)
     */
    protected $releaseNotes;

    /**
     * @var float
     * @ORM\Column(type="float", nullable=true)
     */
    protected $price;

    /**
     * @var float
     * @ORM\Column(type="string", nullable=true)
     */
    protected $currency;

    /**
     * @var string
     * @ORM\Column(type="string", nullable=true)
     */
    protected $vendor;

    /**
     * @var string
     * @ORM\Column(type="array", nullable=true)
     */
    protected $screenshots;

    /**
     * @var array
     * @ORM\Column(type="json_array", length=65532, nullable=true)
     */
    protected $supports;

    /**
     * "default" required because we use native SQL in AppEntityManager for
     * INSERT.
     * @var boolean
     * @ORM\Column(type="boolean", options={"default"=0})
     */
    protected $managed = false;

    /**
     * "default" required because we use native SQL in AppEntityManager for
     * INSERT.
     * @var boolean
     * @ORM\Column(type="boolean", name="force_install", options={"default"=0})
     */
    protected $forceInstall = false;

    /**
     * @var \Doctrine\Common\Collections\ArrayCollection
     */
    protected $states;

    /**
     * @var ArrayCollection
     * @ORM\ManyToMany(targetEntity="Topalis\ConfigGroup\Entity\ConfigGroup", cascade={"persist"})
     */
    protected $configGroups;

    /**
     * Not persisted. See isUploadedPackageChanged()
     * @var boolean
     */
    protected $uplaodedPackageChanged = false;

    public function __construct()
    {
        $this->states = new ArrayCollection();
        $this->configGroups = new ArrayCollection();
        $this->screenshots = new ArrayCollection();
    }

    /**
     * Set a list of rules to use when validating an instance of this Class
     *
     * @param ClassMetadata $metadata
     */
    public static function loadValidatorMetadata(ClassMetadata $metadata)
    {
        $metadata->addConstraint(new UniqueEntity(array(
            "fields"  => "identifier",
            "message" => "app.identifierAlreadyInUse"
        )));
    }

    /**
     * merge data from other app (e.g. from app store app)
     *
     * @param App $app
     * @param boolean $force If false only defined data will be copied,
     *     otherwise defined data my be set to null.
     *
     * @return $this
     */
    public function merge(App $app, $force = false)
    {
        if (null !== $app->getIdentifier() || $force) {
            $this->setIdentifier($app->getIdentifier());
        }
        if (null !== $app->getIcon() || $force) {
            $this->setIcon($app->getIcon());
        }
        if (null !== $app->getIconLarge() || $force) {
            $this->setIconLarge($app->getIconLarge());
        }
        if (null !== $app->getName() || $force) {
            $this->setName($app->getName());
        }
        if (null !== $app->getDescription() || $force) {
            $this->setDescription($app->getDescription());
        }
        if (null !== $app->getVersion() || $force) {
            $this->setVersion($app->getVersion());
        }
        if (null !== $app->getReleaseNotes() || $force) {
            $this->setReleaseNotes($app->getReleaseNotes());
        }
        if (null !== $app->getScreenshots() || $force) {
            $this->setScreenshots($app->getScreenshots());
        }
        if (null !== $app->getPrice() || $force) {
            $this->setPrice($app->getPrice());
        }
        if (null !== $app->getCurrency() || $force) {
            $this->setCurrency($app->getCurrency());
        }
        if (null !== $app->getVendor() || $force) {
            $this->setVendor($app->getVendor());
        }
        if (null !== $app->getSupports()) {
            $this->setSupports($app->getSupports());
        }

        return $this;
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    public function setId($id)
    {
        $this->id = $id;

        return $this;
    }

    /**
     * @return string
     */
    public function getType()
    {
        return $this->type;
    }

    /**
     * @param string $type
     *
     * @return $this
     */
    public function setType($type)
    {
        $this->type = $type;

        return $this;
    }

    /**
     * @param string $identifier
     *
     * @return $this
     */
    public function setIdentifier($identifier)
    {
        $this->identifier = $identifier;

        return $this;
    }

    /**
     * @return string
     */
    public function getIdentifier()
    {
        return $this->identifier;
    }

    /**
     * @param string $name
     *
     * @return $this
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * @param string $icon
     *
     * @return $this
     */
    public function setIcon($icon)
    {
        $this->icon = $icon;

        return $this;
    }

    /**
     * @return string
     */
    public function getIcon()
    {
        return $this->icon;
    }

    /**
     * @return string
     */
    public function getIconLarge()
    {
        return $this->iconLarge;
    }

    /**
     * @param string $iconLarge
     *
     * @return $this
     */
    public function setIconLarge($iconLarge)
    {
        $this->iconLarge = $iconLarge;

        return $this;
    }

    /**
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * @param string $description
     *
     * @return $this
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * @return float
     */
    public function getPrice()
    {
        return $this->price;
    }

    /**
     * @param float $price
     *
     * @return $this
     */
    public function setPrice($price)
    {
        $this->price = $price;

        return $this;
    }

    /**
     * @return string
     */
    public function getCurrency()
    {
        return $this->currency;
    }

    /**
     * @param string $currency
     *
     * @return $this
     */
    public function setCurrency($currency)
    {
        $this->currency = $currency;

        return $this;
    }

    /**
     * @return string
     */
    public function getReleaseNotes()
    {
        return $this->releaseNotes;
    }

    /**
     * @param string $releaseNotes
     *
     * @return $this
     */
    public function setReleaseNotes($releaseNotes)
    {
        if (strlen($releaseNotes) > 4096) {
            $releaseNotes = substr($releaseNotes, 0, 4093) + "...";
        }
        $this->releaseNotes = $releaseNotes;

        return $this;
    }

    /**
     * @return string
     */
    public function getScreenshots()
    {
        return $this->screenshots;
    }

    /**
     * @param string $screenshots
     *
     * @return $this
     */
    public function setScreenshots($screenshots)
    {
        $this->screenshots = $screenshots;

        return $this;
    }

    /**
     * @return string
     */
    public function getVendor()
    {
        return $this->vendor;
    }

    /**
     * @param string $vendor
     *
     * @return $this
     */
    public function setVendor($vendor)
    {
        $this->vendor = $vendor;

        return $this;
    }

    /**
     * @return string
     */
    public function getVersion()
    {
        return $this->version;
    }

    /**
     * @param string $version
     *
     * @return $this
     */
    public function setVersion($version)
    {
        $this->version = $version;

        return $this;
    }

    /**
     * @param array $supports The supported devices or operating systems.
     * @return App
     */
    public function setSupports(array $supports)
    {
        $this->supports = $supports;

        return $this;
    }

    /**
     * @return array|NULL The supported devices or operating systems.
     */
    public function getSupports()
    {
        return $this->supports;
    }

    /**
     * @return boolean
     */
    public function isManaged()
    {
        return $this->managed;
    }

    /**
     * @param boolean $managed
     *
     * @return $this
     */
    public function setManaged($managed)
    {
        $this->managed = $managed;

        return $this;
    }

    /**
     * @return boolean
     */
    public function isForceInstall()
    {
        return $this->forceInstall;
    }

    /**
     * @param boolean $force
     *
     * @return $this
     */
    public function setForceInstall($force)
    {
        $this->forceInstall = $force;

        return $this;
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup[]
     */
    public function getConfigGroups()
    {
        return $this->configGroups;
    }

    public function addConfigGroup(ConfigGroup $configGroup)
    {
        if (!$this->configGroups->contains($configGroup)) {
            $this->configGroups->add($configGroup);
        }

        return $this;
    }

    /**
     * @param ConfigGroup[] $configGroups
     *
     * @return $this
     */
    public function setConfigGroups(array $configGroups)
    {
        $this->configGroups->clear();
        foreach ($configGroups as $configGroup) {
            $this->addConfigGroup($configGroup);
        }

        return $this;
    }

    public function removeConfigGroup(ConfigGroup $configGroup)
    {
        $this->configGroups->removeElement($configGroup);

        return $this;
    }

    public function hasConfigGroups()
    {
        return !$this->configGroups->isEmpty();
    }

    /**
     * @return AppState[]
     */
    public function getStates()
    {
        return $this->states;
    }

    public function setStates(array $states)
    {
        $this->states->clear();
        foreach ($states as $state) {
            $this->addState($state);
        }

        return $this;
    }

    public function addState(AppState $state)
    {
        if (!$this->states->contains($state)) {
            $this->states->add($state);
            $state->setApp($this);
        }

        return $this;
    }

    public function removeState(AppState $state)
    {
        $this->states->removeElement($state);

        return $this;
    }

    public function hasStates()
    {
        return !$this->states->isEmpty();
    }

    /**
     * @param boolean $uploadedPackageChanged true if the package of an uploaded app has been changed in the current
     *                                        request. It is used to know if an uploaded apps has to be updated on
     *                                        devices. It's not persisted.
     * @return $this
     */
    public function setUploadedPackageChanged($uploadedPackageChanged)
    {
        $this->uplaodedPackageChanged = $uploadedPackageChanged;

        return $this;
    }

    /**
     * @return boolean true if the package of an uploaded app has been changed
     *     in the current request. It is used to know if an uploaded apps has to
     *     be updated on devices. It's not persisted.
     */
    public function isUploadedPackageChanged()
    {
        return $this->uplaodedPackageChanged;
    }
}
