<?php
namespace Topalis\App\Entity;

use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Database\EntityManager;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class AppEntityManager extends EntityManager
{
    /**
     * Return an array with identifier as key and app id as value
     *
     * @param array $criteria Optional array with required field-value-pairs.
     * @param array $notCriteria Optional array with field-value-pairs that must
     *     not be equal.
     * @return int[]
     */
    public function getAppIds(array $criteria = array(), array $notCriteria = array())
    {
        $qb = $this->createQueryBuilder("a")
            ->select("a.id, a.identifier");
        $i = 0;
        foreach ($criteria as $field => $value) {
            $qb->andWhere("a." . $field . " = ?$i");
            $qb->setParameter($i, $value);
            $i++;
        }
        foreach ($notCriteria as $field => $value) {
            $qb->andWhere("a." . $field . " != ?$i");
            $qb->setParameter($i, $value);
            $i++;
        }
        $apps = $qb
            ->getQuery()
            ->getArrayResult();
        $ids = array();
        foreach ($apps as $app) {
            $ids[$app['identifier']] = $app['id'];
        }

        return $ids;
    }

    /**
     * Returns all managed apps that are assigned to the specified group.
     *
     * @param ConfigGroup|int $configGroup The group or its id to get the apps
     *    for.
     * @return App[]
     */
    public function getAppsForConfigGroup($configGroup)
    {
        return $this->createQueryBuilder("a")
            ->select("a")
            ->where("a.managed = 1 AND :configGroup MEMBER OF a.configGroups")
            ->setParameter("configGroup", $configGroup)
            ->getQuery()
            ->getResult();
    }

    /**
     * @param integer $count The number of top installed apps in the result. All
     *     apps, if not specified.
     * @return array Array with the following key for each app: id, identifier,
     *     name, installCount. The array is ordered descendent by installCount.
     */
    public function getTopApps($count = null)
    {
        $qb = $this->createQueryBuilder("a")
            ->select("a.id", "a.identifier", "a.name", "COUNT(a) as installCount")
            ->join("a.states", "s")
            ->where("s.state IN (:states)")
            ->groupBy("a.id")
            ->orderBy("installCount", "DESC")
            ->setParameter("states", AppState::getInstalledStates())
        ;
        if ($count !== null) {
            $qb->setMaxResults($count);
        }

        return $qb->getQuery()->getResult();
    }

    /**
     * Returns the app with the specified identifier. If it doesn't exist it
     * will be created.
     *
     * @param string $identifier The app identifier
     * @param string $name The app name. Will be used if a new app is created.
     * @return \Topalis\App\Entity\App
     */
    public function getOrCreate($identifier, $name)
    {
        $appEntity = $this->findOneBy(array("identifier" => $identifier));
        if (null !== $appEntity) {
            return $appEntity;
        }

        if ($this->em->getConnection()->getDatabasePlatform()->getName() == "mysql") {
            // avoid duplicate keys in case a concurrenlty running process
            // inserted the app before
            $cm = $this->em->getClassMetadata($this->class);
            $table = $cm->table['name'];
            $this->em->getConnection()->executeQuery(
                "INSERT INTO $table (identifier, name, type) VALUES (:identifier, :name, :type)
                ON DUPLICATE KEY UPDATE name = :name",
                array("identifier" => $identifier, "name" => $name, "type" => App::TYPE_UNKNOWN));

            $appEntity = $this->findOneBy(array("identifier" => $identifier));
            if (null !== $appEntity) {
                return $appEntity;
            }
        }

        $appEntity = $this->create();
        $appEntity->setIdentifier($identifier);
        $appEntity->setName($name);
        $appEntity->setType(App::TYPE_UNKNOWN);
        $this->save($appEntity);
    }
}
