<?php
namespace Topalis\App\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\HttpFoundation\File\File;
use Topalis\App\Ios\IpaFile;

/**
 * NOTE: Check IosAppEditFormHandler when adding new attributes that aren't
 * copied in merge().
 *
 * @ORM\Entity
 * @ORM\Table(name="iosapps", indexes={
 *  @ORM\Index(name="managed_idx", columns={"managed"})
 * })
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class IosApp extends App
{
    /**
     * Purchase Method: Volume Purchase Program
     */
    const PURCHASE_METHOD_VPP = 0;

    /**
     * Purchase Method: Volume Purchase Program App Assignment
     */
    const PURCHASE_METHOD_VPP_ASSIGNMENT = 1;

    /**
     * App will be automatically removed when mdm profile ist deleted and app is managed
     *
     * @var boolean
     * @ORM\Column(type="boolean", options={"default" = 1})
     */
    protected $removeWheneMdmIsRemoved = true;

    /**
     * Prevent backup of the app data.
     *
     * @var boolean
     * @ORM\Column(type="boolean", options={"default" = 1})
     */
    protected $preventBackup = true;

    /**
     * @var integer
     * @ORM\Column(type="bigint", nullable=true, unique=true, options={"unsigned"=true})
     */
    protected $iTunesStoreID;

    /**
     * @var string|File
     * @ORM\Column(name="ipa", type="text", nullable=true)
     */
    protected $ipaPath;

    /**
     * @var IpaFile
     */
    protected $ipa;

    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="Topalis\App\Entity\IosAppState", mappedBy="app", cascade={"persist"})
     */
    protected $states;

    /**
     * @var ArrayCollection
     * @ORM\OneToMany(targetEntity="Topalis\App\Entity\VppLicense", mappedBy="app", cascade={"persist", "remove"})
     */
    protected $vppLicenses;

    /**
     * @var boolean App uses vpp
     * @ORM\Column(type="boolean", options={"default" = 0})
     */
    protected $useVpp = false;

    public function __construct()
    {
        parent::__construct();
        $this->vppLicenses = new ArrayCollection();
    }

    public function merge(App $app, $force = false)
    {
        /** @var self $app */
        parent::merge($app, $force);
        if (null !== $app->getITunesStoreID() || $force) {
            $this->setITunesStoreID($app->getITunesStoreID());
        }

        return $this;
    }

    /**
     * @return boolean
     */
    public function isPreventBackup()
    {
        return $this->preventBackup;
    }

    /**
     * @param boolean $preventBackup
     *
     * @return $this
     */
    public function setPreventBackup($preventBackup)
    {
        $this->preventBackup = $preventBackup;

        return $this;
    }

    /**
     * @return boolean
     */
    public function isRemoveWheneMdmIsRemoved()
    {
        return $this->removeWheneMdmIsRemoved;
    }

    /**
     * @param boolean $removeWheneMdmIsRemoved
     *
     * @return $this
     */
    public function setRemoveWheneMdmIsRemoved($removeWheneMdmIsRemoved)
    {
        $this->removeWheneMdmIsRemoved = $removeWheneMdmIsRemoved;

        return $this;
    }

    /**
     * @return integer
     */
    public function getITunesStoreID()
    {
        return $this->iTunesStoreID;
    }

    /**
     *
     * @param integer $iTunesStoreID
     *
     * @return self
     */
    public function setITunesStoreID($iTunesStoreID)
    {
        $this->iTunesStoreID = $iTunesStoreID;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getIpaPath()
    {
        return $this->ipaPath;
    }

    /**
     * @param mixed $ipaPath
     *
     * @return self
     */
    public function setIpaPath($ipaPath)
    {
        // Objects of type File are converted to their path on comparison. So
        // we have to check for non-identity.
        if ($this->ipaPath !== $ipaPath) {
            $this->ipaPath = $ipaPath;
            $this->setUploadedPackageChanged(true);
        }

        return $this;
    }

    /**
     * @return IpaFile
     */
    public function getIpa()
    {
        return $this->ipa;
    }

    /**
     * @param IpaFile $ipa
     *
     * @return self
     */
    public function setIpa(IpaFile $ipa)
    {
        $this->ipa = $ipa;

        return $this;
    }

    public function getManagementFlags()
    {
        $flags = 0;
        if ($this->isRemoveWheneMdmIsRemoved()) {
            $flags |= 1;
        }
        if ($this->isPreventBackup()) {
            $flags |= 4;
        }

        return $flags;
    }

    /**
     * @param VppLicense[] $vppLicenses
     *
     * @return $this
     */
    public function setVppLicenses(array $vppLicenses)
    {
        $this->vppLicenses->clear();
        foreach ($vppLicenses as $vppLicense) {
            $this->addVppLicense($vppLicense);
        }

        return $this;
    }

    /**
     * @param VppLicense $vppLicense
     *
     * @return $this
     */
    public function addVppLicense(VppLicense $vppLicense)
    {
        if (!$this->vppLicenses->contains($vppLicense)) {
            $this->vppLicenses->add($vppLicense);
            $vppLicense->setApp($this);
        }

        return $this;
    }

    /**
     * @param VppLicense $vppLicense
     *
     * @return $this
     */
    public function removeVppLicense(VppLicense $vppLicense)
    {
        if ($this->vppLicenses->removeElement($vppLicense)) {
            $vppLicense->setApp(null);
        }

        return $this;
    }

    /**
     * @return VppLicense[]
     */
    public function getVppLicenses()
    {
        return $this->vppLicenses;
    }

    /**
     * @return boolean
     */
    public function isUseVpp()
    {
        return $this->useVpp;
    }

    /**
     * @param boolean $useVpp
     *
     * @return $this
     */
    public function setUseVpp($useVpp)
    {
        $this->useVpp = $useVpp;

        return $this;
    }
}
