<?php
namespace Topalis\App\EventListener\MDM;

use Doctrine\ORM\Query;
use Doctrine\ORM\Query\Expr\Join;
use Psr\Log\LoggerInterface;
use Topalis\App\Entity\App;
use Topalis\App\Entity\AppEntityManager;
use Topalis\App\Entity\AppState;
use Topalis\App\Entity\AppStateEntityManager;
use Topalis\App\Entity\IosAppState;
use Topalis\Configuration\AuralisConfig;
use Topalis\Device\Entity\Device;
use Topalis\Device\Entity\DeviceEntityManager;
use Topalis\Device\Type\IosDeviceType;
use Topalis\MDM\MdmAdmin;
use Topalis\MDM\MdmEvent;
use Topalis\MDM\MdmEvents;
use Topalis\User\Entity\UserEntityManager;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class IosAppMdmListener extends AppMdmListener
{

    /**
     * @var AuralisConfig
     */
    private $config;

    /**
     * @var DeviceEntityManager
     */
    private $deviceEntityManager;

    /**
     * @var UserEntityManager
     */
    private $userEntityManager;

    public function __construct(
        MdmAdmin $mdmAdmin,
        AppEntityManager $appManager,
        AppStateEntityManager $appStateEntityManager,
        UserEntityManager $userEntityManager,
        DeviceEntityManager $deviceEntityManager,
        AuralisConfig $config,
        LoggerInterface $logger)
    {
        parent::__construct($mdmAdmin, $appManager, $appStateEntityManager, $logger);

        $this->config = $config;
        $this->userEntityManager = $userEntityManager;
        $this->deviceEntityManager = $deviceEntityManager;
    }

    public static function getSubscribedEvents()
    {
        return parent::getSubscribedEvents() + array(MdmEvents::INVITE_TO_PROGRAM => "handleInviteToProgram");
    }

    public function supports(Device $device)
    {
        return $device->getType() instanceof IosDeviceType;
    }

    public function handleInviteToProgram(MdmEvent $event)
    {
        if (!$event->isAcknowledged()) {
            return;
        }

        $invitationAcknowledged = "Acknowledged" == $event->getContext()->getMdmRequest()->get("InvitationResult");
        $user = $event->getDevice()->getUser();
        $devices = $user->getDevices();
        $eventCommand = $event->getContext()->getMdmCommand();
        // cancel invitation requests for other devices of the same user
        foreach ($devices as $device) {
            if ($device == $event->getDevice() || !$this->supports($device)) {
                continue;
            }

            foreach ($device->getMdmQueue() as $queueEntry) {
                $command = $queueEntry->getCommand();
                if ($command->getRequestType() != $eventCommand->getRequestType()) {
                    continue;
                }

                // cancel invitation request if the invitation is acknowledged
                // or if invitation links are equal
                if ($invitationAcknowledged || $command->getInfo() == $eventCommand->getInfo()) {
                    $this->mdmAdmin->cancelCommand($command);
                }
            }
        }
    }

    public function handleManagedApplicationList(MdmEvent $event)
    {
        parent::handleManagedApplicationList($event);

        // get states to retry installation
        /** @var IosAppState[] $states */
        $states = $this->appStateEntityManager->createQueryBuilder("s")
            ->select("s, a")
            ->join("s.app", "a")
            ->where("a.managed = 1 AND a.forceInstall = 1 AND (s.rejected = :userRejected
                OR (s.state = :removed AND s.managed = 1))
                AND s.installAttempts < :installAttempts
                AND s.state != :installed AND s.state != :removalRequested")
            ->setParameter("userRejected", AppState::REJECTED_USER)
            ->setParameter("removed", AppState::REMOVED)
            ->setParameter("installAttempts", $this->config->getAppInstallAttempts())
            ->setParameter("installed", AppState::INSTALLED)
            ->setParameter("removalRequested", AppState::REMOVAL_REQUESTED)
            ->getQuery()
            ->getResult()
        ;

        foreach ($states as $state) {
            if (null !== $state->getLastInstallAttempt()
                && $state->getLastInstallAttempt()->getTimestamp() + 86400 >= time()) {
                continue;
            }
            $this->mdmAdmin->installApp($event->getDevice(), $state->getApp(), true);
        }

        $device = $event->getDevice();
        if (!$device->isFlag(Device::FLAG_APP_STORE_TEMPORARY_ENABLED)) {
            return;
        }

        $existsRequest = $this->appStateEntityManager->createQueryBuilder("s")
            ->select("1")
            ->join("s.app", "a", Join::WITH, "a.type = :typeStore")
            ->where("s.device = :device")
            ->andWhere("s.state = :installationRequested OR s.state = :updateRequested")
            ->setParameter("typeStore", App::TYPE_STORE)
            ->setParameter("device", $device)
            ->setParameter("installationRequested", AppState::INSTALLATION_REQUESTED)
            ->setParameter("updateRequested", AppState::UPDATE_REQUESTED)
            ->getQuery()->setMaxResults(1)->getOneOrNullResult(Query::HYDRATE_SCALAR);

        if (!$existsRequest) {
            // no pending install request, disable store
            $device->setFlag(Device::FLAG_APP_STORE_TEMPORARY_ENABLED, false);
            $this->deviceEntityManager->save($device, false);
            $this->mdmAdmin->installDeviceProfile($device);
        }
    }
}
