<?php
namespace Topalis\App\EventListener\ORM;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\OnFlushEventArgs;
use Doctrine\ORM\Event\PostFlushEventArgs;
use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\App\Entity\App;
use Topalis\App\Entity\MdmAppState;

/**
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ManagedAppListener extends ContainerAware
{
    protected $inOnFlush = false;
    protected $inPostFlush = false;

    /**
     * @var App[]
     */
    protected $appsToInstall = array();

    /**
     * @var App[]
     */
    protected $appsToRemove = array();

    /**
     * @return \Doctrine\ORM\EntityManagerInterface
     */
    private function getEntityManager()
    {
        return $this->container->get("doctrine.orm.entity_manager");
    }

    /**
     * @return \Topalis\MDM\MdmAdmin
     */
    private function getMdmAdmin()
    {
        return $this->container->get("mdm.admin");
    }

    public function postPersist(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if (!$entity instanceof App || !$entity->isManaged()) {
            return;
        }

        $this->appsToInstall[$entity->getId()] = $entity;
    }

    public function postRemove(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if ($entity instanceof MdmAppState) {
            $app = $entity->getApp();
            if (!$app->isManaged() && !$app->hasStates()) {
                // app is neither managed nor installed, so it can be removed
                $eventArgs->getEntityManager()->remove($app);
                $eventArgs->getEntityManager()->flush($app);
            }
        }
    }

    public function onFlush(OnFlushEventArgs $eventArgs)
    {
        $em = $eventArgs->getEntityManager();
        if ($this->inOnFlush) {
            return;
        }

        $this->inOnFlush = true;

        $uow = $em->getUnitOfWork();
        foreach ($uow->getScheduledEntityDeletions() as $entity) {
            if (!$entity instanceof App) {
                continue;
            }

            if ($entity->hasStates()) {
                /*
                 * As long as there are associated states an app cannot be removed.
                 * A not managed app will be removed if there are no associated
                 * states left, i.e. it is not installed on any device, anymore.
                 */
                $entity->setManaged(false);
                $uow->persist($entity);
                $md = $em->getClassMetadata(get_class($entity));
                $uow->recomputeSingleEntityChangeSet($md, $entity);
            }
        }

        foreach ($uow->getScheduledEntityUpdates() as $entity) {
            if (!$entity instanceof App) {
                continue;
            }

            $changeSet = $uow->getEntityChangeSet($entity);
            if (!$entity->getConfigGroups()->isDirty() && !isset($changeSet['managed'])
                && !isset($changeSet['supports']) && !$entity->isUploadedPackageChanged()) {
                continue;
            }

            if ($entity->isManaged()) {
                $this->appsToInstall[$entity->getId()] = $entity;
            } else if ($entity->hasStates()) {
                $this->appsToRemove[$entity->getId()] = $entity;
                // not managed apps can't have config groups
                $entity->setConfigGroups(array());
                $md = $em->getClassMetadata(get_class($entity));
                $uow->recomputeSingleEntityChangeSet($md, $entity);
            } else {
                // app is not installed on any device so it can be removed
                // from the db
                $em->remove($entity);
            }
        }

        $this->inOnFlush = false;
    }

    /**
     * avoid flush cycles during installation
     *
     * @param PostFlushEventArgs $eventArgs
     */
    public function postFlush(PostFlushEventArgs $eventArgs)
    {
        $eventArgs; // unused

        if ($this->inPostFlush) {
            return;
        }

        $this->inPostFlush = true;

        $mdmAdmin = $this->getMdmAdmin();
        foreach ($this->appsToRemove as $key => $app) {
            $mdmAdmin->removeApps($app);
            unset($this->appsToRemove[$key]);
        }

        foreach ($this->appsToInstall as $key => $app) {
            $mdmAdmin->installApps($app);
            $mdmAdmin->removeUnassignedApps($app);
            unset($this->appsToInstall[$key]);
        }

        $this->inPostFlush = false;
    }
}
