<?php
namespace Topalis\App\EventListener\ORM;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\OnFlushEventArgs;
use Doctrine\ORM\Query\Expr\Join;
use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\App\Entity\AppState;
use Topalis\App\Entity\IosApp;
use Topalis\App\Entity\IosAppState;
use Topalis\App\Entity\VppLicense;
use Topalis\User\Entity\User;

/**
 * VppListener
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class VppListener extends ContainerAware
{
    /**
     * Used to avoid endless loop
     * @var User[]
     */
    private $updatedUsers = array();

    public function preUpdate(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if ($entity instanceof User && !in_array($entity, $this->updatedUsers)) {
            $this->updatedUsers[] = $entity;
            $this->updateUser($entity, $eventArgs);
        }
    }

    public function postPersist(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if ($entity instanceof IosAppState) {
            $this->updateAppState($entity, $eventArgs);
        } else if ($entity instanceof VppLicense) {
            $this->updateLicense($entity);
        }
    }

    public function postUpdate(LifecycleEventArgs $eventArgs)
    {
        $entity = $eventArgs->getEntity();
        if ($entity instanceof IosAppState) {
            $this->updateAppState($entity, $eventArgs);
        } else if ($entity instanceof VppLicense) {
            $this->updateLicense($entity);
        }
    }

    public function onFlush(OnFlushEventArgs $eventArgs)
    {
        $em = $eventArgs->getEntityManager();
        $eventManager = $em->getEventManager();

        // remove event to avoid infinite recursion loop if we call $em->flush()
        $eventManager->removeEventListener("onFlush", $this);

        $uow = $em->getUnitOfWork();
        foreach ($uow->getScheduledEntityDeletions() as $entity) {
            if (!$entity instanceof IosAppState) {
                continue;
            }
            if (null === $entity->getVppLicense()) {
                continue;
            }

            // state with associated license must not be deleted
            $em = $eventArgs->getEntityManager();
            $uow = $em->getUnitOfWork();
            $em->persist($entity);
            $md = $em->getClassMetadata(get_class($entity));
            $uow->recomputeSingleEntityChangeSet($md, $entity);
            $this->getVppAssociateQueue()->disassociate($entity);
        }

        // re-attach since we're done
        $eventManager->addEventListener("onFlush", $this);
    }

    protected function updateUser(User $user, LifecycleEventArgs $eventArgs)
    {
        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($user);
        if (isset($changeSet['vppInviteLink']) && $user->getVppInviteLink()
            && $user->getVppStatus() == User::VPP_STATUS_REGISTERED) {
            // send invitation via mdm
            foreach ($user->getDevices() as $device) {
                if (!$device->isActive() || $device->getType()->getName() != "ios"
                    || !$device->getType()->isRolloutFinished($device)) {
                    continue;
                }

                $this->getMdmAdmin()->inviteToVolumePurchaseProgram($device, $user->getVppInviteLink());
            }
        } else if (isset($changeSet['vppStatus']) && $user->getVppStatus() == User::VPP_STATUS_ASSOCIATED) {
            // associate licenses if user becomes associated
            $waitingStates = $this->getAppStateEntityManager()->createQueryBuilder("s")
                ->select("partial s.{id}")
                ->where("s.state = :waitForLicense")
                ->join("s.device", "d")
                ->join("d.user", "u", Join::WITH, "u = :user")
                ->setParameter("waitForLicense", AppState::WAIT_FOR_LICENSE)
                ->setParameter("user", $user)
                ->getQuery()->getResult();
            foreach ($waitingStates as $state) {
                $this->getVppAssociateQueue()->associate($state);
            }
        }
    }

    protected function updateAppState(IosAppState $state, LifecycleEventArgs $eventArgs)
    {
        /** @var IosApp $app */
        $app = $state->getApp();
        if (!$app->isUseVpp()) {
            return;
        }
        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($state);
        if (!isset($changeSet['state']) && !isset($changeSet['mdmState'])) {
            return;
        }
        if (null === $state->getVppLicense()) {
            if (AppState::INSTALLATION_REJECTED === $state->getState()
                && "NeedsRedemption" === $state->getMdmState()
            ) {
                $this->getVppAssociateQueue()->associate($state);
            }
        }
    }

    protected function updateLicense(VppLicense $vppLicense)
    {
        if (VppLicense::STATUS_AVAILABLE !== $vppLicense->getStatus()) {
            return;
        }
        // find wating installations for this app
        $states = $this->getAppStateEntityManager()->findBy(array(
            "app" => $vppLicense->getApp(),
            "state" => AppState::WAIT_FOR_LICENSE,
        ));
        foreach ($states as $state) {
            $this->getVppAssociateQueue()->associate($state);
        }
    }

    /**
     * @return \Topalis\App\Job\VppAssociateQueue
     */
    protected function getVppAssociateQueue()
    {
        return $this->container->get("app.job.queue.vpp_associate");
    }

    /**
     * @return \Topalis\App\Entity\AppStateEntityManager
     */
    protected function getAppStateEntityManager()
    {
        return $this->container->get("app.entity.manager.ios_app_state");
    }

    /**
     * @return \Topalis\MDM\MdmAdmin
     */
    protected function getMdmAdmin()
    {
        return $this->container->get("mdm.admin");
    }
}
