<?php
namespace Topalis\App\Ios;

use Buzz\Browser;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\App\AppStore;
use Topalis\App\Entity\IosApp;
use Topalis\Configuration\Client\AuralisCurl;
use Topalis\App\Entity\App;

/**
 * @author Dominic Tubach <dominic.tubach@to.com>
 * @see    https://www.apple.com/itunes/affiliates/resources/documentation/itunes-store-web-service-search-api.html
 */
class IosAppStore extends ContainerAware implements AppStore
{
    /**
     * @var \Buzz\Browser
     */
    protected $browser;

    /**
     * @var string
     */
    protected $lookupUrl = "https://itunes.apple.com/%store%/lookup?";

    protected $searchUrl = "https://itunes.apple.com/%store%/search?media=software&entity=software,iPadSoftware";

    /**
     * @param integer $id iTunes Store ID to lookup.
     *
     * @return IosApp|NULL
     */
    public function lookup($id)
    {
        $apps = $this->lookupBy(array("id" => $id));

        return isset($apps[0]) ? $apps[0] : null;
    }

    /**
     * @param string $bundleId Bundle identifier to lookup.
     *
     * @return IosApp|NULL
     */
    public function lookupByBundleId($bundleId)
    {
        $apps = $this->lookupBy(array("bundleId" => $bundleId));

        return isset($apps[0]) ? $apps[0] : null;
    }

    public function lookupByBundleIds(array $bundleIds)
    {
        return $this->lookupBy(array("bundleId" => implode(",", $bundleIds)));
    }

    public function lookupByIdentifier($identifier)
    {
        return $this->lookupByBundleId($identifier);
    }

    public function lookupByITunesStoreId($storeId)
    {
        $apps = $this->lookupBy(array("id" => $storeId));

        return isset($apps[0]) ? $apps[0] : null;
    }

    public function lookupByITunesStoreIds(array $storeIds)
    {
        return $this->lookupBy(array("id" => implode(",", $storeIds)));
    }

    /**
     * @param array $params Associative array of lookup parameters.
     *
     * @return IosApp[] Array containing IosApp objects.
     */
    public function lookupBy(array $params)
    {
        if (empty($params)) {
            return null;
        }

        $url = $this->lookupUrl;
        foreach ($params as $key => $value) {
            $url .= "&" . $key . "=" . $value;
        }

        $result = $this->getResult($url);
        if (null === $result) {
            return;
        }

        $apps = array();
        foreach ($result['results'] as $data) {
            $apps[] = $this->resultToApp($data);
        }

        return $apps;
    }

    /**
     * @param string $term searchterm
     *
     * @return IosApp[]
     */
    public function search($term)
    {
        $url = $this->searchUrl . "&term=" . urlencode($term);
        $result = $this->getResult($url);
        if (null === $result) {
            return array();
        }
        $apps = array();
        foreach ($result['results'] as $data) {
            $apps[] = $this->resultToApp($data);
        }

        return $apps;
    }

    /**
     * @param IosApp[] $apps
     */
    public function refreshApps(array $apps)
    {
        /** @var IosApp[] $storeIds */
        $storeIds = array();
        /** @var IosApp[] $bundleIds */
        $bundleIds = array();
        foreach ($apps as $app) {
            if ($app->getType() == App::TYPE_UPLOADED) {
                continue;
            }
            if (null !== $app->getITunesStoreID()) {
                $storeIds[$app->getITunesStoreID()] = $app;
            } else {
                $bundleIds[$app->getIdentifier()] = $app;
            }
        }
        if (!empty($storeIds)) {
            $storeApps = null;
            try {
                $storeApps = $this->lookupByITunesStoreIds(array_keys($storeIds));
            } catch (\Buzz\Exception\ExceptionInterface $e) {
                // ignore Buzz exceptions and do not log. because this always happens if a firewall does not allow it,
                // which is not unlikely because itunes.apple.com is hosted by Akamai and has no fixed ip. #AUR-417
            } catch (\Exception $e) {
                // if something else fails here we have to continue #AUR-417
                $this->getLogger()->error("Error looking up app data from Apple", array("exception" => $e));
            }
            if (null !== $storeApps) {
                foreach ($storeApps as $storeApp) {
                    if (!isset($storeIds[$storeApp->getItunesStoreID()])) {
                        // this should never happen...
                        $this->getLogger()->warning("The iTunes Store identifier "
                            . $storeApp->getIdentifier() . " was not found in the storeIds array");
                        continue;
                    }

                    $app = $storeIds[$storeApp->getITunesStoreID()];
                    $app->merge($storeApp);
                    $app->setType(App::TYPE_STORE);
                }
            }
        }
        if (!empty($bundleIds)) {
            $storeApps = null;
            try {
                $storeApps = $this->lookupByBundleIds(array_keys($bundleIds));
            } catch (\Buzz\Exception\ExceptionInterface $e) {
                // ignore Buzz exceptions and do not log. because this always happens if a firewall does not allow it,
                // which is not unlikely because itunes.apple.com is hosted by Akamai and has no fixed ip. #AUR-417
            } catch (\Exception $e) {
                // if something else fails here we have to continue #AUR-417
                $this->getLogger()->error("Error looking up app data from Apple", array("exception" => $e));
            }
            if (null !== $storeApps) {
                foreach ($storeApps as $storeApp) {
                    if (!isset($bundleIds[$storeApp->getIdentifier()])) {
                        // this happens if identifiers differs in there case sensitivity #AUR-603
                        continue;
                    }

                    $app = $bundleIds[$storeApp->getIdentifier()];
                    $app->merge($storeApp);
                    if (null !== $app->getITunesStoreID()) {
                        $app->setType(App::TYPE_STORE);
                    }
                }
            }
        }
    }

    protected function getResult($url)
    {
        $stores = array();
        $store = $this->getAuralisConfig()->getItunesAppStore();
        if ($store) {
            $stores[] = $store;
        }
        if (!in_array("us", $stores)) {
            $stores[] = "us"; // fallback to us app store
        }
        foreach ($stores as $store) {
            $response = $this->getBrowser()->get(str_replace("%store%", $store, $url));
            $result = json_decode($response->getContent(), true);
            if (isset($result['resultCount']) && $result['resultCount'] > 0) {
                return $result;
            }
        }

        return null;
    }

    /**
     * @param array $data
     *
     * @return IosApp
     */
    protected function resultToApp(array $data)
    {
        $app = new IosApp();
        if (isset($data['trackId'])) {
            $app->setITunesStoreID($data['trackId']);
            $app->setType(App::TYPE_STORE);
        }
        if (isset($data['bundleId'])) {
            $app->setIdentifier($data['bundleId']);
        }
        if (isset($data['trackName'])) {
            $app->setName($data['trackName']);
        }
        if (isset($data['artworkUrl60'])) {
            $app->setIcon($data['artworkUrl60']);
        }
        if (isset($data['artworkUrl100'])) {
            $app->setIconLarge($data['artworkUrl100']);
        }
        if (isset($data['price'])) {
            $app->setPrice($data['price']);
        }
        if (isset($data['currency'])) {
            $app->setCurrency($data['currency']);
        }
        if (isset($data['description'])) {
            $app->setDescription($data['description']);
        }
        if (isset($data['releaseNotes'])) {
            $app->setReleaseNotes($data['releaseNotes']);
        }
        if (isset($data['version'])) {
            $app->setVersion($data['version']);
        }
        if (isset($data['artistName'])) {
            $app->setVendor($data['artistName']);
        }
        if (isset($data['supportedDevices'])) {
            $app->setSupports(array("devices" => $data['supportedDevices']));
        }

        return $app;
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    protected function getAuralisConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @return \Buzz\Browser
     */
    protected function getBrowser()
    {
        if (null !== $this->browser) {
            return $this->browser;
        }

        return $this->browser = new Browser(new AuralisCurl($this->container->get("config.auralis")));
    }

    /**
     * @return LoggerInterface
     */
    public function getLogger()
    {
        return $this->container->get("logger");
    }
}
