<?php
namespace Topalis\App\Ios;

use Ashb\PngUncrush\PPngUncrush\PPngUncrush;
use CFPropertyList\CFPropertyList;
use Symfony\Component\HttpFoundation\File\File;
use Topalis\App\Exception\IpaFileException;
use Topalis\Util\FileSystemUtil;

/**
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class IpaFile
{
    // @codingStandardsIgnoreStart
    /**
     * Array of device keys in app store to supported capabilities.
     * @see https://developer.apple.com/library/ios/documentation/DeviceInformation/Reference/iOSDeviceCompatibility/DeviceCompatibilityMatrix/DeviceCompatibilityMatrix.html
     * @see https://github.com/ArtSabintsev/UIDevice-SupportedDevices/blob/master/UIDevice%2BSupportedDevices/UIDevice%2BSupportedDevices.m
     * @var array
     */
    private static $devices = array(
        "iPhone" => array("accelerometer", "armv6", "location-services", "microphone", "opengles-1", "sms", "still-camera", "telephony", "wifi"),
        "iPhone-3G" => array("accelerometer", "armv6", "gps", "location-services", "microphone", "opengles-1", "peer-peer", "sms", "still-camera", "telephony", "wifi"),
        "iPhone-3GS" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "gamekit", "gps", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "sms", "still-camera", "telephony", "video-camera", "wifi"),
        "iPhone4" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "camera-flash", "front-facing-camera", "gamekit", "gps", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "sms", "still-camera", "telephony", "video-camera", "wifi"),
        "iPhone4S" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "camera-flash", "front-facing-camera", "gamekit", "gps", "gyroscope", "healthkit", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "sms", "still-camera", "telephony", "video-camera", "wifi"),
        "iPhone5" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "camera-flash", "front-facing-camera", "gamekit", "gps", "gyroscope", "healthkit", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "sms", "still-camera", "telephony", "video-camera", "wifi"),
        "iPhone5c" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "camera-flash", "front-facing-camera", "gamekit", "gps", "gyroscope", "healthkit", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "sms", "still-camera", "telephony", "video-camera", "wifi"),
        "iPhone5s" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "camera-flash", "front-facing-camera", "gamekit", "gps", "gyroscope", "healthkit", "location-services", "magnetometer", "metal", "microphone", "opengles-1", "opengles-2", "opengles-3", "peer-peer", "sms", "still-camera", "telephony", "video-camera", "wifi"),
        "iPhone6" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "camera-flash", "front-facing-camera", "gamekit", "gps", "gyroscope", "healthkit", "location-services", "magnetometer", "metal", "microphone", "opengles-1", "opengles-2", "opengles-3", "peer-peer", "sms", "still-camera", "telephony", "video-camera", "wifi"),
        "iPhone6Plus" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "camera-flash", "front-facing-camera", "gamekit", "gps", "gyroscope", "healthkit", "location-services", "magnetometer", "metal", "microphone", "opengles-1", "opengles-2", "opengles-3", "peer-peer", "sms", "still-camera", "telephony", "video-camera", "wifi"),

        // not all features are supported by all devices hat have the deivce name iPadMini or iPadMini4G
        // note: iPad4,5 (cellular) seem to have iPadMini as device name
        "iPadMini" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "front-facing-camera", "gamekit", "gps", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "still-camera", "video-camera", "wifi"),
        "iPadMini4G" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "front-facing-camera", "gamekit", "gps", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "still-camera", "video-camera", "wifi"),

        "iPadWifi" => array("accelerometer", "armv6", "armv7", "gamekit", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "wifi"),
        "iPad3G" => array("accelerometer", "armv6", "armv7", "gamekit", "gps", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "wifi"),
        "iPad2Wifi" => array("accelerometer", "armv6", "armv7", "front-facing-camera", "gamekit", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "still-camera", "video-camera", "wifi"),
        "iPad23G" => array("accelerometer", "armv6", "armv7", "front-facing-camera", "gamekit", "gps", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "still-camera", "video-camera", "wifi"),
        "iPadThirdGen" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "front-facing-camera", "gamekit", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "still-camera", "video-camera", "wifi"),
        "iPadThirdGen4G" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "front-facing-camera", "gamekit", "gps", "gyroscope", "location-services", "magnetometer", "microphone", "opengles-1", "opengles-2", "peer-peer", "still-camera", "video-camera", "wifi"),
        "iPadFourthGen" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "front-facing-camera", "gamekit", "gyroscope", "location-services", "magnetometer", "metal", "microphone", "opengles-1", "opengles-2", "opengles-3", "peer-peer", "still-camera", "video-camera", "wifi"),
        "iPadFourthGen4G" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "front-facing-camera", "gamekit", "gps", "gyroscope", "location-services", "magnetometer", "metal", "microphone", "opengles-1", "opengles-2", "opengles-3", "peer-peer", "still-camera", "video-camera", "wifi"),

        "iPod-touch" => array("accelerometer", "armv6", "location-services", "opengles-1", "wifi"),
        "iPod-touch-with-mic" => array("accelerometer", "armv6", "location-services", "microphone", "opengles-1", "wifi"),
        "iPodTouchSecondGen" => array("accelerometer", "armv6", "gamekit", "location-services", "microphone", "opengles-1", "peer-peer", "wifi"),
        "iPodTouchThirdGen" => array("accelerometer", "armv6", "armv7", "gamekit", "location-services", "microphone", "opengles-1", "opengles-2", "peer-peer", "wifi"),
        // this is not a typo
        "iPodTouchourthGen" => array("accelerometer", "armv6", "armv7", "front-facing-camera", "gamekit", "gyroscope", "location-services", "microphone", "opengles-1", "opengles-2", "peer-peer", "still-camera", "video-camera", "wifi"),
        // there exists a iPod touch 5 withoud rear-facing camera, but no extra device key
        "iPodTouchFifthGen" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "camera-flash", "front-facing-camera", "gamekit", "gyroscope", "healthkit", "location-services", "microphone", "opengles-1", "opengles-2", "peer-peer", "still-camera", "video-camera", "wifi"),
        // TODO is magnetometer supported? This model is not not in the device compatibility reference, yet...
        // metal and opengles-3 are supported by the Apple A8 GPU
        "iPodTouchSixthGen" => array("accelerometer", "armv6", "armv7", "auto-focus-camera", "bluetooth-le", "camera-flash", "front-facing-camera", "gamekit", "gyroscope", "healthkit", "location-services", "magnetometer", "metal", "microphone", "opengles-1", "opengles-2", "opengles-3", "peer-peer", "still-camera", "video-camera", "wifi"),
    );
    // @codingStandardsIgnoreEnd

    /**
     * @var \SplFileInfo
     */
    private $file;

    /**
     * @var \ZipArchive
     */
    private $archive;

    /**
     * basedir with app resources
     * @var string
     */
    private $baseDir;

    /**
     * Content of Info.plist
     * @var array
     */
    private $info;

    /**
     * cached icons
     * @var array
     */
    private $icons;

    /**
     * Array of required capabilities
     * @var array
     */
    private $requiredCapabilities;

    public function __construct($pathname)
    {
        $this->file = new File($pathname);
    }

    /**
     * extracts info form Info.plist in ipa files
     * @see https://developer.apple.com/library/ios/documentation/general/Reference/InfoPlistKeyReference/Articles/AboutInformationPropertyListFiles.html
     * @return array|mixed
     */
    public function getInfo()
    {
        if (null !== $this->info) {
            return $this->info;
        }

        $fileContent = $this->getFile("Info.plist");
        if ($fileContent === false) {
            throw new IpaFileException("Could not read Info.plist");
        }
        $plist = new CFPropertyList();
        $plist->parse($fileContent, CFPropertyList::FORMAT_AUTO);

        return $this->info = $plist->toArray();
    }

    public function getIdentifier()
    {
        $this->getInfo();

        return $this->info['CFBundleIdentifier'];
    }

    public function getVersion()
    {
        $this->getInfo();

        return $this->info['CFBundleVersion'];
    }

    /**
     * @return string The minimum required OS version.
     */
    public function getMinOSVersion()
    {
            $info = $this->getInfo();
            return $info['MinimumOSVersion'];
    }

    public function getName()
    {
        $this->getInfo();

        return $this->info['CFBundleDisplayName'];
    }

    public function getFile($filename)
    {
        $archive = $this->prepareArchive();

        return $archive->getFromName($this->baseDir . $filename);
    }

    /**
     * @return array Array with required capabilities.
     */
    public function getRequiredCapabilities()
    {
        if (null !== $this->requiredCapabilities) {
            return $this->requiredCapabilities;
        }

        $this->requiredCapabilities = array();
        $info = $this->getInfo();
        if (!isset($info['UIRequiredDeviceCapabilities'])) {
            return $this->requiredCapabilities;
        }

        // can be an array or a dictionary with the key as capability and a value of true or false
        foreach ($this->info['UIRequiredDeviceCapabilities'] as $key => $value) {
            if (is_bool($value)) {
                if (!$value) {
                    // capability not required
                    continue;
                }
                $this->requiredCapabilities[] = $key;
            } else {
                $this->requiredCapabilities[] = $value;
            }
        }

        return $this->requiredCapabilities;
    }

    public function getStream($filename)
    {
        $archive = $this->prepareArchive();

        return $archive->getStream($this->baseDir . $filename);
    }

    public function getSupportedDevices()
    {
        $supportedDevices = self::$devices;

        // can be an array or a dictionary with the key as capability and a value of true or false
        foreach ($this->getRequiredCapabilities() as $capability) {
            foreach ($supportedDevices as $key => $supportedCapabilities) {
                if (!in_array($capability, $supportedCapabilities)) {
                    unset($supportedDevices[$key]);
                }
            }
        }

        return array_keys($supportedDevices);
    }

    public function getIcons()
    {
        if (null !== $this->icons) {
            return $this->icons;
        }
        $this->getInfo();
        if (!isset($this->info['CFBundleIcons']['CFBundlePrimaryIcon']['CFBundleIconFiles'])) {
            return array();
        }

        $iconList = array();
        $icons = $this->info['CFBundleIcons']['CFBundlePrimaryIcon']['CFBundleIconFiles'];
        foreach ($icons as $icon) {
            $suffixList = array("", "@2x", "~ipad", "@2x~ipad");
            foreach ($suffixList as $suffix) {
                $iconFile = $this->getFile($icon . $suffix . ".png");
                if (false === $iconFile) {
                    continue;
                }
                $tmpName = tempnam(sys_get_temp_dir(), 'ipa-icon.png');
                $uncrushed = tempnam(sys_get_temp_dir(), 'ipa-icon.uncrushed.png');
                file_put_contents($tmpName, $iconFile);
                $this->uncrushPng($tmpName, $uncrushed);
                $size = getimagesize($uncrushed);
                unlink($tmpName);
                if (isset($iconList[$size[0]])) {
                    unlink($uncrushed);
                } else {
                    $iconList[$size[0]] = array("name" => $icon.$suffix, "size" => $size, "file" => $uncrushed);
                }
            }
        }
        ksort($iconList);

        return $this->icons = $iconList;
    }

    /**
     * @param int $size icon size
     *
     * @return array|null
     */
    public function getIconForSize($size)
    {
        $icons = $this->getIcons();
        if (empty($icons)) {
            return null;
        }

        foreach ($icons as $iconSize => $icon) {
            if ($iconSize >= $size) {
                return $icon;
            }
        }

        return $icons[max(array_keys($icons))];
    }

    public function getIcon()
    {
        $icon = $this->getIconForSize(40);

        return $icon;
    }

    public function getLargeIcon()
    {
        $icon = $this->getIconForSize(100);

        return $icon;
    }

    private function prepareArchive()
    {
        if (null === $this->archive) {
            $this->archive = new \ZipArchive();
            if (true !== ($resultCode = $this->archive->open($this->file->getPathname()))) {
                throw new IpaFileException(sprintf("failed to open ipa (%s)", $resultCode));
            }
        }
        if (null === $this->baseDir) {
            for ($i = 0; $i < $this->archive->numFiles; ++$i) {
                $name = $this->archive->getNameIndex($i);
                if (preg_match("=^Payload/([^/]+).app=s", $name, $matches)) {
                    $this->baseDir = "Payload/" . $matches[1] . ".app/";
                    break;
                }
            }
        }

        return $this->archive;
    }

    private function uncrushPng($imagePath, $outPath = null)
    {
        $uncrush = PPngUncrush::getInstance();
        $uncrush->setFilePath($imagePath);

        return $uncrush->decode($outPath);
    }

    public function __destruct()
    {
        // cleanup icons
        if (null === $this->icons) {
            return;
        }
        foreach ($this->icons as $icon) {
            FileSystemUtil::tryUnlinkFile($icon['file']);
        }
    }
}
