<?php
namespace Topalis\App\Ios;

use Buzz\Exception\ClientException;
use Topalis\App\Exception\VppApiException;
use Topalis\App\Exception\VppClientException;
use Topalis\App\Exception\VppException;
use Topalis\App\Exception\VppResponseException;
use Topalis\Configuration\AuralisConfig;
use Topalis\Kernel\Exception\Http\RequestFailedException;
use Topalis\Kernel\Http\Browser;
use Topalis\Kernel\Http\Response;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class VppApi implements VppApiInterface
{
    const ERROR_LOGIN_REQUIRED = 9601;
    const ERROR_INVALID_AUTH_TOKEN = 9622;

    const SERVICE_CLIENT_CONFIG = "clientConfigSrvUrl";

    const SERVICE_GET_USERS = "getUsersSrvUrl";
    const SERVICE_REGISTER_USER = "registerUserSrvUrl";
    const SERVICE_RETIRE_USER = "retireUserSrvUrl";

    const SERVICE_GET_LICENSES = "getLicensesSrvUrl";
    const SERVICE_ASSOCIATE_LICENSE = "associateLicenseSrvUrl";
    const SERVICE_DISASSOCIATE_LICENSE = "disassociateLicenseSrvUrl";

    /**
     * @var AuralisConfig
     */
    protected $config;

    /**
     * @var Browser
     */
    private $browser;

    /**
     * @var array
     */
    private $services;

    /**
     * @param Browser       $browser
     * @param AuralisConfig $config
     */
    public function __construct(Browser $browser, AuralisConfig $config)
    {
        $this->browser = $browser;
        $this->config = $config;
    }

    public function getServiceUrl($name)
    {
        if (null === $this->services) {
            $this->services = $this->sendRawRequest(
                "https://vpp.itunes.apple.com/WebObjects/MZFinance.woa/wa/VPPServiceConfigSrv"
            );
        }

        return $this->services[$name];
    }

    public function getToken()
    {
        return $this->config->getVppToken();
    }

    public function hasToken()
    {
        return strlen($this->getToken()) > 0;
    }

    public function hasValidToken()
    {
        return $this->isTokenValid($this->getToken());
    }

    public function isAvailable(&$message = null)
    {
        try {
            if (!$this->hasValidToken()) {
                if (null !== $message) {
                    $message = "Invalid VPP token";
                }
                return false;
            }

            return true;
        } catch (VppException $e) {
            if (null !== $message) {
                $message = $e->getMessage();
            }
            return false;
        }
    }

    public function isTokenValid($token)
    {
        if (empty($token)) {
            return false;
        }

        try {
            $this->sendRequest(self::SERVICE_CLIENT_CONFIG, array(
                "sToken" => $token,
            ));
            return true;
        } catch (VppApiException $e) {
            if ($e->getCode() == self::ERROR_INVALID_AUTH_TOKEN || $e->getCode() == self::ERROR_LOGIN_REQUIRED) {
                return false;
            }

            throw $e;
        }
    }

    public function sendRequest($service, array $params = array())
    {
        return $this->sendRawRequest($this->getServiceUrl($service), $params);
    }

    public function sendBatchRequest($service, $key, array $params = array())
    {
        $data = array();
        $lastModified = null;
        do {
            $content = $this->sendRequest($service, $params);
            if (isset($content[$key])) {
                foreach ($content[$key] as $value) {
                    $data[] = $value;
                }
            }
            $params['batchToken'] = isset($content['batchToken']) ? $content['batchToken'] : null;
            if (isset($content['sinceModifiedToken'])) {
                $lastModified = $content['sinceModifiedToken'];
            }
        } while (null !== $params['batchToken']);

        return array("data" => $data, "lastModified" => $lastModified);
    }

    /**
     * @param string $url HTTP URL
     * @param array  $params HTTP GET params
     *
     * @return array decoded json result
     *
     * @throws VppClientException If an exception occurs when trying to connect.
     * @throws VppResponseException If the status code of the HTTP response is
     *     not between 200 and 299.
     * @throws VppApiException If the VPP server rejects executing the request
     *     for a specific reason (status != 0).
     */
    protected function sendRawRequest($url, array $params = array())
    {
        if (!isset($params['sToken'])) {
            $params['sToken'] = $this->getToken();
        }
        $url .= "?";
        foreach ($params as $key => $value) {
            $url .= $key . "=" . urlencode($value) . "&";
        }
        $url = substr($url, 0, -1);

        try {
            /** @var Response $response */
            $response = $this->browser->get($url);
        } catch (ClientException $e) {
            throw new VppClientException($e->getMessage(), $e->getCode(), $e);
        } catch (RequestFailedException $e) {
            throw new VppResponseException($e->getResponse()->getRequest(), $e->getResponse(),
                $e->getMessage(), $e->getCode(), $e);
        }
        $content = json_decode($response->getContent(), true);
        if (0 !== $content['status']) {
            throw new VppApiException($response, $content);
        }

        return $content;
    }
}
