<?php
namespace Topalis\App\Ios;

use Psr\Log\LoggerInterface;
use Topalis\App\Entity\AppEntityManager;
use Topalis\App\Entity\AppState;
use Topalis\App\Entity\AppStateEntityManager;
use Topalis\App\Entity\IosApp;
use Topalis\App\Entity\IosAppState;
use Topalis\App\Entity\VppLicense;
use Topalis\App\Entity\VppLicenseEntityManager;
use Topalis\App\Exception\NoVppLicenseAvailableException;
use Topalis\App\Exception\VppAlreadyAssociatedException;
use Topalis\App\Exception\VppApiException;
use Topalis\App\Exception\VppException;
use Topalis\Configuration\AuralisConfig;
use Topalis\User\Entity\User;
use Topalis\User\Entity\UserEntityManager;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class VppLicenseService
{
    const ERROR_LICENSE_NOT_ASSOCIATED = 9619;

    /**
     * @var VppApi
     */
    protected $vppApi;

    /**
     * @var AuralisConfig
     */
    protected $config;

    /**
     * @var AppEntityManager
     */
    protected $appEntityManager;

    /**
     * @var UserEntityManager
     */
    protected $userEntityManager;

    /**
     * @var AppStateEntityManager
     */
    protected $appStateEntityManager;

    /**
     * @var VppLicenseEntityManager
     */
    protected $vppLicenseEntityManager;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @param VppApi                  $vppApi
     * @param AuralisConfig           $config
     * @param AppEntityManager        $appEntityManager
     * @param UserEntityManager       $userEntityManager
     * @param AppStateEntityManager   $appStateEntityManager
     * @param VppLicenseEntityManager $vppLicenseEntityManager
     * @param LoggerInterface         $logger
     */
    public function __construct(
        VppApi $vppApi,
        AuralisConfig $config,
        AppEntityManager $appEntityManager,
        UserEntityManager $userEntityManager,
        AppStateEntityManager $appStateEntityManager,
        VppLicenseEntityManager $vppLicenseEntityManager,
        LoggerInterface $logger
    )
    {
        $this->vppApi = $vppApi;
        $this->config = $config;
        $this->appEntityManager = $appEntityManager;
        $this->userEntityManager = $userEntityManager;
        $this->appStateEntityManager = $appStateEntityManager;
        $this->vppLicenseEntityManager = $vppLicenseEntityManager;
        $this->logger = $logger;
    }

    /**
     *
     * @param boolean $incremental
     *
     * @throws VppException
     */
    public function syncLicenses($incremental = true)
    {
        $this->logger->info("Sync VPP licensens" . (($incremental) ? " incrementally" : ""));
        $this->updateLicenses($incremental);
        $this->reclaimLicenses();
        $this->cleanupLicenses();
    }

    public function isAssociated(User $user, IosApp $app)
    {
        return $this->vppLicenseEntityManager->count(
            array(
                "user"   => $user,
                "app"    => $app,
                "status" => VppLicense::STATUS_ASSOCIATED,
            )
        ) > 0;
    }

    /**
     * @param string $message Contains the reason if the VPP service is not
     *    available.
     *
     * @return bool true if the VPP service is available, false otherwise.
     */
    public function isAvailable(&$message = null)
    {
        return $this->vppApi->isAvailable($message);
    }

    /**
     * bool true if a VPP token is configured, false otherwise.
     */
    public function isTokenConfigured()
    {
        return $this->vppApi->hasToken();
    }

    /**
     * @param IosAppState $appState
     *
     * @throws NoVppLicenseAvailableException
     * @throws VppAlreadyAssociatedException
     * @throws VppException
     */
    public function associate(IosAppState $appState)
    {
        $user = $appState->getDevice()->getUser();
        $app = $appState->getApp();
        $this->logger->info(sprintf("Associate license to AppState %s", $appState->getId()), array(
            "user"          => $user->getId(),
            "userLogin"     => $user->getLogin(),
            "app"           => $app->getId(),
            "appIdentifier" => $app->getIdentifier(),
        ));

        if (null === $user->getVppUserId()) {
            throw new VppException("User " . $user->getLogin() . " has no VPP user id");
        }
        $userLicense = $this->vppLicenseEntityManager->findOneBy(array(
            "user"   => $user,
            "app"    => $app,
            "status" => VppLicense::STATUS_ASSOCIATED,
        ));
        if (null !== $userLicense) {
            $appState->setVppLicense($userLicense);
            $this->appStateEntityManager->save($appState);

            throw new VppAlreadyAssociatedException();
        }

        // find unused license
        /** @var VppLicense $license */
        $license = $this->vppLicenseEntityManager->findOneBy(array(
            "app"    => $app,
            "status" => VppLicense::STATUS_AVAILABLE,
        ));
        if (null === $license) {
            $this->logger->info(sprintf("No more VPP licenses available for app %s", $app->getIdentifier()), array(
                "app"           => $app->getId(),
                "state"         => $appState->getId(),
            ));
            $appState
                ->setState(AppState::REJECTED_SYSTEM)
                ->setRejectionReason("NoVPPLicense")
            ;
            $this->appStateEntityManager->save($appState);
            throw new NoVppLicenseAvailableException();
        }
        $this->logger->info(sprintf("Using license %s for AppState %s", $license->getLicenseId(), $appState->getId()),
            array("license" => $license->getId()));
        $this->vppApi->sendRequest(
            VppApi::SERVICE_ASSOCIATE_LICENSE,
            array(
                "userId"    => $user->getVppUserId(),
                "licenseId" => $license->getLicenseId(),
            )
        );

        $license
            ->setStatus(VppLicense::STATUS_ASSOCIATED)
            ->setUser($user)
        ;
        $this->vppLicenseEntityManager->save($license);

        $appState->setVppLicense($license);
        $this->appStateEntityManager->save($appState);
    }

    /**
     *
     * @param IosAppState $appState
     *
     * @throws VppException
     */
    public function disassociate(IosAppState $appState)
    {
        /** @var VppLicense $license */
        $license = $appState->getVppLicense();
        if (null === $license) {
            throw new VppException(sprintf("No VPP license found for app state %s", $appState->getId()));
        }
        $count = $this->appStateEntityManager->createQueryBuilder("s")
            ->select("COUNT(s)")
            ->where("s.vppLicense = :license")
            ->setParameter("license", $license)
            ->getQuery()
            ->getSingleScalarResult()
        ;
        if ($count == 1) {
            $this->logger->info(
                sprintf("Disassociate license for AppState %s", $appState->getId()),
                array(
                    "license"       => $appState->getVppLicense()->getId(),
                    "licenseId"     => $appState->getVppLicense()->getLicenseId(),
                    "app"           => $appState->getApp()->getId(),
                    "appIdentifier" => $appState->getApp()->getIdentifier(),
                )
            );
            $this->disassociateLicense($license);
        } else {
            $this->logger->info(
                sprintf("Disassociate not possible: The license is used by %s AppStates on other devices", $count),
                array(
                    "state"   => $appState->getId(),
                    "license" => $license->getId(),
                )
            );
        }
        $appState->setVppLicense(null);
        $this->appStateEntityManager->save($appState);
    }

    /**
     *
     * @param VppLicense $license
     *
     * @throws VppException
     */
    protected function disassociateLicense(VppLicense $license)
    {
        $this->logger->info("Disassociate license " . $license->getLicenseId(), array(
            "license"       => $license->getId(),
            "app"           => $license->getApp()->getId(),
            "appIdentifier" => $license->getApp()->getIdentifier(),
            "user"          => $license->getUser() ? $license->getUser()->getId() : null,
            "userLogin"     => $license->getUser() ? $license->getUser()->getLogin() : null,
        ));
        try {
            $this->vppApi->sendRequest(
                VppApi::SERVICE_DISASSOCIATE_LICENSE,
                array(
                    "licenseId" => $license->getLicenseId(),
                )
            );
            $license
                ->setStatus(VppLicense::STATUS_AVAILABLE)
                ->setUser(null)
            ;
        } catch (VppApiException $e) {
            if ($e->getCode() === self::ERROR_LICENSE_NOT_ASSOCIATED) {
                $this->logger->info("Failed to disassociate license " . $license->getLicenseId()
                    . ": license is not associated", array(
                    "license"       => $license->getId(),
                    "app"           => $license->getApp()->getId(),
                    "appIdentifier" => $license->getApp()->getIdentifier(),
                    "user"          => $license->getUser() ? $license->getUser()->getId() : null,
                    "userLogin"     => $license->getUser() ? $license->getUser()->getLogin() : null,
                ));
                $license
                    ->setStatus(VppLicense::STATUS_AVAILABLE)
                    ->setUser(null)
                ;
            } else {
                throw $e;
            }
        }
        $this->vppLicenseEntityManager->save($license);
    }

    /**
     * @param bool $incremental
     *
     * @return mixed
     *
     * @throws VppException
     */
    protected function updateLicenses($incremental)
    {
        $params = array();
        if ($incremental) {
            $lastModified = $this->config->getVppLicenseLastModified();
            if (null !== $lastModified) {
                $params['sinceModifiedToken'] = $lastModified;
            }
        }
        $result = $this->vppApi->sendBatchRequest(VppApi::SERVICE_GET_LICENSES, "licenses", $params);

        $licenseList = array();
        foreach ($result['data'] as $license) {
            // only handle app licenses. Other value would be Publication for books
            if ("Application" !== $license['productTypeName']) {
                continue;
            }
            /** @var VppLicense $entity */
            $entity = $this->vppLicenseEntityManager->findOneBy(array("licenseId" => $license['licenseId']));
            if (null === $entity) {
                $entity = new VppLicense();
                $entity->setLicenseId($license['licenseId']);
            }
            $licenseList[] = $entity;
            // find or create app by iTunesStoreID
            $app = $this->appEntityManager->findOneBy(array("iTunesStoreID" => $license['adamId']));
            if (null === $app) {
                $app = new IosApp();
                $app
                    ->setITunesStoreID($license['adamId'])
                    // set tmp values, which will be replaced with AppStore data
                    ->setIdentifier("auralis.vpp.tmp." . $license['adamId'])
                    ->setName($license['adamId'])
                    ->setType(IosApp::TYPE_STORE)
                    ->setRemoveWheneMdmIsRemoved(true)
                    ->setPreventBackup(true)
                ;
            }
            $app
                ->setManaged(true)
                ->setUseVpp(true)
            ;
            $entity->setApp($app);
            switch ($license['status']) {
                case "Available":
                    $entity->setStatus(VppLicense::STATUS_AVAILABLE);
                    $entity->setUser(null);
                    break;
                case "Associated":
                    $entity->setStatus(VppLicense::STATUS_ASSOCIATED);
                    /** @var User $user */
                    $user = $this->userEntityManager->findOneBy(array("vppUserId" => $license['userId']));
                    if (null !== $user) {
                        $entity->setUser($user);
                    }
                    break;
            }
            $this->vppLicenseEntityManager->save($entity);
        }
        // Delete license not returned by api, but persisted in DB
        if (!$incremental) {
            $qb = $this->vppLicenseEntityManager->createQueryBuilder("l");
            if ($licenseList) {
                $qb->where("l NOT IN (:licenses)")->setParameter("licenses", $licenseList);
            }
            /** @var VppLicense[] $licenses */
            $licenses = $qb->getQuery()->getResult();
            foreach ($licenses as $license) {
                $this->vppLicenseEntityManager->delete($license);
            }
        }

        if (null !== $result['lastModified']) {
            $this->config->setVppLicenseLastModified($result['lastModified']);
        }
    }

    /**
     * reclaim unused licenses
     *
     * @throws VppException
     */
    protected function reclaimLicenses()
    {
        $this->logger->debug("Reclaiming unused licenses");
        /** @var IosAppState[] $states */
        $states = $this->appStateEntityManager->createQueryBuilder("s")
            ->where("s.vppLicense IS NOT NULL")
            ->andWhere("s.state != :installation_requested AND s.state NOT IN (:licenseRequiredStates)")
            ->setParameter("installation_requested", AppState::INSTALLATION_REQUESTED)
            ->setParameter("licenseRequiredStates", IosAppState::getLicenseRequiredStates())
            ->getQuery()
            ->getResult()
        ;
        foreach ($states as $state) {
            try {
                $this->disassociate($state);
            } catch (VppApiException $e) {
                $this->logger->error("Failed to reclaim unused license", array(
                    "state"     => $state->getId(),
                    "exception" => $e,
                ));
            }
        }

        // disassociate licenses without app states
        /** @var \Topalis\App\Entity\VppLicense[] $licensesWithoutState */
        $licensesWithoutState = $this->vppLicenseEntityManager->createQueryBuilder("l")
            ->where("l.status = :associated")
            ->andWhere("SIZE(l.states) = 0")
            ->setParameter("associated", VppLicense::STATUS_ASSOCIATED)
            ->getQuery()
            ->getResult();
        foreach ($licensesWithoutState as $license) {
            try {
                $this->disassociateLicense($license);
            } catch (VppApiException $e) {
                $this->logger->error("Failed to reclaim unused license '" . $license->getLicenseId() . "'", array(
                    "license"     => $license->getId(),
                    "exception"   => $e,
                ));
            }
        }
    }

    /**
     * Disassociate associated licenses without assigned states.
     *
     * @throws VppException
     */
    protected function cleanupLicenses()
    {
        /** @var VppLicense[] $licenses */
        $licenses = $this->vppLicenseEntityManager->createQueryBuilder("l")
            ->select("l")
            ->leftJoin("l.states", "s")
            ->where("l.status = :associated")
            ->setParameter("associated", VppLicense::STATUS_ASSOCIATED)
            ->having("COUNT(s) = 0")
            ->groupBy("l.id")
            ->getQuery()
            ->getResult()
        ;
        foreach ($licenses as $license) {
            try {
                $this->disassociateLicense($license);
            } catch (VppApiException $e) {
                $this->logger->error("Failed to disassociate license " . $license->getLicenseId(), array(
                    "license"   => $license->getId(),
                    "exception" => $e,
                ));
            }
        }
    }
}
