<?php
namespace Topalis\App\Ios;

/**
 * Encodes a VPP token and provides access to its values.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class VppToken
{
    /**
     * @var array
     */
    private $decoded;

    /**
     * @var string
     */
    private $encoded;

    /**
     * Constructor.
     *
     * @param string $encoded The encoded VPP token.
     */
    public function __construct($encoded)
    {
        $this->setEncoded($encoded);
    }

    /**
     * @param string $key
     *
     * @return scalar the value with the specified key from the decoded VPP
     *    token.
     */
    public function __get($key)
    {
        return $this->decoded[$key];
    }

    /**
     * @param string $key
     *
     * @return bool if the value with the specified key is available in the
     *    decoded VPP token, false otherwise.
     */
    public function __isset($key)
    {
        return isset($this->decoded[$key]);
    }

    /**
     * @param string $key
     *
     * @return scalar|null the value with the specified key from the decoded VPP
     *    token, or null if it does not exist.
     */
    public function get($key)
    {
        return isset($this->decoded[$key]) ? $this->decoded[$key] : null;
    }

    /**
     * @return array|null Returns the decoded VPP token, or null if the encoded
     *    VPP token couldn't be decoded.
     */
    public function getDecoded()
    {
        return $this->decoded;
    }

    /**
     * @return string The encoded VPP token.
     */
    public function getEncoded()
    {
        return $this->encoded;
    }

    /**
     * Sets the encoded VPP token.
     *
     * @param string $encoded The encoded VPP token.
     */
    public function setEncoded($encoded)
    {
        $this->encoded = $encoded;
        $this->decoded = json_decode(base64_decode($encoded), true);
        if (!is_array($this->decoded) || empty($this->decoded)) {
            $this->decoded = null;
        }
    }

    /**
     *
     * @return int|null The token expiration date as UNIX timestamp, or null if
     *    it's not available.
     */
    public function getExpDate()
    {
        $expDate = $this->get("expDate");
        return null === $expDate ? null : strtotime($expDate);
    }

    /**
     * @return string|null The name of the organization for which the token is
     *    issued, or null if the name is not available.
     */
    public function getOrgName()
    {
        return $this->get("orgName");
    }

    /**
     * @return int|null The remaining days (ceiled) the VPP token is valid, or
     *    null if the expiration date is not available.
     */
    public function getRemainingDays()
    {
        $expDate = $this->getExpDate();
        if (null === $expDate) {
            return null;
        }

        return max(0, (int) ceil(($this->getExpDate() - time()) / 86400));
    }

    /**
     * @return string|null The actual token, or null if it is not available.
     */
    public function getToken()
    {
        return $this->get("token");
    }

    /**
     * @return bool true if the token could be decoded, false otherwise.
     */
    public function isValid()
    {
        return null !== $this->decoded;
    }
}
