<?php
namespace Topalis\App\Ios;

use Topalis\App\Entity\AppEntityManager;
use Topalis\App\Exception\VppException;
use Topalis\Configuration\AuralisConfig;
use Topalis\MDM\MdmQueueInterface;
use Topalis\MDM\Request\InviteToProgramRequest;

/**
 * VppTokenManager.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class VppTokenManager implements VppTokenManagerInterface
{
    /**
     * @var AuralisConfig
     */
    private $config;

    /**
     * @var AppEntityManager
     */
    private $iosAppEntityManager;

    /**
     * @var MdmQueueInterface
     */
    private $mdmQueue;

    /**
     * @var VppApi
     */
    private $vppApi;

    /**
     * @var VppUserService
     */
    private $vppUserService;

    public function __construct(VppApiInterface $vppApi, VppUserServiceInterface $vppUserService,
        MdmQueueInterface $mdmQueue, AppEntityManager $iosAppEntityManager, AuralisConfig $config)
    {
        $this->vppApi = $vppApi;
        $this->vppUserService = $vppUserService;
        $this->mdmQueue = $mdmQueue;
        $this->iosAppEntityManager = $iosAppEntityManager;
        $this->config = $config;
    }

    public function getToken()
    {
        $encodedToken = $this->config->getVppToken();
        if (empty($encodedToken)) {
            return null;
        }

        return new VppToken($encodedToken);
    }

    public function removeToken()
    {
        // verify that VPP service is reachable; if the token is invalid we
        // perform the actions below, anyway
        $this->vppApi->hasValidToken();

        try {
            $this->config->setVppSyncDisabled(true);
            $this->mdmQueue->cancelCommandsBy(array("requestType" => InviteToProgramRequest::REQUEST_TYPE));
            $this->vppUserService->retireAllUsers(true);
            foreach ($this->iosAppEntityManager->findBy(array("useVpp" => true)) as $app) {
                $app->setUseVpp(false);
                $app->setManaged(false);
                // app remove MDM commands will be created on save
                $this->iosAppEntityManager->save($app);
            }
            $this->config->setVppToken(null);
            $this->config->setVppSyncDisabled(false);
        } catch (\Exception $e) {
            $this->config->setVppSyncDisabled(false);
            throw $e;
        }
    }

    public function setToken(VppToken $token)
    {
        $this->assertToken($token);
        $currentToken = $this->getToken();
        if (null !== $currentToken && $currentToken->getToken() != $token->getToken()) {
            $this->removeToken();
        }
        $this->config->setVppToken($token->getEncoded());
        $this->config->setVppSyncDisabled(false);
    }

    /**
     * Asserts that the specified token is valid.
     *
     * @param VppToken $token
     *
     * @throws VppClientException If an exception occurs when trying to connect.
     * @throws VppResponseException If the status code of the HTTP response is
     *     not between 200 and 299.
     * @throws VppApiException If the VPP server rejects executing the request
     *     for a specific reason (status != 0).
     * @throws VppException If the token is invalid.
     */
    protected function assertToken(VppToken $token)
    {
        if (!$this->vppApi->isTokenValid($token->getEncoded())) {
            throw new VppException("Invalid VPP token");
        }
    }
}
