<?php
namespace Topalis\App\Ios;

use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;
use Topalis\App\Exception\VppApiException;
use Topalis\App\Exception\VppException;
use Topalis\Configuration\AuralisConfig;
use Topalis\User\Entity\User;
use Topalis\User\Entity\UserEntityManager;

/**
 * VppUserService
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class VppUserService implements VppUserServiceInterface, LoggerAwareInterface
{
    const ERROR_USER_ALREADY_RETIRED = 9618;
    const ERROR_USER_ALREADY_DELETED = 9620;

    /**
     * @var VppApiInterface
     */
    protected $vppApi;

    /**
     * @var AuralisConfig
     */
    protected $config;

    /**
     * @var UserEntityManager
     */
    protected $userEntityManager;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @param VppApi            $vppApi
     * @param AuralisConfig     $config
     * @param UserEntityManager $userEntityManager
     */
    public function __construct(
        VppApiInterface $vppApi,
        AuralisConfig $config,
        UserEntityManager $userEntityManager
    )
    {
        $this->vppApi = $vppApi;
        $this->config = $config;
        $this->userEntityManager = $userEntityManager;
    }

    public function isAvailable(&$message = null)
    {
        return $this->vppApi->isAvailable($message);
    }

    public function isTokenConfigured()
    {
        return $this->vppApi->hasToken();
    }

    public function registerUser(User $user)
    {
        // user already registered
        if ($user->getVppStatus() !== User::VPP_STATUS_UNREGISTERED) {
            return;
        }

        // don't add email address because it might change later
        $response = $this->vppApi->sendRequest(
            VppApi::SERVICE_REGISTER_USER,
            array(
                "clientUserIdStr" => $user->getId(), // unique id that will not be changed
            )
        );
        $data = $response['user'];
        $user
            ->setVppUserId($data['userId'])
            ->setVppInviteLink($data['inviteUrl'])
            ->setVppStatus(User::VPP_STATUS_REGISTERED)
        ;
        $this->userEntityManager->save($user);
    }

    public function retireAllUsers($ignoreInvalidToken = false)
    {
        $validToken = $this->vppApi->hasValidToken();
        if (!$ignoreInvalidToken || $validToken) {
            $this->syncUsers();
        }
        foreach ($this->userEntityManager->findBy(array("vppStatus" => array(
            User::VPP_STATUS_ASSOCIATED,
            User::VPP_STATUS_REGISTERED,
        ))) as $user) {
            if (!$validToken && $ignoreInvalidToken) {
                $user->setVppStatus(User::VPP_STATUS_UNREGISTERED);
                $user->setVppUserId(null);
                $this->userEntityManager->save($user);
            } else {
                $this->retireUser($user);
            }
        }
    }

    public function retireUser(User $user)
    {
        if ($user->getVppStatus() === User::VPP_STATUS_UNREGISTERED) {
            return;
        }

        $this->logger->info("Retire user " . $user->getLogin(), array(
            "user" => $user->getId(),
            "vppUserId" => $user->getVppUserId(),
        ));
        $this->retireUserByVppId($user->getVppUserId());
        $user->setVppStatus(User::VPP_STATUS_UNREGISTERED);
        $user->setVppUserId(null);
        $this->userEntityManager->save($user);
    }

    public function syncUsers($incremental = true)
    {
        $params = array();
        if ($incremental) {
            $lastModified = $this->config->getVppUserLastModified();
            if (null !== $lastModified) {
                $params['sinceModifiedToken'] = $lastModified;
            }
            // exclude retired users
            $params['includeRetired'] = 0;
        }
        $result = $this->vppApi->sendBatchRequest(VppApi::SERVICE_GET_USERS, "users", $params);

        $userList = array();
        foreach ($result['data'] as $user) {
            $vppUserId = $user['userId'];
            /** @var User $entity */
            $entity = $this->userEntityManager->findOneBy(array("vppUserId" => $vppUserId));
            if (null === $entity) {
                try {
                    $this->retireUserByVppId($vppUserId);
                } catch (VppException $e) {
                    $this->logger->error("Failed to retire VPP user with id " . $vppUserId, array(
                        "clientUserIdStr" => $result['clientUserIdStr'],
                        "status"          => $result['status'],
                        "exception"       => $e,
                    ));
                }
                continue;
            }

            $userList[] = $entity;
            switch ($user['status']) {
                case "Registered":
                    $entity->setVppStatus(User::VPP_STATUS_REGISTERED);
                    break;
                case "Associated":
                    $entity->setVppStatus(User::VPP_STATUS_ASSOCIATED);
                    break;
            }
            $this->userEntityManager->save($entity, false);
        }

        if (null !== $result['lastModified']) {
            $this->config->setVppUserLastModified($result['lastModified']);
        }

        // set VPP_STATUS_UNREGISTERED to other users
        if (!$incremental) {
            $qb = $this->userEntityManager->createQueryBuilder("u");
            if ($userList) {
                $qb->where("u NOT IN (:users)")->setParameter("users", $userList);
            }
            /** @var User[] $users */
            $users = $qb->getQuery()->getResult();
            foreach ($users as $user) {
                $user->setVppStatus(User::VPP_STATUS_UNREGISTERED);
                $user->setVppUserId(null);
                $this->userEntityManager->save($user, false);
            }
        }

        $this->userEntityManager->flush();
    }

    public function setLogger(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    /**
     * Retires the user with the specified VPP user id.
     *
     * @param string $userId The id of the user to retire.
     */
    protected function retireUserByVppId($userId)
    {
        $this->logger->info("Retire user with VPP user id " . $userId);
        try {
            $this->vppApi->sendRequest(
                VppApi::SERVICE_RETIRE_USER,
                array(
                    "userId" => $userId,
                )
            );
        } catch (VppApiException $e) {
            if ($e->getCode() === self::ERROR_USER_ALREADY_RETIRED) {
                $this->logger->info("Failed to retire user with VPP user id " . $userId
                    .": the user has already been retired");
            } else if ($e->getCode() === self::ERROR_USER_ALREADY_DELETED) {
                $this->logger->info("Failed to retire user with VPP user id " . $userId
                    .": the user has already been deleted");
            } else {
                throw $e;
            }
        }
    }
}
