<?php
namespace Topalis\App\Mdm;

use Symfony\Component\DependencyInjection\ContainerAware;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;
use Topalis\Admin\Entity\Admin;
use Topalis\App\Entity\App;
use Topalis\App\Entity\AppState;
use Topalis\App\Entity\MdmAppState;
use Topalis\Device\Entity\Device;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\MDM\MdmCommandGenerator;
use Topalis\MDM\Request\InstallApplicationRequest;
use Topalis\MDM\Request\RemoveApplicationRequest;
use Doctrine\ORM\ORMInvalidArgumentException;
use Doctrine\ORM\Query\Expr\Join;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
abstract class GenericAppMdmAdmin extends ContainerAware implements AppMdmAdminInterface
{
    public function getAppStates(Device $device, array $states = null)
    {
        $qb = $this->getAppStateEntityManager()->createQueryBuilder("s")
            ->where("s.device = :device")
            ->setParameter("device", $device)
        ;
        if (null !== $states) {
            $qb->andWhere("s.state in (:states)")
                ->setParameter("states", $states);
        }
        return $qb->getQuery()->getResult();
    }

    public function isAppInstalled(Device $device, $appIdentifiers)
    {
        $qb = $this->getAppStateEntityManager()->createQueryBuilder("s")->select("1")
            ->where("s.device = :device")
            ->andWhere("s.state in (:installedStates)")
            ->setParameter("device", $device)
            ->setParameter("installedStates", AppState::getInstalledStates())
            ->setParameter("appIdentifiers", $appIdentifiers)
            ->setMaxResults(1);
        if (is_string($appIdentifiers)) {
            $qb->join("s.app", "a", Join::WITH, "a.identifier = :appIdentifiers");
        } else {
            $qb->join("s.app", "a", Join::WITH, "a.identifier in (:appIdentifiers)");
        }
        return count($qb->getQuery()->execute()) == 1;
    }


    public function installApp(Device $device, App $app, $force = false, $removeBeforeInstall = false)
    {
        $this->getLogger()->debug("Install app " . get_class($app) . "::" . $app->getId()
            . " on device " . $device->getId());

        $state = $this->getAppState($device, $app);
        if (null === $state) {
            // app is not installed
            $state = $this->getAppStateEntityManager()->create();
            $state
                ->setApp($app)
                ->setDevice($device)
                ->setState(AppState::INSTALLATION_REQUESTED);
            ;
        } else if ($app->isUploadedPackageChanged() && $state->isInstalled()) {
                // update uploaded app
            if ($state->getRemoveCommand()) {
                $this->getMdmQueue()->cancelCommand($state->getRemoveCommand());
            }
            if ($state->getInstallCommand()) {
                $this->getMdmQueue()->cancelCommand($state->getInstallCommand());
            }

            $state->setState(AppState::UPDATE_REQUESTED);
            // reset number of install attempts
            $state->setInstallAttempts(0);
        } else {
            if ($state->getRemoveCommand()) {
                $this->getMdmQueue()->cancelCommand($state->getRemoveCommand());
            }

            if ($state->getInstallCommand()) {
                $installCommandStatus = $state->getInstallCommand()->getStatus();
                switch ($installCommandStatus) {
                    case MdmCommand::STATUS_NEW:
                        /* fall through */
                    case MdmCommand::STATUS_QUEUED:
                        /* fall through */
                    case MdmCommand::STATUS_NOT_NOW:
                        return;
                    case MdmCommand::STATUS_ACKNOWLEDGED:
                        if (!$force) {
                            return;
                        }
                }
            }

            if (($state->getRejected() === AppState::REJECTED_DEVICE
                || $state->getRejected() === AppState::REJECTED_USER) && !$force) {
                return;
            }

            if ($state->getState() == AppState::UPDATE_REQUESTED
                || $state->getState() == AppState::UPDATE_REJECTED
                || $state->getState() == AppState::UPDATE_UNSUPPORTED
                || $state->getState() == AppState::UPDATE_FAILED) {
                // TODO shall we remove the app and reinstall if update failed?
                $state->setState(AppState::UPDATE_REQUESTED);
            } else {
                if ($state->getState() == AppState::REMOVAL_REQUESTED) {
                    // the app might have been removed in the mean time
                    try {
                        $this->getAppStateEntityManager()->refresh($state);
                        if ($state->getState() != AppState::REMOVAL_REQUESTED) {
                            // call installApp again so the previous checks have to be passed again
                            $this->installApp($device, $app, $force);
                            return;
                        }
                    } catch (ORMInvalidArgumentException $e) {
                        $this->getLogger()->error("Failed to refresh app state", array(
                            "exception" => $e,
                            "state" => $state->getId()
                        ));
                        return;
                    }
                }
                if ($state->isInstalled()) {
                    if ($state->getState() == AppState::REMOVAL_REQUESTED) {
                        $state->setInstallAttempts(0);
                    }
                    if ($app->getType() == App::TYPE_UPLOADED && $app->getVersion() != $state->getVersion()) {
                        $state->setState(AppState::UPDATE_REQUESTED);
                    } else {
                        // no need to send an install command again
                        $state->setState(AppState::INSTALLED);
                        $this->getAppStateEntityManager()->save($state);
                        return;
                    }
                } else {
                    $state->setState(AppState::INSTALLATION_REQUESTED);
                }
            }
        }

        $rejectionReason = "";
        if ($this->deviceSupportsApp($device, $app) === false) {
            switch ($state->getState()) {
                case AppState::UPDATE_REQUESTED:
                    $state->setState(AppState::UPDATE_UNSUPPORTED);
                    break;
                default:
                    $state->setState(AppState::UNSUPPORTED);
                    $state->setManaged(false);
                    break;
            }
            $state->setRejected(AppState::REJECTED_SYSTEM);
            $state->setRejectionReason("unsupported");
        } else if ($this->isInstallRejectedBySystem($device, $app, $rejectionReason)) {
            $this->getLogger()->info(sprintf("Cannot install %s::%d on device %s: %s",
                get_class($app), $app->getId(), $device->getCombinedId(), $rejectionReason));
            if ($state->getState() === AppState::UPDATE_REQUESTED) {
                $state->setState(AppState::UPDATE_REJECTED);
            } else {
                $state->setState(AppState::INSTALLATION_REJECTED);
            }
            $state->setRejected(AppState::REJECTED_SYSTEM);
            $state->setRejectionReason($rejectionReason);
        } else if ($this->isNeedsLicense($state)) {
            // install will fail if license is not associated before sending InstallationRequest
            $state
                ->setState(AppState::WAIT_FOR_LICENSE)
                ->setRejected(AppState::REJECTED_NONE)
            ;
            $this->associateLicense($state);
        } else {
            $this->cancelMdmCommands($state);
            if ($removeBeforeInstall) {
                $this->removeApp($state, true);
            }
            $state
                ->addInstallAttempt()
                ->setLastInstallAttempt(new \DateTime("now"))
                ->setRejected(AppState::REJECTED_NONE)
            ;

            $request = $this->getInstallationRequest($app);
            $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
            $mdmCommand = $mdmCommandGenerator->createFromRequest($request, $app->getIdentifier());
            $this->beforeInstallCommand($state);
            $state->setInstallCommand($mdmCommand);
        }

        $this->getAppStateEntityManager()->save($state);
    }

    public function installApps(App $app, $force = false)
    {
        $this->getLogger()->debug("Install app " . get_class($app) . "::" . $app->getId());
        foreach ($app->getConfigGroups() as $group) {
            foreach ($group->getDevices() as $device) {
                if (!$this->supportsDevice($device)) {
                    continue;
                }
                if ($device->isManaged() && $device->isActive()) {
                    $this->installApp($device, $app, $force);
                }
            }
        }
    }

    public function installAppsForDevice(Device $device)
    {
        $this->getLogger()->debug("Install apps on " . $device->getId());
        $apps = $this->getAppEntityManager()->getAppsForConfigGroup($device->getConfigGroup());
        foreach ($apps as $app) {
            $this->installApp($device, $app);
        }
    }

    public function removeApp(AppState $state, $force = false)
    {
        if (!$state instanceof MdmAppState) {
            throw new UnexpectedTypeException($state, '\Topalis\App\Entity\MdmAppState');
        }
        $device = $state->getDevice();
        $this->getLogger()->debug("Remove app " . $state->getApp()->getIdentifier()
            . " from device " . $device->getId());

        $stateEm = $this->getAppStateEntityManager();

        /*
         * Pending install commands must be canceled for unmanaged apps, too.
         * Otherwise the app will be installed later, even though it is not
         * intended, anymore
         */
        $this->cancelMdmCommands($state);
        if (!$force && !$this->isRemoveCommandSafe($state)) {
            // if the app has been removed by the user the remove command has to
            // be sent to make it unmanaged, otherwise the app state can be
            // deleted.
            if ($state->getState() == AppState::INSTALLATION_REQUESTED
                && $state->getInstallCommand()->getStatus() == MdmCommand::STATUS_ACKNOWLEDGED) {
                // the app might be installed, but we don't know it, yet
                $this->getMdmAdmin()->requestManagedApplicationList($device);
            } else if (!$state->isInstalled()) {
                $stateEm->delete($state);
            } else {
                $this->getLogger()->debug("app " . $state->getApp()->getIdentifier()
                    . " could not be removed from device " . $device->getId()
                    . " as this app wasn't installed through auralis");
            }

            return false;
        }

        $removeCommand = $state->getRemoveCommand();
        if ($removeCommand) {
            switch ($removeCommand->getStatus()) {
                case MdmCommand::STATUS_NEW:
                    /* fall through */
                case MdmCommand::STATUS_QUEUED:
                    return true;
            }
        }
        $state->setState(AppState::REMOVAL_REQUESTED);

        $removeRequest = new RemoveApplicationRequest();
        $removeRequest->setIdentifier($state->getApp()->getIdentifier());
        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->createFromRequest($removeRequest, $state->getApp()->getIdentifier());
        $state->setRemoveCommand($mdmCommand);
        $stateEm->save($state);
        return true;
    }

    /**
     * Called before an app installation commando is persisted.
     *
     * @param MdmAppState $state
     */
    protected function beforeInstallCommand(MdmAppState $state)
    {
    }

    protected function cancelMdmCommands(MdmAppState $state)
    {
        if ($state->getRemoveCommand()) {
            $this->getMdmQueue()->cancelCommand($state->getRemoveCommand());
        }
        if ($state->getInstallCommand()) {
            $this->getMdmQueue()->cancelCommand($state->getInstallCommand());
        }
    }

    public function removeApps(App $app)
    {
        $this->getLogger()->debug("Remove app " . $app->getId());
        $stateEm = $this->getAppStateEntityManager();
        foreach ($stateEm->findBy(array("app" => $app)) as $state) {
            $this->removeApp($state);
        }
    }

    public function removeUnassignedApps(App $app)
    {
        $this->getLogger()->debug(sprintf("Remove app %s from old config groups", get_class($app)."::".$app->getId()));
        $states = $this->getAppStateEntityManager()->findUnassigned($app);
        foreach ($states as $state) {
            $this->removeApp($state);
        }
    }

    public function removeUnassignedAppsFromDevice(Device $device)
    {
        $this->getLogger()->debug("Remove unassigned apps from device " . $device->getCombinedId());
        $states = $this->getAppStateEntityManager()->findUnassignedForDevice($device);
        foreach ($states as $state) {
            $this->removeApp($state);
        }
    }


    /**
     * @return \Topalis\Admin\Entity\Admin
     */
    protected function getAdmin()
    {
        return $this->getMdmAdmin()->getAdmin();
    }

    /**
     * @param Device $device
     * @param App    $app
     *
     * @return MdmAppState|null
     */
    protected function getAppState(Device $device, App $app)
    {
        return $this->getAppStateEntityManager()->findOneBy(array("device" => $device, "app" => $app));
    }

    /**
     * @param App $app
     *
     * @return InstallApplicationRequest
     */
    abstract protected function getInstallationRequest(App $app);

    /**
     * @return \Symfony\Bridge\Monolog\Logger
     */
    protected function getLogger()
    {
        return $this->container->get("logger");
    }

    /**
     * @return \Topalis\App\Entity\AppEntityManager
     */
    abstract protected function getAppEntityManager();

    /**
     * @return \Topalis\App\Entity\AppStateEntityManager
     */
    abstract protected function getAppStateEntityManager();

    /**
     * @return \Topalis\MDM\MdmAdmin
     */
    protected function getMdmAdmin()
    {
        return $this->container->get("mdm.admin");
    }

    /**
     * @return \Topalis\MDM\MdmQueue
     */
    protected function getMdmQueue()
    {
        return $this->container->get("mdm.queue");
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    protected function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @param Device $device The device the app shall be installed on.
     * @param App $app The app that shall be installed.
     * @param string $rejectionReason Contains the rejection reason if the
     *     installation is rejected.
     * @return boolean true If the installation of the specified app is rejected
     *     by the system, false otherwise.
     */
    protected function isInstallRejectedBySystem(Device $device, App $app, &$rejectionReason)
    {
        return false;
    }

    /**
     * Check whether a license is needed for installing this app on this device
     *
     * @param AppState $state
     *
     * @return bool
     */
    protected function isNeedsLicense(AppState $state)
    {
        return false;
    }

    /**
     * @param AppState $state
     *
     * @return bool true if it is safe to send a remove command for the state,
     *    false otherwise.
     */
    protected function isRemoveCommandSafe(AppState $state)
    {
        return $state->isManaged();
    }

    /**
     * Associate a license to an app state
     *
     * @param AppState $state
     */
    protected function associateLicense(AppState $state)
    {
    }
}
