<?php
namespace Topalis\App\WinPhone;

use Symfony\Component\DependencyInjection\ContainerAware;
use Symfony\Component\DomCrawler\Crawler;
use Topalis\App\Entity\App;
use Topalis\App\Entity\WinPhoneApp;
use Topalis\Configuration\Client\AuralisCurl;
use Buzz\Browser;
use Topalis\Configuration\AuralisConfig;
use Topalis\App\AppStore;
use Topalis\App\Util\CrawlerUtil;

class WinPhoneStore extends ContainerAware implements AppStore
{
    const GUID_PATTERN = "/^[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}$/";

    /**
     * @var \Buzz\Browser
     */
    private $browser;

    private $lookupUrl = "https://www.windowsphone.com/s?appid={id}";

    private $searchUrl = "https://www.windowsphone.com/{locale}/search?q={term}";

    /**
     * @param string $id The id of the app to lookup.
     * @return \Topalis\App\Entity\WinPhoneApp|NULL
     */
    public function lookup($id)
    {
        if (!preg_match(self::GUID_PATTERN, $id)) {
            throw new \InvalidArgumentException("The id '" . $id . "' is not a valid app store id");
        }

        $url = str_replace("{id}", $id, $this->lookupUrl);
        $headers = array(
            "Host" => "www.windowsphone.com",
            "User-Agent" => "Mozilla/5.0 (X11; Linux x86_64; rv:33.0) Gecko/20100101 Firefox/33.0",
            "Accept" => "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
            "Accept-Language" => $this->container->get("config.auralis")->getWinPhoneStoreLocale()
        );

        $response = $this->getBrowser()->get($url, $headers);
        if (!$response->isSuccessful()) {
            return null;
        }
        $result = $response->getContent();

        $app = null;
        try {
            $app = $this->parseAppDetails($result);
            $app->setIdentifier($id);
        } catch (\Exception $e) {
            $this->getLogger()->error(
                sprintf("failed to get details for Windows Phone app %s: %s", $id, $e->getMessage()),
                array(
                    "exception" => $e,
                    "identifier" => $id
                ));
        }

        return $app;
    }

    public function lookupByIdentifier($identifier)
    {
        return $this->lookup($identifier);
    }

    public function refreshApps(array $apps)
    {
        foreach ($apps as $app) {
            if ($app->getType() == App::TYPE_UPLOADED) {
                continue;
            }
            $storeApp = $this->lookup($app->getIdentifier());
            if (null === $storeApp) {
                continue;
            }
            $app->setType(App::TYPE_STORE);
            $app->merge($storeApp);
        }
    }

    public function search($term)
    {
        $url = str_replace("{term}", $term, $this->searchUrl);

        $locales = array($this->container->get("config.auralis")->getWinPhoneStoreLocale());
        if (!in_array("en-us", $locales)) {
            // fallback
            $locales[] = "en-us";
        }

        $apps = array();
        foreach ($locales as $locale) {
            $headers = array(
                "User-Agent" => "Mozilla/5.0 (X11; Linux x86_64; rv:33.0) Gecko/20100101 Firefox/33.0",
                "Accept" => "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
                "Accept-Language" => $this->container->get("config.auralis")->getWinPhoneStoreLocale()
            );

            $response = $this->getBrowser()->get(str_replace("{locale}", $locale, $url), $headers);
            if (!$response->isSuccessful()) {
                continue;
            }

            $apps = $this->parseSearchResult($response->getContent());
        }

        return $apps;
    }

    private function parseAppDetails($html)
    {
        $crawler = new Crawler();
        $crawler->addHtmlContent($html);

        $app = new WinPhoneApp();
        $app->setType(App::TYPE_STORE);

        // don't search for the GUID as it would be too complicated and seems
        // to be replaced by the product id

        $productId = CrawlerUtil::getFromCrawler($crawler, "meta[name='ms.prod_id']", "attr", "content");
        $app->setProductId($productId);

        $name = CrawlerUtil::getFromCrawler($crawler, "meta[name='ms.prod']", "attr", "content");
        if (!$name) {
            throw new \InvalidArgumentException("App name not found");
        }
        $app->setName($name);

        $publisherNode = $crawler->filter("dt.metadata-list-title");
        if ($publisherNode->count() > 0) {
            $vendor = trim(CrawlerUtil::getFromCrawler($publisherNode->nextAll(), ".content"));
            $app->setVendor($vendor);
        }

        $description = CrawlerUtil::getFromCrawler($crawler, "meta[name='description']", "attr", "content");
        $app->setDescription($description);

        $price = self::splitPrice(trim(CrawlerUtil::getFromCrawler($crawler, "div.price > .header-sub")));
        $app->setPrice($price['price']);
        $app->setCurrency($price['currency']);

        $largeIcon = CrawlerUtil::getFromCrawler($crawler, "div.ph-logo img", "attr", "src");
        if ($largeIcon) {
            $largeIcon = preg_replace("/(h|w)=[0-9]+/", "\\1=100", $largeIcon);
            $largeIcon = "https:" . $largeIcon;
            $app->setIconLarge($largeIcon);

            $smallIcon = preg_replace("/(h|w)=[0-9]+/", "\\1=40", $largeIcon);
            $app->setIcon($smallIcon);
        }

        return $app;
    }

    private function parseSearchResult($html)
    {
        $crawler = new Crawler();
        $crawler->addHtmlContent($html);

        $apps = array();

        foreach ($crawler->filter("table.appList tbody tr td") as $node) {
            $apps[] = $this->parseSingleSearchResult(new Crawler($node));
        }
        if (!$apps) {
            // if there's no table the result might be in a <ul>
            foreach ($crawler->filter("ul.appList li") as $node) {
                $apps[] = $this->parseSingleSearchResult(new Crawler($node));
            }
        }
        return $apps;
    }

    private function parseSingleSearchResult(Crawler $node)
    {
        $app = new WinPhoneApp();
        $app->setType(App::TYPE_STORE);

        $url = CrawlerUtil::getFromCrawler($node, "a.title", "attr", "href");
        $path = parse_url($url, PHP_URL_PATH);
        $identifier = pathinfo($path, PATHINFO_FILENAME);
        if (!$identifier) {
            throw new \InvalidArgumentException("App identifier not found");
        }
        $app->setIdentifier($identifier);

        $name = trim(CrawlerUtil::getFromCrawler($node, "a.title"));
        if (!$name) {
            throw new \InvalidArgumentException("App name not found");
        }
        $app->setName($name);

        $smallIcon = CrawlerUtil::getFromCrawler($node, "a.appImage img", "attr", "src");
        $app->setIcon($smallIcon);

        $largeIcon = str_replace("ws_icon_tiny", "ws_icon_large", $smallIcon);
        $app->setIconLarge($largeIcon);

        $price = self::splitPrice(trim(CrawlerUtil::getFromCrawler($node, "div.cost")));
        $app->setPrice($price['price']);
        $app->setCurrency($price['currency']);

        return $app;
    }

    protected static function splitPrice($price)
    {
        $matches = array();
        preg_match("/([^\d]*)(\d+(\.|,)\d{2})([^\d]*)/", $price, $matches);
        if (count($matches) !== 5) {
            return array("price" => "0", "currency" => null);
        }

        $price = str_replace(",", ".", $matches[2]);
        // currency is either at the beginning or at the end
        $currency = trim($matches[1]);
        if (!$currency) {
            $currency = trim($matches[4]);
        }

        return array(
            "price" => $price,
            "currency" => $currency
        );
    }

    /**
     * @return \Symfony\Bridge\Monolog\Logger
     */
    protected function getLogger()
    {
        return $this->container->get("logger");
    }

    /**
     * @return \Buzz\Browser
     */
    protected function getBrowser()
    {
        if (null !== $this->browser) {
            return $this->browser;
        }

        $curl = new AuralisCurl($this->container->get("config.auralis"));
        $curl->setConnectTimeout(5);
        $curl->setTimeout(5);
        return $this->browser = new Browser($curl);
    }
}
