<?php
namespace Topalis\Certificate;

use Topalis\Certificate\Entity\ClientCertificate;
use Topalis\Certificate\Util\CertUtil;
use Topalis\Device\Entity\Device;
use Topalis\Kernel\AbstractAuralisService;
use Topalis\Kernel\Exception\Certificate\RevokeCertificateException;
use Topalis\Util\CryptUtil;
use Topalis\Util\FileSystemUtil;
use Topalis\Util\Process;
use Topalis\Util\Random;

/**
 * Class with functions for managing certificates.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class CertificateManager extends AbstractAuralisService
{
    /**
     * @var boolean
     */
    private $createRevocationList = false;

    /**
     * @var string
     */
    private $sslConfigFilename;

    /**
     * @return string
     */
    public function getSslConfigFilename()
    {
        return $this->sslConfigFilename;
    }

    /**
     * @param string $sslConfigFilename
     */
    public function setSslConfigFilename($sslConfigFilename)
    {
        $this->sslConfigFilename = $sslConfigFilename;
    }

    private function createReverseProxyCACert()
    {
        $config = $this->getConfig();

        $caCert = $this->getCACertificate();
        $certificate = new Certificate($caCert->getCertificate(), null,
            array("config" => $this->getSslConfigFilename()));
        $certificate->setCommonName("auralis sub CA");

        $serial = $config->getAndDecrementCertNextSerial();
        $cert = $certificate->export($serial, $caCert);
        $key = $certificate->exportPrivateKey();

        $config->setReverseProxyCACert($cert);
        $config->setReverseProxyCAPrivateKey($key);

        return $certificate;
    }

    /**
     * Fetch CA organization.
     * @return string The organization of the CA certificate or an empty string if none exists.
     */
    public function getCAOrg()
    {
        $cert = $this->getCACertificate();
        if (null === $cert) {
            return "";
        }

        return $cert->getOrganization();
    }

    /**
     * @return boolean true if CA certificate exists, false otherwise.
     * @throws \Exception On error.
     */
    public function existsCACert()
    {
        return strlen($this->getConfig()->getCACert()) > 0 && strlen($this->getConfig()->getCAPrivateKey()) > 0;
    }

    /**
     * @return boolean true if server certificate exists, false otherwise.
     * @throws \Exception On error.
     */
    public function existsServerCert()
    {
        return strlen($this->getConfig()->getServerCert()) > 0 && strlen($this->getConfig()->getServerPrivateKey()) > 0;
    }

    /**
     * Generate apache related certificate files from entries in config table and set those in apache config.
     * @throws \Exception On error.
     */
    public function exportServerCert()
    {
        $chain = $this->getConfig()->getServerChain();
        $serverCert = $this->getServerCert();

        file_put_contents("/etc/auralis/pki/tls/certs/servercert.pem", $serverCert->getPem(), LOCK_EX);
        $this->writeKeyFile("/etc/auralis/pki/tls/certs/private/serverkey.pem", $serverCert->exportPrivateKey());
        file_put_contents("/etc/auralis/pki/tls/certs/serverchain.pem", $chain, LOCK_EX);
        $this->getApache()->setServerCerts("/etc/auralis/pki/tls/certs");
    }

    /**
     * Generate apache related CA certificate files from entries in config table and set those in apache config.
     * @throws \Exception On error.
     */
    public function exportCACert()
    {
        $config = $this->getConfig();
        $caCert = $this->getCACertificate();

        $revProxyCa = $this->getReverseProxyCACert();
        if ($revProxyCa->getSignerKeyId() != $caCert->getKeyId()) {
            $revProxyCa = $this->createReverseProxyCACert();
        } else {
            $revProxyCa->setCountry($caCert->getCountry());
            $revProxyCa->setEmail($caCert->getEmail());
            $revProxyCa->setLocation($caCert->getLocation());
            $revProxyCa->setOrganization($caCert->getOrganization());
            $revProxyCa->setOrganizationalUnit($caCert->getOrganizationalUnit());
            $revProxyCa->setState($caCert->getState());
            $revProxyCa->sign($revProxyCa->getSerial(), $caCert);
            $config->setReverseProxyCACert($revProxyCa->getPem());
            $config->setReverseProxyCAPrivateKey($revProxyCa->exportPrivateKey());
        }

        // reset RA certificates so they will be regenerated when requested
        $config->setRACryptCert(null);
        $config->setRACryptPrivateKey(null);
        $config->setRASignCert(null);
        $config->setRASignPrivateKey(null);

        $chainedCA = $this->getChainedCACert();
        file_put_contents("/etc/auralis/pki/CA/cacert.pem", $chainedCA, LOCK_EX);
        $this->writeKeyFile("/etc/auralis/pki/CA/private/cakey.pem", $caCert->exportPrivateKey());

        $revProxyCaChain = $chainedCA;
        array_unshift($revProxyCaChain, $revProxyCa->getPem());
        file_put_contents("/etc/auralis/pki/CA/revproxy-cachain.pem", $revProxyCaChain, LOCK_EX);
        file_put_contents("/etc/auralis/pki/CA/revproxy-cacert.pem", $revProxyCa->getPem(), LOCK_EX);
        $this->writeKeyFile("/etc/auralis/pki/CA/private/revproxy-cakey.pem", $revProxyCa->exportPrivateKey());

        $this->getApache()->setClientCA("/etc/auralis/pki/CA/revproxy-cachain.pem",
            "/etc/auralis/pki/CA/cacert.pem",
            "/etc/auralis/pki/CA/revproxy-cacert.pem");
    }

    /**
     * @return string[] PEM encoded CA certificate with its chain.
     */
    public function getChainedCACert()
    {
        $chaindata = $this->getConfig()->getCAChain();
        if (!empty($chaindata)) {
            $chain = unserialize($chaindata);
        } else {
            $chain = array();
        }
        $cacert = $this->getConfig()->getCACert();
        array_unshift($chain, $cacert);

        return $chain;
    }

    /**
     * @return \Topalis\Certificate\Certificate The CA certificate.
     */
    public function getCACert()
    {
        $cert = $this->getConfig()->getCACert();
        if ($cert === null) {
            return null;
        }

        $key = $this->getConfig()->getCAPrivateKey();
        return new Certificate($cert, $key, array("config" => $this->getSslConfigFilename()));
    }

    /**
     * @return \Topalis\Certificate\Certificate The CA certificate.
     *
     * @deprecated Use getCACert()
     */
    public function getCACertificate()
    {
        return $this->getCACert();
    }

    /**
     * @return \Topalis\Certificate\Certificate
     */
    public function getReverseProxyCACert()
    {
        $cert = $this->getConfig()->getReverseProxyCACert();
        $key = $this->getConfig()->getReverseProxyCAPrivateKey();

        if (!$cert || !$key) {
            return $this->createReverseProxyCACert();
        }

        return new Certificate($cert, $key, array("config" => $this->getSslConfigFilename()));
    }

    /**
     * @return null|\Topalis\Certificate\ServerCertificate
     */
    public function getServerCert()
    {
        $cert = $this->getConfig()->getServerCert();
        if (empty($cert)) {
            return null;
        }

        $key = $this->getConfig()->getServerPrivateKey();
        return new ServerCertificate($this, $cert, $key);
    }

    /**
     * @return Certificate The RA certificate to encrypt and decrypt messages.
     */
    public function getRACryptCert()
    {
        $config = $this->getConfig();
        $cert = $config->getRACryptCert();
        $key = $config->getRACryptPrivateKey();
        $configArgs = array(
            "config" => $this->getSslConfigFilename(),
            "x509_extensions" => "racrypt_cert"
        );

        if ($cert && $key) {
            return new Certificate($cert, $key, $configArgs);
        }

        $caCert = $this->getReverseProxyCACert();
        $certificate = new Certificate($caCert->getCertificate(), null, $configArgs);
        $certificate->setCommonName("auralis RA crypt");

        $serial = $config->getAndDecrementCertNextSerial();
        $cert = $certificate->export($serial, $caCert);
        $key = $certificate->exportPrivateKey();

        $config->setRACryptCert($cert);
        $config->setRACryptPrivateKey($key);

        return $certificate;
    }

    /**
     * @return Certificate The RA certificate to sign messages.
     */
    public function getRASignCert()
    {
        $config = $this->getConfig();
        $cert = $config->getRASignCert();
        $key = $config->getRASignPrivateKey();
        $configArgs = array(
            "config" => $this->getSslConfigFilename(),
            "x509_extensions" => "rasign_cert"
        );

        if ($cert && $key) {
            return new Certificate($cert, $key, $configArgs);
        }

        $caCert = $this->getReverseProxyCACert();
        $certificate = new Certificate($caCert->getCertificate(), null, $configArgs);
        $certificate->setCommonName("auralis RA sign");

        $serial = $config->getAndDecrementCertNextSerial();
        $cert = $certificate->export($serial, $caCert);
        $key = $certificate->exportPrivateKey();

        $config->setRASignCert($cert);
        $config->setRASignPrivateKey($key);

        return $certificate;
    }

    public function setDeviceCertsUnused(Device $device)
    {
        /** @var $clientCerts ClientCertificate[] */
        $clientCerts = $this->getClientCertificateEntityManager()->createQueryBuilder("c")
            ->select("c")
            ->where("c.device = :device")
            ->andWhere("c.unused = 0")
            ->setParameter("device", $device)
            ->getQuery()
            ->getResult()
        ;
        foreach ($clientCerts as $clientCert) {
            $clientCert->setUnused(true);
            $this->getClientCertificateEntityManager()->save($clientCert, false);
        }
        $this->getClientCertificateEntityManager()->flush();
    }

    /**
     * Generate a PKCS7 container from the specified content.
     *
     * @param string     $content The content to be PKCS7-signed.
     * @param mixed      $cert    The certificate to sign with. If not specified the CA
     *                            certificate is used.
     * @param mixed      $key     The private key that corresponds to the certificate.
     * @param null|array $chain   Certificate chain as array
     *
     * @return string The signed content.
     */
    public function sign($content, $cert = null, $key = null, $chain = null)
    {
        if ($cert === null) {
            $ca = $this->getCACertificate();
            $cert = $ca->getResource();
            $key = $ca->getPrivateKey();
        }

        return CertUtil::signPKCS7($content, $cert, $key, $chain);
    }


    /**
     * Decrypts an S/MIME encrypted message.
     * If $cert is not specified the CA certificate and key will be used.
     *
     * @param string $content The encrypted content,
     * @param mixed $cert
     * @param mixed $key
     * @return string The decrypted content, or false if decryption fails.
     * @throws \Exception On error.
     */
    public function decryptPKCS7($content, $cert = null, $key = null)
    {
        if ($cert === null) {
            $ca = $this->getCACertificate();
            $cert = $ca->getResource();
            $key = $ca->getPrivateKey();
        }

        return CryptUtil::decryptPKCS7($content, $cert, $key);
    }

    /**
     * Generates the CRL file for apache from the revoked client certificates in the database.
     * @throws \Exception On error.
     */
    public function doCreateRevocationList()
    {
        $this->getLogger()->debug("create certificate revocation list");
        putenv('HOME=/etc/auralis/pki/CA');

        if ($this->getLicense()->isCertRevokeRequired()) {
            $certs = $this->getEntityManager()->findAll();
        } else {
            /*
             * Revoke all certificates that belong to a disabled device or to non
             * device. Do not revoke MDM certs that may be still in use, so we can
             * still wipe a device with an HTTP 401 response, even though the device
             * hadn't received the MDM wipe command.
             * Revoke all certificates of devices that are successfully wiped,
             * the wipe status is unknown, or the wipe failed to ensure that
             * the other services cannot be requested, anymore.
             *
             * The query used to have "OR unused = 1" as final condition.
             * However the iPhone seems to use the last generated certificate for
             * the MDM service, not that one that was created during the installation
             * of the MDM profile.
             */
            $this->getDeviceEntityManager()->disableDeleteFilter();
            $certs = $this->getEntityManager()->createQueryBuilder("c")
                ->leftJoin("c.device", "d")
                ->where("(d.disabled = 1 OR d.deleted = 1 OR d.activeSyncDisabled = 1 OR c.device IS NULL)
                AND (d.mdmCert IS NULL
                    OR d.wipestatus = :wipeexecuted
                    OR d.wipestatus = :wipeunknown
                    OR d.wipestatus = :wipeerror)")
                ->setParameter("wipeexecuted", Device::WIPE_EXECUTED)
                ->setParameter("wipeunknown", Device::WIPE_UNKNOWN)
                ->setParameter("wipeerror", Device::WIPE_ERROR)
                ->getQuery()->getResult();
            ;
            $this->getDeviceEntityManager()->enableDeleteFilter();
        }
        $this->getLogger()->debug(count($certs) . " revoked certificates");

        // we have to create the CRL with the CA that signed the certificate
        $issuers = array("default", "revproxy");

        foreach ($issuers as $issuer) {
            if ($issuer == "default") {
                $prefix = "";
            } else {
                $prefix = $issuer . "-";
            }
            file_put_contents("/etc/auralis/pki/CA/" . $prefix . "index.txt", "");
        }

        $out = "";
        foreach ($certs as $cert) {
            try {
                if (strlen($cert->getCert()) == 0) {
                    continue;
                }

                $issuer = $cert->getIssuer();
                if (!in_array($issuer, $issuers)) {
                    $issuer = "default";
                }
                $tmpName = tempnam(sys_get_temp_dir(), 'auralisRev');
                file_put_contents($tmpName, $cert->getCert());
                $cmd = "openssl ca -config " . $this->getSslConfigFilename()
                    . " -name CA_" . $issuer . " -revoke " . $tmpName;
                Process::execThrow($cmd, $out);
                unlink($tmpName);
            } catch (\Exception $e) {
                FileSystemUtil::tryUnlinkFile($tmpName);
                throw new RevokeCertificateException("Revoking certificate failed", $cert->getId(), $e);
            }
        }

        $crlChanged = false;
        foreach ($issuers as $issuer) {
            if ($issuer == "default") {
                $prefix = "";
            } else {
                $prefix = $issuer . "-";
            }
            $crlPemFile = "/etc/auralis/pki/CA/crl/" . $prefix . "crl.pem";
            if (!$crlChanged && file_exists($crlPemFile)) {
                $oldsum = md5_file($crlPemFile);
            } else {
                $oldsum = "";
            }
            $cmd = "openssl ca -config " . $this->getSslConfigFilename() . " -name CA_"
                . $issuer . " -gencrl -out " . $crlPemFile;
            Process::execThrow($cmd, $out);
            $cmd = "ln -sf " . $crlPemFile
                . " /etc/auralis/pki/CA/crl/`openssl crl -hash -noout -in ". $crlPemFile ."`.r0";
            Process::execThrow($cmd, $out);

            if (!$crlChanged && md5_file($crlPemFile) != $oldsum) {
                $crlChanged = true;
            }
        }

        if ($crlChanged) {
            $this->getLogger()->debug("revocation list changed");
            $this->getApache()->setClientCRL("/etc/auralis/pki/CA/crl");
        }
    }

    /**
     * Enables the creation of the CRL file for apache from the revoked client
     * certificates in the database on terminate event.
     */
    public function enableCreateRevocationList()
    {
        $this->getApache()->enableReload();
        $this->createRevocationList = true;
    }

    /**
     * @return boolean if the creation of the CRL on termination is enabled,
     *     false otherwise.
     */
    public function isCreateRevocationList()
    {
        return $this->createRevocationList;
    }

    /**
     * @throws \Exception If no APNS certificate is defined or a topic is not
     *     set in the certificate.
     * @return string The topic that MDM listens to for push notifications.
     */
    public function getMdmTopic()
    {
        $pem = $this->getConfig()->getApnsCert();
        if ($pem === null) {
            throw new \Exception("No APNS certificate defined");
        }
        $cert = openssl_x509_parse($pem);
        if (!isset($cert['subject']['UID'])) {
            throw new \Exception("Invalid APNS certificate");
        }
        return $cert['subject']['UID'];
    }

    /**
     * Generate a PKCS12 client certificate container. This method can only
     * used for server side created certificates, i.e. when SCEP was not used
     * (non iPhone devices).
     *
     * @param \Topalis\Certificate\Entity\ClientCertificate $cc The certificate
     *     entity.
     * @param string $epw The export password for encryption of the PKCS12 data.
     * @return string A binary string, containing the generated PKCS12 data.
     * @throws \Exception On error.
     * @see createClientCert()
     */
    public function createClientP12(ClientCertificate $cc, $epw = null)
    {
        $chainedCA = $this->getChainedCACert();
        $ret = "";
        openssl_pkcs12_export($cc->getCert(), $ret, $cc->getPrivateKey(), $epw, array('extracerts' => $chainedCA));
        return $ret;
    }

    /**
     * Initializes a ClientCertificate object and stores it in the database.
     *
     * @param Device $device The device the certificate is going to be used
     *     for.
     * @param boolean $withChallenge Adds challenge to the entity  for use in
     *     SCEP, if true.
     * @return ClientCertificate Initialized certificate db entry with id and challenge.
     * @see signClientCert
     */
    public function initClientCert(Device $device, $withChallenge = true)
    {
        $cert = new ClientCertificate();

        if ($withChallenge) {
            $repo = $this->getEntityManager()->getRepository();
            do {
                // find unique challenge
                // challenge passwords must not be longer than 20 chars
                $challenge = Random::getString(20);
            } while (null !== $repo->findOneBy(array("challenge" => sha1($challenge))));
            $cert->setPlainChallenge($challenge);
        }

        $cert->setDevice($device);
        $this->getEntityManager()->save($cert);

        return $cert;
    }

    /**
     * @param string $challenge The challenge to get the ClientCertificate for.
     * @return ClientCertificate|NULL The ClientCertificate object with the
     *     specified challenge, or null if no such certificate was found.
     */
    public function getChallengedClientCert($challenge)
    {
        return $this->getEntityManager()->findOneBy(array("challenge" => sha1($challenge)));
    }

    /**
     * Signs the certificate in the specified certificate signing request and
     * stores the certificate in the database. This method is intended for
     * certificates created on client side using SCEP.
     *
     * @param string $challenge The challenge used for SCEP.
     * @param string $csr A PEM encoded certificate signing request.
     * @param Certificate $cacert The CA certificate to sign the certificate.
     *     If not specified the default CA is used.
     * @return string|NULL The PEM encoded signed certificate, or null if no
     *     certificate with the specified challenge was found.
     * @see initClientCert()
     */
    public function signChallengedClientCert($challenge, $csr, Certificate $cacert = null)
    {
        $cert = $this->getChallengedClientCert($challenge);
        if ($cert === null) {
            return null;
        }

        return $this->signClientCert($cert, $csr, $cacert);
    }

    /**
     * Signs the certificate in the specified certificate signing request and
     * stores the certificate in the database.
     *
     * @param ClientCertificate $cert The entity to store the signed cetificate.
     * @param string $csr A PEM encoded certificate signing request.
     * @param Certificate $cacert The CA certificate to sign the certificate.
     *     If not specified a default CA is used.
     * @param array $subject Values in this array override the corresponding
     *     values of the CSR subject.
     * @return string The PEM encoded signed certificate.
     * @see initClientCert()
     */
    public function signClientCert(ClientCertificate $cert, $csr, Certificate $cacert = null, array $subject = null)
    {
        $csrSubject = openssl_csr_get_subject($csr);
        if ($subject) {
            foreach ($csrSubject as $key => $value) {
                if (!isset($subject[$key])) {
                    $subject[$key] = $value;
                }
            }
        } else {
            $subject = $csrSubject;
        }

        $cn = isset($subject['CN']) ? $subject['CN'] : null;
        if ($cacert === null) {
            if ($cn == "SecureIntranet") {
                // Sign the certificate for SecureIntranet with the sub CA
                $cacert = $this->getReverseProxyCACert();
            } else {
                $cacert = $this->getCACertificate();
            }
        }

        $validityDays = 3650;
        $serial = $cert->getId();

        // we have to use phpseclib to modify the CSR subject.
        $x509Csr = new \File_X509();
        $x509Csr->loadCSR($csr);
        // remove null character that shouldn't be there (from Windows Phone 8)
        $subject['CN'] = preg_replace("/\\x00$/", "", $cn);
        $x509Csr->setDN($subject);
        $x509Csr->setExtension("netscape-cert-type", array("SSLClient"));
        $x509Csr->setExtension("id-ce-keyUsage",
            array("nonRepudiation", "digitalSignature", "keyEncipherment"), true);
        if ($cert->getDevice()->getType()->getName() == "winphone") {
            // only needed for SyncML
            $x509Csr->setExtension("id-ce-extKeyUsage", array("id-kp-clientAuth"), true);
        }

        $key = new \Crypt_RSA();
        $keyPem = "";
        openssl_pkey_export($cacert->getPrivateKey(), $keyPem);
        $key->loadKey($keyPem);
        $ca = new \File_X509();
        $ca->loadX509($cacert->getPem());
        $ca->setPrivateKey($key);
        $x509 = new \File_X509();
        $x509->setSerialNumber($serial, 10);
        $x509->setStartDate("-2 hour");
        $x509->setEndDate("+$validityDays day");
        $clientcertPem = $x509->saveX509($x509->sign($ca, $x509Csr, "sha256WithRSAEncryption"));

        if ($cacert->getHash() == $this->getReverseProxyCACert()->getHash()) {
            $cert->setIssuer("revproxy");
        } else {
            $cert->setIssuer("default");
        }
        $cert->setCert($clientcertPem);
        $cert->setChallenge(null);
        $this->getEntityManager()->save($cert);

        return $clientcertPem;
    }

    /**
     * Creates a certificate for the specified device and stores it in
     * the database. Required when we do not use SCEP (i.e. for non iPhone devices.)
     *
     * @param Device $device The device to create the certificate for.
     * @throws \Exception On error.
     * @return ClientCertificate The client certificate.
     */
    public function createClientCertForDevice(Device $device)
    {
        $user = $device->getUser();
        if ($user === null) {
            throw new \Exception("Failed to retrieve user data");
        }
        $config = $this->getConfig();
        $exdomain = $config->getExchangeDomain();
        $email = $user->getEmail();
        if (strlen($exdomain) == 0 || !$config->isExchangeAltLoginType()) {
            $exLogin = $user->getLogin();
        } else {
            $exLogin = $user->getLogin() . "@" . $exdomain;
        }
        try {
            return $this->createClientCert($device, $exLogin, $email);
        } catch (\Exception $e) {
            throw new \Exception("Generating client certificate failed", $e->getCode(), $e);
        }
    }

    /**
     * Create a client certificate.
     *
     * @param Device $device The device the certificate is created for.
     * @param string $cn     The subject common name for the certificate to be generated.
     * @param string $email  The subject email for the certificate to be generated.
     *
     * @return ClientCertificate The certificate object.
     */
    public function createClientCert(Device $device, $cn, $email = '')
    {
        $certEntity = new ClientCertificate();
        $certEntity->setDevice($device);

        $ca = $this->getReverseProxyCACert();
        $clientCertificate = new Certificate($ca->getCertificate(), null, array(
            "config" => $this->getSslConfigFilename(),
            "x509_extensions" => "usr_cert"
        ));
        $clientCertificate->setCommonName($cn);
        $clientCertificate->setEmail($email);

        // save the certificate so we can use the id as serial
        $this->getEntityManager()->save($certEntity);

        $cert = $clientCertificate->export($certEntity->getId(), $ca);
        $privateKey = $clientCertificate->exportPrivateKey();
        $certEntity->setCert($cert);
        $certEntity->setPrivateKey($privateKey);
        $this->getEntityManager()->save($certEntity);

        return $certEntity;
    }

    /**
     * @return \Topalis\Apache\Apache
     */
    private function getApache()
    {
        return $this->container->get("apache");
    }

    /**
     * @return \Topalis\Certificate\Entity\ClientCertificateEntityManager
     */
    private function getClientCertificateEntityManager()
    {
        return $this->container->get("certificate.entity.manager.certificate");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\Certificate\Entity\ClientCertificateEntityManager
     */
    private function getEntityManager()
    {
        return $this->container->get("certificate.entity.manager.certificate");
    }

    private function writeKeyFile($filename, $data)
    {
        $keyExists = is_file($filename);
        file_put_contents($filename, $data, LOCK_EX);
        if (!$keyExists) {
            /*
             * Key file newly created so it's required to set permissions. If
             * the file aready existed the current process may not be allowed to
             * do this.
             */
            chmod($filename, 0600);
        }
    }
}
