<?php

namespace Topalis\Certificate\Entity;

use Doctrine\ORM\Mapping as ORM;

/**
 * ClientCertificate
 *
 * @ORM\Table(name="clientcerts", indexes={
 *     @ORM\Index(name="idx_clientcerts_unused", columns={"unused"})
 * })
 * @ORM\Entity
 */
class ClientCertificate
{
    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string
     *
     * @ORM\Column(name="cert", type="string", length=8192, nullable=true,
     *     options={"comment"="The PEM encoded X.509 certificate"})
     */
    protected $cert;

    /**
     * @var string
     *
     * @ORM\Column(name="privatekey", type="string", length=8192, nullable=true,
     *     options={"comment"="The certificates private key (only available if generated on server side)"})
     */
    protected $privateKey;

    /**
     * @var string
     *
     * @ORM\Column(name="challenge", type="string", length=65, nullable=true, unique=true)
     */
    protected $challenge;

    /**
     *
     * @var string
     */
    protected $plainChallange;

    /**
     * @var boolean
     *
     * @ORM\Column(name="unused", type="boolean", nullable=false, options={"default"=0})
     */
    protected $unused = false;

    /**
     * @var string
     *
     * @ORM\Column(name="issuer", type="string", length=30, nullable=true)
     */
    protected $issuer;

    /**
     * @var \Topalis\Device\Entity\Device
     *
     * @ORM\ManyToOne(targetEntity="Topalis\Device\Entity\Device")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="device_id", referencedColumnName="id")
     * })
     */
    protected $device;

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string PEM encoded certificate.
     */
    public function getCert()
    {
        return $this->cert;
    }

    /**
     * Sets the PEM encoded certificate.
     *
     * @param string $cert The PEM encoded certificate to set.
     */
    public function setCert($cert)
    {
        $this->cert = $cert;
    }

    /**
     * @return string|null The certificate's private key, or null if not set.
     */
    public function getPrivateKey()
    {
        return $this->privateKey;
    }

    /**
     * Sets the certificate's private key.
     * @param string $privateKey The private key to set.
     */
    public function setPrivateKey($privateKey)
    {
        $this->privateKey = $privateKey;
    }

    /**
     * @return boolean true if the certificate is not in use, false otherwise.
     */
    public function isUnused()
    {
        return (boolean) $this->unused;
    }

    /**
     * Sets the unused state of the certificate.
     *
     * @param boolean $unused true if the certificate is not in use, false
     *     otherwise.
     */
    public function setUnused($unused)
    {
        $this->unused = $unused;
    }

    /**
     *
     * @return string The issuer name referencing a CA section name in the
     *     openssl config.
     */
    public function getIssuer()
    {
        return $this->issuer;
    }

    /**
     * Sets the issuer name referencing a CA section name in the openssl config.
     *
     * @param string $issuer The issuer name to set.
     */
    public function setIssuer($issuer)
    {
        $this->issuer = $issuer;
    }

    /**
     * @return string The SHA1 encoded challenge for SCEP.
     */
    public function getChallenge()
    {
        return $this->challenge;
    }

    /**
     * Sets the SHA1 encoded challenge for SCEP.
     *
     * @param string $challenge The SHA1 encoded challenge for SCEP.
     */
    public function setChallenge($challenge)
    {
        $this->challenge = $challenge;
    }

    /**
     * @return string The challenge for SCEP. It is not persited.
     */
    public function getPlainChallenge()
    {
        return $this->plainChallenge;
    }

    /**
     * Sets the challenge for SCEP. The specified challenge will be persisted as
     * SHA1 hash. The plain challenge will be stored in memory only.
     *
     * @param string $plainChallenge
     */
    public function setPlainChallenge($plainChallenge)
    {
        $this->plainChallenge = $plainChallenge;
        $this->setChallenge(sha1($plainChallenge));
    }

    /**
     * Set device
     *
     * @param \Topalis\Device\Entity\Device $device
     * @return ClientCertificate
     */
    public function setDevice(\Topalis\Device\Entity\Device $device = null)
    {
        $this->device = $device;

        return $this;
    }

    /**
     * Get device
     *
     * @return \Topalis\Device\Entity\Device
     */
    public function getDevice()
    {
        return $this->device;
    }
}
