<?php
namespace Topalis\Compliance\Action;

use Topalis\Compliance\ComplianceVerificationResultInterface;
use Topalis\Compliance\Entity\RuleValidation;
use Topalis\Compliance\Exception\InvalidArgumentException;
use Topalis\Compliance\PreviousComplianceVerificationResultInterface;
use Topalis\Device\Entity\Device;
use Topalis\Util\TypeCheck;

/**
 * This abstract used can be extended by compliance actions that shall be
 * performed with a delay.
 */
abstract class AbstractDelayedComplianceAction implements ComplianceActionInterface
{
    public function perform(array $options, Device $device, ComplianceVerificationResultInterface $verificationResult,
        PreviousComplianceVerificationResultInterface $previousVerificationResult, array $actions)
    {
        $this->assertOptions($options);

        if (count($verificationResult->getViolated()) === 0) {
            $this->onNoViolation($options, $device, $actions);
            return new ComplianceActionResult(false, null);
        }

        $delay = (int) $options['delay'];
        $now = time();
        $minAllowedViolationTime = $now - $delay;
        $minViolationTime = $now;

        foreach ($verificationResult->getViolated() as $violated) {
            $ruleValidation = $previousVerificationResult->getRuleValidation($violated['rule']);
            if (null === $ruleValidation || !$ruleValidation->isViolated()
                || $ruleValidation->getViolationTime() >= $minViolationTime) {
                continue;
            }

            $minViolationTime = $ruleValidation->getViolationTime();
            if ($minViolationTime < $minAllowedViolationTime) {
                // rule is violated for more than the specified delay
                $this->onViolation($options, $device, $ruleValidation, $actions);
                return new ComplianceActionResult(true, time());
            }
        }

        $this->onActionDelayed($options, $device, $actions);
        return new ComplianceActionResult(false, $minViolationTime + $delay + 1);
    }

    /**
     * Asserts the specified options and throws an InvalidArgumentException if
     * the options aren't valid.
     *
     * @param array $options
     *
     * @throws \Topalis\Compliance\Exception\InvalidArgumentException
     */
    protected function assertOptions(array $options)
    {
        if (!isset($options['delay'])) {
            throw new InvalidArgumentException(sprintf(
                "No delay given for compliance action %s", get_class($this)
            ));
        }

        $delay = $options['delay'];
        if (!TypeCheck::isInt($delay)) {
            throw new InvalidArgumentException("Expected an integer as delay, got \"" . $delay . "\"");
        }
    }

    /**
     * Called if no violated rule exceeds the delay.
     *
     * @param array $options
     * @param Device $device
     * @param array $actions
     */
    protected function onActionDelayed(array $options, Device $device, array $actions)
    {
    }

    /**
     * Called if no rule is violated.
     *
     * @param array $options
     * @param Device $device
     * @param array $actions
     */
    protected function onNoViolation(array $options, Device $device, array $actions)
    {
    }

    /**
     * Called if a violated rule exceeds the delay.
     *
     * @param array $options
     * @param Device $device
     * @param RuleValidation $ruleValidation Rule validation with violation time
     *    that exceeds the delay.
     * @param array $actions
     */
    abstract protected function onViolation(array $options, Device $device,
        RuleValidation $ruleValidation, array $actions);
}
