<?php
namespace Topalis\Compliance\Action;

use Topalis\Device\Entity\Device;
use Topalis\Compliance\ComplianceVerificationResultInterface;
use Topalis\Mail\MailerInterface;
use Symfony\Component\Translation\TranslatorInterface;
use Psr\Log\LoggerInterface;
use Topalis\Compliance\Util\ComplianceVerificationUtils;
use Topalis\Compliance\PreviousComplianceVerificationResultInterface;

/**
 * This compliance action sends an email to the administrator if there are new
 * rule violations.
 */
class SendAdminMailComplianceAction implements ComplianceActionInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Topalis\Mail\MailerInterface
     */
    private $mailer;

    /**
     * @var \Symfony\Component\Translation\TranslatorInterface
     */
    private $translator;

    /**
     * Constructor.
     *
     * @param MailerInterface $mailer
     * @param TranslatorInterface $translator
     * @param LoggerInterface $logger
     */
    public function __construct(MailerInterface $mailer, TranslatorInterface $translator, LoggerInterface $logger)
    {
        $this->mailer = $mailer;
        $this->translator = $translator;
        $this->logger = $logger;
    }

    public function perform(array $options, Device $device, ComplianceVerificationResultInterface $verificationResult,
        PreviousComplianceVerificationResultInterface $previousVerificationResult, array $actions)
    {
        $actions; // unused
        $options; // unused

        if (!$this->mailer->isConfigured()) {
            $this->getLogger()->info("Mailer is not configured");
            return new ComplianceActionResult(false, null);
        }

        if (!$verificationResult->hasNewViolations($previousVerificationResult)) {
            $this->getLogger()->debug("No new violations");
            return new ComplianceActionResult(false, null);
        }

        try {
            $this->getLogger()->debug("Send violation mail to admin");
            // user might be deleted
            $user = $device->getUser() ? $device->getUser()->getLogin() : null;
            $subject = $this->translator->trans("admin.compliance.action.sendAdminMail.subject", array(
                "%user%" => $user,
                "%device%" => $device->getCombinedName(),
            ));
            $text = $this->composeMailText($device, $verificationResult, $actions);
            $this->mailer->sendAdminMessage($subject, $text);
            return new ComplianceActionResult(true, time());
        } catch (\Exception $e) {
            $this->getLogger()->error("Sending compliance violation message to admin failed", array("exception" => $e));
        }
    }

    public function onRemove(Device $device)
    {
    }

    private function composeMailText(Device $device, ComplianceVerificationResultInterface $verificationResult,
        array $actions)
    {
        $violationMessages = ComplianceVerificationUtils::getViolationMessages($verificationResult,
            $this->translator);

        $violations = "* " . join("\n* ", $violationMessages);
        // user might be deleted
        $user = $device->getUser() ? $device->getUser()->getLogin() : null;
        $deviceName = $device->getCombinedName();
        $text = $this->translator->trans("admin.compliance.action.sendAdminMail.text", array(
            "%user%" => $user,
            "%device%" => $deviceName,
            "%violations%" => $violations
        ));

        $delayedActions = array();
        foreach ($actions as $name => $options) {
            if (isset($options['delay'])) {
                $delayHours = ceil($options['delay'] / 3600);
                $actionTransId = "admin.compliance.action.sendAdminMail.delayedAction." . $name;
                $delayedActionText = $this->translator->transChoice(
                    $actionTransId, $delayHours, array("%hours%" => $delayHours));
                $delayedActions[] = $delayedActionText;
            }
        }
        if ($delayedActions) {
            $delayedActionsText = "- " . join("\n- ", $delayedActions);
            $text .= "\n" . $this->translator->trans("admin.compliance.action.sendAdminMail.delayedActions", array(
                "%actions%" => $delayedActionsText,
            ));
        }

        return $text;
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    private function getLogger()
    {
        return $this->logger;
    }
}
