<?php
namespace Topalis\Compliance\Action;

use Topalis\Device\Entity\Device;
use Topalis\Compliance\ComplianceVerificationResultInterface;
use Topalis\Mail\MailerInterface;
use Symfony\Component\Translation\TranslatorInterface;
use Psr\Log\LoggerInterface;
use Topalis\Compliance\Util\ComplianceVerificationUtils;
use Topalis\Compliance\PreviousComplianceVerificationResultInterface;

/**
 * This compliance action sends a email to the user if there are new rule
 * violations.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class SendUserMailComplianceAction implements ComplianceActionInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Topalis\Mail\MailerInterface
     */
    private $mailer;

    /**
     * @var \Symfony\Component\Translation\TranslatorInterface
     */
    private $translator;

    /**
     * Constructor.
     *
     * @param MailerInterface $mailer
     * @param TranslatorInterface $translator
     * @param LoggerInterface $logger
     */
    public function __construct(MailerInterface $mailer, TranslatorInterface $translator, LoggerInterface $logger)
    {
        $this->mailer = $mailer;
        $this->translator = $translator;
        $this->logger = $logger;
    }

    public function perform(array $options, Device $device, ComplianceVerificationResultInterface $verificationResult,
        PreviousComplianceVerificationResultInterface $previousVerificationResult, array $actions)
    {
        $options; // unused

        if (!$this->mailer->isConfigured()) {
            $this->getLogger()->info("Mailer is not configured");
            return new ComplianceActionResult(false, null);
        }

        if (!$verificationResult->hasNewViolations($previousVerificationResult)) {
            $this->getLogger()->debug("No new violations");
            return new ComplianceActionResult(false, null);
        }

        $user = $device->getUser();
        if (null === $user) {
            $this->getLogger()->info("User of device " . $device->getCombinedId() . " has been deleted");
            return new ComplianceActionResult(false, null);
        }

        try {
            $this->getLogger()->debug("Send violation mail to " . $user->getEmail());
            $subject = $this->translator->trans(
                "admin.compliance.action.sendUserMail.subject", array(), null, $device->getLocale());
            $text = $this->composeMailText($device, $verificationResult, $actions);
            $message = $this->mailer->createMessage($subject);
            $message->setTo($user->getEmail(), $user->getRealname());
            $message->setBody($text);
            $this->mailer->sendMessage($message);
            return new ComplianceActionResult(true, time());
        } catch (\Exception $e) {
            $this->getLogger()->error("Sending compliance violation message to " . $user->getEmail() . " failed",
                array("exception" => $e));
        }
    }

    public function onRemove(Device $device)
    {
    }

    private function composeMailText(Device $device,
        ComplianceVerificationResultInterface $verificationResult, array $actions)
    {
        $violationMessages = ComplianceVerificationUtils::getViolationMessages($verificationResult,
            $this->translator, $device->getLocale());

        $violations = "* " . join("\n\n* ", $violationMessages);
        $user = $device->getUser()->getLogin();
        $realName = $device->getUser()->getRealname();
        $deviceName = $device->getName();
        $text = $this->translator->trans("admin.compliance.action.sendUserMail.text", array(
            "%realName%" => $realName,
            "%user%" => $user,
            "%device%" => $deviceName,
            "%violations%" => $violations
        ), null, $device->getLocale());

        $delayedActions = array();
        foreach ($actions as $name => $options) {
            if (isset($options['delay'])) {
                $delayHours = ceil($options['delay'] / 3600);
                $actionTransId = "admin.compliance.action.sendUserMail.delayedAction." . $name;
                $delayedActionText = $this->translator->transChoice(
                    $actionTransId, $delayHours, array("%hours%" => $delayHours), null, $device->getLocale());
                if ($delayedActionText != $actionTransId) {
                    // show only those actions for which a translation exist
                    $delayedActions[] = $delayedActionText;
                }
            }
        }
        if ($delayedActions) {
            $delayedActionsText = "- " . join("\n- ", $delayedActions);
            $text .= "\n" . $this->translator->trans("admin.compliance.action.sendUserMail.delayedActions", array(
                "%actions%" => $delayedActionsText,
            ), null, $device->getLocale());
        }

        return $text;
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    private function getLogger()
    {
        return $this->logger;
    }
}
