<?php
namespace Topalis\Compliance;

use Psr\Log\LoggerInterface;
use Topalis\Compliance\Action\ComplianceActionContainerInterface;
use Topalis\Compliance\Entity\Compliance;
use Topalis\Compliance\Entity\ComplianceEntityManager;
use Topalis\Compliance\Entity\DeviceActionEntityManager;
use Topalis\Compliance\Entity\RuleValidation;
use Topalis\Compliance\Entity\RuleValidationEntityManager;
use Topalis\Compliance\Rule\PreviousRuleValidationResultInterface;
use Topalis\Compliance\Rule\RuleTypeContainerInterface;
use Topalis\Compliance\Rule\RuleValidationResultInterface;
use Topalis\Device\Entity\Device;

/**
 * ComplianceManager
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ComplianceManager implements ComplianceManagerInterface
{
    /**
     * @var \Topalis\Compliance\Action\ComplianceActionContainerInterface
     */
    private $actionContainer;

    /**
     * @var \Topalis\Compliance\Entity\ComplianceEntityManager
     */
    private $complianceEntityManager;

    /**
     * @var \Topalis\Compliance\Entity\DeviceActionEntityManager
     */
    private $deviceActionEntityManager;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Topalis\Compliance\Rule\RuleTypeContainerInterface
     */
    private $ruleTypeContainer;

    /**
     * @var \Topalis\Compliance\Entity\RuleValidationEntityManager
     */
    private $ruleValidationEntityManager;

    /**
     * @var \Topalis\Compliance\ComplianceVerifierInterface
     */
    private $verifier;

    /**
     * Constructor.
     *
     * @param ComplianceActionContainerInterface $actionContainer
     * @param RuleTypeContainerInterface $ruleTypeContainer
     * @param ComplianceVerifierInterface $verifier
     * @param ComplianceEntityManager $complianceEntityManager
     * @param RuleValidationEntityManager $ruleValidationEntityManager
     * @param DeviceActionEntityManager $deviceActionEntityManager
     * @param LoggerInterface $logger
     */
    public function __construct(ComplianceActionContainerInterface $actionContainer,
        RuleTypeContainerInterface $ruleTypeContainer, ComplianceVerifierInterface $verifier,
        ComplianceEntityManager $complianceEntityManager, RuleValidationEntityManager $ruleValidationEntityManager,
        DeviceActionEntityManager $deviceActionEntityManager, LoggerInterface $logger)
    {
        $this->actionContainer = $actionContainer;
        $this->ruleTypeContainer = $ruleTypeContainer;
        $this->verifier = $verifier;
        $this->complianceEntityManager = $complianceEntityManager;
        $this->ruleValidationEntityManager = $ruleValidationEntityManager;
        $this->deviceActionEntityManager = $deviceActionEntityManager;
        $this->logger = $logger;
    }

    public function getComplianceForDevice(Device $device)
    {
        return $this->getComplianceEntityManager()->getComplianceForDevice($device);
    }

    public function hasComplianceForDevice(Device $device)
    {
        return $this->getComplianceEntityManager()->hasComplianceForDevice($device);
    }

    public function getLastResult(Device $device)
    {
        $previousResult = new PreviousComplianceVerificationResult($this->getRuleTypeContainer());
        foreach ($this->getRuleValidationEntityManager()->findBy(array("device" => $device)) as $ruleValidation) {
            $previousResult->addRuleValidation($ruleValidation);
        }

        return $previousResult;
    }

    public function getPerformedActions(Device $device)
    {
        $actions = $this->deviceActionEntityManager->findBy(
            array("device" => $device, "performed" => true), array("time" => "ASC"));
        $rv = array();
        foreach ($actions as $action) {
            $rv[$action->getAction()] = $action->getTime();
        }

        return $rv;
    }

    public function getPlannedActions(Device $device)
    {
        $actions = $this->deviceActionEntityManager->findBy(
            array("device" => $device, "performed" => false), array("time" => "ASC"));
        $rv = array();
        foreach ($actions as $action) {
            $rv[$action->getAction()] = $action->getTime();
        }

        return $rv;
    }

    public function verify(Device $device)
    {
        $this->logger->debug("Verify compliance of device '" . $device->getCombinedName() . "'",
            array("device" => $device->getCombinedId()));
        $em = $this->getComplianceEntityManager();
        $compliance = $this->getComplianceForDevice($device);
        if (null === $compliance) {
            $this->logger->debug("No compliance assigned to device '" . $device->getCombinedName() . "'",
                array("device" => $device->getCombinedId()));
            return;
        }

        $verifier = $this->getVerifier();
        $verificationStart = time();
        $previousResult = $this->getLastResult($device);
        $result = $verifier->verify($compliance, $device, $previousResult);

        $em->refresh($compliance);
        if ($compliance->getLastRuleChange() > $verificationStart) {
            $this->getLogger()->notice(
                "The rules of compliance \"" . $compliance->getName()
                . "\" have changed during compliance check of device \"" . $device->getCombinedName()
                . "\", so neither the actions are performed nor the result is stored.",
                array("compliance" => $compliance->getId(), "device" => $device->getCombinedId())
            );
            return;
        }

        $this->performActions($device, $compliance, $result, $previousResult);
        $this->storeResult($verificationStart, $device, $result, $previousResult);
        $this->logger->debug("Verify compliance of device '" . $device->getCombinedName() . "' finished",
            array("device" => $device->getCombinedId()));
    }

    /**
     * @return \Topalis\Compliance\Action\ComplianceActionContainerInterface
     */
    private function getActionContainer()
    {
        return $this->actionContainer;
    }

    /**
     * @return \Topalis\Compliance\Entity\ComplianceEntityManager
     */
    private function getComplianceEntityManager()
    {
        return $this->complianceEntityManager;
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    private function getLogger()
    {
        return $this->logger;
    }

    /**
     * @return \Topalis\Compliance\Rule\RuleTypeContainerInterface
     */
    private function getRuleTypeContainer()
    {
        return $this->ruleTypeContainer;
    }

    /**
     * @return \Topalis\Compliance\Entity\RuleValidationEntityManager
     */
    private function getRuleValidationEntityManager()
    {
        return $this->ruleValidationEntityManager;
    }

    /**
     * @return ComplianceVerifierInterface
     */
    private function getVerifier()
    {
        return $this->verifier;
    }

    private function performActions(Device $device, Compliance $compliance,
        ComplianceVerificationResultInterface $result,
        PreviousComplianceVerificationResultInterface $previousResult)
    {
        $actions = $compliance->getActions();
        $actionContainer = $this->getActionContainer();
        foreach ($actions as $name => $options) {
            $action = $actionContainer->get($name);
            try {
                $this->getLogger()->debug("Perform compliance action '" . $name . "'",
                    array("device" => $device->getCombinedId()));
                $actionResult = $action->perform($options, $device, $result, $previousResult, $actions);
                if ($actionResult->performed || null !== $actionResult->time) {
                    $deviceAction = $this->deviceActionEntityManager->findOrCreate($device, $name);
                    $deviceAction->setPerformed($actionResult->performed);
                    $deviceAction->setTime($actionResult->time);
                    $this->deviceActionEntityManager->save($deviceAction);
                }
                $this->getLogger()->debug("Perform compliance action '" . $name . "' finished",
                    array("device" => $device->getCombinedId()));
            } catch (\Exception $e) {
                $this->getLogger()->error("Error performing compliance action '" . $name . "'", array(
                    "device" => $device->getCombinedId(),
                    "compliance" => $compliance->getId(),
                    "exception" => $e
                ));
            }
        }
    }

    /**
     * Stores the result of a compliance check.
     *
     * @param integer $verificationStart UNIX timestamp of the beginning of the
     *     check. It is used as last check time and violation time. By using the
     *     beginning it is ensured that conditions depending on the time don't
     *     give wrong results if the compliance is changed during the validation.
     * @param Device $device
     * @param ComplianceVerificationResultInterface $result
     * @param PreviousComplianceVerificationResultInterface $previousResult
     */
    private function storeResult($verificationStart, Device $device, ComplianceVerificationResultInterface $result,
        PreviousComplianceVerificationResultInterface $previousResult)
    {
        $this->logger->debug("Store compliance verification result of device '" . $device->getCombinedName() . "'",
            array("device" => $device->getCombinedId()));

        /*
         * Note: The previous result doesn't contain results of rules not in the
         * new result because results of deleted rules are automatically
         * removed. So we don't need to (and won't) find rule validation results
         * that are in the previous result but not in the new one.
         */
        $em = $this->getRuleValidationEntityManager();

        foreach ($result->getPassed() as $passed) {
            $ruleValidation = $previousResult->getRuleValidation($passed['rule']);
            if ($ruleValidation === null) {
                $ruleValidation = new RuleValidation();
                $ruleValidation->setDevice($device);
                $ruleValidation->setRule($passed['rule']);
            }
            /*
             * Check if previous result is used. If that's the case we don't
             * need to update $ruleValidation
             */
            if (! $passed['result'] instanceof PreviousRuleValidationResultInterface) {
                $ruleValidation->setParameters($passed['result']->getParameters());
                $ruleValidation->setViolated(false);
                $ruleValidation->setLastCheckTime($verificationStart);
            }
            $em->save($ruleValidation, false);
        }

        foreach ($result->getViolated() as $violated) {
            $ruleValidation = $previousResult->getRuleValidation($violated['rule']);
            if ($ruleValidation === null) {
                $ruleValidation = new RuleValidation();
                $ruleValidation->setDevice($device);
                $ruleValidation->setRule($violated['rule']);
            }
            /*
             * Check if previous result is used. If that's the case we don't
             * need to update $ruleValidation
             */
            if (! $violated['result'] instanceof PreviousRuleValidationResultInterface) {
                $ruleValidation->setParameters($violated['result']->getParameters());
                if (!$ruleValidation->isViolated()) {
                    // rule previously wasn't violated so we have a new violation time
                    $ruleValidation->setViolationTime($verificationStart);
                }
                $ruleValidation->setViolated(true);
                $ruleValidation->setLastCheckTime($verificationStart);
            }
            $em->save($ruleValidation, false);
        }

        if (!$result->getViolated()) {
            // delete actions because no rule is violated
            $this->deviceActionEntityManager->deleteBy(array("device" => $device));
        }

        $em->flush();

        $device->setLastComplianceCheck($verificationStart);
    }
}
