<?php
namespace Topalis\Compliance;

use Topalis\Compliance\Rule\RuleValidatorFactoryInterface;
use Topalis\Device\Entity\Device;
use Topalis\Compliance\Entity\Compliance;
use Topalis\Compliance\Rule\RuleValidatorInterface;
use Topalis\Compliance\Rule\PreviousRuleValidationResult;
use Topalis\Compliance\Rule\PreviousRuleValidationResultInterface;
use Topalis\Compliance\Entity\Rule;
use Psr\Log\LoggerInterface;

/**
 * ComplianceVerifier
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ComplianceVerifier implements ComplianceVerifierInterface
{
    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Topalis\Compliance\Rule\RuleValidatorFactoryInterface
     */
    private $validatorFactory;

    public function __construct(RuleValidatorFactoryInterface $validatorFactory, LoggerInterface $logger)
    {
        $this->validatorFactory = $validatorFactory;
        $this->logger = $logger;
    }

    public function verify(Compliance $compliance, Device $device,
        PreviousComplianceVerificationResultInterface $previousResult)
    {
        $this->logger->debug("Verify compliance '" . $compliance->getName() . "' on device '"
            . $device->getCombinedName() . "'",
            array("compliance" => $compliance->getId(), "device" => $device->getCombinedId()));
        $result = new ComplianceVerificationResult();
        foreach ($compliance->getRules() as $rule) {
            if (null === $rule->getOs() || $rule->getOs() == $device->getType()->getName()) {
                $validationResult = $this->validate($rule, $previousResult, $device);
                $result->addValidationResult($rule, $validationResult);
            }
        }

        return $result;
    }

    private function validate(Rule $rule, PreviousComplianceVerificationResultInterface $previousResult,
        Device $device)
    {
        $previousValidationResult = $previousResult->getValidationResult($rule);
        $validator = $this->validatorFactory->create($rule);
        if ($previousValidationResult
            && !$this->isValidationRequired($rule, $validator, $previousValidationResult, $device)) {
            $this->getLogger()->debug(sprintf("Use previous validation result for rule \"%s\"", $rule->getType()),
                array("device" => $device->getCombinedId(), "rule" => $rule->getId()));
            return $previousValidationResult;
        }

        $this->getLogger()->debug(sprintf("Validate rule \"%s\"", $rule->getType()),
            array("device" => $device->getCombinedId(), "rule" => $rule->getId()));
        $validationResult = $validator->validate($device, $rule->getCondition());
        return $validationResult;
    }

    /**
     * Determines if a validation is required. This is the case on one of the
     * following conditions:
     * <ul>
     * <li>The validator doesn't depend on device information.</li>
     * <li>There was no compliance check.</li>
     * <li>The last device information update is after the last rule validation.</li>
     * <li>The last condition change is after the last rule validation.</li>
     *
     * @param Rule $rule
     * @param RuleValidatorInterface $validator
     * @param PreviousRuleValidationResultInterface $previousValidationResult
     * @param Device $device
     * @return boolean true if a check is required, false otherwise.
     */
    private function isValidationRequired(Rule $rule, RuleValidatorInterface $validator,
        PreviousRuleValidationResultInterface $previousValidationResult, Device $device)
    {
        $lastCheckTime = $previousValidationResult->getLastCheckTime();
        return !$validator->isDependsOnDeviceInformation() || !$device->getLastComplianceCheck()
            || $device->getLastDeviceInformationUpdate() > $lastCheckTime
            || $rule->getLastConditionChange() > $lastCheckTime;
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    protected function getLogger()
    {
        return $this->logger;
    }
}
