<?php
namespace Topalis\Compliance\Controller;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Topalis\Compliance\Entity\Compliance;
use Topalis\Compliance\Form\ComplianceForm;
use Topalis\Compliance\Form\Handler\ComplianceFormHandler;
use Topalis\Compliance\Util\ComplianceVerificationUtils;
use Topalis\Kernel\Controller;

/**
 * Controller for compliances.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/compliance", service="compliance.controller")
 */
class ComplianceController extends Controller
{
    /**
     * @Route("", name="admin_compliance")
     * @Template()
     *
     * @return array
     */
    public function indexAction()
    {
        $em = $this->getComplianceEntityManager();
        $compliances = $em->findAll();
        $deviceCount = array();
        $violatingDeviceCount = array();
        foreach ($compliances as $compliance) {
            $deviceCount[$compliance->getId()] = $em->getDeviceCount($compliance);
            $violatingDeviceCount[$compliance->getId()] = $em->getViolatingDeviceCount($compliance);
        }

        return array(
            "compliances"          => $compliances,
            "deviceCount"          => $deviceCount,
            "violatingDeviceCount" => $violatingDeviceCount,
        );
    }

    /**
     * @Route("/add", name="admin_compliance_add")
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function addAction()
    {
        $compliance = new Compliance();
        return $this->forward("compliance.controller:editAction", array("compliance" => $compliance));
    }

    /**
     * @param Request $request
     * @param Compliance $compliance
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     *
     * @Route("/{compliance}/edit", name="admin_compliance_edit")
     * @Template
     */
    public function editAction(Request $request, Compliance $compliance)
    {
        $em = $this->getComplianceEntityManager();
        $form = $this->createForm(new ComplianceForm($compliance, $this->container->get("app.winphone.store")));
        $formHandler = new ComplianceFormHandler($em, $this->getRequest());

        if ($formHandler->process($form)) {
            $em->beginTransaction();
            try {
                $em->save($formHandler->getComplianceEntity());
                $em->commit();
                $this->addSuccess($this->tr("admin.compliance.save.success"));
            } catch (\Exception $e) {
                $em->rollback();
                $this->getLogger()->error("Error saving compliance", array("exception" => $e));
                $msg = $this->tr("admin.compliance.save.error", array("%errmsg%" => $e->getMessage()));
                $this->addError($msg);
            }
        }

        $violatingDevices = null === $compliance->getId() ? array() : $em->getViolatingDevices($compliance);
        $violationMessages = array();
        $performedActions = array();
        $plannedActions = array();
        $complianceManager = $this->getComplianceManager();
        $translator = $this->getTranslator();
        foreach ($violatingDevices as $device) {
            $verificationResult = $complianceManager->getLastResult($device);
            $violationMessages[$device->getId()] =
                ComplianceVerificationUtils::getShortViolationMessages($verificationResult, $translator);
            $performedActions[$device->getId()] = $complianceManager->getPerformedActions($device);
            $plannedActions[$device->getId()] = $complianceManager->getPlannedActions($device);
        }

        return array(
            "form"              => $form->createView(),
            "compliance"        => $compliance,
            "violatingDevices"  => $violatingDevices,
            "violationMessages" => $violationMessages,
            "performedActions"  => $performedActions,
            "plannedActions"    => $plannedActions,
            "tabRestore"        => "POST" == $request->getMethod(),
        );
    }

    /**
     * @Route("/{compliance}/delete", name="admin_compliance_delete")
     *
     * @param Compliance $compliance
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function deleteAction(Compliance $compliance)
    {
        try {
            $this->getComplianceEntityManager()->delete($compliance);
            $this->addSuccess($this->tr("admin.compliance.delete.success"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error deleting compliance with id " . $compliance->getId(),
                array("exception" => $e));
            $msg = $this->tr(
                "admin.compliance.delete.error",
                array("%name%" => $compliance->getName(), "%errmsg%" => $e->getMessage())
            );
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_compliance"));
    }



    /**
     * @Route("/winphone/store/lookupName/{id}", name="admin_compliance_winphone_store_lookup_name",
     *   options={"expose" = true})
     *
     * @param string $id The identifier of the app to lookup
     * @return Response Contains a string with "name (vendor)" on success.
     */
    public function winPhoneStoreLookupNameAction($id)
    {
        try {
            $app = $this->container->get("app.winphone.store")->lookup($id);
            if ($app === null) {
                $response = new Response(Response::$statusTexts[404], 404);
            } else {
                $response = new Response($app->getName() . " (" . $app->getVendor() . ")");
            }
        } catch (\InvalidArgumentException $e) {
            $response = new Response($e->getMessage(), 400);
        } catch (\Exception $e) {
            $response = new Response($e->getMessage(), 500);
        }

        $response->headers->set("Content-Type", "plain/text; charset=UTF-8");
        return $response;
    }


    /**
     * @return \Topalis\Compliance\Entity\ComplianceEntityManager
     */
    private function getComplianceEntityManager()
    {
        return $this->container->get("compliance.entity.manager.compliance");
    }

    /**
     * @return \Topalis\Compliance\ComplianceManagerInterface
     */
    private function getComplianceManager()
    {
        return $this->container->get("compliance.manager");
    }
}
