<?php
namespace Topalis\Compliance\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Topalis\ConfigGroup\Entity\ConfigGroup;

/**
 * Represents a compliance.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 *
 * @ORM\Entity
 * @ORM\Table(name="compliances")
 * @UniqueEntity(fields="name")
 */
class Compliance
{
    /**
     * @var integer
     *
     * @ORM\Column(type="integer", options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string
     *
     * @ORM\Column(type="string", unique=true)
     */
    protected $name;

    /**
     * @var string
     *
     * @ORM\Column(type="string")
     */
    protected $description;

    /**
     * Every group may only have one compliance so we make the inverse join
     * column unique.
     *
     * @var \Doctrine\Common\Collections\Collection
     *
     * @ORM\ManyToMany(targetEntity="Topalis\ConfigGroup\Entity\ConfigGroup", cascade={"persist"})
     * @ORM\JoinTable(name="compliances_configgroups",
     *     inverseJoinColumns={@ORM\JoinColumn(name="group_id", referencedColumnName="id", onDelete="cascade")})
     */
    protected $groups;

    /**
     * @var \Doctrine\Common\Collections\Collection
     *
     * @ORM\OneToMany(targetEntity="Rule", mappedBy="compliance", cascade={"persist"},
     *     orphanRemoval=true)
     */
    protected $rules;

    /**
     * @var WinPhoneAppPolicy
     *
     * @ORM\OneToOne(targetEntity="WinPhoneAppPolicy", inversedBy="compliance",
     *     cascade={"persist"}, orphanRemoval=true)
     * @ORM\JoinColumn(onDelete="SET NULL")
     */
    protected $winPhoneAppPolicy;

    /**
     * @var array
     *
     * @ORM\Column(type="json_array", length=65532, nullable=true)
     */
    protected $actions;

    /**
     * @var int
     *
     * @ORM\Column(type="integer", options={"unsigned"=true})
     */
    protected $checkInterval;

    /**
     * @var int
     *
     * @ORM\Column(type="bigint", nullable=true, options={"unsigned"=true})
     */
    protected $jobId;

    /**
     * UNIX timestamp of the last rule change.
     * @var integer
     *
     * @ORM\Column(type="integer", nullable=true, options={"unsigned"=true})
     */
    protected $lastRuleChange;

    /**
     * Constructor
     */
    public function __construct()
    {
        $this->groups = new \Doctrine\Common\Collections\ArrayCollection();
        $this->rules = new \Doctrine\Common\Collections\ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set name
     *
     * @param string $name
     * @return Compliance
     */
    public function setName($name)
    {
        $this->name = $name;

        return $this;
    }

    /**
     * Get name
     *
     * @return string
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     * Set description
     *
     * @param string $description
     * @return Compliance
     */
    public function setDescription($description)
    {
        $this->description = $description;

        return $this;
    }

    /**
     * Get description
     *
     * @return string
     */
    public function getDescription()
    {
        return $this->description;
    }

    /**
     * Add groups
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $group
     * @return Compliance
     */
    public function addGroup(ConfigGroup $group)
    {
        $this->groups[] = $group;

        return $this;
    }

    /**
     * Remove groups
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $group
     */
    public function removeGroup(ConfigGroup $group)
    {
        $this->groups->removeElement($group);
    }

    /**
     * Get groups
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getGroups()
    {
        return $this->groups;
    }

    /**
     * Add rules
     *
     * @param \Topalis\Compliance\Entity\Rule $rule
     * @return Compliance
     */
    public function addRule(Rule $rule)
    {
        if (!$this->rules->contains($rule)) {
            $this->rules->add($rule);
            $rule->setCompliance($this);
        }

        return $this;
    }

    /**
     * Remove rules
     *
     * @param \Topalis\Compliance\Entity\Rule $rule
     */
    public function removeRule(Rule $rule)
    {
        $this->rules->removeElement($rule);
    }

    /**
     * Get rules
     *
     * @return \Doctrine\Common\Collections\Collection
     */
    public function getRules()
    {
        return $this->rules;
    }

    /**
     * Get winPhoneAppPolicy
     *
     * @return WinPhoneAppPolicy
     */
    public function getWinPhoneAppPolicy()
    {
        return $this->winPhoneAppPolicy;
    }

    /**
     * Set winPhoneAppPolicy
     *
     * @param WinPhoneAppPolicy $winPhoneAppPolicy
     *
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup self
     */
    public function setWinPhoneAppPolicy(WinPhoneAppPolicy $winPhoneAppPolicy = null)
    {
        if ($this->winPhoneAppPolicy != $winPhoneAppPolicy) {
            $this->winPhoneAppPolicy = $winPhoneAppPolicy;
            if (null !== $this->winPhoneAppPolicy) {
                $this->winPhoneAppPolicy->setCompliance($this);
            }
        }

        return $this;
    }

    /**
     * Set actions
     *
     * @param array $actions
     * @return Compliance
     */
    public function setActions(array $actions)
    {
        $this->actions = $actions;

        return $this;
    }

    /**
     * Get actions
     *
     * @return array
     */
    public function getActions()
    {
        return $this->actions;
    }

    /**
     * Set checkInterval
     *
     * @param integer $checkInterval
     * @return Compliance
     */
    public function setCheckInterval($checkInterval)
    {
        $this->checkInterval = $checkInterval;

        return $this;
    }

    /**
     * Get checkInterval
     *
     * @return integer
     */
    public function getCheckInterval()
    {
        return $this->checkInterval;
    }

    /**
     * Set jobId
     *
     * @param integer $jobId
     * @return Compliance
     */
    public function setJobId($jobId)
    {
        $this->jobId = $jobId;

        return $this;
    }

    /**
     * Get jobId
     *
     * @return integer
     */
    public function getJobId()
    {
        return $this->jobId;
    }

    /**
     * Set lastRuleChange
     *
     * @param integer $lastRuleChange UNIX timestamp of the last rule change.
     * @return Compliance
     */
    public function setLastRuleChange($lastRuleChange)
    {
        $this->lastRuleChange = $lastRuleChange;

        return $this;
    }

    /**
     * Get lastRuleChange
     *
     * @return integer UNIX timestamp of the last rule change.
     */
    public function getLastRuleChange()
    {
        return $this->lastRuleChange;
    }
}
