<?php
namespace Topalis\Compliance\Entity;

use Doctrine\ORM\AbstractQuery;
use Doctrine\ORM\Query\Expr\Join;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Database\EntityManager;
use Topalis\Device\Entity\Device;

/**
 * ComplianceEntityManager
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ComplianceEntityManager extends EntityManager
{
    public function __construct(\Doctrine\ORM\EntityManager $em)
    {
        parent::__construct($em, "Topalis\Compliance\Entity\Compliance");
    }

    /**
     * @param Device $device The device to get the compliance for.
     *
     * @return Compliance|null The compliance for the specified device, or null
     *     if there's no compliance configured for the device.
     */
    public function getComplianceForDevice(Device $device)
    {
        return $this->createQueryBuilder("c")
            ->join("c.groups", "g")
            ->join("Topalis\Device\Entity\Device", "d", Join::WITH, "d = :device AND d.configGroup = g")
            ->setParameter("device", $device)
            ->getQuery()->getOneOrNullResult();
    }

    /**
     * @param Device $device The device to check if it has a compliance.
     *
     * @return bool true if there exists a compliance for the device, false
     *     otherwise.
     */
    public function hasComplianceForDevice(Device $device)
    {
        return null !== $this->createQueryBuilder("c")
            ->select("1")
            ->join("c.groups", "g")
            ->join("Topalis\Device\Entity\Device", "d", Join::WITH, "d = :device AND d.configGroup = g")
            ->setParameter("device", $device)
            ->getQuery()->getOneOrNullResult();
    }

    /**
     * @param ConfigGroup $group The group to get the compliance for.
     *
     * @return Compliance|null The compliance for the specified group, or null
     *     if there's no compliance configured for the group.
     */
    public function getComplianceForGroup(ConfigGroup $group)
    {
        return $this->getComplianceForGroupId($group->getId());
    }

    /**
     * @param int $id The id of the group to get the compliance for.
     *
     * @return Compliance|null The compliance for the specified id, or null if
     *    there's no compliance configured for the group.
     */
    public function getComplianceForGroupId($id)
    {
        return $this->createQueryBuilder("c")
            ->join("c.groups", "g", Join::WITH, "g.id = :id")
            ->setParameter("id", $id)
            ->getQuery()->getOneOrNullResult();
    }

    /**
     * @param Compliance $compliance
     *
     * @return int The number of devices belonging to the specified compliance.
     */
    public function getDeviceCount(Compliance $compliance)
    {
        return $this->createQueryBuilder("c")->select("COUNT(c)")
            ->join("c.groups", "g")
            ->join("g.devices", "d")
            ->where("c = :compliance")
            ->setParameter("compliance", $compliance)
            ->getQuery()->getSingleScalarResult();
    }

    /**
     * @param Compliance $compliance The compliance to get the violating devices
     *     for.
     *
     * @return Device[] Array of devices that violate the specified compliance.
     */
    public function getViolatingDevices(Compliance $compliance)
    {
        $qb = $this->em->createQueryBuilder()->select("1")->from("Topalis\Compliance\Entity\RuleValidation", "rv")
            ->join("rv.rule", "r")
            ->where("rv.device = d")
            ->andWhere("rv.violated = 1")
            ->andWhere("r.compliance = :compliance");
        $devices = $this->em->createQueryBuilder()->select("d")->from("Topalis\Device\Entity\Device", "d")
            ->join("d.configGroup", "g", Join::WITH, "g IN (:groups)")
            ->where("EXISTS (" . $qb->getDQL() . ")")
            ->setParameter("groups", $compliance->getGroups()->toArray())
            ->setParameter("compliance", $compliance)
            ->getQuery()->getResult();

        return $devices;
    }

    /**
     * @param Compliance $compliance
     *
     * @return int The number of devices that violate the specified compliance.
     *    If no compliance is specified all violating devices are counted.
     */
    public function getViolatingDeviceCount(Compliance $compliance = null)
    {
        $qb = $this->em->createQueryBuilder()->select("COUNT(DISTINCT d)")
            ->from("Topalis\Compliance\Entity\RuleValidation", "rv")
            ->join("rv.rule", "r")
            ->join("rv.device", "d")
            ->where("rv.violated = 1");
        if (null !== $compliance) {
            $qb->andWhere("r.compliance = :compliance")
                ->setParameter("compliance", $compliance);
        }
        $count = $qb->getQuery()->getOneOrNullResult(AbstractQuery::HYDRATE_SCALAR);

        return $count ? reset($count) : 0;
    }
}
