<?php
namespace Topalis\Compliance\Event;

use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Topalis\Compliance\ComplianceManagerInterface;
use Topalis\Compliance\Job\ComplianceVerifyQueueInterface;
use Topalis\MDM\MdmEvent;
use Topalis\MDM\MdmEvents;
use Topalis\MDM\MdmQueueInterface;

/**
 * Initiates a compliance verify job after the device information of a device
 * were received.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class MdmSubscriber implements EventSubscriberInterface
{
    private static $dependendMdmCommands = array(
        "DeviceInformation",
        "InstalledApplications",
        "ManagedApplicationList",
    );

    /**
     * @var \Topalis\Compliance\ComplianceManagerInterface
     */
    private $complianceManager;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Topalis\MDM\MdmQueueInterface
     */
    private $mdmQueue;

    /**
     * @var \Topalis\Compliance\Job\ComplianceVerifyQueueInterface
     */
    private $verifyQueue;

    /**
     * @param ComplianceVerifyQueueInterface $verifyQueue
     * @param MdmQueueInterface $mdmQueue
     * @param ComplianceManagerInterface $complianceManager
     * @param LoggerInterface $logger
     */
    public function __construct(ComplianceVerifyQueueInterface $verifyQueue,
        MdmQueueInterface $mdmQueue, ComplianceManagerInterface $complianceManager,
        LoggerInterface $logger)
    {
        $this->verifyQueue = $verifyQueue;
        $this->mdmQueue = $mdmQueue;
        $this->complianceManager = $complianceManager;
        $this->logger = $logger;
    }

    public static function getSubscribedEvents()
    {
        return array(
            MdmEvents::DEVICE_INFORMATION => "handleEvent",
            MdmEvents::INSTALLED_APPLICATION_LIST => "handleEvent",
            MdmEvents::MANAGED_APPLICATION_LIST => "handleEvent",
        );
    }

    /**
     * @param MdmEvent $event
     */
    public function handleEvent(MdmEvent $event)
    {
        $device = $event->getDevice();
        if (!$this->complianceManager->hasComplianceForDevice($device)) {
            return;
        }

        $command = $event->getContext()->getMdmCommand()->getType();
        foreach (self::$dependendMdmCommands as $dependendCommand) {
            if ($command == $dependendCommand) {
                // don't check command existance for just answered command
                continue;
            }

            if ($this->mdmQueue->existsCommand($dependendCommand, $device)) {
                $this->getLogger()->debug(
                    "Don't verifiy compliance of device " . $device->getCombinedId()
                    . " because a request for device information is still in the queue");
                return;
            }
        }

        $this->verifyQueue->putDevice($device);
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    private function getLogger()
    {
        return $this->logger;
    }
}
