<?php
namespace Topalis\Compliance\Event;

use Topalis\Compliance\Entity\Compliance;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Doctrine\ORM\Query\Expr\Join;
use Topalis\Compliance\Entity\Rule;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Doctrine\ORM\Event\PostFlushEventArgs;
use Doctrine\ORM\Event\OnFlushEventArgs;
use Doctrine\ORM\PersistentCollection;
use Topalis\Compliance\Entity\WinPhoneAppPolicy;

/**
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ORMListener
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    /**
     * @var ORMListenerFlushContext
     */
    private $flushContext;

    /**
     * Groups of which the last compliance check time has been reset on devices
     * belonging to the group.
     *
     * @var array
     */
    private $groupLastComplianceCheckReset = array();

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    public function onFlush(OnFlushEventArgs $eventArgs)
    {
        $em = $eventArgs->getEntityManager();
        $eventManager = $em->getEventManager();

        // remove event to avoid infinite recursion loop if we call $em->flush()
        $eventManager->removeEventListener("onFlush", $this);

        $this->flushContext = new ORMListenerFlushContext($this->getEntityManager());
        $this->flushContext->update($eventArgs);

        foreach ($this->flushContext->getGroupCloned() as $group) {
            $this->handleGroupClonedOnFlush($group, $eventArgs);
        }

        foreach ($this->flushContext->getComplianceRulesChanged() as $compliance) {
            $this->handleChangedRulesOnFlush($compliance, $eventArgs);
        }

        foreach ($this->flushContext->getGroupComplianceChanged() as $changed) {
            $group = $changed['group'];
            if (null !== $changed['previous']) {
                $this->handleComplianceChangeOnFlush($group, $changed['previous'],
                    $changed['new'], $eventArgs);
            }
        }

        foreach ($this->flushContext->getGroupComplianceRemoved() as $removed) {
            $this->handleComplianceRemoveOnFlush($removed['group'], $removed['compliance'], $eventArgs);
        }

        foreach ($this->flushContext->getGroupWinPhoneAppPolicyChanged() as $group) {
            $this->handleWinPhoneAppPolicyChangeOnFlush($group, $eventArgs);
        }

        foreach ($this->flushContext->getComplianceActionsChanged() as $changed) {
            $this->handleChangedActionsOnFlush($changed['compliance'], $changed['previous'],
                $changed['new'], $eventArgs);
        }

        // re-attach since we're done
        $eventManager->addEventListener("onFlush", $this);
    }

    public function postFlush(PostFlushEventArgs $eventArgs)
    {
        $em = $eventArgs->getEntityManager();
        $eventManager = $em->getEventManager();

        // remove event to avoid infinite recursion loop if we call $em->flush()
        $eventManager->removeEventListener("postFlush", $this);

        $queue = $this->container->get("compliance.job.control_queue");
        foreach ($this->flushContext->getCompliancesToValidate() as $compliance) {
            // put the job with a delay to avoid unnecessary checks if the user
            // updates the compliance again after a short time
            $queue->putCompliance($compliance, true, 120);
        }

        $this->flushContext->clear();
        $this->groupLastComplianceCheckReset = array();

        // re-attach since we're done
        $eventManager->addEventListener("postFlush", $this);
    }

    /**
     * @return \Topalis\Compliance\Action\ComplianceActionContainerInterface
     */
    private function getActionContainer()
    {
        return $this->container->get("compliance.action.container");
    }

    /**
     * @return \Topalis\Compliance\Entity\ComplianceEntityManager
     */
    private function getEntityManager()
    {
        return $this->container->get("compliance.entity.manager.compliance");
    }

    /**
     * @param PersistentCollection $collection
     *
     * @return array The elements that were and are still in the collection.
     */
    private function getUnchanged(PersistentCollection $collection)
    {
        return array_udiff_assoc(
            $collection->getSnapshot(),
            $collection->getDeleteDiff(),
            function ($a, $b) {
                return $a === $b ? 0 : 1;
            }
        );
    }

    private function clearDeviceActions(ConfigGroup $group, OnFlushEventArgs $eventArgs)
    {
        $em = $eventArgs->getEntityManager();
        // doctrine doesn't allow joins in delete statements so we have to
        // fetch the device actions to delete first
        $deviceActions = $em->createQueryBuilder()->select("a")
            ->from("TopalisComplianceBundle:DeviceAction", "a")
            ->join("a.device", "d", Join::WITH, "d.configGroup = :group")
            ->setParameter("group", $group)
            ->getQuery()->getResult();
        foreach ($deviceActions as $deviceAction) {
            $em->remove($deviceAction);
        }
        $em->flush();
    }

    private function handleChangedActionsOnFlush(Compliance $compliance, array $previousActions,
        array $newActions, OnFlushEventArgs $eventArgs)
    {
        $compliancesToValidate = $this->flushContext->getCompliancesToValidate();
        if (!isset($compliancesToValidate[$compliance->getId()])) {
            foreach ($newActions as $key => $options) {
                if (!isset($previousActions[$key]) || $options != $previousActions[$key]) {
                    /*
                     * actually only the actions have to be performed on
                     * devices in groups that were not added just now, but
                     * lets do the compliance check again to avoid
                     * additional complexity.
                     */
                    $this->flushContext->addComplianceToValidate($compliance);
                    break;
                }
            }
        }

        $deletedActions = array_diff(array_keys($previousActions), array_keys($newActions));
        foreach ($this->getUnchanged($compliance->getGroups()) as $group) {
            $this->handleDeletedActionsOnFlush($deletedActions, $group, $eventArgs);
        }
    }

    private function handleChangedRulesOnFlush(Compliance $compliance, OnFlushEventArgs $eventArgs)
    {
        $em = $eventArgs->getEntityManager();
        $uow = $em->getUnitOfWork();
        $compliance->setLastRuleChange(time());
        $em->persist($compliance);
        $md = $em->getClassMetadata(get_class($compliance));
        $uow->recomputeSingleEntityChangeSet($md, $compliance);

        foreach ($compliance->getGroups() as $group) {
            $this->resetLastComplianceCheck($group);
        }
    }

    private function handleComplianceChangeOnFlush(ConfigGroup $group,
        Compliance $previousCompliance, Compliance $newCompliance,
        OnFlushEventArgs $eventArgs)
    {
        $em = $eventArgs->getEntityManager();
        $uow = $em->getUnitOfWork();

        // remove group from previous compliance
        $previousCompliance->removeGroup($group);
        $em->persist($previousCompliance);
        $md = $em->getClassMetadata(get_class($previousCompliance));
        $uow->recomputeSingleEntityChangeSet($md, $previousCompliance);

        // doctrine doesn't allow joins in delete statements so we have to
        // fetch the rule validations to delete first
        // we could compare the rules of the previous and the new compliance
        // for equal rules we could set the rule in the rule validation and
        // remove the other rules. Is it worth to do it?
        $ruleValidations = $em->createQueryBuilder()->select("rv")
            ->from("TopalisComplianceBundle:RuleValidation", "rv")
            ->join("rv.device", "d", Join::WITH, "d.configGroup = :group")
            ->setParameter("group", $group)
            ->getQuery()->getResult();
        foreach ($ruleValidations as $ruleValidation) {
            $em->remove($ruleValidation);
        }

        $this->resetLastComplianceCheck($group);
        $this->clearDeviceActions($group, $eventArgs);

        $previousActions = $previousCompliance->getActions();
        $newActions = $newCompliance->getActions();
        $deletedActions = array_diff(array_keys($previousActions), array_keys($newActions));
        $this->handleDeletedActionsOnFlush($deletedActions, $group, $eventArgs);
    }

    private function handleComplianceRemoveOnFlush(ConfigGroup $group, Compliance $compliance,
        OnFlushEventArgs $eventArgs)
    {
        $em = $eventArgs->getEntityManager();
        // doctrine doesn't allow joins in delete statements so we have to
        // fetch the rule validations to delete first
        $ruleValidations = $em->createQueryBuilder()->select("rv")
            ->from("TopalisComplianceBundle:RuleValidation", "rv")
            ->join("rv.device", "d", Join::WITH, "d.configGroup = :group")
            ->setParameter("group", $group)
            ->getQuery()->getResult();
        foreach ($ruleValidations as $ruleValidation) {
            $em->remove($ruleValidation);
        }

        $this->resetLastComplianceCheck($group);

        $deletedActions = array_keys($compliance->getActions());
        $this->handleDeletedActionsOnFlush($deletedActions, $group, $eventArgs);
    }

    private function handleDeletedActionsOnFlush(array $actionKeys, ConfigGroup $group, OnFlushEventArgs $eventArgs)
    {
        $actionContainer = $this->getActionContainer();
        foreach ($actionKeys as $actionKey) {
            $action = $actionContainer->get($actionKey);
            foreach ($group->getDevices() as $device) {
                $action->onRemove($device);
            }

            $em = $eventArgs->getEntityManager();
            // doctrine doesn't allow joins in delete statements so we have to
            // fetch the device actions to delete first
            $deviceActions = $em->createQueryBuilder()->select("a")
                ->from("TopalisComplianceBundle:DeviceAction", "a")
                ->join("a.device", "d", Join::WITH, "d.configGroup = :group")
                ->where("a.action = :action")
                ->setParameter("group", $group)
                ->setParameter("action", $actionKey)
                ->getQuery()->getResult();
            foreach ($deviceActions as $deviceAction) {
                $em->remove($deviceAction);
            }
            $em->flush();
        }
    }

    private function handleGroupClonedOnFlush(ConfigGroup $group, OnFlushEventArgs $eventArgs)
    {
        $compliance = $this->getEntityManager()->getComplianceForGroupId($group->getOriginId());
        if (null === $compliance) {
            return;
        }

        $em = $eventArgs->getEntityManager();
        $uow = $em->getUnitOfWork();
        // ensure that the new group has the same compliance as the cloned group
        $compliance->addGroup($group);
        $em->persist($compliance);
        $md = $em->getClassMetadata(get_class($compliance));
        $uow->computeChangeSet($md, $compliance);
    }

    private function handleWinPhoneAppPolicyChangeOnFlush(ConfigGroup $group, OnFlushEventArgs $eventArgs)
    {
        $eventArgs; // unused
        $syncMLAdmin = $this->container->get("winphone.oma.syncml.admin");
        foreach ($group->getDevices() as $device) {
            if ($syncMLAdmin->isActiveManagedDevice($device) && version_compare($device->getVersion(), "8.1", ">=")) {
                $syncMLAdmin->installPolicyManagerCSP($device);
            }
        }
    }

    private function resetLastComplianceCheck(ConfigGroup $group)
    {
        if (isset($this->groupLastComplianceCheckReset[$group->getId()])) {
            return;
        }

        $this->getEntityManager()->createQueryBuilder("c")->update("TopalisDeviceBundle:Device", "d")
            ->set("d.lastComplianceCheck", "NULL")
            ->where("d.configGroup = :group")
            ->setParameter("group", $group)
            ->getQuery()->execute();

        $this->groupLastComplianceCheckReset[$group->getId()] = $group;
    }
}
