<?php
namespace Topalis\Compliance\Event;

use Doctrine\ORM\Event\OnFlushEventArgs;
use Doctrine\ORM\UnitOfWork;
use Topalis\Compliance\Entity\Compliance;
use Topalis\Compliance\Entity\ComplianceEntityManager;
use Topalis\Compliance\Entity\Rule;
use Topalis\Compliance\Entity\WinPhoneAppPolicy;
use Topalis\ConfigGroup\Entity\ConfigGroup;

/**
 * The ORMListenerFlushContext stores information about entities that are
 * scheduled for delete, insert, or update.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ORMListenerFlushContext
{
    /**
     * Compliances with changed actions.
     *
     * @var array
     */
    private $complianceActionsChanged = array();

    /**
     * Compliances with changed rules.
     *
     * @var Compliance[]
     */
    private $complianceRulesChanged = array();

    /**
     * Compliances that need to be validated.
     *
     * @var Compliance[]
     */
    private $compliancesToValidate = array();

    /**
     * New groups cloned from other groups.
     *
     * @var \Topalis\ConfigGroup\Entity\ConfigGroup[]
     */
    private $groupCloned = array();

    /**
     * Groups with changed compliance.
     *
     * @var array
     */
    private $groupComplianceChanged = array();

    /**
     * Groups with removed compliance.
     *
     * @var array
     */
    private $groupComplianceRemoved = array();

    /**
     * Groups with changed Windows Phone App policies.
     *
     * @var \Topalis\ConfigGroup\Entity\ConfigGroup[]
     */
    private $groupWinPhoneAppPolicyChanged = array();

    /**
     * @var ComplianceEntityManager
     */
    private $complianceEm;

    public function __construct(ComplianceEntityManager $complianceEm)
    {
        $this->complianceEm = $complianceEm;
    }

    /**
     * @return array Compliances with changed actions.
     */
    public function getComplianceActionsChanged()
    {
        return $this->complianceActionsChanged;
    }

    /**
     * @return Compliance[] Compliances with changed rules.
     */
    public function getComplianceRulesChanged()
    {
        return $this->complianceRulesChanged;
    }

    /**
     * Adds the specified compliance to the list of compliances to validate.
     *
     * @param Compliance $compliance
     */
    public function addComplianceToValidate(Compliance $compliance)
    {
        $this->compliancesToValidate[$compliance->getId()] = $compliance;
    }

    /**
     * @return Compliance[] Compliances that need to be validated.
     */
    public function getCompliancesToValidate()
    {
        return $this->compliancesToValidate;
    }

    /**
     *
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup[] New groups cloned from
     *    other groups.
     */
    public function getGroupCloned()
    {
        return $this->groupCloned;
    }

    /**
     * @return array Groups with changed compliance.
     */
    public function getGroupComplianceChanged()
    {
        return $this->groupComplianceChanged;
    }

    /**
     * @return array Groups with removed compliance.
     */
    public function getGroupComplianceRemoved()
    {
        return $this->groupComplianceRemoved;
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup[] Groups with changed
     *     Windows Phone App policies.
     */
    public function getGroupWinPhoneAppPolicyChanged()
    {
        return $this->groupWinPhoneAppPolicyChanged;
    }

    public function clear()
    {
        $this->complianceActionsChanged = array();
        $this->complianceRulesChanged = array();
        $this->compliancesToValidate = array();
        $this->groupCloned = array();
        $this->groupComplianceChanged = array();
        $this->groupComplianceRemoved = array();
        $this->groupWinPhoneAppPolicyChanged = array();
    }

    public function update(OnFlushEventArgs $eventArgs)
    {
        $uow = $eventArgs->getEntityManager()->getUnitOfWork();

        $this->handleWinPhoneAppPolicyChanged($uow);
        $this->handleComplianceRulesChanged($uow);
        $this->handleClonedGroup($uow);
        // must be called before handleComplianceUpdated and handleComplianceInserted
        $this->handleComplianceRemoved($uow);
        $this->handleComplianceUpdated($uow);
        $this->handleComplianceInserted($uow);
    }

    /**
     *
     * @param UnitOfWork $uow
     */
    private function handleComplianceRulesChanged(UnitOfWork $uow)
    {
        $entities = array_merge(
            $uow->getScheduledEntityInsertions(),
            $uow->getScheduledEntityUpdates(),
            $uow->getScheduledEntityDeletions()
        );

        foreach ($entities as $entity) {
            if (!$entity instanceof Rule) {
                continue;
            }

            $compliance = $entity->getCompliance();
            if ($uow->isScheduledForDelete($compliance)) {
                // ignore rules of compliances scheduled for deletion
                continue;
            }

            $this->complianceRulesChanged[$compliance->getId()] = $compliance;
            $this->compliancesToValidate[$compliance->getId()] = $compliance;
        }
    }

    /**
     *
     * @param UnitOfWork $uow
     */
    private function handleWinPhoneAppPolicyChanged(UnitOfWork $uow)
    {
        $entities = array_merge(
            $uow->getScheduledEntityInsertions(),
            $uow->getScheduledEntityUpdates(),
            $uow->getScheduledEntityDeletions()
        );

        foreach ($entities as $entity) {
            if (!$entity instanceof WinPhoneAppPolicy) {
                continue;
            }

            $compliance = $entity->getCompliance();
            foreach ($compliance->getGroups() as $group) {
                $this->groupWinPhoneAppPolicyChanged[$group->getId()] = $group;
            }
        }
    }

    private function handleClonedGroup(UnitOfWork $uow)
    {
        foreach ($uow->getScheduledEntityInsertions() as $entity) {
            if (!$entity instanceof ConfigGroup) {
                continue;
            }

            if ($entity->isCloned()) {
                $this->groupCloned[] = $entity;
            }
        }
    }

    private function handleComplianceInserted(UnitOfWork $uow)
    {
        foreach ($uow->getScheduledEntityInsertions() as $entity) {
            if (!$entity instanceof Compliance) {
                continue;
            }

            foreach ($entity->getGroups() as $group) {
                $previousCompliance = $this->complianceEm->getComplianceForGroup($group);
                $this->groupComplianceChanged[$group->getId()] = array(
                    "group" => $group,
                    "previous" => $previousCompliance,
                    "new" => $entity
                );
                // group has a new compliance
                unset($this->groupComplianceRemoved[$group->getId()]);
            }
        }
    }



    /**
     *
     * @param UnitOfWork $uow
     */
    private function handleComplianceRemoved(UnitOfWork $uow)
    {
        foreach ($uow->getScheduledEntityDeletions() as $entity) {
            if (!$entity instanceof Compliance) {
                continue;
            }

            foreach ($entity->getGroups() as $group) {
                $this->groupComplianceRemoved[$group->getId()] = array(
                    "group" => $group,
                    "compliance" => $entity
                );
                $this->groupWinPhoneAppPolicyChanged[$group->getId()] = $group;
            }
        }
    }

    private function handleComplianceUpdated(UnitOfWork $uow)
    {
        foreach ($uow->getScheduledEntityUpdates() as $entity) {
            if (!$entity instanceof Compliance) {
                continue;
            }

            $changeSet = $uow->getEntityChangeSet($entity);
            $insertDiff = $entity->getGroups()->getInsertDiff();
            foreach ($insertDiff as $group) {
                $previousCompliance = $this->complianceEm->getComplianceForGroup($group);
                $this->groupComplianceChanged[$group->getId()] = array(
                    "group" => $group,
                    "previous" => $previousCompliance,
                    "new" => $entity
                );
                $this->groupWinPhoneAppPolicyChanged[$group->getId()] = $group;
                // group has a new compliance
                unset($this->groupComplianceRemoved[$group->getId()]);
            }

            if ($insertDiff) {
                /*
                 * actually we would only need to validate the devices
                 * of the added groups, if rules and actions have not
                 * changed, but lets do it for the complete compliance
                 * to avoid additional complexity.
                 */
                $this->compliancesToValidate[$entity->getId()] = $entity;
            }

            foreach ($entity->getGroups()->getDeleteDiff() as $group) {
                if (!isset($this->groupComplianceChanged[$group->getId()])) {
                    $this->groupComplianceRemoved[$group->getId()] = array(
                        "group" => $group,
                        "compliance" => $entity
                    );
                }
            }

            if (isset($changeSet['actions'])) {
                $this->complianceActionsChanged[$entity->getId()] = array(
                    "compliance" => $entity,
                    "previous" => $changeSet['actions'][0],
                    "new" => $changeSet['actions'][1]
                );
            }
        }
    }
}
