<?php
namespace Topalis\Compliance\Job;

use Psr\Log\LoggerInterface;
use Topalis\Compliance\Entity\ComplianceEntityManager;
use Topalis\Compliance\Exception\NoSuchComplianceException;
use Topalis\Kernel\Job\AbstractJobRunner;
use Topalis\Util\StringUtil;

/**
 * Job runner for jobs in a ComplianceControlQueue.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ComplianceControlJobRunner extends AbstractJobRunner
{
    /**
     * @var \Topalis\Compliance\Entity\Compliance
     */
    private $compliance;

    /**
     * @var boolean
     */
    private $complianceChanged;

    /**
     * @var ComplianceEntityManager
     */
    private $entityManager;

    /**
     * @var ComplianceControlWorkerInterface
     */
    private $worker;

    public function __construct(
        ComplianceControlQueueInterface $queue,
        ComplianceControlWorkerInterface $worker,
        ComplianceEntityManager $entityManager,
        LoggerInterface $logger
    )
    {
        parent::__construct($entityManager->getConnection(), $logger, $queue);
        $this->entityManager = $entityManager;
        $this->worker = $worker;
    }

    protected function preHandle(\Pheanstalk_Job $job)
    {
        $data = json_decode($job->getData(), true);
        $complianceId = $data['compliance'];
        $this->complianceChanged = StringUtil::strtobool($data['changed']);

        /* @var $compliance \Topalis\Compliance\Entity\Compliance */
        $this->compliance = $this->entityManager->find($complianceId);
        if (null === $this->compliance) {
            throw new NoSuchComplianceException(
                sprintf("A compliance with id '%s' doesn't exist", $complianceId));
        }

        $this->logger->debug("Control job for compliance " . $this->compliance->getId());
        $jobId = $job->getId();
        if ($jobId != $this->compliance->getJobId()) {
            // ensure that we have only one job for each compliance.
            // this happens if a new job has been put to the queue while
            // the previous job has been reserved.
            $this->logger->debug(
                sprintf("There's a new control job for compliance with id '%s'", $complianceId),
                array("old job id" => $jobId, "new job id" => $this->compliance->getJobId())
            );
            $this->queue->delete($job);
            // stop further handling of the job
            return false;
        }
    }

    protected function cleanup()
    {
        $this->entityManager->flush();
        $this->entityManager->clear();
    }

    protected function handle(\Pheanstalk_Job $job)
    {
        $job; // unused
        $this->worker->handleCompliance($this->compliance);
    }

    protected function postHandle(\Pheanstalk_Job $job)
    {
        if ($this->complianceChanged) {
            $this->queue->delete($job);
            // if there's a new job for the compliance we don't replace
            // it, otherwise we (propably) would increase the delay and
            // overwrite the "changed" flag.
            $this->queue->putCompliance($this->compliance, false, $this->compliance->getCheckInterval(), false);
        } else {
            $this->queue->release($job,
                \Pheanstalk_PheanstalkInterface::DEFAULT_PRIORITY, $this->compliance->getCheckInterval());
        }
    }

    protected function onException(\Pheanstalk_Job $job, \Exception $exception)
    {
        $exception; // unused

        if (null !== $this->compliance) {
            // lets try again later
            $this->queue->release($job,
                \Pheanstalk_PheanstalkInterface::DEFAULT_PRIORITY, $this->compliance->getCheckInterval());
        } else {
            $this->queue->delete($job);
        }
    }
}
