<?php
namespace Topalis\Compliance\Job;

use Doctrine\DBAL\LockMode;
use Psr\Log\LoggerInterface;
use Topalis\Compliance\Entity\Compliance;
use Topalis\Compliance\Entity\ComplianceEntityManager;
use Topalis\Kernel\Exception\Job\JobReservedException;
use Topalis\Kernel\Job\AbstractQueue;
use Topalis\Util\StringUtil;

/**
 * ComplianceControlQueue
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ComplianceControlQueue extends AbstractQueue implements ComplianceControlQueueInterface
{
    const JOB_TTR = 60;

    /**
     * @var \Topalis\Compliance\Entity\ComplianceEntityManager
     */
    private $em;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * Constructor.
     *
     * @param \Pheanstalk_PheanstalkInterface $pheanstalk
     * @param string $tube
     * @param \Topalis\Compliance\Entity\ComplianceEntityManager $em
     * @param \Psr\Log\LoggerInterface $logger
     */
    public function __construct(\Pheanstalk_PheanstalkInterface $pheanstalk, $tube, ComplianceEntityManager $em,
        LoggerInterface $logger)
    {
        parent::__construct($pheanstalk, $tube);
        $this->em = $em;
        $this->logger = $logger;
    }

    public function containsCompliance(Compliance $compliance)
    {
        return null !== $this->peekForCompliance($compliance);
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    protected function getLogger()
    {
        return $this->logger;
    }

    public function peekForCompliance(Compliance $compliance)
    {
        return $this->peek($compliance->getJobId());
    }

    public function putCompliance(Compliance $compliance, $complianceChanged = false, $delay = 0, $replace = true)
    {
        $this->em->beginTransaction();
        try {
            // we have to lock the compliance to ensure the job id is not
            // changed in between.
            $this->em->lock($compliance, LockMode::PESSIMISTIC_WRITE);
            $this->em->refresh($compliance);
            $jobId = $compliance->getJobId();
            $job = null !== $jobId ? $this->peek($jobId) : null;
            $doPutJob = true;

            if (null !== $job) {
                if ($replace) {
                    $this->getLogger()->info("Replace compliance control job", array(
                       "compliance" => $compliance->getId(),
                        "job" => $job->getId()
                    ));
                    try {
                        $this->delete($job);
                    } catch (JobReservedException $e) {
                        // the job is currently running so we delay the execution
                        // of the new job.
                        $this->getLogger()->info(
                            "The compliance control job is reserved so the new one will be delayed", array(
                                "compliance" => $compliance->getId(),
                                "job" => $job->getId()
                            )
                        );
                        $delay = max(array($delay, self::JOB_TTR * 2));
                    }
                } else {
                    // don't replace the existing job
                    $doPutJob = false;
                    $jobData = json_decode($job->getData());
                    if ($jobData['changed']) {
                        // kick the job if the changed flag is set
                        try {
                            $this->getLogger()->info("Kick compliance control job for changed compliance", array(
                                "compliance" => $compliance->getId(),
                                "job" => $job->getId()
                            ));
                            $this->pheanstalk->kickJob($job);
                        } catch (\Pheanstalk_Exception_ServerException $e) {
                            // ignore if the job could not be found; it propably
                            // has changed into the ready state in the mean time.
                            if (!$this->isNotFound($e)) {
                                throw $e;
                            }
                        }
                    }
                }
            }

            if ($doPutJob) {
                $this->getLogger()->info("Put compliance control job", array("compliance" => $compliance->getId()));
                $data = array(
                    "compliance" => $compliance->getId(),
                    "changed" => StringUtil::booltostr($complianceChanged),
                );
                $newJobId = $this->put($data,
                    \Pheanstalk_PheanstalkInterface::DEFAULT_PRIORITY, $delay, self::JOB_TTR);
                $compliance->setJobId($newJobId);
                $this->em->save($compliance);
            }

            $this->em->commit();
        } catch (\Exception $e) {
            $this->em->rollback();
            throw $e;
        }

        return $newJobId;
    }
}
