<?php
namespace Topalis\Compliance\Job;

use Psr\Log\LoggerInterface;
use Topalis\Compliance\ComplianceManagerInterface;
use Topalis\Compliance\Entity\Compliance;
use Topalis\Compliance\Entity\ComplianceEntityManager;
use Topalis\Device\Entity\Device;
use Topalis\Device\MDM\MdmFacadeInterface;

/**
 * ComplianceControlWorker
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class ComplianceControlWorker implements ComplianceControlWorkerInterface
{
    /**
     * @var \Topalis\Compliance\Entity\ComplianceEntityManager
     */
    private $em;

    /**
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    /**
     * @var \Topalis\Device\MDM\MdmFacadeInterface
     */
    private $mdmFacade;

    /**
     * @var ComplianceControlQueueInterface
     */
    private $queue;

    /**
     * @var ComplianceVerifyQueueInterface
     */
    private $verifyQueue;

    public function __construct(ComplianceControlQueueInterface $queue, ComplianceVerifyQueueInterface $verifyQueue,
        MdmFacadeInterface $mdmFacade, ComplianceEntityManager $em, LoggerInterface $logger)
    {
        $this->queue = $queue;
        $this->verifyQueue = $verifyQueue;
        $this->mdmFacade = $mdmFacade;
        $this->em = $em;
        $this->logger = $logger;
    }

    private function getActiveManagedDevices(Compliance $compliance)
    {
        $devices = array();
        /* @var $group \Topalis\ConfigGroup\Entity\ConfigGroup */
        foreach ($compliance->getGroups() as $group) {
            foreach ($group->getDevices() as $device) {
                if ($device->isActive() && $device->isManaged()) {
                    $devices[] = $device;
                }
            }
        }

        return $devices;
    }

    public function handleCompliance(Compliance $compliance)
    {
        $jobId = $compliance->getJobId();
        $job = $this->queue->peek($jobId);

        $i = 0;
        $devices = $this->getActiveManagedDevices($compliance);
        $this->logger->debug("Got " . count($devices) . " devices for compliance " . $compliance->getId());
        /* @var $device \Topalis\Device\Entity\Device */
        foreach ($devices as $device) {
            $this->mdmFacade->requestDeviceInfo($device);

            if (time() - $device->getLastComplianceCheck() > $compliance->getCheckInterval()
                && $device->getType()->isRolloutFinished($device)) {
                /*
                 * perform verification even though there was no information
                 * update to ensure that there's at least one check and delayed
                 * actions will be executed.
                 */
                $this->verifyQueue->putDevice($device);
            }

            $i++;
            if ($i % 100 === 0) {
                $this->em->refresh($compliance);
                if ($jobId != $compliance->getJobId()) {
                    $this->logger->info("Compliance '" . $compliance->getName()
                        . "' has been changed while consuming the control job",
                        array(
                            "compliance" => $compliance->getId(),
                            "job" => $jobId,
                        )
                    );
                    return;
                } else {
                    // ensure that the job is not automatically released
                    $this->queue->touch($job);
                }
            }
        }
    }
}
