<?php
namespace Topalis\Compliance\Rule;

use Topalis\App\Mdm\AppMdmAdminRegistry;
use Topalis\Compliance\Exception\InvalidConditionException;
use Topalis\Compliance\Rule\RuleValidatorInterface;
use Topalis\Device\DeviceManager;
use Topalis\Device\Entity\Device;
use Topalis\Device\MDM\MdmFacadeInterface;

/**
 * Validator for the TimeoutRule.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class TimeoutRuleValidator implements RuleValidatorInterface
{
    /**
     * @var \Topalis\Device\DeviceManager
     */
    private $deviceManager;

    /**
     * @var \Topalis\Device\MDM\MdmFacadeInterface
     */
    private $mdmFacade;

    public function __construct(DeviceManager $deviceManager, MdmFacadeInterface $mdmFacade)
    {
        $this->deviceManager = $deviceManager;
        $this->mdmFacade = $mdmFacade;
    }

    public function isDependsOnDeviceInformation()
    {
        return false;
    }

    public function validate(Device $device, $condition)
    {
        $timeout = (int) $condition;
        if ((string) $timeout !== $condition) {
            throw new InvalidConditionException(sprintf(
                "Invalid timeout for rule validator %s", __CLASS__
            ));
        }

        $violated = false;

        if ($device->isManaged()) {
            // for managed devices we use the time of the last MDM command
            // response
            $lastCommand = $this->mdmFacade->getLastExecutedCommand($device);
            if (null !== $lastCommand) {
                $lastAccess = $lastCommand->getResponseTime();
            } else {
                $lastAccess = false;
            }
        } else {
            $lastAccess = strtotime($this->deviceManager->getLastAccess($device));
        }

        // only check for devices with at least one access
        if ($lastAccess !== false && time() - $lastAccess > $condition) {
            $violated = true;
        }

        return new TimeoutRuleValidationResult($violated, $lastAccess, $timeout);
    }
}
