<?php
namespace Topalis\ConfigGroup\Form\Type;

use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolverInterface;
use Symfony\Component\Validator\Constraints\NotBlank;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\Form\FormError;
use Symfony\Component\Validator\Constraints\EqualTo;
use Topalis\ConfigGroup\Entity\ConfigGroupEntityManager;

/**
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ConfigGroupRemoveForm extends AbstractType
{
    /**
     * @var \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    private $em;

    /**
     * @var \Topalis\ConfigGroup\Entity\ConfigGroup
     */
    private $group;

    /**
     * @var boolean
     */
    private $lastDeviceGroup;

    /**
     * @var boolean
     */
    private $lastGroup;

    /**
     * @var boolean
     */
    private $validated = false;

    /**
     *
     * @param ConfigGroup $group The group going to be deleted.
     * @param boolean $lastDeviceGroup Flag indicating if it is there are no
     *     other groups the devices can be moved into.
     * @param boolean $lastGroup Flag indicating if it is the last group
     *     altogether.
     * @param ConfigGroupEntityManager $em
     */
    public function __construct(ConfigGroup $group, $lastDeviceGroup, $lastGroup, ConfigGroupEntityManager $em)
    {
        $this->group = $group;
        $this->lastDeviceGroup = $lastDeviceGroup;
        $this->lastGroup = $lastGroup;
        $this->em = $em;
    }

    public function buildForm(FormBuilderInterface $builder, array $options)
    {
        $options; // unused

        $hasUsers = !$this->group->getUsers()->isEmpty();
        $hasDevices = false;
        foreach ($this->group->getDevices() as $device) {
            if (!$device->isDeleted()) {
                $hasDevices = true;
            }
        }

        $choices = array(
            "change" => "admin.group.delete.changeGroup",
            "delete" => "Delete"
        );

        if ($hasUsers) {
            if ($this->lastGroup) {
                $builder->add("userAction", "hidden", array(
                    "data" => "delete",
                    "constraints" => new EqualTo(array("value" => "delete")),
                ));
            } else {
                $userGroupQb = $this->em->createQueryBuilder("cg")
                    ->where("cg != :cg")->orderBy("cg.name", "ASC")
                    ->setParameter("cg", $this->group);

                $builder
                    ->add("userAction", "choice", array(
                        "label" => "Action",
                        "choices" => $choices,
                        "constraints" => new NotBlank(),
                    ))
                    ->add("userConfigGroup", "entity", array(
                        "label" => "admin.group.delete.newGroup",
                        "class" => "TopalisConfigGroupBundle:ConfigGroup",
                        "query_builder" => $userGroupQb,
                        "property" => "name",
                        "required" => false,
                        "empty_value" => false,
                    ));
            }
        }

        if ($hasDevices) {
            if ($this->lastDeviceGroup) {
                $builder->add("deviceAction", "hidden", array(
                    "data" => "delete",
                    "constraints" => new EqualTo(array("value" => "delete")),
                ));
            } else {
                $deviceGroupQb = $this->em->createQueryBuilder("cg")
                    ->where("cg != :cg")->andWhere("cg.noExchange = :noExchange")
                    ->orderBy("cg.name", "ASC")
                    ->setParameter("cg", $this->group)
                    ->setParameter("noExchange", $this->group->isNoExchange());

                $builder
                    ->add("deviceAction", "choice", array(
                        "label" => "Action",
                        "choices" => $choices,
                        "constraints" => new NotBlank(),
                    ))
                    ->add("deviceConfigGroup", "entity", array(
                        "label" => "admin.group.delete.newGroup",
                        "class" => "TopalisConfigGroupBundle:ConfigGroup",
                        "query_builder" => $deviceGroupQb,
                        "property" => "name",
                        "required" => false,
                        "empty_value" => false,
                    ));
            }
        }
    }

    public function setDefaultOptions(OptionsResolverInterface $resolver)
    {
        $resolver->setDefaults(array(
            "validation_groups" => array(
                $this,
                "validate",
            ),
        ));
    }

    public function getName()
    {
        return "configGroupRemove";
    }

    public function validate(FormInterface $form)
    {
        if ($this->validated) {
            // this method is called multiple times; once is enough
            return;
        }

        $notBlank = new NotBlank();
        if ($form->has("userAction")) {
            $userAction = $form->get("userAction")->getData();
            if ($userAction != "delete") {
                $newGroupUser = $form->get("userConfigGroup")->getData();
                if ($newGroupUser === null) {
                    $form->get("userConfigGroup")->addError(new FormError($notBlank->message));
                }
            }
        }

        if ($form->has("deviceAction")) {
            $deviceAction = $form->get("deviceAction")->getData();
            if ($deviceAction != "delete") {
                $newGroupDevice = $form->get("deviceConfigGroup")->getData();
                if ($newGroupDevice === null) {
                    $form->get("deviceConfigGroup")->addError(new FormError($notBlank->message));
                }
            }
        }

        $this->validated = true;
    }
}
