<?php
namespace Topalis\Configuration;

use Symfony\Component\HttpFoundation\File\File;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\Process;
use Symfony\Component\Process\ProcessBuilder;
use Topalis\ActiveSyncFilter\Apache\ModConfig;
use Topalis\Configuration\Exception\VersionMismatchException;
use Topalis\Kernel\AbstractAuralisService;
use Topalis\Util\Exception\Process\ExecException;

class BackupManager extends AbstractAuralisService
{
    /**
     * Create and return an auralis data backup including a complete database
     * dump. Restoring dumps is possible if major and minor version has not
     * changed. Hence patch level updates of aurais MUST NOT change in a way
     * that restoring config dumps is not possible, anymore.
     *
     * @return string filename of the backup
     * @throws ExecException If executing mysqldump fails.
     * @throws \Exception On error.
     */
    public function createDump()
    {
        try {
            $tmpDir = null;
            while (null === $tmpDir || is_dir($tmpDir)) {
                $tmpDir = sys_get_temp_dir() . "/" . uniqid("auralisBackup");
            }
            $this->getFilesystem()->mkdir($tmpDir);

            // serialize metadata into metadata
            $metadata = array(
                "name"           => "auralis",
                "version"        => $this->getAuralisVersion(),
                "schema_version" => $this->getConfig()->getSchemaVersion(),
                "timestamp"      => $this->getTimeProvider()->getTime(),
            );
            $this->getFilesystem()->dumpFile($tmpDir . "/metadata", json_encode($metadata));
            $dbName = $this->getDB()->getName();
            $process = new Process(
                "mysqldump --ignore-table="
                . $dbName . ".accesslog --ignore-table="
                . $dbName . ".asf_log --ignore-table="
                . $dbName . ".asf_cache  --disable-keys "
                . $this->getDB()->getConfigAsCommandLine()
                . " > " . $tmpDir . "/auralis.sql"
            );
            $process
                ->setTimeout(3600)
                ->run()
            ;
            if (!$process->isSuccessful()) {
                throw new ProcessFailedException($process);
            }

            // serialize asf config into asfApacheMod.conf
            $asfApacheMod = $this->getASFApacheMod();
            $asfApacheModConfig = $asfApacheMod->getConfig();
            $asfModConfig = array(
                "enabled"    => $asfApacheMod->isEnabled(),
                "configLine" => (string) $asfApacheModConfig->toApacheConfigLine()
            );
            $this->getFilesystem()->dumpFile($tmpDir . "/asfApacheMod.conf", json_encode($asfModConfig));

            // archive tmpDir in zip file
            $zipFile = $tmpDir . ".zip";
            $builder = new ProcessBuilder(array(
                "zip",
                "-r",
                $zipFile,
                ".",
            ));
            $builder->setWorkingDirectory($tmpDir);
            $process = $builder->getProcess();
            $process->run();
            if (!$process->isSuccessful()) {
                throw new ProcessFailedException($process);
            }
            $this->getFilesystem()->remove($tmpDir);

            $this->backupUploads($zipFile);

            return $zipFile;
        } catch (\Exception $e) {
            $this->getFilesystem()->remove($tmpDir);
            if (null !== $zipFile) {
                $this->getFilesystem()->remove($zipFile);
            }
            throw $e;
        }
    }

    /**
     * Restore a dump of auralis data.
     * ATTENTATION: all backed up tables are dropped and created on restore.
     *
     * @param File $backup The backup file
     *
     * @throws VersionMismatchException If the version that created the backup
     *     differs from the current version.
     * @throws ExecException If executing mysql or the db upgrade script fails.
     * @throws \Exception On error.
     */
    public function restoreDump(File $backup)
    {
        ini_set("max_execution_time", 300);
        if ("application/zip" !== $backup->getMimeType()) {
            throw new VersionMismatchException($this->getAuralisVersion(), null);
        }

        try {
            $tmpDir = null;
            while (null === $tmpDir || is_dir($tmpDir)) {
                $tmpDir = sys_get_temp_dir() . "/" . uniqid("auralisBackupRestore");
            }

            // unzip
            $builder = new ProcessBuilder(array(
                "unzip",
                $backup->getRealPath(),
                // set target dir
                "-d",
                $tmpDir,
            ));
            $process = $builder->getProcess();
            $process->run();
            if (!$process->isSuccessful()) {
                throw new ProcessFailedException($process);
            }

            // extract metadata and compare versions
            $metadata = json_decode(file_get_contents($tmpDir . "/metadata"), true);
            if (version_compare(
                self::chopVersion($metadata['version'], 2), self::chopVersion($this->getAuralisVersion(), 2), "!=")) {
                throw new VersionMismatchException($this->getAuralisVersion(), $metadata['version']);
            }
            $msg = sprintf(
                "restore configuration of %s version %s created on %s",
                $metadata['name'],
                $metadata['version'],
                date(\DateTime::RFC1123, $metadata['timestamp'])
            );
            $this->getLogger()->info($msg);

            $asfModConfig = json_decode(file_get_contents($tmpDir . "/asfApacheMod.conf"), true);

            // stop ASF Server, because restoring SQL dump affects its tables
            $this->getFilterServer()->disable();

            $this->restoreSqlDump($tmpDir . "/auralis.sql");
            $this->restoreUploads($tmpDir);
            $this->restoreASFModConfig($asfModConfig);

            $this->getFilterServer()->enable();

            $this->getLogger()->info("restore configuration finished");
            $this->getFilesystem()->remove($tmpDir);
        } catch (\Exception $e) {
            $this->getFilesystem()->remove($tmpDir);
            throw $e;
        }

    }

    /**
     * @return \Topalis\ActiveSyncFilter\Apache\ASFApacheMod
     */
    private function getASFApacheMod()
    {
        return $this->container->get("asf.apache_mod");
    }

    /**
     * @return \Topalis\Apache\Apache
     */
    private function getApache()
    {
        return $this->container->get("apache");
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\ActiveSyncFilter\FilterServer
     */
    private function getFilterServer()
    {
        return $this->container->get("asf.filter_server");
    }

    /**
     * @return \Topalis\Configuration\Snmpd\SnmpdManager
     */
    private function getSnmpdManager()
    {
        return $this->container->get("config.snmpd.manager");
    }

    /**
     * @return \Topalis\Log\WebDavManager
     */
    private function getWebDavManager()
    {
        return $this->container->get("log.webdav.manager");
    }

    /**
     * @return \Symfony\Component\Filesystem\Filesystem
     */
    private function getFilesystem()
    {
        return $this->container->get("filesystem");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\Device\Entity\AccessLogEntityManager
     */
    private function getAccessLogEntityManager()
    {
        return $this->container->get("device.entity.manager.access_log");
    }

    /**
     * @return string symfony web directory
     */
    private function getWebDir()
    {
        return $this->container->getParameter("kernel.root_dir") . "/../web";
    }

    /**
     * Add files from uploads directory to backup zip files
     *
     * @param string $zipFile backup file
     *
     * @throws ProcessFailedException
     */
    private function backupUploads($zipFile)
    {
        $builder = new ProcessBuilder(array(
            "zip",
            "-r",
            $zipFile,
            "uploads",
        ));
        // change dir for relative paths in zip file
        $builder->setWorkingDirectory($this->getWebDir());
        $process = $builder->getProcess();
        $process->run();
        if (!$process->isSuccessful()) {
            throw new ProcessFailedException($process);
        }
    }

    /**
     * Restore backuped uploads to uploads directory
     *
     * @param string $tmpDir extracted backup
     */
    private function restoreUploads($tmpDir)
    {
        $uploadDir = $tmpDir . "/uploads";
        if (!is_dir($uploadDir)) {
            return;
        }
        // copy and override files from backup
        $this->getFilesystem()->mirror($uploadDir, $this->getWebDir() . "/uploads", null, array(
            "override" => true,
        ));
    }

    /**
     * Restores the configuration of the ActiveSync Filter Apache module.
     * Activates the ActiveSync Filter Server if the module is enabled.
     *
     * @param array $asfModConfig
     * @throws \Exception On error.
     */
    private function restoreASFModConfig(array $asfModConfig)
    {
        try {
            $asfApacheModConfig = ModConfig::fromApacheConfigLine(
                ApacheConfigLine::parse($asfModConfig['configLine'])
            );
            if ($asfModConfig['enabled']) {
                $this->getASFApacheMod()->enable($asfApacheModConfig);
            } else {
                $this->getASFApacheMod()->disable($asfApacheModConfig);
            }
        } catch (\Exception $e) {
            throw new \Exception("ActiveSync Filter: " . $e->getMessage(), $e->getCode(), $e);
        }
    }

    /**
     * @param string $data path to sql file
     *
     * @throws \Exception
     */
    private function restoreSqlDump($data)
    {
        $process = new Process("mysql " . $this->getDB()->getConfigAsCommandLine() . " < " . $data);
        $process->setTimeout(3600);
        $process->run();
        if (!$process->isSuccessful()) {
            throw new ProcessFailedException($process);
        }

        $this->deleteInvalidDBEntries();
        $this->restoreServicesConfig();
    }

    /**
     * Delete entries from accesslog table, where the device doesn't exist (anymore).
     */
    private function deleteInvalidDBEntries()
    {
        $devices = $this->getDeviceEntityManager()->createQueryBuilder("d")
            ->select("d.id")
            ->getQuery()
            ->getArrayResult()
        ;
        $ids = array();
        foreach ($devices as $device) {
            $ids[] = $device['id'];
        }
        $this->getAccessLogEntityManager()->createQueryBuilder("a")
            ->delete("TopalisDeviceBundle:AccessLog", "a")
            ->where("a.device NOT IN(:devices)")
            ->setParameter("devices", $ids)
            ->getQuery()
            ->execute()
        ;
    }

    /**
     * Restores the configuration of external services based on settings stored
     * in the database.
     *
     * @throws \Exception On error.
     */
    private function restoreServicesConfig()
    {
        $config = $this->getConfig();
        try {
            $this->getApache()->setHost($config->getHostname());
            $this->getApache()->setProxy($config->isExchangeSsl(), $config->getExchangeHost());
            $this->getApache()->setAllowFrom($config->getAllowFrom());
        } catch (\Exception $e) {
            throw new \Exception("Apache: " . $e->getMessage(), $e->getCode(), $e);
        }
        try {
            $this->getWebDavManager()->generateHtpasswd();
        } catch (\Exception $e) {
            throw new \Exception("WebDAV: " . $e->getMessage(), $e->getCode(), $e);
        }
        try {
            $this->getCertificateManager()->exportCACert();
            $this->getCertificateManager()->exportServerCert();
            $this->getCertificateManager()->enableCreateRevocationList();
        } catch (\Exception $e) {
            throw new \Exception("Certificate: " . $e->getMessage(), $e->getCode(), $e);
        }
        try {
            $this->getSnmpdManager()->setEnabled($this->getSnmpdManager()->isEnabled());
        } catch (\Exception $e) {
            throw new \Exception("SNMP: " . $e->getMessage(), $e->getCode(), $e);
        }
    }

    /**
     * @param string $version The version number to chop.
     * @param integer $sequences The number of sequences in the returned version
     *     number.
     * @return string The chopped version containing the specified number of
     *     sequences.
     */
    private static function chopVersion($version, $sequences)
    {
        $versionSplit = explode(".", $version);
        $sequences = min(array($sequences, count($versionSplit)));
        $rv = $versionSplit[0];
        for ($i = 1; $i < $sequences; $i++) {
            $rv .= "." . $versionSplit[$i];
        }

        return $rv;
    }
}
