<?php
namespace Topalis\Configuration\Controller\Section;

use DateTime;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Bundle\FrameworkBundle\Controller\Controller;
use Symfony\Component\Form\FormError;
use Symfony\Component\Form\FormInterface;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\HttpFoundation\Request;
use Topalis\App\Exception\VppApiException;
use Topalis\App\Exception\VppException;
use Topalis\App\Ios\VppApi;
use Topalis\App\Ios\VppToken;
use Topalis\Configuration\Form\AppConfigurationForm;

/**
 * AppConfigController
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 *
 * @Route("/admin/config/app")
 */
class AppConfigController extends Controller
{
    /**
     * @param Request $request
     *
     * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
     *
     * @Route("", name="admin_config_app")
     * @Template()
     */
    public function indexAction(Request $request)
    {
        $tokenManager = $this->getVppTokenManager();
        $token = $tokenManager->getToken();
        $validDiff = null;
        $validClass = "success";
        $remainingDays = 0;
        if (null !== $token && $token->isValid()) {
            $remainingDays = $token->getRemainingDays();
            $validDiff = $this->get("translator")->transChoice("date.diff.day", $remainingDays,
                array("%days%" => $remainingDays));
            $important = 30 ; // 1 month
            $warning = 30 * 3; // 3 months
            if ($remainingDays < $important) {
                $validClass = "important";
            } else if ($remainingDays < $warning) {
                $validClass = "warning";
            }
        }

        $config = $this->container->get("config.auralis");
        $form = $this->createForm(new AppConfigurationForm(), array(
            "itunesStore" => strtoupper($config->getItunesAppStore()),
            "googlePlayStoreLang" => $config->getGooglePlayStoreLang(),
            "winPhoneStoreLocale" => $config->getWinPhoneStoreLocale(),
            "appInstallAttempts" => $config->getAppInstallAttempts(),
        ));

        $form->handleRequest($request);
        if ($form->isValid()) {
            $data = $form->getData();
            /** @var UploadedFile $tokenFile */
            $tokenFile = $data['vppToken'];
            if (null !== $tokenFile) {
                $rawToken = file_get_contents($tokenFile->getPathname());
                $this->verifyVppToken($rawToken, $form->get("vppToken"));
            }
        }

        if ($form->isValid() && null !== $tokenFile) {
            try {
                $tokenManager->setToken(new VppToken($rawToken));
            } catch (VppException $e) {
                $form->get("vppToken")->addError(new FormError($e->getMessage()));
            }
        }

        if ($form->isValid()) {
            $config->setItunesAppStore(strtolower($data['itunesStore']));
            $config->setGooglePlayStoreLang($data['googlePlayStoreLang']);
            $config->setWinPhoneStoreLocale($data['winPhoneStoreLocale']);
            $config->setAppInstallAttempts($data['appInstallAttempts']);

            $this->get("session")->getFlashBag()->add("success",
                $this->get("translator")->trans("admin.global.saveSuccess")
            );
            return $this->redirect($this->generateUrl("admin_config_app"));
        }

        return array(
            "sections"       => $this->container->getParameter("config.sections"),
            "active_section" => "app",
            "token"          => $token,
            "form"           => $form->createView(),
            "validDiff"      => $validDiff,
            "validClass"     => $validClass,
            "remainingDays"  => $remainingDays,
        );
    }

    /**
     * @param Request $request
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     *
     * @Route("/vpp/token/remove", name="admin_config_app_vpp_token_remove")
     */
    public function removeVppTokenAction(Request $request)
    {
        try {
            $this->getVppTokenManager()->removeToken();
            $this->get("session")->getFlashBag()->add("success",
                $this->get("translator")->trans("admin.global.saveSuccess")
            );
        } catch (\Exception $e) {
            $this->get("logger")->error("Error removing VPP token", array("exception" => $e));
            $this->get("session")->getFlashBag()->add("error",
                $this->get("translator")->trans("admin.global.app.vppToken.removeError",
                    array("%errmsg%" => $e->getMessage()))
            );
        }

        return $this->redirect($this->generateUrl("admin_config_app"));
    }

    /**
     * @return \Topalis\App\Ios\VppTokenManagerInterface
     */
    private function getVppTokenManager()
    {
        return $this->get("app.ios.vpp_token_manager");
    }

    private function verifyVppToken($token, FormInterface $form)
    {
        $tokenData = json_decode(base64_decode($token), true);
        if (empty($tokenData)) {
            $form->addError(new FormError("app.invalidVppToken"));
        } else {
            try {
                if (!$this->get("app.ios.vpp_api")->isTokenValid($token)) {
                    $form->addError(new FormError("app.invalidVppToken"));
                }
            } catch (VppApiException $e) {
                $form->addError(new FormError($e->getReason()));
            } catch (\Exception $e) {
                $form->addError(new FormError($e->getMessage()));
            }
        }
    }
}
