<?php
namespace Topalis\Configuration\Controller\Section;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Symfony\Component\HttpFoundation\File\UploadedFile;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Topalis\Certificate\Form\PushCertForm;
use Topalis\Certificate\Form\PushCertUploadForm;
use Topalis\Certificate\PushCertificate;
use Topalis\Certificate\Util\CertUtil;
use Topalis\Configuration\Form\GcmConfigurationForm;
use Topalis\Kernel\Controller;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @Route("/admin/config/push")
 */
class PushConfigController extends Controller
{
    /**
     * @Route("", name="admin_config_push")
     * @Template()
     * @param Request $request
     *
     * @return array
     */
    public function indexAction(Request $request)
    {
        $config = $this->getConfig();
        $created = false;
        $x509 = array();
        $remainingTime = null;
        $validDiff = null;
        $validClass = "success";
        $csr = null;
        if (null !== $cert = $config->getApnsCert()) {
            $created = true;
            $x509 = openssl_x509_parse($cert);
            $remainingTime = $x509['validTo_time_t'] - $this->getTimeProvider()->getTime();
            $remainingDays = max(0, intval(floor($remainingTime / 86400)) + 1);
            $validDiff = $this->getTranslator()->transChoice("date.diff.day", $remainingDays,
                array("%days%" => $remainingDays));
            $important = 30 * 86400; // 1 month
            $warning = 30 * 3 * 86400; // 3 months
            if ($remainingTime < $important) {
                $validClass = "important";
            } else if ($remainingTime < $warning) {
                $validClass = "warning";
            }

            $csr = openssl_csr_get_subject($config->getApnsCsr());
            $extra = preg_split("/\/([^\/]+)=/", $csr['name'], -1, PREG_SPLIT_DELIM_CAPTURE);
            if (empty($extra[0])) {
                // since 2.2.1 firstname and lastname are seperated
                // street and houseNo as well, but not used, so we don't need
                // to take extra action
                // $csr['name'] starts with "/firstname=" so $extra[0] is empty
                // firstname lastname
                $csr['name'] = $extra[2] . " " . $extra[4];
            } else {
                $csr['name'] = $extra[0];
            }

            $count = count($extra);
            for ($i = 2; $i < $count; $i += 2) {
                $csr[$extra[$i - 1]] = $extra[$i];
            }
        }

        $step = 1;
        // determine wizzard step
        if (!$created) {
            if (null !== $config->getApnsCsr()) {
                $step = 2;
            }
            if (2 == $step && $config->isApnsCsrExported()) {
                $step = 3;
            }
        }


        try {
            $form = $this->createForm(new PushCertUploadForm());
            $form->handleRequest($request);
            if ($form->isValid()) {
                $apnsCertificateExists = $this->getLicense()->existsApnsCertificate();
                /** @var $file UploadedFile */
                $file = $form->get("file")->getData();
                $content = file_get_contents($file->getRealPath());
                $this->addFlash("success", $this->tr("admin.cert.push.message.uploadSuccess"));
                $this->getConfig()->setApnsCert($content);
                if (!$apnsCertificateExists) {
                    // we have an APNS certificate for the first time
                    // enable port ${auralis_primary_port}
                    $this->getApache()->setPortEnabled('${auralis_primary_port}', true);
                    // enable features
                    $this->getLicenseManager()->updateFeatureStatus();
                }

                return $this->redirect();
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("Error uploading APNS certificate", array("exception" => $e));
            $msg = $this->tr("admin.cert.push.error.upload", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
            $this->redirect();
        }

        return array(
            "form"           => $form->createView(),
            "created"        => $created,
            "step"           => $step,
            "x509"           => $x509,
            "remainingTime"  => $remainingTime,
            "validDiff"      => $validDiff,
            "validClass"     => $validClass,
            "mailConfigured" => $this->getMailer()->isConfigured(),
            "csr"            => $csr,
            "sections"       => $this->container->getParameter("config.sections"),
            "sub_section"    => "apns",
            "active_section" => "push",
        );
    }

    /**
     * @Route("/apns/download", name="admin_config_push_apns_download")
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    public function downloadAction()
    {
        $config = $this->getConfig();
        $csr = $config->getApnsCsr();
        if (null == $csr) {
            return $this->redirect($this->generateUrl("admin_config_push"));
        }
        $csr = CertUtil::pem2der($csr);

        /**
         * @see http://symfony.com/doc/master/components/http_foundation/introduction.html#downloading-files
         */
        $response = new Response($csr);
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            "auralis_apple_push.csr"
        );
        $response->headers->set("Content-Disposition", $disposition);
        $response->headers->set("Content-Type", "application/pkcs10");

        $config->setApnsCsrExported(true);

        return $response;
    }

    /**
     * @Route("/apns/send", name="admin_config_push_apsn_send")
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    public function sendAction()
    {
        try {
            $config = $this->getConfig();

            if (null !== $config->getApnsCert()) {
                return $this->redirect($this->generateUrl("admin_config_push"));
            }

            $csr = $config->getApnsCsr();
            if (null === $csr) {
                return $this->redirect($this->generateUrl("admin_config_push"));
            }
            $csr = CertUtil::pem2der($csr);

            $mailer = $this->getMailer();
            $attachment = \Swift_Attachment::newInstance($csr, "auralis_apple_push.csr", "application/octet-stream");
            $message = $mailer->createAdminMessage($this->tr("admin.cert.push.title"));
            $message->attach($attachment);
            $text = $mailer->prepareAdminText($this->tr("admin.cert.push.mailText"));
            $message->setBody($text);

            $mailer->sendMessage($message);
            $config->setApnsCsrExported(true);
            $this->addFlash("success", $this->tr("admin.cert.push.message.mailSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error sending APNS certificate", array("exception" => $e));
            $msg = $this->tr("admin.cert.push.error.send", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_config_push"));
    }

    /**
     * @Route("/apns/create", name="admin_config_push_apns_create")
     * @Template()
     *
     * @param Request $request The request.
     * @return \Symfony\Component\HttpFoundation\RedirectResponse|\Symfony\Component\HttpFoundation\Response
     */
    public function createAction(Request $request)
    {
        if (null !== $this->getConfig()->getApnsCert()) {
            return $this->redirect($this->generateUrl("admin_config_push"));
        }

        try {
            $form = $this->createForm(new PushCertForm());

            $caCert = $this->getCertificateManager()->getCACertificate();

            // use values of CA cert as base
            $certificate = new PushCertificate($this->getLicense()->getAuralisId(), $caCert->getCertificate());
            $form->setData($certificate);
            $form->handleRequest($request);

            if ($form->isValid()) {
                $config = $this->getConfig();
                $csr = $certificate->exportCsr();
                $config->setApnsCsr($csr);
                $privateKey = $certificate->exportPrivateKey();
                $config->setApnsPrivKey($privateKey);
                $this->addFlash("success", $this->tr("admin.cert.push.message.createSuccess"));

                return $this->redirect($this->generateUrl("admin_config_push"));
            }

            return array(
                "form"           => $form->createView(),
                "sections"       => $this->container->getParameter("config.sections"),
                "sub_section"    => "apns",
                "active_section" => "push",
            );
        } catch (\Exception $e) {
            $this->getLogger()->error("Error creating APNS certificate", array("exception" => $e));
            $msg = $this->tr("admin.cert.push.error.create", array("%errmsg%" => $e->getMessage()));
            $this->addError($msg);
            $this->redirect($this->generateUrl("admin_config_push"));
        }
    }

    public function uploadAction()
    {
        if (null !== $this->getConfig()->getApnsCert()) {
            return $this->redirect($this->generateUrl("admin_config_push"));
        }
    }

    /**
     * @Route("/gcm", name="admin_config_push_gcm")
     * @Template()
     *
     * @param Request $request The request.
     * @return array
     */
    public function gcmAction(Request $request)
    {
        $config = $this->getConfig();
        $form = $this->createForm(new GcmConfigurationForm(), $config->getGcmConfig());
        $form->handleRequest($request);
        if ($form->isValid()) {
            $config->setGcmConfig($form->getData());

            $this->addFlash("success", $this->tr("admin.global.saveSuccess"));

            return $this->redirect($this->generateUrl("admin_config_push_gcm"));
        }
        return array(
            "form"           => $form->createView(),
            "sections"       => $this->container->getParameter("config.sections"),
            "sub_section"    => "gcm",
            "active_section" => "push",
        );
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->container->get("mailer");
    }
}
