<?php
namespace Topalis\Configuration;

class Ini implements Configuration
{
    private $properties = array();

    public static function load($filename, $processSections = false)
    {
        $ini = new Ini();
        // Silencing errors required to get no error if "#" is used to mark a
        // comment.
        // @codingStandardsIgnoreStart
        $ini->properties = @parse_ini_file($filename, $processSections, INI_SCANNER_RAW);
        // @codingStandardsIgnoreEnd
        if (false === $ini->properties) {
            throw new \RuntimeException(sprintf("Can't parse ini %s", $filename));
        }
        return $ini;
    }

    public static function parse($string, $processSections = false, $allowSemicolons = false)
    {
        $ini = new Ini();
        if ($allowSemicolons) {
            // workaround to allow ";" in values
            $string = str_replace(";", chr(0x1a), $string);
        }
        // Silencing errors required to get no error if "#" is used to mark a
        // comment.
        // @codingStandardsIgnoreStart
        $ini->properties = @parse_ini_string($string, $processSections, INI_SCANNER_RAW);
        // @codingStandardsIgnoreEnd
        if (false === $ini->properties) {
            throw new \RuntimeException(sprintf("Can't parse ini string"));
        }
        if ($allowSemicolons) {
            foreach ($ini->properties as &$value) {
                // workaround to allow ";" in values
                $value = str_replace(chr(0x1a), ";", $value);
            }
        }
        return $ini;
    }

    public function save($filename, $processSections = false)
    {
        return $this->writeIniFile($this->properties, $filename, $processSections);
    }

    public function getProperties()
    {
        $rv = array();
        foreach ($this->properties as $key => $value) {
            if (is_array($value)) {
                foreach ($value as $subkey => $subvalue) {
                    $rv[$key . '.' . $subkey] = $subvalue;
                }
            } else {
                $rv[$key] = $value;
            }
        }
        return $rv;
    }

    public function getProperty($key, $default = null)
    {
        $pos = strpos($key, ".");
        if ($pos == false) {
            return $this->containsProperty($key) ? $this->properties[$key] : $default;
        } else {
            $section = substr($key, 0, $pos);
            $key = substr($key, $pos + 1);
            return $this->getSectionProperty($section, $key, $default);
        }
    }

    public function getSectionProperty($section, $key, $default = null)
    {
        return $this->containsSectionProperty($section, $key) ? $this->properties[$section][$key] : $default;
    }

    public function clear()
    {
        $this->properties = array();
    }

    public function containsProperty($key)
    {
        $pos = strpos($key, ".");
        if ($pos == false) {
            return array_key_exists($key, $this->properties) && !is_array($this->properties[$key]);
        } else {
            $section = substr($key, 0, $pos);
            $key = substr($key, $pos + 1);
            return $this->containsSectionProperty($section, $key);
        }
    }

    public function containsSection($section)
    {
        return array_key_exists($section, $this->properties) && is_array($this->properties[$section]);
    }

    public function containsSectionProperty($section, $key)
    {
        if (!$this->containsSection($section)) {
            return false;
        }

        $sectionArr = $this->properties[$section];
        return array_key_exists($key, $sectionArr);
    }

    public function setProperty($key, $value)
    {
        if ($this->getProperty($key) == $value) {
            return false;
        }
        $pos = strpos($key, ".");
        if ($pos == false) {
            $this->properties[$key] = $value;
        } else {
            $section = substr($key, 0, $pos);
            $key = substr($key, $pos + 1);
            $this->setSectionProperty($section, $key, $value);
        }
        return true;
    }

    public function setSectionProperty($section, $key, $value)
    {
        if ($this->getSectionProperty($section, $key) == $value) {
            return false;
        }
        $this->properties[$section][$key] = $value;
        return true;
    }

    public function removeProperty($key)
    {
        if ($this->containsProperty($key)) {
            unset($this->properties[$key]);
            return true;
        }

        return false;
    }

    public function removeSection($section)
    {
        if ($this->containsSection($section)) {
            unset($this->properties[$section]);
            return true;
        }

        return false;
    }

    public function removeSectionProperty($section, $key)
    {
        if ($this->containsSectionProperty($section, $key)) {
            unset($this->properties[$section][$key]);
            return true;
        }

        return false;
    }

    /**
     * from http://stackoverflow.com/questions/1268378/create-ini-file-write-values-in-php
     * @param array $assocArr
     * @param string $path file name to write into.
     * @param boolean $hasSections
     */
    private function writeIniFile(array $assocArr, $path, $hasSections = false)
    {
        if ($hasSections) {
            $content = "";
            foreach ($assocArr as $key => $elem) {
                $content .= "[".$key."]\n";
                $content .= $this->toString($elem);
            }
        } else {
            $content = $this->toString($assocArr);
        }

        if (!$handle = fopen($path, 'w')) {
            return false;
        }
        if (!fwrite($handle, $content)) {
            return false;
        }
        fclose($handle);
        return true;
    }

    private function toString($assocArr)
    {
        $content = "";
        foreach ($assocArr as $key => $elem) {
            if (is_array($elem)) {
                $elemCount = count($elem);
                for ($i = 0; $i < $elemCount; $i++) {
                    $content .= $key . "[] = \"" . $elem[$i] . "\"\n";
                }
            } else if ($elem == "") {
                $content .= $key. " = \n";
            } else {
                $content .= $key. " = \"" . $elem . "\"\n";
            }
        }

        return $content;
    }
}
