<?php
namespace Topalis\Configuration\Snmpd;

use Topalis\Kernel\AbstractAuralisService;
use Topalis\Util\Process;
use Topalis\Util\FileSystemUtil;

/**
 * Helper class for managing snmpd setup.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class SnmpdManager extends AbstractAuralisService
{
    private $configTemplateFilename;
    private $helperCommand = "/usr/sbin/aursnmpd";
    private $passPersistCommand = "/usr/sbin/auralis-snmpd";

    public function __construct()
    {
        $this->configTemplateFilename = __DIR__ . "/../Resources/snmpd/snmpd.template";
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * Disables snmpd.
     *
     * @throws \Topalis\Util\Exception\Process\ExecException If executing the
     *     helper command fails.
     */
    public function disable()
    {
        $cmd = $this->helperCommand . " disable";
        Process::execThrow($cmd);
        $this->getConfig()->setSnmpEnabled(false);
    }
    /**
     * Enables snmpd.
     *
     * @throws \Topalis\Util\Exception\Process\ExecException If executing the
     *     helper command fails.
     */
    public function enable()
    {
        $community = $this->getConfig()->getSnmpCommunity();
        if (strlen($community) == 0) {
            return;
        }

        $serverCert = $this->getCertificateManager()->getServerCert();
        $snmpdConfig = file_get_contents($this->configTemplateFilename);
        $snmpdConfig = str_replace("__ACL__", $this->generateAcl(), $snmpdConfig);
        $snmpdConfig = str_replace("__SYSCONTACT__", $serverCert->getEmail(), $snmpdConfig);
        $snmpdConfig = str_replace("__SYSLOCATION__", $serverCert->getLocation(), $snmpdConfig);
        $snmpdConfig = str_replace("__PASS_PERSIST_COMMAND__", $this->passPersistCommand, $snmpdConfig);
        try {
            $tmpConfgFilename = tempnam(sys_get_temp_dir(), "auralisSnmpd");
            file_put_contents($tmpConfgFilename, $snmpdConfig);
            $cmd = $this->helperCommand . " enable " . $tmpConfgFilename;
            Process::execThrow($cmd);
            $this->getConfig()->setSnmpEnabled(true);
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpConfgFilename);
            throw $e;
        }
    }

    /**
     * @return string The SNMP community.
     */
    public function getCommunity()
    {
        return $this->getConfig()->getSnmpCommunity();
    }

    /**
     * Sets the SNMP community.
     * NOTE: You have to (re)enable SNMPD to apply changes!
     *
     * @param string $community The community to set.
     */
    public function setCommunity($community)
    {
        $this->getConfig()->setSnmpCommunity($community);
    }

    /**
     * Sets the command used to control SNMPD.
     *
     * @param string $helperCommand The command to set.
     */
    public function setHelperCommand($helperCommand)
    {
        $this->helperCommand = $helperCommand;
    }

    /**
     * Sets the command used for the pass_persist directive in the SNMPD config.
     *
     * @param string $passPersistCommand The command to set.
     */
    public function setPassPersistCommand($passPersistCommand)
    {
        $this->passPersistCommand = $passPersistCommand;
    }

    /**
     * @return boolean true if SNMPD is enabled, false otherwise.
     */
    public function isEnabled()
    {
        return $this->getConfig()->isSnmpEnabled();
    }

    /**
     * Enables or disables SNMPD.
     *
     * @param boolean $enabled true to enable, false to disable SNMPD.
     */
    public function setEnabled($enabled)
    {
        if ($enabled) {
            $this->enable();
        } else {
            $this->disable();
        }
    }

    private function generateAcl()
    {
        $acl = "";
        $community = $this->getCommunity();
        $allowFrom = $this->getConfig()->getAllowFrom();
        $allowFromSplit = explode(",", $allowFrom);
        $allowFromSplit[] = "127.0.0.1/32";
        foreach ($allowFromSplit as $from) {
            $acl .= sprintf("com2sec auralis-snmp   %s  %s\n", $from, $community);
        }

        return $acl;
    }
}
