<?php
namespace Topalis\Database\Migrations;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;
use Doctrine\DBAL\Migrations\AbstractMigration;

/**
 * Abstract class for auralis database migrations.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
abstract class AbstractAuralisMigration extends AbstractMigration implements ContainerAwareInterface
{
    /**
     * @var ContainerInterface
     */
    protected $container;

    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    protected function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @return \Doctrine\ORM\EntityManager
     */
    protected function getEntityManager()
    {
        return $this->container->get("doctrine.orm.entity_manager");
    }

    /**
     * @return \Topalis\ActiveSyncFilter\FilterServer
     */
    protected function getFilterServer()
    {
        return $this->container->get("asf.filter_server");
    }

    /**
     * @return \Symfony\Bridge\Doctrine\RegistryInterface
     */
    protected function getDoctrine()
    {
        return $this->container->get("doctrine");
    }

    /**
     * Get a database escaped string
     *
     * @param string $string The string to be escaped
     * @return string The escaped string.
     */
    public function getEscaped($string)
    {
        $ret = $this->connection->quote($string);
        // remove quotes
        $ret = substr($ret, 1, -1);
        return $ret;
    }

    /**
     * Disable all doctrine event listeners.
     */
    protected function disableEventListeners()
    {
        $eventManager = $this->connection->getEventManager();
        $listeners = $eventManager->getListeners();
        foreach ($listeners as $event => $listenersHash) {
            foreach ($listenersHash as $listener) {
                $eventManager->removeEventListener($event, $listener);
            }
        }
    }

    /**
     * Disable all doctrine filters.
     */
    protected function disableFilters()
    {
        foreach ($this->getDoctrine()->getManagers() as $em) {
            $filterCollection = $em->getFilters();
            foreach (array_keys($filterCollection->getEnabledFilters()) as $filterName) {
                $filterCollection->disable($filterName);
            }
        }
    }

    /**
     * Checks if an index exist on a table.
     *
     * @param string $table Table name.
     * @param string $name Index name.
     * @return boolean true if the specified exists on the specified table,
     *     false otherwise.
     */
    protected function existsIndex($table, $name)
    {
        $this->abortIf($this->connection->getDatabasePlatform()->getName() != "mysql",
            "Index existance check can only be executed on 'mysql'.");

        $rv = false;
        $stmt = $this->connection->executeQuery(
            "SHOW index FROM " . $this->getEscaped($table) . " WHERE Key_name = :name",
            array("name" => $name)
        );
        if ($stmt->fetch(\PDO::FETCH_NUM)) {
            $rv = true;
        }
        $stmt->closeCursor();

        return $rv;
    }
}
