<?php
namespace Topalis\Device\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Topalis\Device\Entity\AccessLog;

/**
 * This command logs the access of a device in the database.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class LogDeviceAccessCommand extends ContainerAwareCommand
{
    /**
     * @return \Topalis\Device\Entity\AccessLogEntityManager
     */
    private function getAccessLogEntityManager()
    {
        return $this->getContainer()->get("device.entity.manager.access_log");
    }

    /**
     * @return \Topalis\Certificate\Entity\ClientCertificateEntityManager
     */
    private function getCertificateEntityManager()
    {
        return $this->getContainer()->get("certificate.entity.manager.certificate");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->getContainer()->get("device.entity.manager.device");
    }

    /**
     * @return \Symfony\Bridge\Doctrine\RegistryInterface
     */
    private function getDoctrine()
    {
        return $this->getContainer()->get("doctrine");
    }

    protected function configure()
    {
        $this
        ->setName("device:log_access")
        ->setDescription("Logs a device access with data read from STDIN.")
        ->setHelp("Logs a device access with data read from STDIN.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        try {
            $this->getDoctrine()->getConnection()->setAutoCommit(true);
            // avoid overhead
            $this->disableDoctrineFilters();
            $this->removeDoctrineListeners();
            openlog("auralis-logger", LOG_PID, LOG_DAEMON);
            return $this->logAccess();
        } catch (\Exception $e) {
            if ($output instanceof \Symfony\Component\Console\Output\ConsoleOutput) {
                $output = $output->getErrorOutput();
            }
            $output->writeln("Error logging device access:");
            $output->writeln((string) $e);
            return -1;
        }
    }

    private function disableDoctrineFilters()
    {
        $managers = $this->getDoctrine()->getManagers();
        foreach ($managers as $em) {
            $filterCollection = $em->getFilters();
            $enabledFilters = $filterCollection->getEnabledFilters();
            foreach (array_keys($enabledFilters) as $filterName) {
                $filterCollection->disable($filterName);
            }
        }
    }

    /**
     * Checks if a DB connection is established. If not, a new connection will
     * be opened.
     */
    private function ensureDBConnection()
    {
        try {
            $connection = $this->getDoctrine()->getConnection();
            $connection->query("select 1;")->fetch();
        } catch (\Exception $e) {
            $connection->close();
            $connection->connect();
        }
    }

    private function logAccess()
    {
        while (($line = fgets(STDIN)) !== false) {
            $a = explode(" ", $line);
            if (5 === count($a)) {
                try {
                    $this->ensureDBConnection();
                    $accessLog = new AccessLog();
                    $timestamp = intval($a[0]);
                    $dt = \DateTime::createFromFormat("U", $timestamp);
                    $dt->setTimezone($this->getContainer()->get("kernel.time_provider")->getTimezone());
                    $accessLog->setTstamp($dt);
                    $accessLog->setIp($a[1]);
                    $certId = hexdec($a[2]);
                    $cert = $this->getCertificateEntityManager()->find($certId);
                    if ($cert === null) {
                        $this->logError(
                            sprintf("Could not store access data: Invalid certificate id '%s'", $certId));
                        return;
                    }
                    $device = $cert->getDevice();
                    $accessLog->setCert($cert);
                    $accessLog->setStatus(intval($a[3]));
                    $accessLog->setTraffic(intval($a[4]));
                    $accessLog->setDevice($device);
                    $this->getAccessLogEntityManager()->save($accessLog);

                    // perform update without transaction to avoid dealocks
                    $this->getDeviceEntityManager()->createQueryBuilder("d")
                        ->update()->set("d.lastAccess", ":lastAccess")
                        ->where("d = :device")
                        ->setParameter("lastAccess", $timestamp)
                        ->setParameter("device", $device)
                        ->getQuery()->execute()
                    ;
                } catch (\Exception $e) {
                    $this->logError("Could not store access data: " . $e->getMessage());
                }
            }
        }
        if (!feof(STDIN)) {
            $this->logError("Unexpected fgets() failure, exiting");
            return -1;
        }
    }

    private function logError($message)
    {
        syslog(LOG_ERR, $message);
    }

    private function removeDoctrineListeners()
    {
        $managers = $this->getDoctrine()->getManagers();
        foreach ($managers as $em) {
            $eventManager = $em->getEventManager();
            $listeners = $eventManager->getListeners();
            foreach ($listeners as $event => $listenersHash) {
                foreach ($listenersHash as $listener) {
                    $eventManager->removeEventListener($event, $listener);
                }
            }
        }
    }
}
