<?php
namespace Topalis\Device;

use Topalis\Device\Entity\DeviceEntityManager;
use Topalis\Kernel\AbstractAuralisService;
use Topalis\Device\Entity\Device;
use Topalis\Kernel\Exception\Database\DatabaseException;
use Topalis\Kernel\Exception\Http\NotFoundException;
use Doctrine\ORM\Query;
use Topalis\Util\DateUtil;
use Topalis\Device\Entity\AccessStatDaily;
use Topalis\Util\Interpolator;
use Doctrine\ORM\AbstractQuery;

/**
 * Class containing methods to manage devices.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class DeviceManager extends AbstractAuralisService
{
    /**
     * Deletes the specified device.
     *
     * @param int $id The id of the device to delete.
     * @throws DatabaseException On database error.
     * @throws \Exception On error.
     */
    public function deleteDevice($id)
    {
        $this->getLogger()->info("Deleting device '" . $id . "'");
        $device = $this->getDevice($id);
        $this->getEntityManager()->delete($device);
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\Device\MDM\MdmFacadeInterface
     */
    private function getMdmFacade()
    {
        return $this->container->get("device.mdm.facade");
    }

    /**
     * Returns the device with the specified id.
     *
     * @param int $id The id of the device to return.
     * @throws DatabaseException On database error.
     * @throws NotFoundException If no device with the specified id exists.
     * @throws \Exception On error.
     * @return Device The device with the specified id.
     */
    public function getDevice($id)
    {
        $id = intval($id);
        $device = $this->getEntityManager()->findById($id);
        if ($device === null) {
            throw new NotFoundException("device does not exist");
        }
        return $device;
    }

    /**
     * Returns the devices using the specified ordering. The list of devices may
     * be reduced to those of a specified user.
     *
     * @param string $sortOrder The sort field.
     * @param string $sortDir The sort direction (ASC or DESC).
     * @param array $condition Condition that returned devices must match.
     * @return Device[] Array of devices.
     * @throws DatabaseException On database error.
     * @throws \Exception On error.
     */
    public function getDevices($sortOrder = "d.deviceId", $sortDir = "ASC", array $condition = null)
    {
        $query = $this->getEntityManager()->createQueryBuilder("d")
            ->select("d, u, c, q")
            ->leftJoin("d.user", "u")
            ->leftJoin("d.configGroup", "c")
            ->leftJoin("d.mdmQueue", "q")
            ->orderBy($sortOrder, $sortDir)
        ;
        $i = 0;
        foreach ($condition as $key => $value) {
            $query->andWhere(sprintf("%s = :param%d", $key, $i));
            $query->setParameter("param".$i, $value);
            $i++;
        }
        $result = $query->getQuery()->getResult();

        return $result;
    }

    /**
     * @return \DateTime
     */
    public function getLogStartDate()
    {
        $statDate = $this->container->get("device.entity.manager.access_stat_daily")->createQueryBuilder("s")
            ->select("MIN(s.date)")->getQuery()->getSingleScalarResult();
        $logDate = $this->container->get("device.entity.manager.access_log")->createQueryBuilder("l")
            ->select("MIN(l.tstamp)")->getQuery()->getSingleScalarResult();

        $statDate = $statDate === null ? $this->getTimeProvider()->getTime() : strtotime($statDate);
        $logDate = $logDate === null ? $this->getTimeProvider()->getTime() : strtotime($logDate);
        $minDate = $statDate < $logDate ? $statDate : $logDate;
        return \DateTime::createFromFormat("U", $minDate);
    }

    /**
     * @param integer|string $from The date to start from as timestamp or as
     *     string PHP can interpret as date.
     * @param integer|string $to The date to end as timestamp or as string PHP
     *     can interpret as date.
     * @return array Array of AccessStatDaily objects.
     */
    public function getStatDay($from = null, $to = null)
    {
        if ($from === null) {
            $from = 0;
        }
        if (is_string($from)) {
            $from = strtotime($from);
        }

        if ($to === null) {
            $to = $this->getTimeProvider()->getTime();
        } else {
            if (is_string($to)) {
                $to = strtotime($to);
            }
        }
        $stats = $this->container->get("device.entity.manager.access_stat_daily")
            ->createQueryBuilder("s")
            ->where("s.date >= :from")
            ->andWhere("s.date <= :to")
            ->orderBy("s.id")
            ->setParameter("from", date("Y-m-d", $from))
            ->setParameter("to", date("Y-m-d", $to))
            ->getQuery()->getResult();

        // add not aggregated data
        if ($stats) {
            // start from next day
            $fromLatest = end($stats)->getDate()->format("U") + 86400;
        } else {
            $fromLatest = $from;
        }

        $em = $this->container->get("device.entity.manager.access_log");
        $latestTraffic = $em->createQueryBuilder("a")
        ->select("DATE(a.tstamp) AS date", "SUM(a.traffic) AS traffic", "COUNT(a.id) AS accessCount",
            "COUNT(DISTINCT a.device) AS distinctDevices", "COUNT(DISTINCT a.ip) AS distinctIps")
        ->where("a.tstamp >= :from")
        ->andWhere("a.tstamp <= :to")
        ->orderBy("a.id")
        ->groupBy("date")
        ->setParameter("from", date("Y-m-d", $fromLatest))
        ->setParameter("to", date("Y-m-d H:i:s", $to))
        ->getQuery()->getArrayResult();
        ;

        foreach ($latestTraffic as $trafficData) {
            $stat = new AccessStatDaily();
            $stat->setDate(new \DateTime(date("Y-m-d", $fromLatest)));
            $stat->setAccessCount($trafficData['accessCount']);
            $stat->setDistinctDevices($trafficData['distinctDevices']);
            $stat->setDistinctIps($trafficData['distinctIps']);
            $stat->setTraffic($trafficData['traffic']);
            $stats[] = $stat;
            $fromLatest += 86400;
        }

        return $stats;
    }

    /**
     * @param integer|string $from The date to start from as timestamp or as
     *     string PHP can interpret as date.
     * @param integer|string $to The date to end as timestamp or as string PHP
     *     can interpret as date.
     * @return array Array of timestamp-traffic pairs. Values at the beginning
     *     and the end are interpolated if possible.
     */
    public function getStatDayArray($from = null, $to = null)
    {
        if ($from === null) {
            $from = 0;
        }
        if (is_string($from)) {
            $from = strtotime($from);
        }

        if ($to === null) {
            $to = $this->getTimeProvider()->getTime();
        } else {
            if (is_string($to)) {
                $to = strtotime($to);
            }
        }

        $rv = array();
        $origStart = $from;
        $origEnd = $to;
        $from -= 86399; // for interpolation
        $to += 10; // for interpolation

        $stats = $this->container->get("device.entity.manager.access_stat_daily")
            ->createQueryBuilder("s")
            ->select("s.date AS date", "s.traffic AS traffic")
            ->where("s.date >= :from")
            ->andWhere("s.date <= :to")
            ->orderBy("s.id")
            ->setParameter("from", date("Y-m-d", $from))
            ->setParameter("to", date("Y-m-d", $to))
            ->getQuery()->getArrayResult();

        $lastTime = $from;
        if ($stats) {
            foreach ($stats as $stat) {
                $time = $stat['date']->format("U");
                for ($betweenTime = $lastTime + 86400; $betweenTime < $time; $betweenTime += 86400) {
                    // no traffic for one or more days
                    $rv[$betweenTime] = 0;
                }
                $rv[$time] = intval($stat['traffic']);
                $lastTime = $time;
            }
            $lastTime = $from = $lastTime + 86400; // start from next day
        }

        // add not aggregated data, aggregate in 10 minute steps
        $em = $this->container->get("device.entity.manager.access_log");
        $latestTraffic = $em->createQueryBuilder("a")
            ->select("DIV(a.tstamp, 1000) * 10 AS date", "SUM(a.traffic) AS traffic")
            ->where("a.tstamp >= :from")
            ->andWhere("a.tstamp <= :to")
            ->orderBy("date")
            ->groupBy("date")
            ->setParameter("from", date("Y-m-d", $from))
            ->setParameter("to", date("Y-m-d H:i:s", $to))
            ->getQuery()->getArrayResult();
        ;

        foreach ($latestTraffic as $trafficData) {
            $dt = \DateTime::createFromFormat("YmdHi", $trafficData['date']);
            $dt->setTimezone($this->getTimeProvider()->getTimezone());
            $time = $dt->format("U") + 300;
            for ($betweenTime = $lastTime + 600; $betweenTime < $time; $betweenTime += 600) {
                // no traffic for more than 10 minutes
                $rv[$betweenTime] = 0;
            }
            $rv[$time] = intval($trafficData['traffic']);
            $lastTime = $time;
        }

        // add not aggregated data, aggregate in 1 minute steps
        $from = $lastTime + 300;
        if ($from > $origEnd) {
            $from = $origEnd;
        }
        $latestTraffic = $em->createQueryBuilder("a")
        ->select("DIV(a.tstamp, 100) AS date", "SUM(a.traffic) AS traffic")
        ->where("a.tstamp >= :from")
        ->andWhere("a.tstamp <= :to")
        ->orderBy("date")
        ->groupBy("date")
        ->setParameter("from", date("Y-m-d H:i", $from))
        ->setParameter("to", date("Y-m-d H:i:s", $to))
        ->getQuery()->getArrayResult();
        ;

        foreach ($latestTraffic as $trafficData) {
            $dt = \DateTime::createFromFormat("YmdHi", $trafficData['date']);
            $dt->setTimezone($this->getTimeProvider()->getTimezone());
            $time = $dt->format("U") + 30;
            for ($betweenTime = $lastTime + 60; $betweenTime < $time; $betweenTime += 60) {
                // no traffic for more than 1 minute
                $rv[$betweenTime] = 0;
            }
            $rv[$time] = intval($trafficData['traffic']);
            $lastTime = $time;
        }

        // add last value
        $from = $origEnd;
        $trafficData = $em->createQueryBuilder("a")
        ->select("a.tstamp AS date", "a.traffic AS traffic")
        ->where("a.tstamp >= :from")
        ->andWhere("a.tstamp <= :to")
        ->orderBy("a.id", "DESC")
        ->setParameter("from", date("Y-m-d H:i:s", $from))
        ->setParameter("to", date("Y-m-d H:i:s", $to))
        ->getQuery()->setMaxResults(1)->getOneOrNullResult();
        ;

        if ($trafficData) {
            $dt = $trafficData['date'];
            $time = $dt->format("U");
            $rv[$time] = intval($trafficData['traffic']);
        }

        return Interpolator::interpolateArray($rv, $origStart, $origEnd);
    }

    /**
     * @return \stdClass The statistics of last month
     * @throws DatabaseException On database error.
     */
    public function getStatLastMonth()
    {
        static $statLastMonth = null;
        if ($statLastMonth === null) {
            $stats = $this->getStatMonthEach("-1 month");
            if (isset($stats[0])) {
                $statLastMonth = $stats[0];
            } else {
                $statLastMonth = new \stdClass();
                $statLastMonth->date = DateUtil::getFirstOfMonth("-1 month", $this->getTimeProvider()->getTime());
                $statLastMonth->traffic = 0;
                $statLastMonth->access_count = 0;
                $statLastMonth->distinct_devices = 0;
                $statLastMonth->distinct_ips = 0;
            }
        }
        return $statLastMonth;
    }

    /**
     * @return \stdClass The statistics of the current month.
     * @throws DatabaseException On database error.
     */
    public function getStatThisMonth()
    {
        static $statThisMonth = null;
        if ($statThisMonth === null) {
            $stats = $this->getStatMonthEach($this->getTimeProvider()->getTime());
            if (isset($stats[0])) {
                $statThisMonth = $stats[0];
            } else {
                $statThisMonth = new \stdClass();
                $statThisMonth->date = DateUtil::getFirstOfMonth("-1 month", $this->getTimeProvider()->getTime());
                $statThisMonth->traffic = 0;
                $statThisMonth->access_count = 0;
                $statThisMonth->distinct_devices = 0;
                $statThisMonth->distinct_ips = 0;
            }
        }
        return $statThisMonth;
    }

    /**
     * Returns the traffic for each month in the specified period.
     * Note: the returned objects will always contain the values for a full month,
     * meaning that e.g. $from is 2013-01-22, you will get the complete traffic
     * in January 2013.
     *
     * @param array|string|int $from The beginning of the requested period.
     *     Might be an array with the entries 'year' and 'month', a date string,
     *     that can be parsed by strtotime(), or a UNIX timestamp.
     * @param array|string|int $to The end of the requested period.
     *     Might be an array with the entries 'year' and 'month', a date string,
     *     that can be parsed by strtotime(), or a UNIX timestamp.
     * @return \stdClass[] Array with a stdClass object for each month. Each object has the
     *     properties date, traffic, access_count, distinct_devices, and distinct_ips.
     *     The property date is a string in the form YYYY-MM-01. The entries are
     *     sorted descending by date.
     */
    public function getStatMonthEach($from, $to = null)
    {
        static $sql = "SELECT `date`, traffic, access_count, distinct_ips, distinct_devices
            FROM %s WHERE `date` >= :from AND `date` <= :to";
        $curTime = $this->getTimeProvider()->getTime();
        $from = DateUtil::getFirstOfMonth($from, $curTime);
        if ($to === null) {
            $to = $from;
        } else {
            $to = DateUtil::getFirstOfMonth($to, $curTime);
        }

        $statMonths = $this->getDB()->queryObjects(
            sprintf($sql, "access_stat_year_month"), array("from" => $from, "to" => $to)
        );

        if ($to == DateUtil::getFirstOfMonth($curTime)) {
            if (!isset($statMonths[0]) || $statMonths[0]->date != $to) {
                $statThisMonth = new \stdClass();
                $statThisMonth->date = $to;
                $statThisMonth->traffic = 0;
                $statThisMonth->access_count = 0;
                $statThisMonth->distinct_devices = 0;
                $statThisMonth->distinct_ips = 0;
                array_merge(array($statThisMonth), $statMonths);
            } else {
                $statThisMonth = $statMonths[0];
            }
            // use (almost) current values for current month
            $statToday = $this->getStatToday();
            $statThisMonth->traffic += $statToday->traffic;
            $statThisMonth->access_count += $statToday->access_count;
            $statThisMonth->distinct_devices = max(
                array($statThisMonth->distinct_devices, $statToday->distinct_devices)
            );
            $statThisMonth->distinct_ips = max(
                array($statThisMonth->distinct_ips, $statToday->distinct_ips)
            );
        }

        return $statMonths;
    }

    /**
     * @return \stdClass The statistics of the current day.
     * @throws DatabaseException On database error.
     */
    public function getStatToday()
    {
        static $statToday = null;
        if ($statToday === null) {
            $now = date("Y-m-d");
            $sql = "SELECT IFNULL(SUM(traffic), 0) AS traffic, IFNULL(COUNT(id), 0) AS access_count,
            IFNULL(COUNT(DISTINCT ip), 0) AS distinct_ips, IFNULL(COUNT(DISTINCT device_id), 0) AS distinct_devices
            FROM accesslog WHERE tstamp >= :starttime";
            $statToday = $this->getDB()->queryObject($sql, array("starttime" => $now));
        }
        return $statToday;
    }

    /**
     * @return \stdClass The overall statistics.
     */
    public function getStatTotal()
    {
        static $statTotal = null;
        if ($statTotal === null) {
            $conn = $this->getDoctrine()->getConnection();
            $statTotal = $conn
                ->executeQuery("SELECT SUM(traffic) AS traffic, SUM(access_count) AS access_count
                    FROM access_stat_year_month")
                ->fetch(Query::HYDRATE_SIMPLEOBJECT);

            $statToday = $this->getStatToday();
            $statTotal->traffic += $statToday->traffic;
            $statTotal->access_count += $statToday->access_count;
            $statTotal->distinct_devices = $conn
                ->executeQuery("SELECT COUNT(DISTINCT device_id) AS distinct_devices FROM access_cert_stat")
                ->fetchColumn() + 0;
        }
        return $statTotal;
    }

    /**
     * @return integer The traffic in bytes for the last 24 hours.
     */
    public function getTrafficLast24h()
    {
        $em = $this->container->get("device.entity.manager.access_log");
        $now = $this->getTimeProvider()->getTime();
        return $em->createQueryBuilder("l")
            ->select("SUM(l.traffic)")
            ->where("l.tstamp >= :from")
            ->setParameter("from", date("Y-m-d H:i:s", $now - 86400))
            ->getQuery()->getSingleScalarResult() ?: 0;
    }


    /**
     * Disables or enables ActiveSync for the specified device. For managed
     * devices the ActiveSync configuration will be removed or added. For not
     * managed devices the certificates will be revoked if disabled.
     *
     * @param Device $device The device to disable or enable ActiveSync.
     * @param boolean $disabled true to disable, or false to enable ActiveSync.
     */
    public function setActiveSyncDisabled(Device $device, $disabled)
    {
        if ($device->isActiveSyncDisabled() == $disabled) {
            return;
        }

        if ($disabled) {
            $this->getLogger()->info("Disable ActiveSync for device " . $device->getCombinedName(),
                array("device" => $device->getCombinedId()));
        } else {
            $this->getLogger()->info("Enable ActiveSync for device " . $device->getCombinedName(),
                array("device" => $device->getCombinedId()));
        }

        $device->setActiveSyncDisabled($disabled);
        $this->getEntityManager()->save($device, false);
        if ($device->isActive() && $device->isManaged()) {
            $mdmFacade = $this->getMdmFacade();
            if ($disabled) {
                foreach ($device->getMailAccounts() as $mailAccount) {
                    $mdmFacade->removeMailAccount($mailAccount);
                }
            } else {
                foreach ($device->getMailAccounts() as $mailAccount) {
                    $mdmFacade->installMailAccount($mailAccount);
                }
            }
        }
        $this->getEntityManager()->flush();
    }

    /**
     * Enables or disables the device with the specified id by activating or
     * deactivating its certificate.
     *
     * @param int $id The id of the device to enable or disable.
     * @param boolean $enabled true to enable, or false to disable the device.
     * @throws DatabaseException On database error.
     * @throws \Exception On error.
     */
    public function setDeviceEnabled($id, $enabled)
    {
        $device = $this->getDevice($id);
        if ($enabled) {
            if ($device->isManaged()) {
                throw new \Exception("Managed devices cannot be enabled");
            }
            $device->setDisabled(false);
            $this->getEntityManager()->save($device);
        } else {
            $this->disableDevice($id);
        }
    }

    /**
     * Disables the device with the specified id.
     * @param Device|int $device The device or the id of the device to disable.
     * @param string $action If set to "erase" managed devices will be erased
     *     instead of doing an enterprise wipe. This parameter has no meaning
     *     for non managed devices.
     * @throws \Exception
     */
    public function disableDevice($device, $action = null)
    {
        if (! $device instanceof Device) {
            $device = $this->getDevice($device);
        }
        if ($device->isManaged() && $device->getWipeStatus() != Device::WIPE_EXECUTED) {
            if ($action == "erase") {
                $this->getMdmFacade()->eraseDevice($device);
            } else {
                $this->getMdmFacade()->wipeDevice($device);
            }
            $device->setWipeStatus(Device::WIPE_PENDING);
        }
        $device->setDisabled(true);
        $this->getEntityManager()->save($device);
    }

    public function getDeviceTraffic(Device $device)
    {
        return $this->container->get("doctrine.orm.entity_manager")->createQueryBuilder()
            ->select("SUM(a.traffic)")
            ->from("TopalisDeviceBundle:AccessCertStat", "a")
            ->where("a.device = :device")
            ->setParameter("device", $device)
            ->getQuery()->getOneOrNullResult(AbstractQuery::HYDRATE_SINGLE_SCALAR) ?: 0;
    }

    /**
     * @return array The number of devices by type with the OS id as key.
     */
    public function getDeviceTypeCount()
    {
        $typeCount = $this->getEntityManager()->createQueryBuilder("d")
            ->select("d.os AS os", "COUNT(d.os) AS number")
            ->groupBy("d.os")
            ->getQuery()->getArrayResult();
        $rv = array();
        foreach ($typeCount as $t) {
            $rv[$t['os']] = $t['number'];
        }

        return $rv;
    }
}
