<?php

namespace Topalis\Device\Entity;

use Doctrine\ORM\Mapping as ORM;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;

/**
 * MailAccount
 *
 * @ORM\Table(name="mailaccounts", uniqueConstraints={
 *     @ORM\UniqueConstraint(name="idx_mailaccounts_device_id_email", columns={"device_id", "email"})
 * })
 * @ORM\Entity
 */
class MailAccount
{
    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="user", type="string", length=255, nullable=false,
     *     options={"comment"="email account login user"})
     */
    private $user;

    /**
     * @var string
     *
     * @ORM\Column(name="email", type="string", length=255, nullable=false,
     *     options={"comment"="email address"})
     */
    private $email;

    /**
     * The device the mail account belongs to.
     * @var \Topalis\Device\Entity\Device
     *
     * @ORM\ManyToOne(targetEntity="Topalis\Device\Entity\Device", inversedBy="mailAccounts")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="device_id", referencedColumnName="id", nullable=false)
     * })
     */
    private $device;

    /**
     * The certificate used for the mail account.
     * @var \Topalis\Certificate\Entity\ClientCertificate
     *
     * @ORM\ManyToOne(targetEntity="Topalis\Certificate\Entity\ClientCertificate", cascade={"persist"})
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="cert_id", referencedColumnName="id")
     * })
     */
    private $cert;

    /**
     * @var string
     *
     * @ORM\Column(name="uuid", type="string", length=255, nullable=true,
     *     options={"comment"="UUID of the mail account (Windows Phone)"})
     */
    private $uuid;

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set user
     *
     * @param string $user
     *
     * @throws \Symfony\Component\Validator\Exception\UnexpectedTypeException
     * @return MailAccount
     */
    public function setUser($user)
    {
        if (null !== $user && !is_string($user)) {
            throw new UnexpectedTypeException($user, "string");
        }
        $this->user = $user;

        return $this;
    }

    /**
     * Get user
     *
     * @return string
     */
    public function getUser()
    {
        return $this->user;
    }

    /**
     * Set email
     *
     * @param string $email
     * @return MailAccount
     */
    public function setEmail($email)
    {
        $this->email = $email;

        return $this;
    }

    /**
     * Get email
     *
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * Set device
     *
     * @param \Topalis\Device\Entity\Device $device
     * @return MailAccount
     */
    public function setDevice(\Topalis\Device\Entity\Device $device)
    {
        if ($this->device === $device) {
            return $this;
        }

        $this->device = $device;
        $device->addMailAccount($this);

        return $this;
    }

    /**
     * Get device
     *
     * @return \Topalis\Device\Entity\Device
     */
    public function getDevice()
    {
        return $this->device;
    }

    /**
     * Set cert
     *
     * @param \Topalis\Certificate\Entity\ClientCertificate $cert
     * @return MailAccount
     */
    public function setCert(\Topalis\Certificate\Entity\ClientCertificate $cert = null)
    {
        $this->cert = $cert;

        return $this;
    }

    /**
     * Get cert
     *
     * @return \Topalis\Certificate\Entity\ClientCertificate
     */
    public function getCert()
    {
        return $this->cert;
    }

    /**
     * @return string The UUID of the mail account.
     */
    public function getUuid()
    {
        return $this->uuid;
    }

    /**
     * Sets the UUID.
     *
     * @param string $uuid The UUID to set.
     *
     * @return $this
     */
    public function setUuid($uuid)
    {
        $this->uuid = $uuid;

        return $this;
    }
}
