<?php
namespace Topalis\Device\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\Device\Entity\Device;

/**
 * This listener sends an email to the owner of a newly created device.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class SendUserMailEventListener extends ContainerAware
{
    /**
     * @var boolean
     */
    private $enabled;

    public function postPersist(LifecycleEventArgs $eventArgs)
    {
        if (!$this->isEnabled()) {
            return;
        }

        $entity = $eventArgs->getEntity();
        if (!$entity instanceof Device) {
            return;
        }

        try {
            $mailer = $this->getMailer();
            if (!$mailer->isConfigured()) {
                return;
            }

            $config = $this->getConfig();
            $user = $entity->getUser();
            $translator = $this->getTranslator();
            $subject = $translator->trans("admin.devices.userMail.subject", array(
                "%company%" => $this->getCertificateManager()->getCAOrg(),
            ));
            $params = array(
                "%name%" => $user->getRealname(),
                "%auralisUrl%" => "http://www.auralis.de/",
                "%login%" => $user->getLogin(),
            );
            $type = $entity->getType()->getName();
            switch ($type) {
                case "android":
                    $hostname = $config->getHostname();
                    if ($config->getSecondaryPort() != 8843) {
                        $hostname .= ":" . $config->getSecondaryPort();
                    }
                    $params['%hostname%'] = $hostname;
                    break;
                case "ios":
                    /* fall through */
                case "other":
                    $params['%rolloutUrl%'] = "https://" . $config->getHostname() . ":" . $config->getSecondaryPort();
                    break;
                case "winphone":
                    $params['%email%'] = $user->getEmail();
                    $params['%server%'] = $config->getHostname() . ":" . $config->getSecondaryPort();
                    break;
            }
            $text = $translator->trans("admin.devices.userMail.text." . $type, $params);

            $message = $mailer->createMessage($subject);
            $message->setBody($text);
            $message->setTo($user->getEmail(), $user->getRealname());
            $mailer->sendMessage($message);
        } catch (\Exception $e) {
            $this->getLogger()->error("Sending welcome message to " . $user->getEmail() . "failed",
                array("exceptione" => $e));
        }
    }

    /**
     * @return boolean
     */
    public function isEnabled()
    {
        if ($this->enabled === null) {
            $this->enabled = $this->getConfig()->isSendUserMail();
        }
        return $this->enabled;
    }

    /**
     * @param boolean $enabled
     */
    public function setEnabled($enabled)
    {
        $this->enabled = $enabled;
        $this->getConfig()->setSendUserMail($enabled);
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     *
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @return \Psr\Log\LoggerInterface
     */
    private function getLogger()
    {
        return $this->container->get("logger");
    }

    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->container->get("mailer");
    }

    /**
     * @return \Symfony\Component\Translation\TranslatorInterface
     */
    private function getTranslator()
    {
        return $this->container->get("translator");
    }
}
