<?php
namespace Topalis\Device\Type\Info;

use Topalis\Util\StringUtil;
use Symfony\Component\PropertyAccess\PropertyAccess;
use Symfony\Component\PropertyAccess\Exception\NoSuchPropertyException;

/**
 * Device information for Windows Phone devices.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class WinPhoneDeviceInfo implements DeviceInfo
{
    /**
     *
     * @var array
     */
    private $data;

    public function __construct(array $data)
    {
        $this->data = $data;

        if (StringUtil::startsWith($this->getData('DevId'), "urn:uuid:")) {
            $this->data['DevId'] = substr($this->getData('DevId'), strlen("urn:uuid:"));
        }

        if (isset($this->data['LocalTime'])) {
            try {
                $this->data['LocalTime'] = new \DateTime($this->data['LocalTime']);
            } catch (\Exception $e) {
                // LocalTime couldn't be parsed
            }
        }
    }

    public function getVendor()
    {
        return $this->getData('Man');
    }

    public function getModelName()
    {
        return $this->getData('Mod');
    }

    public function getName()
    {
        return $this->getData('DeviceName');
    }

    public function getOsVersion()
    {
        return $this->getData('OSPlatform') . " " . $this->getData('SwV');
    }

    public function isEncrypted()
    {
        return StringUtil::strtobool($this->getData("Encrypted"), true);
    }

    public function isRoaming()
    {
        return StringUtil::strtobool($this->getData("Roaming"), true);
    }

    public function getCarrier()
    {
        return null;
    }

    public function getPhoneNumber()
    {
        return $this->getData("PhoneNumber");
    }

    public function getUDID()
    {
        return $this->getData('DevId');
    }

    public function getIMEI()
    {
        return $this->getData("IMEI");
    }

    public function getWifiMac()
    {
        return $this->getData('WLANMACAddress');
    }

    public function getSerial()
    {
        return null;
    }

    public function getOther()
    {
        $rv = $this->data;
        $usedKeys = array(
            "Man", "Mod", "DeviceName", "OSPlatform", "SwV", "DevId",
            "WLANMACAddress", "Roaming", "PhoneNumber", "IMEI",
        );
        foreach ($usedKeys as $key) {
            unset($rv[$key]);
        }
        ksort($rv);

        return $rv;
    }

    public function getIcon()
    {
        return "bundles/topalisdevice/images/winphone.png";
    }

    public function getBluetoothMac()
    {
        return null;
    }

    public function contains($key)
    {
        return isset($this->data[$key]);
    }

    public function get($key)
    {
        $accessor = PropertyAccess::createPropertyAccessor();
        try {
            $value = $accessor->getValue($this, $key);
            return $value;
        } catch (NoSuchPropertyException $e) {
            // ignore
        }

        switch ($key) {
            case "Model":
                $key = "DevTyp";
                break;
            case "DeviceEncrypted":
                $key = "Encrypted";
                break;
            case "BuildVersion":
                $key = "FwV";
                break;
        }

        return $this->getData($key);
    }

    public function getAvailableCapacity()
    {
        return null;
    }

    public function getCapacity()
    {
        return null;
    }

    public function hasExternalStorage()
    {
        return false;
    }

    public function getAvailableExternalCapacity()
    {
        return null;
    }

    public function getExternalCapacity()
    {
        return null;
    }

    public function getBatteryLevel()
    {
        return null;
    }

    protected function getData($key)
    {
        return isset($this->data[$key]) ? $this->data[$key] : null;
    }
}
