<?php
namespace Topalis\Gcm;

use Topalis\Device\Entity\Device;
use Topalis\MDM\Entity\MdmCommand;
use Psr\Log\LoggerInterface;
use Topalis\Gcm\Exception\GcmException;
use Topalis\Gcm\Exception\GcmClientException;
use Topalis\Kernel\AbstractAuralisService;

/**
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class GcmServer extends AbstractAuralisService
{
    const NOT_REGISTERED_TIMEOUT = 172800000; // 48h

    /**
     *
     * @var \Psr\Log\LoggerInterface
     */
    private $logger;

    private $invalidApiKey = false;

    private $serverErrorCount = 0;

    public function __construct(LoggerInterface $logger)
    {
        $this->logger = $logger;
    }

    public function run($offline = false)
    {
        $this->logger->info("run GcmServer");
        $em = $this->getEntityManager();

        // offline mode for local testing
        if ($offline) {
            while (true) {
                try {
                    $this->pushMessages();
                } catch (\Exception $e) {
                    // TODO comment why the exception is not handled
                }
                // avoid memory leaks
                $em->flush();
                $em->clear();
                gc_collect_cycles();
                sleep(10);
            }
        }

        while (true) {
            // ensure that we can use GCM
            $retryAfter = 0;
            while (!$this->isGcmUseable($retryAfter)) {
                if ($this->invalidApiKey) {
                    $this->getConfig()->setGcmStatus("INVALID_API_KEY");
                    $this->waitForApiKeyChange();
                } else {
                    $this->getConfig()->setGcmStatus("ERROR");
                    // sleep and try again
                    sleep($retryAfter);
                }
                $this->reloadConfig();
            }

            while (true) {
                pcntl_signal_dispatch(); // ensure possible lograte is noticed
                try {
                    $this->getConfig()->setGcmStatus("OK");
                    $this->pushMessages();
                } catch (\Exception $e) {
                    $this->logger->error($e->getMessage(), array("exception" => $e));
                    $this->getConfig()->setGcmStatus("ERROR");
                    $this->getConfig()->setGcmErrorMessage($e->getMessage());

                    if ($e instanceof GcmException) {
                        break; // ensure that GCM is useable again
                    }

                    // db connection might be lost, check and reconnect
                    $connection = $this->getDoctrine()->getConnection();
                    try {
                        $connection->query("select 1;")->fetch();
                    } catch (\Exception $e) {
                        // db server might be restarted so wait some time and try to reconnect
                        try {
                            sleep(15);
                            $connection->close();
                            $connection->connect();
                            $connection->query("select 1;")->fetch();
                            $this->logger->info("Reconnected to database");
                        } catch (\Exception $e2) {
                            $this->logger->critical("Cannot access database: " . $e2->getMessage(),
                                array("exception" => $e2));
                            return;
                        }
                    }
                }

                // avoid memory leaks
                $em->flush();
                $em->clear();
                gc_collect_cycles();
                sleep(10);

                // config might have changed
                $this->reloadConfig();
            }
        }
    }

    /**
     * Tests if the GCM service is useable. If the API key is invalid the
     * attribute invalidApiKey will be set to true. If the service is not
     * useable the next try should be done after $retryAfter seconds, or if the
     * API key has changed in case of an invalid API key.
     *
     * @param integer $retryAfter The number of seconds to wait for the next
     *     try, or 0 if API key is invalid.
     * @return boolean true if the GCM service is useable, false otherwise.
     */
    protected function isGcmUseable(&$retryAfter)
    {
        try {
            $response = $this->getGcm()->getTestResponse();
            $status = $response->getStatusCode();
            if ($status >= 200 && $status <= 299) {
                $this->serverErrorCount = 0;
                return true;
            }

            if ($status == 401) {
                $this->invalidApiKey = true;
                $this->serverErrorCount = 0;
                return false;
            }
        } catch (GcmClientException $e) {
            // timeout, status 5xx, ...
        }

        $this->serverErrorCount++;

        if (isset($response)) {
            $retryAfter = $response->getHeader("Retry-After");
            if (is_numeric($retryAfter)) {
                $retryAfter = intval($retryAfter);
                if ($retryAfter > 0) {
                    return false;
                }
            }
        }

        // increase retry time after every error
        $retryAfter = $this->serverErrorCount * $this->serverErrorCount;
        if ($retryAfter > 300) {
            // don't wait more then 5 minutes
            $retryAfter = 300;
        }
        return false;
    }

    protected function getDeviceOs()
    {
        static $deviceOs = null;
        if ($deviceOs === null) {
            $deviceOs = $this->getDeviceTypeFactory()->getType("android")->getId();
        }

        return $deviceOs;
    }

    protected function disableDevice(Device $device)
    {
        $device->setDisabled(true);
        $device->setMdmToken(null);
        $device->setWipeStatus(Device::WIPE_UNKNOWN);
        $this->getDeviceEntityManager()->save($device);
        $this->getMdmQueue()->cancelCommandsForDevice($device);
        $this->getCertificateManager()->setDeviceCertsUnused($device);
    }

    protected function handleFailure(Device $device, $error)
    {
        // see https://developer.android.com/google/gcm/http.html#error_codes
        switch ($error) {
            case "Unavailable":
                // retry
                $this->getMdmQueue()->setWaiting($device, false);
                break;
            case "NotRegistered":
                /*
                 * device has a new registration id, or app was uninstalled from
                 * the device
                 * We wait some time for a new registration id. If we don't get
                 * it, we disable the device.
                 */
                $deviceInfo = $device->getDeviceinformation();
                $notRegistered = isset($deviceInfo['NotRegistered']) ? $deviceInfo['NotRegistered'] : null;
                if ($notRegistered) {
                    if ($this->getTimeProvider()->getTime() - $notRegistered > self::NOT_REGISTERED_TIMEOUT) {
                        $this->disableDevice($device);
                        try {
                            // inform admin
                            $mailer = $this->getMailer();
                            if ($mailer->isConfigured()) {
                                $subject = $this->tr("admin.gcm.notRegisteredMail.subject");
                                $text = $this->tr("admin.gcm.notRegisteredMail.text", array(
                                    "%name%" => $device->getName(),
                                    "%user%" => $device->getUser()->getLogin()
                                ));
                                $mailer->sendAdminMessage($subject, $text);
                            }
                        } catch (\Exception $e) {
                            $this->getLogger()->error("Sending not registered message for device "
                                . $device->getId() . " to admin failed");
                        }
                    }
                } else {
                    // store time of first NotRegistered error
                    $deviceInfo['NotRegistered'] = $this->getTimeProvider()->getTime();
                    $device->setDeviceinformation($deviceInfo);
                    $this->getDeviceEntityManager()->save($device);
                }
                break;
            case "MismatchSenderId":
                /*
                 * admin propably changed the API key
                 * we clear the mdm token, cancel remaining commands so we don't
                 * send messages to the GCM server for that device again.
                 */
                $device->setMdmtoken(null);
                $this->getDeviceEntityManager()->save($device);
                $this->getMdmQueue()->cancelCommandsForDevice($device);
                try {
                    // inform admin
                    $mailer = $this->getMailer();
                    if ($mailer->isConfigured()) {
                        $subject = $this->tr("admin.gcm.mismatchSenderId.subject");
                        $text = $this->tr("admin.gcm.mismatchSenderId.text", array(
                            "%name%" => $device->getName(),
                            "%user%" => $device->getUser()->getLogin()
                        ));
                        $mailer->sendAdminMessage($subject, $text);
                    }
                } catch (\Exception $e) {
                    $this->getLogger()->error("Sending mismatch sender id message for device "
                        . $device->getId() . " to admin failed");
                }
                break;
            default:
                /*
                 * other errors should never happen or have an HTTP status
                 * that doesn't match 2xx.
                 */
                break;
        }
    }

    protected function pushMessage(Device $device)
    {
        $this->logger->info("Send message for device " . $device->getId());
        $response = $this->getGcm()->send($device, array("mdm" => "sync"), array(
            "collapse_key" => "auralis-mdm",
            "delay_while_idle" => false,
        ));
        $result = isset($response['results'][0]) ? $response['results'][0] : null;

        // see https://developer.android.com/google/gcm/http.html#success
        if (isset($result['message_id']) && isset($result['registration_id'])
            && strlen($response['registration_id']) > 0) {
            // new registration id
            $device->setMdmtoken($response['registration_id']);
            $deviceInfo = $device->getDeviceinformation();
            if (isset($deviceInfo['NotRegistered'])) {
                // remove time of first NotRegistered error
                unset($deviceInfo['NotRegistered']);
                $device->setDeviceinformation($deviceInfo);
            }
            $this->getDeviceEntityManager()->save($device);
            return;
        }

        if (!isset($response['failure']) || $response['failure'] == 0) {
            // everything is OK
            return;
        }

        $error = isset($result['error']) ? $result['error'] : "Not specified";
        $this->logger->error("Error sending message for device " . $device->getId() . ": " . $error);
        $this->handleFailure($device, $error);
    }

    protected function pushMessages()
    {
        $this->getDeviceEntityManager()->disableDeleteFilter();
        // clear the entity manager otherwise we will get old entity states
        $this->getDeviceEntityManager()->clear();
        // add new commands to queue
        /** @var $newMdmCommands MdmCommand[] */
        $newMdmCommands = $this->getMdmCommandEntityManager()->findByStatusAndOs(
            MdmCommand::STATUS_NEW, $this->getDeviceOs());
        foreach ($newMdmCommands as $mdmCommand) {
            $device = $mdmCommand->getDevice();
            if ($device->getMdmToken() === null) {
                $this->logger->debug("Device '" . $device->getId() . "' has no MDM token");
                continue;
            }
            $this->getMdmQueue()->addCommand($mdmCommand);
            $mdmCommand->setStatus(MdmCommand::STATUS_QUEUED);
            $this->getMdmCommandEntityManager()->save($mdmCommand);
        }

        // send push where required
        $devices = $this->getMdmQueue()->getPushRequired($this->getDeviceOs());
        foreach ($devices as $device) {
            $this->pushMessage($device);
            $this->getMdmQueue()->setWaiting($device, true);
            $this->getMdmQueue()->setPushTime($device, time());
        }
    }

    protected function reloadConfig()
    {
        $this->getGcm()->loadConfig($this->getConfig());
        $this->getMailer()->loadSettingsFromConfig();
    }

    /**
     * Blocks until the API key has changed.
     */
    protected function waitForApiKeyChange()
    {
        $currentApiKey = $this->getGcm()->getApiKey();
        do {
            sleep(60);
            $newApiKey = $this->getConfig()->getGcmConfig()->getApiKey();
        } while ($currentApiKey == $newApiKey);
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     *
     * @return \Topalis\Device\Type\DeviceTypeFactory
     */
    private function getDeviceTypeFactory()
    {
        return $this->container->get("device.type.factory");
    }

    /**
     *
     * @return \Doctrine\ORM\EntityManager
     */
    private function getEntityManager()
    {
        return $this->getDoctrine()->getManager();
    }

    /**
     * @return \Topalis\Gcm\Gcm
     */
    private function getGcm()
    {
        return $this->container->get("mdm.gcm");
    }

    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->container->get("mailer");
    }

    /**
     *
     * @return \Topalis\MDM\Entity\MdmCommandEntityManager
     */
    private function getMdmCommandEntityManager()
    {
        return $this->container->get("mdm.entity.manager.mdm_command");
    }

    /**
     * @return \Topalis\MDM\MdmQueue
     */
    private function getMdmQueue()
    {
        return $this->container->get("mdm.queue");
    }
}
