<?php
namespace Topalis;

use Symfony\Component\HttpFoundation\Response;

/**
 * Class with functions to generate common HTTP error responses.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class HttpError
{
    public static function response($status, $message = null)
    {
        self::closeSession();
        $statusText = Response::$statusTexts[$status];
        $content = "<!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">\n";
        $content .= "<html>\n<head><title>". $status . " " . $statusText . "</title></head>\n";
        $content .= "<body>\n<h1>" . $statusText . "</h1>\n";
        if ($message !== null) {
            $content .= $message . "\n";
        }
        $content .=  "</body>\n</html>\n";
        return new Response($content, $status);
    }

    /**
     * Terminate the current session and generate a "403 Bad Request" response.
     *
     * @param string $message The message to send. Set to "Bad request", if not
     *     specified.
     * @return Response
     */
    public static function badRequestResponse($message = null)
    {
        return self::response(400, $message);
    }

    /**
     * Terminate the current session and generate a "500 Internal Server Error" response.
     *
     * @param string $message
     */
    public static function internal($message = null)
    {
        if ($message === null) {
            $message = "Internal server error";
        }
        self::closeSession();
        header('HTTP/1.0 500 Internal Server Error');
        echo '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">'."\n";
        echo '<html><head><title>500 Internal server error</title></head><body>'."\n";
        echo $message . '</body></html>'."\n";
        exit;
    }

    /**
     * Terminate the current session and generate a "500 Internal Server Error" response.
     *
     * @param string $message The message to send. Set to "Internal server error",
     *     if not specified.
     * @return Response
     */
    public static function internalResponse($message = null)
    {
        return self::response(500, $message);
    }

    /**
     * Terminate the current session and generate a "403 Forbidden" response.
     *
     * @param string $message The message to send. Set to "Forbidden", if not
     *     specified.
     */
    public static function forbidden($message = null)
    {
        if ($message === null) {
            $message = "Access denied";
        }
        self::closeSession();
        header('HTTP/1.0 403 Forbidden');
        echo '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">'."\n";
        echo '<html><head><title>403 Forbidden</title></head><body>'."\n";
        echo $message . '</body></html>'."\n";
        exit;
    }

    /**
     * Terminate the current session and generate a "403 Forbidden" response.
     *
     * @param string $message
     */
    public static function forbiddenResponse($message = null)
    {
        return self::response(403, $message);
    }

    /**
     * Terminate the current session and generate a "401 Unauthorized" response.
     *
     * @param string $realm
     */
    public static function unauthorized($realm = null)
    {
        header('HTTP/1.0 401 Unautorized');
        if ($realm !== null) {
            header('WWW-Authenticate: Basic realm="'.$realm.'"');
        } else {
            header('WWW-Authenticate', 'Basic');
        }
        echo '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">'."\n";
        echo '<html><head><title>401 Unauthorized</title></head><body>'."\n";
        echo 'Authorization required</body></html>'."\n";
        exit;
    }

    /**
     * Terminate the current session and generate a "401 Unauthorized" response.
     *
     * @param string $realm The basic auth realm.
     * @param string $message The message to use instead of the default one.
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public static function unauthorizedResponse($realm = null, $message = null)
    {
        $response = self::response(401, $message);
        if ($realm !== null) {
            $response->headers->set("WWW-Authenticate", "Basic realm=\"$realm\"");
        } else {
            $response->headers->set("WWW-Authenticate", "Basic");
        }
        return $response;
    }

    /**
     * Generate a "404 Not Found" response
     */
    public static function notfound()
    {
        header('HTTP/1.0 404 Not found');
        echo '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">'."\n";
        echo '<html><head><title>404 Not Found</title></head>'."\n";
        echo '<body><h1>Not Found</h1>';
        echo 'The requested URL was not found on this server.</body></html>'."\n";
        exit;
    }

    private static function closeSession()
    {
        try {
            if (self::isSessionActive()) {
                $cookie = session_name();
                session_destroy();
                setcookie($cookie, '', 1, '/', null, true, true);
            }
        } catch (\Exception $e) {
            // do nothing
        }
    }

    private static function isSessionActive()
    {
        // session_status() available since PHP 5.4
        if (PHP_VERSION_ID >= 50400) {
            return session_status() == PHP_SESSION_ACTIVE;
        }
        return session_name() != "" && session_id() != "";
    }
}
