<?php
namespace Topalis\Kernel\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Command\Command;
use Monolog\Handler\StreamHandler;

/**
 * The LogrotateAwareCommand registers a signal handler for the HUP signal that
 * closes all stream handlers so they will open a new file handler. The HUP
 * signal must be sent after a log rotation that affects the command. If you
 * want to use another signal than HUP, override getLogrotateSignals().
 *
 * Sub classes must call the configure() method of this class. To ensure that
 * the signal is not handled late, call pcntl_signal_dispatch() at vital spots
 * or use declare(ticks=1).
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 *
 * @see http://www.php.net/manual/en/function.pcntl-signal-dispatch.php
 * @see http://php.net/manual/en/control-structures.declare.php
 */
abstract class LogrotateAwareCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        foreach ($this->getLogrotateSignals() as $signal) {
            pcntl_signal($signal, array($this, "onLogrotate"));
        }
    }

    public function onLogrotate($signo)
    {
        $signo; // unused

        $logger = $this->getContainer()->get("logger");
        if (!$logger instanceof \Monolog\Logger) {
            return;
        }

        $logger->debug("onLogrotate");
        $handlers = $logger->getHandlers();
        foreach ($handlers as $handler) {
            if (!$handler instanceof StreamHandler) {
                continue;
            }

            $reflClass = new \ReflectionClass($handler);
            if ($reflClass->hasProperty("url")) {
                $reflProp = $reflClass->getProperty("url");
                $reflProp->setAccessible(true);
                $url = $reflProp->getValue($handler);
                if (null === $url) {
                    // StreamHandler was created with a resource, not a filename
                    // so we cannot close it
                    continue;
                }
            }

            $handler->close();
        }
    }

    protected function getLogrotateSignals()
    {
        return array(SIGHUP);
    }
}
