<?php
namespace Topalis\Kernel\Command;

use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Supervisor event listener for PROCESS_STATE_FATAL. It sends an email to the
 * admin if a subprocess is moved to the FATAL state.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class SupervisorEventProcessStateFatalCommand extends LogrotateAwareCommand
{
    protected function configure()
    {
        parent::configure(true);
        $this
            ->setName("supervisor:event:process-state-fatal")
            ->setDescription("Supervisor event listener to send an email to the"
                . " admin if a Supervisor subprocess is moved to the FATAL state.");
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $input; // unused

        $logger = $this->getContainer()->get("logger");
        $logger->debug("Start Supervisor event listener for event PROCESS_STATE_FATAL");
        $this->getContainer()->get("config.auralis")->setCacheEnabled(false);
        $mailer = $this->getContainer()->get("mailer");
        $mailer->setAlwaysLoadSettings(true);
        $translator = $this->getContainer()->get("translator");

        while (true) {
            $output->writeln("READY");
            try {
                $body = null; // reset for correct log output in case of error
                $line = fgets(STDIN);
                if ($line === false) {
                    // probably supervisord was stopped
                    return 0;
                }
                pcntl_signal_dispatch(); // ensure possible lograte is noticed
                $headers = $this->tokenizeSupervisorLine($line);
                $body = fread(STDIN, $headers['len']);
                $bodyTokens = $this->tokenizeSupervisorLine($body);
                $process = $bodyTokens['processname'];
            } catch (\Exception $e) {
                $logger->error("Failed to parse Supervisor event data",
                    array("header" => $line, "body" => $body, "exception" => $e));
                return 1;
            }

            if ($mailer->isConfigured()) {
                $subject = $translator->trans("admin.supervisor.event.processState.fatal.mail.subject");
                $text = $translator->trans("admin.supervisor.event.processState.fatal.mail.text", array(
                    "%process%" => $process,
                ));

                try {
                    $mailer->sendAdminMessage($subject, $text);
                } catch (\Exception $e) {
                    $logger->error("Failed to send email to admin", array("process" => $process, "exception" => $e));
                    return 1;
                }
            } else {
                $logger->warning("The Supervisor subprocess \"" . $process . "\" moved to state FATAL");
            }

            $output->write("RESULT 2\nOK");
        }
    }

    protected function tokenizeSupervisorLine($line)
    {
        $pairs = array();
        $lineSplit = explode(" ", rtrim($line));
        array_walk($lineSplit, function ($val, $key) use (&$pairs) {
            $pair = explode(":", $val);
            $pairs[$pair[0]] = $pair[1];
        });

        return $pairs;
    }
}
