<?php
namespace Topalis\Kernel;

use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Generator\UrlGeneratorInterface;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\HttpFoundation\StreamedResponse;
use Topalis\Kernel\Http\RedirectDelayResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Topalis\Apache\Apache;
use Symfony\Component\BrowserKit\Request;
use Symfony\Bundle\FrameworkBundle\Routing\Router;
use Symfony\Component\HttpFoundation\Session\Session;

/**
 * Abstract class for controllers
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
abstract class Controller extends AbstractAuralisService
{
    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    private $request;

    /**
     * Convenience method for addFlash("error", $message)
     *
     * @param string $message The message, or array of messages to add.
     *
     * @see addFlash()
     */
    protected function addError($message)
    {
        $this->addFlash("error", $message);
    }

    /**
     * Convenience method for addFlash("success", $message)
     *
     * @param string $message The message, or array of messages to add.
     *
     * @see addFlash()
     */
    protected function addSuccess($message)
    {
        $this->addFlash("success", $message);
    }

    /**
     * Adds a flash message for type.
     *
     * @param string $type
     * @param string $message
     */
    protected function addFlash($type, $message)
    {
        $this->getSession()->getFlashBag()->add($type, $message);
    }

    /**
     * Creates and returns a Form instance from the type of the form.
     *
     * @param string|FormTypeInterface $type    The built type of the form
     * @param mixed                    $data    The initial data for the form
     * @param array                    $options Options for the form
     *
     * @return \Symfony\Component\Form\FormInterface
     */
    public function createForm($type, $data = null, array $options = array())
    {
        return $this->getFormFactory()->create($type, $data, $options);
    }

    /**
     * Creates and returns a form builder instance
     *
     * @param mixed $data    The initial data for the form
     * @param array $options Options for the form
     *
     * @return FormBuilder
     */
    public function createFormBuilder($data = null, array $options = array())
    {
        return $this->getFormFactory()->createBuilder("form", $data, $options);
    }

    /**
     * Forwards the request to another controller.
     *
     * @param string $controller The controller name (a string like BlogBundle:Post:index)
     * @param array  $path       An array of path parameters
     * @param array  $query      An array of query parameters
     *
     * @return Response A Response instance
     */
    public function forward($controller, array $path = array(), array $query = array())
    {
        $path['_controller'] = $controller;
        $subRequest = $this->getRequest()->duplicate($query, null, $path);

        return $this->getHttpKernel()->handle($subRequest, HttpKernelInterface::SUB_REQUEST);
    }

    /**
     * Generates a URL from the given parameters.
     *
     * @param string         $route         The name of the route
     * @param mixed          $parameters    An array of parameters
     * @param boolean|string $referenceType The type of reference (one of the constants in UrlGeneratorInterface)
     *
     * @return string The generated URL
     *
     * @see UrlGeneratorInterface
     */
    public function generateUrl($route, $parameters = array(), $referenceType = UrlGeneratorInterface::ABSOLUTE_PATH)
    {
        return $this->getRouter()->generate($route, $parameters, $referenceType);
    }

    /**
     * @return \Topalis\Apache\Apache
     */
    public function getApache()
    {
        return $this->container->get("apache");
    }

    /**
     * @return \Symfony\Component\Form\FormFactoryInterface
     */
    public function getFormFactory()
    {
        return $this->container->get("form.factory");
    }

    /**
     * @return \Symfony\Component\HttpKernel\HttpKernelInterface
     */
    public function getHttpKernel()
    {
        return $this->container->get("http_kernel");
    }

    /**
     * @return \Symfony\Component\HttpFoundation\Request
     */
    public function getRequest()
    {
        if ($this->request === null) {
            if ($this->container->has("request")) {
                $this->request = $this->container->get("request");
            }
            if ($this->request === null) {
                $this->request = \Symfony\Component\HttpFoundation\Request::createFromGlobals();
            }
        }
        return $this->request;
    }

    /**
     * @return \Symfony\Component\Routing\RouterInterface
     */
    public function getRouter()
    {
        return $this->container->get("router");
    }

    /**
     * @return \Symfony\Component\HttpFoundation\Session\Session
     */
    public function getSession()
    {
        return $this->container->get("session");
    }

    /**
     * @return \Symfony\Bundle\FrameworkBundle\Templating\EngineInterface
     */
    public function getTemplating()
    {
        return $this->container->get("templating");
    }

    /**
     * @return \Twig_Environment
     */
    public function getTwig()
    {
        return $this->container->get("twig");
    }

    /**
     * generate redirect response
     *
     * @param string $url
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function redirect($url = null)
    {
        if (strlen($url) == 0) {
            $attributes = $this->getRequest()->attributes;
            $url = $this->generateUrl($attributes->get("_route"), $attributes->get("_route_params"));
        }
        if ($this->getApache()->isReloadEnabled()) {
            return new RedirectDelayResponse($url);
        } else {
            return new RedirectResponse($url);
        }
    }

    /**
     * Generates a redirect reponse to the previous page set in the session, or
     * to the dashboard if no previous page is set.
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function redirectReferer()
    {
        $referer = $this->getRequest()->headers->get("referer");
        if (strlen($referer) > 0) {
            return $this->redirect($referer);
        }

        $lastRoute = $this->getSession()->get("lastRoute", array(
            "name" => "admin_dashboard",
            "params" => array(),
        ));
        return $this->redirect($this->generateUrl($lastRoute['name'], $lastRoute['params']));
    }

    /**
     * Renders a view.
     *
     * @param string   $view       The view name
     * @param array    $parameters An array of parameters to pass to the view
     * @param Response $response   A response instance
     *
     * @return Response A Response instance
     */
    public function render($view, array $parameters = array(), Response $response = null)
    {
        return $this->getTemplating()->renderResponse($view, $parameters, $response);
    }

    /**
     * Returns a rendered view.
     *
     * @param string $view       The view name
     * @param array  $parameters An array of parameters to pass to the view
     *
     * @return string The rendered view
     */
    public function renderView($view, array $parameters = array())
    {
        return $this->getTemplating()->render($view, $parameters);
    }

    /**
     * Streams a view.
     *
     * @param string           $view       The view name
     * @param array            $parameters An array of parameters to pass to the view
     * @param StreamedResponse $response   A response instance
     *
     * @return StreamedResponse A StreamedResponse instance
     */
    public function stream($view, array $parameters = array(), StreamedResponse $response = null)
    {
        $templating = $this->getTemplating();

        $callback = function () use ($templating, $view, $parameters) {
            $templating->stream($view, $parameters);
        };

        if (null === $response) {
            return new StreamedResponse($callback);
        }

        $response->setCallback($callback);

        return $response;
    }
}
