<?php
namespace Topalis\Kernel\Debug;

use Symfony\Component\ClassLoader\DebugClassLoader;
use Symfony\Component\Debug\Exception\FatalErrorException;
use Symfony\Component\Debug\ExceptionHandler;
use Topalis\Kernel\ErrorHandler;

/**
 * Registers all the debug tools. Based on {@link \Symfony\Component\Debug}.
 *
 * @author Fabien Potencier <fabien@symfony.com>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class Debug
{
    private static $enabled = false;

    /**
     * Enables the debug tools.
     *
     * This method registers an error handler and an exception handler.
     *
     * If the Symfony ClassLoader component is available, a special
     * class loader is also registered.
     *
     * @param int  $errorReportingLevel The level of error reporting you want
     * @param bool $displayErrors       Whether to display errors (for development) or just log them (for production)
     */
    public static function enable($errorReportingLevel = null, $displayErrors = true)
    {
        if (static::$enabled) {
            return;
        }

        static::$enabled = true;

        error_reporting(-1);
        ErrorHandler::register($errorReportingLevel);
        if ($displayErrors) {
            $debug = new static();
            register_shutdown_function(array($debug, "handleFatal"));
            $debug->reservedMemory = str_repeat("x", 10240);
        }

        if ("cli" !== PHP_SAPI) {
            ExceptionHandler::register();
        } else if ($displayErrors && (!ini_get("log_errors") || ini_get("error_log"))) {
            // CLI - display errors only if they're not already logged to STDERR
            ini_set("display_errors", 1);
        }

        if (class_exists("Symfony\Component\ClassLoader\DebugClassLoader")) {
            DebugClassLoader::enable();
        }
    }

    public function handleFatal()
    {
        if (null === $error = error_get_last()) {
            return;
        }

        $this->reservedMemory = "";
        $type = $error['type'] & (E_ALL | E_STRICT);
        if (!in_array($type, array(E_ERROR, E_CORE_ERROR, E_COMPILE_ERROR, E_PARSE))) {
            return;
        }

        // @codingStandardsIgnoreStart
        // get current exception handler
        $exceptionHandler = set_exception_handler(function () {});
        // @codingStandardsIgnoreEnd
        restore_exception_handler();

        if (is_array($exceptionHandler) && $exceptionHandler[0] instanceof ExceptionHandler) {
            $level = isset(ErrorHandler::$levels[$type]) ? ErrorHandler::$levels[$type] : $type;
            $message = sprintf("%s: %s in %s line %d", $level, $error['message'], $error['file'], $error['line']);
            $exception = new FatalErrorException($message, 0, $type, $error['file'], $error['line']);
            $exceptionHandler[0]->handle($exception);
        }
    }
}
