<?php
namespace Topalis\Kernel;

use Psr\Log\LoggerInterface;
use Topalis\Kernel\Exception\ContextErrorException;
use Topalis\Util\StringUtil;

/**
 * ErrorHandler. Used to convert errors into exceptions or to log them.
 * Based on Symfony\Component\Debug\ErrorHandler
 *
 * @author Fabien Potencier <fabien@symfony.com>
 * @author Konstantin Myakshin <koc-dp@yandex.ru>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ErrorHandler
{
    private static $levels = array(
        E_WARNING           => "Warning",
        E_NOTICE            => "Notice",
        E_USER_ERROR        => "User Error",
        E_USER_WARNING      => "User Warning",
        E_USER_NOTICE       => "User Notice",
        E_STRICT            => "Runtime Notice",
        E_RECOVERABLE_ERROR => "Catchable Fatal Error",
        E_DEPRECATED        => "Deprecated",
        E_USER_DEPRECATED   => "User Deprecated",
        E_ERROR             => "Error",
        E_CORE_ERROR        => "Core Error",
        E_COMPILE_ERROR     => "Compile Error",
        E_PARSE             => "Parse",
    );

    private $level;

    private $reservedMemory;

    /**
     * @var LoggerInterface[] Loggers for channels
     */
    private static $loggers = array();

    /**
     * Registers the error handler.
     *
     * @param integer $level The level at which the conversion to Exception is
     *     done (null to use the error_reporting() value and 0 to disable)
     *
     * @return static The registered error handler
     */
    public static function register($level = null)
    {
        $handler = new static();
        $handler->setLevel($level);

        set_error_handler(array($handler, "handle"));
        register_shutdown_function(array($handler, "handleFatal"));
        $handler->reservedMemory = str_repeat("x", 10240);

        return $handler;
    }

    public function setLevel($level)
    {
        $this->level = null === $level ? error_reporting() : $level;
    }

    public static function setLogger(LoggerInterface $logger = null, $channel = null)
    {
        if (null !== $channel) {
            self::$loggers[$channel] = $logger;
            return;
        }

        if (!isset(self::$loggers['deprecation'])) {
            self::$loggers['deprecation'] = $logger;
        }
        if (!isset(self::$loggers['error'])) {
            self::$loggers['error'] = $logger;
        }
        if (!isset(self::$loggers['emergency'])) {
            self::$loggers['emergency'] = $logger;
        }
    }

    /**
     * @param integer $level
     * @param string $message
     * @param string $file
     * @param integer $line
     * @param array $context
     * @throws ContextErrorException When error_reporting returns error
     */
    public function handle($level, $message, $file = "unknown", $line = 0, array $context = array())
    {
        if (0 === $this->level) {
            return false;
        }

        $type = isset(self::$levels[$level]) ? self::$levels[$level] : $level;

        if ($level & (E_USER_DEPRECATED | E_DEPRECATED)) {
            // ignore message that comments with '#' in INI files are deprecated
            if (isset(self::$loggers['deprecation'])
                && !StringUtil::startsWith($message, "Comments starting with '#' are deprecated")) {
                $stack = $this->getBacktrace();
                self::$loggers['deprecation']->warning($message, array("type" => $type, "stack" => $stack));
            }

            return false;
        }

        if ($this->level & $level) {
            $errMsg = sprintf("%s: %s in %s line %d", $type, $message, $file, $line);
            // make sure the ContextErrorException class is loaded (https://bugs.php.net/bug.php?id=65322)
            if (!class_exists("Topalis\Kernel\Exception\ContextErrorException")) {
                // @codingStandardsIgnoreStart
                require __DIR__ . "/Exception/ContextErrorException.php";
                // @codingStandardsIgnoreEnd
            }
            throw new ContextErrorException($errMsg, $level, 0, $file, $line, $context);
        } else {
            if (isset(self::$loggers['error'])) {
                $stack = $this->getBacktrace();
                self::$loggers['error']->error($message, array("type" => $type, "stack" => $stack));
            }
        }

        return false;
    }

    public function handleFatal()
    {
        if (null === $error = error_get_last()) {
            return;
        }

        $this->reservedMemory = "";
        $type = $error['type'];
        if (0 === $this->level || !in_array($type, array(E_ERROR, E_CORE_ERROR, E_COMPILE_ERROR, E_PARSE))) {
            return;
        }

        if (isset(self::$loggers['emergency'])) {
            $fatal = array(
                'type' => $type,
                'file' => $error['file'],
                'line' => $error['line'],
            );

            self::$loggers['emergency']->emerg($error['message'], $fatal);
        }
    }

    protected function getBacktrace()
    {
        if (version_compare(PHP_VERSION, "5.4", "<")) {
            $stack = array_map(
                function ($row) {
                    unset($row['args']);

                    return $row;
                },
                array_slice(debug_backtrace(false), 0, 10)
            );
        } else {
            $stack = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 10);
        }

        return $stack;
    }
}
