<?php
namespace Topalis\Kernel\EventListener;

use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\HttpKernelInterface;

/**
 * Event Listener for changing the locale to the preffered one
 *
 * @author Florian Pfitzer <florian.pfitzer@to.com>
 *
 */
class LocaleListener
{
    /**
     * @var array
     */
    private $availableLanguages;

    /**
     * @var string
     */
    private $defaultLocale;

    /**
     * @var \Symfony\Component\HttpFoundation\Request
     */
    private $request;

    /**
     * @var \Symfony\Component\HttpFoundation\Session\SessionInterface
     */
    private $session;

    public function __construct($defaultLocale = "en")
    {
        $this->defaultLocale = $defaultLocale;
    }

    /**
     * @return array
     */
    public function getAvailableLanguages()
    {
        return $this->availableLanguages;
    }

    /**
     * @param array $availableLanguages
     */
    public function setAvailableLanguages(array $availableLanguages)
    {
        $this->availableLanguages = $availableLanguages;
    }

    /**
     * @return \Symfony\Component\HttpFoundation\Request
     */
    public function getRequest()
    {
        if ($this->request === null) {
            $this->request = \Symfony\Component\HttpFoundation\Request::createFromGlobals();
        }
        return $this->request;
    }

    /**
     * @param \Symfony\Component\HttpFoundation\Request $request
     */
    public function setRequest(\Symfony\Component\HttpFoundation\Request $request = null)
    {
        $this->request = $request;
    }

    /**
     * @return \Symfony\Component\HttpFoundation\Session\Session
     */
    public function getSession()
    {
        return $this->session;
    }

    /**
     * @return boolean true if a session has been set, false otherwise.
     */
    public function hasSession()
    {
        return $this->getSession() !== null;
    }

    /**
     * @param \Symfony\Component\HttpFoundation\Session\Session $session
     */
    public function setSession(\Symfony\Component\HttpFoundation\Session\Session $session)
    {
        $this->session = $session;
    }

    public function onKernelRequest(GetResponseEvent $event)
    {
        if (HttpKernelInterface::MASTER_REQUEST !== $event->getRequestType()) {
            return;
        }

        $request = $event->getRequest();
        if ($request->attributes->has("_locale")) {
            return;
        }

        $session = $this->getSession();
        if (null === $session) {
            return;
        }

        if (!$session->get("_locale")) {
            $this->updateLocaleFromRequest($request);
        } else {
            $request->setLocale($session->get("_locale"));
        }
    }

    /**
     * Detects the best combination of language and region from the request and
     * sets it as default locale. It is used to have on the one hand a supported
     * language in the locale and on the other hand a region that is used
     * initially for the iTunes store.
     *
     * @param Request $request The request.
     * @param string $language The language to use instead of the language in
     *     the request.
     */
    public function updateLocaleFromRequest(Request $request, $language = null)
    {
        $availableLanguages = $this->getAvailableLanguages();
        if (null !== $language && !in_array($language, $availableLanguages)) {
            $language = null;
        }
        $region = null;
        $firstRegion = null;
        foreach ($this->getRequest()->getLanguages() as $requestLanguage) {
            $requestRegion = \Locale::getRegion($requestLanguage);
            if (!$firstRegion) {
                $firstRegion = $requestRegion;
            }

            $requestPrimaryLanguage = \Locale::getPrimaryLanguage($requestLanguage);
            if (!$language) {
                if (in_array($requestPrimaryLanguage, $availableLanguages)) {
                    $language = $requestPrimaryLanguage;
                }
            }

            if ($requestPrimaryLanguage == $language && $requestRegion) {
                $region = $requestRegion;
            }

            if ($language && $region) {
                break;
            }
        }

        $defaultLanguage = \Locale::getPrimaryLanguage($this->defaultLocale);
        if (!$language) {
            $language = $defaultLanguage;
        }

        if (!$region) {
            $defaultRegion = \Locale::getRegion($this->defaultLocale);
            if ($firstRegion) {
                // use first region in request
                $region = $firstRegion;
            } else if ($defaultRegion && $language == $defaultLanguage) {
                $region = $defaultRegion;
            } else {
                switch ($language) {
                    case "de":
                        $region = "DE";
                        break;
                    case "en":
                        $region = "US";
                        break;
                }
            }
        }

        $locale = $language;
        if ($region) {
            $locale .= "_" . $region;
        }

        $this->setLocale($locale, $request);
    }

    protected function setLocale($locale, Request $request)
    {
        $session = $this->getSession();
        $session->set("_locale", $locale);
        $request->setLocale($locale);
    }
}
