<?php
namespace Topalis\Kernel\EventListener;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Session\SessionInterface;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\Routing\RouterInterface;
use Symfony\Component\Security\Core\SecurityContextInterface;

/**
 * Automatically logs out a user after a too long idle period.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class SessionIdleListener
{
    /**
     * @var int
     */
    protected $maxIdleTime;

    /**
     * @var RouterInterface
     */
    protected $router;

    /**
     * @var SecurityContextInterface
     */
    protected $securityContext;

    /**
     * @var SessionInterface
     */
    protected $session;

    public function __construct(SessionInterface $session, SecurityContextInterface $securityContext,
        RouterInterface $router, $maxIdleTime = 0)
    {
        $this->session = $session;
        $this->securityContext = $securityContext;
        $this->router = $router;
        $this->maxIdleTime = $maxIdleTime;
    }

    public function onKernelRequest(GetResponseEvent $event)
    {
        if (HttpKernelInterface::MASTER_REQUEST != $event->getRequestType()) {
            return;
        }

        $request = $event->getRequest();
        $this->session->start();
        // we use the value in the session (if available) so routes marked with
        // "_idle" aren't considered
        $lastUsed = $this->session->get("lastUsed") ?: $this->session->getMetadataBag()->getLastUsed();

        // don't update lastUsed for routes marked with "_idle"
        if (!$event->getRequest()->get("_idle")) {
            $this->session->set("lastUsed", $this->session->getMetadataBag()->getLastUsed());
        }

        if ($this->maxIdleTime > 0 && !$this->session->get("rememberLogin")) {
            $lapse = time() - $lastUsed;

            if ($lapse > $this->maxIdleTime) {
                $this->session->invalidate();
                $this->securityContext->setToken(null);
                $format = $request->getRequestFormat();
                if ("json" == $format) {
                    $event->setResponse(new JsonResponse(null, 401));
                } else {
                    $event->setResponse(new RedirectResponse($this->router->generate("admin_login")));
                }
            }
        }
    }
}
