<?php
namespace Topalis\Kernel\Job;

use Doctrine\Common\EventArgs;
use Doctrine\Common\EventSubscriber;
use Doctrine\ORM\EntityManagerInterface;

/**
 * The AbstractDeferredQueue extends AbstractQueue to put deferred jobs.
 *
 * @see putDeferred()
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
abstract class AbstractDeferredQueue extends AbstractQueue implements EventSubscriber
{
    /**
     * @var array
     */
    private $deferredJobs = array();

    /**
     * @var bool
     */
    private $entitiesScheduled;

    /**
     * @var EntityManagerInterface
     */
    private $entityManager;

    /**
     * Constructor.
     *
     * @param \Pheanstalk_PheanstalkInterface $pheanstalk
     * @param string $tube
     * @param EntityManagerInterface $entityManager If specified deferred jobs
     *    are put if no entities are scheduled.
     */
    public function __construct(\Pheanstalk_PheanstalkInterface $pheanstalk, $tube,
        EntityManagerInterface $entityManager = null)
    {
        parent::__construct($pheanstalk, $tube);
        if (null !== $entityManager) {
            $this->entityManager = $entityManager;
            $entityManager->getEventManager()->addEventSubscriber($this);
        }

        register_shutdown_function(array($this, "putDeferredJobs"));
    }

    public function getSubscribedEvents()
    {
        return array(
            "onClear",
            "postFlush",
        );
    }

    public function onClear(EventArgs $eventArgs)
    {
        $eventArgs; // unused

        /*
         * Assume there are no more entities scheduled. There might be still
         * scheduled entities if clear was called only for a single entitiy
         * class. Will be checked again when required.
         */
        $this->entitiesScheduled = false;

        // avoid commit pending check if there are no deferred jobs
        if ($this->deferredJobs && !$this->isCommitPending()) {
            $this->putDeferredJobs();
        }
    }

    public function postFlush(EventArgs $eventArgs)
    {
        $eventArgs; // unused

        /*
         * Assume there are no more entities scheduled. There might be still
         * scheduled entities if flush was called only for a single entitiy.
         * Will be checked again when required.
         */
        $this->entitiesScheduled = false;

        // avoid commit pending check if there are no deferred jobs
        if ($this->deferredJobs && !$this->isCommitPending()) {
            $this->putDeferredJobs();
        }
    }

    /**
     * Puts the deferred jobs into the queue.
     *
     * @see putDeferred()
     */
    public function putDeferredJobs()
    {
        while ($job = array_shift($this->deferredJobs)) {
            call_user_func_array(array($this, "put"), $job);
        }
    }

    /**
     * @return true if a deferred put is required, false otherwise.
     */
    protected function isDeferredPutRequired()
    {
        return null === $this->entityManager || $this->isCommitPending();
    }

    /**
     * Puts a job in the queue. The actual put will be deferred until
     * {@link putDeferredJobs()} is called if there are changed entities or no
     * entity manager was speciied. {@link putDeferredJobs()} will be called if
     * all scheduled entities are flushed, the entity manager is cleared, or
     * on script shutdown.
     *
     * @param array $data
     * @param int $priority
     * @param int $delay
     * @param int $ttr
     */
    protected function putDeferred(array $data, $priority = \Pheanstalk_PheanstalkInterface::DEFAULT_PRIORITY,
        $delay = \Pheanstalk_PheanstalkInterface::DEFAULT_DELAY, $ttr = \Pheanstalk_PheanstalkInterface::DEFAULT_TTR)
    {
        if ($this->isDeferredPutRequired()) {
            $this->deferredJobs[] = func_get_args();
        } else {
            $this->put($data, $priority, $delay, $ttr);
        }
    }

    /**
     * @return bool true if there's an active transaction or entities with
     *    uncommited changes, false otherwise.
     */
    private function isCommitPending()
    {
        if ($this->entityManager->getConnection()->isTransactionActive()) {
            return true;
        }

        if (!$this->entitiesScheduled) {
            $uow = $this->entityManager->getUnitOfWork();
            $uow->computeChangeSets();
            $this->entitiesScheduled = $uow->getScheduledEntityDeletions() ||
                $uow->getScheduledEntityInsertions() ||
                $uow->getScheduledEntityUpdates() ||
                $uow->getScheduledCollectionDeletions() ||
                $uow->getScheduledCollectionUpdates();
        }

        return $this->entitiesScheduled;
    }
}
