<?php
namespace Topalis\Kernel\Job;

use Doctrine\ORM\EntityManagerInterface;
use Psr\Log\LoggerAwareInterface;
use Psr\Log\LoggerInterface;
use Topalis\Database\Util\DatabaseUtils;
use Doctrine\DBAL\Driver\Connection;

/**
 * AbstractJobRunner
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
abstract class AbstractJobRunner implements JobRunnerInterface
{
    /**
     * @var Connection
     */
    protected $connection;

    /**
     * @var LoggerInterface
     */
    protected $logger;

    /**
     * @var QueueInterface
     */
    protected $queue;

    /**
     *
     * @param Connection $connection In case of an exception it will be checked
     *     if the connection is still open, otherwise it will be reopened. If
     *     that's not possible the run method will return 1.
     * @param LoggerInterface $logger
     * @param QueueInterface $queue
     */
    public function __construct(
        Connection $connection,
        LoggerInterface $logger,
        QueueInterface $queue
    )
    {
        $this->connection = $connection;
        $this->logger = $logger;
        $this->queue = $queue;
    }

    public function run()
    {
        while ($job = $this->queue->reserve()) {
            pcntl_signal_dispatch(); // ensure possible lograte is noticed
            try {
                $this->logger->debug("Process job " . $job->getId());
                DatabaseUtils::ensureConnected($this->connection);
                if (false === $this->preHandle($job)) {
                    continue;
                }
                $this->handle($job);
                $this->postHandle($job);
            } catch (\Exception $e) {
                $this->logger->error(
                    "Failed processing job",
                    array(
                        "job_id"    => $job->getId(),
                        "data"      => $job->getData(),
                        "exception" => $e,
                    )
                );
                $this->onException($job, $e);
                // db connection might be lost
                try {
                    DatabaseUtils::ensureConnected($this->connection);
                } catch (\Exception $e2) {
                    $this->logger->critical(
                        "Cannot access database: " . $e2->getMessage(),
                        array(
                            "exception" => $e2,
                        )
                    );

                    return 1;
                }
            }

            // avoid memory leaks
            $this->cleanup();
            gc_collect_cycles();
        }
    }

    /**
     * Called after the processing of a job is finished. This method will be
     * called no matter whether the handling of the job was successful or not.
     */
    protected function cleanup()
    {
    }

    abstract protected function handle(\Pheanstalk_Job $job);

    /**
     * Called after handle.
     *
     * @param \Pheanstalk_Job $job The handled job.
     */
    protected function postHandle(\Pheanstalk_Job $job)
    {
        $this->queue->delete($job);
    }

    /**
     * Called before handle.
     *
     * @param \Pheanstalk_Job $job The job to handle.
     *
     * @return boolean|NULL false if the job shall not be handled, otherwise
     *     true or null. In the first case neither handle nor postHandle will
     *     be called.
     */
    protected function preHandle(\Pheanstalk_Job $job)
    {
        return true;
    }

    /**
     * Called in case of an exception.
     *
     * @param \Pheanstalk_Job $job The processed job when the exception occurred.
     * @param \Exception $exception The occurred exception.
     */
    protected function onException(\Pheanstalk_Job $job, \Exception $exception)
    {
        $exception; // unused
        $this->queue->delete($job);
    }
}
