<?php
namespace Topalis\Kernel\Job;

use Topalis\Kernel\Exception\Job\JobReservedException;
use Topalis\Util\StringUtil;

/**
 * AbstractQueue
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
abstract class AbstractQueue implements QueueInterface
{
    /**
     * @var \Pheanstalk_PheanstalkInterface
     */
    protected $pheanstalk;

    /**
     * @var string name of the tube to use for this queue
     */
    protected $tube;

    public function __construct(\Pheanstalk_PheanstalkInterface $pheanstalk, $tube)
    {
        $this->pheanstalk = $pheanstalk;
        $this->tube = $tube;
    }

    public function delete(\Pheanstalk_Job $job)
    {
        try {
            $this->pheanstalk->delete($job);
            return true;
        } catch (\Pheanstalk_Exception_ServerException $e) {
            // check if job exists
            if ($this->isNotFound($e)) {
                try {
                    if ($this->isInState($job, "reserved")) {
                        throw new JobReservedException(
                            sprintf("Job %u cannot be deleted because it is reserved by another client", $job->getId())
                        );
                    }
                } catch (\Pheanstalk_Exception_ServerException $e2) {
                    if ($this->isNotFound($e2)) {
                        return false; // job does not exist
                    }
                }
            }

            throw $e;
        }
    }

    public function peek($jobId)
    {
        try {
            return $this->pheanstalk->peek($jobId);
        } catch (\Pheanstalk_Exception_ServerException $e) {
            if ($this->isNotFound($e)) {
                return null;
            }

            throw $e;
        }
    }

    public function release(\Pheanstalk_Job $job,
        $priority = \Pheanstalk_PheanstalkInterface::DEFAULT_PRIORITY, $delay = 0)
    {
        $this->pheanstalk->release($job, $priority, $delay);
    }

    public function reserve($timeout = null)
    {
        return $this->pheanstalk->reserveFromTube($this->tube, $timeout);
    }

    public function touch(\Pheanstalk_Job $job)
    {
        $this->pheanstalk->touch($job);
    }

    /**
     * @param \Pheanstalk_Job $job
     * @param string $state
     *
     * @return boolean true if the specified job is in the specified state,
     *     false otherwise.
     */
    protected function isInState(\Pheanstalk_Job $job, $state)
    {
        $jobStats = $this->pheanstalk->statsJob($job);
        return $jobStats['state'] === $state;
    }

    /**
     * @param \Pheanstalk_Exception_ServerException $e
     * @return boolean true if the exception was thrown because beanstalkd
     *     returned "NOT_FOUND", false otherwise.
     */
    protected function isNotFound(\Pheanstalk_Exception_ServerException $e)
    {
        return StringUtil::startsWith($e->getMessage(), \Pheanstalk_Response::RESPONSE_NOT_FOUND)
        || StringUtil::endsWith($e->getMessage(), \Pheanstalk_Response::RESPONSE_NOT_FOUND);
    }

    protected function put(array $data, $priority = \Pheanstalk_PheanstalkInterface::DEFAULT_PRIORITY,
        $delay = \Pheanstalk_PheanstalkInterface::DEFAULT_DELAY, $ttr = \Pheanstalk_PheanstalkInterface::DEFAULT_TTR)
    {
        return $this->pheanstalk->putInTube($this->tube, json_encode($data), $priority, $delay, $ttr);
    }
}
