<?php
namespace Topalis\Kernel\Security;

use Symfony\Component\Security\Core\Authentication\Provider\AuthenticationProviderInterface;
use Symfony\Component\Security\Core\Authentication\Token\TokenInterface;
use Symfony\Component\Security\Core\Authentication\Token\UsernamePasswordToken;
use Symfony\Component\Security\Core\Encoder\EncoderFactoryInterface;
use Symfony\Component\Security\Core\Exception\AuthenticationServiceException;
use Symfony\Component\Security\Core\Exception\BadCredentialsException;
use Symfony\Component\Security\Core\Exception\UsernameNotFoundException;
use Symfony\Component\Security\Core\User\UserInterface;
use Symfony\Component\Security\Core\User\UserProviderInterface;
use Topalis\Device\Entity\Device;

/**
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 */
class DeviceAuthenticationProvider implements AuthenticationProviderInterface
{
    private $hideUserNotFoundExceptions;
    private $providerKey;
    /**
     * @var \Symfony\Component\Security\Core\Encoder\EncoderFactoryInterface
     */
    private $encoderFactory;
    /**
     * @var \Symfony\Component\Security\Core\User\UserProviderInterface
     */
    private $userProvider;

    /**
     * Constructor.
     *
     * @param \Symfony\Component\Security\Core\User\UserProviderInterface $userProvider
     * @param string $providerKey A provider key
     * @param \Symfony\Component\Security\Core\Encoder\EncoderFactoryInterface $encoderFactory
     * @param boolean $hideUserNotFoundExceptions Whether to hide user not found exception or not
     *
     * @throws \InvalidArgumentException
     */
    public function __construct(UserProviderInterface $userProvider, $providerKey,
        EncoderFactoryInterface $encoderFactory, $hideUserNotFoundExceptions = true)
    {
        if (empty($providerKey)) {
            throw new \InvalidArgumentException('$providerKey must not be empty.');
        }
        $this->providerKey = $providerKey;
        $this->hideUserNotFoundExceptions = $hideUserNotFoundExceptions;
        $this->encoderFactory = $encoderFactory;
        $this->userProvider = $userProvider;
    }

    /**
     * Attempts to authenticates a TokenInterface object.
     *
     * @param TokenInterface $token The TokenInterface instance to authenticate
     *
     * @throws BadCredentialsException
     * @throws \Exception|UsernameNotFoundException
     * @return TokenInterface An authenticated TokenInterface instance, never null
     */
    public function authenticate(TokenInterface $token)
    {
        if (!$this->supports($token)) {
            return null;
        }

        $device = null;
        $username = $token->getUsername();
        if (empty($username)) {
            $username = 'NONE_PROVIDED';
        }

        try {
            $device = $this->retrieveDevices($username, $token);
        } catch (UsernameNotFoundException $notFound) {
            if ($this->hideUserNotFoundExceptions) {
                throw new BadCredentialsException('Bad credentials', 0, $notFound);
            }
            $notFound->setUsername($username);

            throw $notFound;
        }

        $authenticatedToken = new UsernamePasswordToken($device, $token->getCredentials(), $this->providerKey, array());
        $authenticatedToken->setAttributes($token->getAttributes());
        if (!$device->isIPhone()) {
            // used to encrypt the PKCS12 client certificate container
            $authenticatedToken->setAttribute("password", $token->getCredentials());
        }

        return $authenticatedToken;
    }

    protected function retrieveDevices($username, UsernamePasswordToken $token)
    {
        $user = $token->getUser();
        if ($user instanceof Device) {
            return $user;
        }

        try {
            $devices = $this->userProvider->loadUserByUsername($username);

            $deviceFound = false;
            $device = null;
            foreach ($devices as $device) {
                /** @var $device Device */

                try {
                    $this->checkAuthentication($device, $token);
                    $deviceFound = true;

                    break;
                } catch (BadCredentialsException $e) {
                    // empty
                }
            }

            if (!$deviceFound) {
                throw new BadCredentialsException('The presented password is invalid.');
            }

            if (!$device instanceof Device) {
                throw new AuthenticationServiceException('The user provider must return a UserInterface object.');
            }

            return $device;
        } catch (UsernameNotFoundException $notFound) {
            $notFound->setUsername($username);
            throw $notFound;
        } catch (\Exception $repositoryProblem) {
            $ex = new AuthenticationServiceException($repositoryProblem->getMessage(), 0, $repositoryProblem);
            $ex->setToken($token);
            throw $ex;
        }
    }

    protected function checkAuthentication(Device $device, UsernamePasswordToken $token)
    {
        $currentUser = $token->getUser();
        if ($currentUser instanceof UserInterface) {
            if ($currentUser->getPassword() !== $device->getPassword()) {
                throw new BadCredentialsException('The credentials were changed from another session.');
            }
        } else {
            if ("" === ($presentedPassword = $token->getCredentials())) {
                throw new BadCredentialsException('The presented password cannot be empty.');
            }

            list($salt, $password) = explode(":", $device->getPassword(), 2);
            if (!$this->encoderFactory->getEncoder($device)->isPasswordValid($password, $presentedPassword, $salt)) {
                throw new BadCredentialsException('The presented password is invalid.');
            }
        }
    }

    /**
     * Checks whether this provider supports the given token.
     *
     * @param TokenInterface $token A TokenInterface instance
     *
     * @return Boolean true if the implementation supports the Token, false otherwise
     */
    public function supports(TokenInterface $token)
    {
        return $token instanceof UsernamePasswordToken && $this->providerKey === $token->getProviderKey();
    }
}
