<?php
namespace Topalis\Log\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Topalis\Log\Form\LogForm;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpFoundation\Response;
use Topalis\Kernel\Controller;
use Topalis\Kernel\Exception\Http\NotFoundException;
use Topalis\Log\FilteredLog;
use Topalis\Log\Log;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Symfony\Component\HttpFoundation\BinaryFileResponse;

/**
 * Controller for log viewer
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 *
 * @Route("/admin/log", service="log.controller")
 */
class LogController extends Controller
{
    /**
     * @return \Topalis\Log\LogManager
     */
    private function getLogManager()
    {
        return $this->container->get("log.manager");
    }

    /**
     * @return \Topalis\Log\WebDavManager
     */
    private function getWebDavManager()
    {
        return $this->container->get("log.webdav.manager");
    }

    /**
     * @Route("/dav", name="admin_log_dav", methods="POST")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function davAction()
    {
        $form = $this->getForm();
        $form->bind($this->getRequest());

        if (!$form->isValid()) {
            return $this->indexAction();
        }

        $data = $form->getData();
        $this->getWebDavManager()->setEnabled($data['davEnabled']);

        $this->addFlash("success", $this->tr("admin.log.savedConfigSuccess"));
        return $this->redirect($this->generateUrl("admin_log"));
    }

    /**
     * @Route("", name="admin_log", methods="GET")
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function indexAction()
    {
        $logNames = array();
        $logs = $this->getLogManager()->getLogs();
        foreach ($logs as $log) {
            /** @var $log Log */
            $logNames[] = $log->getName();
        }
        array_multisort($logNames, $logs);

        return $this->render("TopalisLogBundle::log.html.twig", array(
            "logs" => $logs, "form" => $this->getForm()->createView(),
            "url" => $this->getRequest()->getSchemeAndHttpHost() . "/logs"
        ));
    }

    /**
     * @Route("/download", name="admin_log_download_all")
     * @return \Symfony\Component\HttpFoundation\BinaryFileResponse
     */
    public function downloadAllAction()
    {
        $tmpFile = tempnam(sys_get_temp_dir(), 'auralisLogs');
        register_shutdown_function(array("\Topalis\Util\FileSystemUtil", "tryUnlinkFile"), $tmpFile);
        $responseFilename = "auralis-logs_" . date("Ymd_His") . ".tar.gz";
        $this->getLogManager()->compressLogs($tmpFile);
        $response = new BinaryFileResponse($tmpFile, 200, array(), false);
        $response->setContentDisposition(ResponseHeaderBag::DISPOSITION_ATTACHMENT, $responseFilename);
        return $response;
    }

    /**
     * @Route("/{id}/download", name="admin_log_download")
     *
     * @param int $id The log id.
     * @throws NotFoundException
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function downloadAction($id)
    {
        $log = $this->getLogManager()->getById($id);
        if (null === $log) {
            throw new NotFoundException();
        }

        $response = new Response($log->getContent());
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            $log->getFilename()
        );
        $response->headers->set("Content-Disposition", $disposition);
        $response->headers->set("Content-Type", "text/plain");
        return $response;

    }

    /**
     * @Route("/{id}/view", name="admin_log_view")
     *
     * @param int $id The log id.
     * @throws NotFoundException
     * @return \Symfony\Component\HttpFoundation\Response
     */
    public function viewAction($id)
    {
        $log = $this->getLogManager()->getById($id);
        if (null === $log) {
            throw new NotFoundException();
        }

        return $this->render("TopalisLogBundle::log_view.html.twig", array("log" => $log));
    }

    /**
     * @Route("/{id}/refresh", name="admin_log_refresh", options={"expose" = true})
     *
     * @param int $id The log id.
     * @throws NotFoundException
     * @return \Symfony\Component\HttpFoundation\JsonResponse
     */
    public function refreshAction($id)
    {
        $log = $this->getLogManager()->getById($id);
        if (null === $log) {
            throw new NotFoundException();
        }

        $line = $this->getRequest()->query->getInt("lines") + 1;
        $filter = $this->getRequest()->query->get("filter");

        if (!empty($filter)) {
            $log = new FilteredLog($log, $filter, true);
        }
        if ($line == 0) {
            $content = $log->getContent();
        } else {
            $content = $log->getContentFromLine($line);
        }
        return new JsonResponse(array("content" => nl2br($content), "lines" => $log->getLines()));
    }

    /**
     *
     * @return \Symfony\Component\Form\FormInterface
     */
    protected function getForm()
    {
        static $form = null;
        if ($form === null) {
            $form = $this->createForm(new LogForm());
            $data = array("davEnabled" => $this->getConfig()->isDavEnabled());
            $form->setData($data);
        }
        return $form;
    }
}
