<?php
namespace Topalis\Log;

use Topalis\Util\Process;

/**
 * Class for getting log files
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class LogManager
{
    /**
     * @var \Symfony\Component\Translation\TranslatorInterface
     */
    private $translator;

    /**
     * @var string
     */
    private $generalLogFilename;

    /**
     * @var string
     */
    private $httpdLogDir = "/var/log/httpd";

    /**
     * Saves the log files into a compressed tar archive.
     *
     * @param string $filename The archive file to use.
     * @param string|NULL $algorithm The compression algorighm to use. Valid
     *     values: bzip2, xz, gzip, or null or an empty string to avoid
     *     compression. (Default: gzip)
     * @throws \InvalidArgumentException If an invalid algorithm is specified.
     * @throws \Topalis\Util\Exception\Process\ExecException If executing the
     *     compression command fails.
     */
    public function compressLogs($filename, $algorithm = "gzip")
    {
        $cmd = "tar -cf " . $filename . " ";
        switch ($algorithm) {
            case "bzip2":
                $cmd .= "-j";
                break;
            case "xz":
                $cmd .= "-J";
                break;
            case "gzip":
                $cmd .= "-z";
                break;
            case "":
                /* fall through */
            case null:
                // no compression
                break;
            default:
                throw new \InvalidArgumentException("'" . $algorithm . "' is not a valid compression algorithm");
        }

        foreach ($this->getLogs() as $log) {
            $cmd .= " " . $log->getRealPath();
        }

        Process::execThrow($cmd);
    }

    public function getLogs()
    {
        static $logs = null;
        if ($logs === null) {
            $logs = array(
                new Log($this->httpdLogDir . "/device.ssl_error_log", $this->tr("admin.log.logs.deviceError")),
                new Log($this->httpdLogDir . "/device.ssl_access_log", $this->tr("admin.log.logs.deviceAccess")),
                new Log($this->httpdLogDir . "/admin.ssl_error_log", $this->tr("admin.log.logs.adminError")),
                new Log($this->httpdLogDir . "/admin.ssl_access_log", $this->tr("admin.log.logs.adminAccess"), false),
                new Log("/var/log/auralis/php/error.log", $this->tr("admin.log.logs.error")),
                new Log($this->generalLogFilename, $this->tr("admin.log.logs.general")),
                new Log("/var/log/auralis/activesyncfilterd/activesyncfilterd.log",
                    $this->tr("admin.log.logs.activesyncfilterd")),
                new Log("/var/log/auralis/apnsd.log", $this->tr("admin.log.logs.apnsd")),
                new Log("/var/log/auralis/gcmd.log", $this->tr("admin.log.logs.gcmd")),
                new Log("/var/log/auralis/update.log", $this->tr("admin.log.logs.update")),
                new Log("/var/log/auralis/php-fpm/device-error.log", $this->tr("admin.log.logs.phpFpmDeviceError")),
                new Log("/var/log/auralis/php-fpm/admin-error.log", $this->tr("admin.log.logs.phpFpmAdminError")),
            );
            // remove Logs that don't exists as they can cause errors while rendering
            for ($i = count($logs) - 1; $i >= 0; --$i) {
                if (!($logs[$i]->isFile())) {
                     unset($logs[$i]);
                }
            }
        }
        return $logs;
    }

    /**
     * find a log by id, will return null if no log found
     *
     * @param string $id ID of the log to find
     *
     * @return null|Log
     */
    public function getById($id)
    {
        $logs = self::getLogs();
        foreach ($logs as $log) {
            if ($id == $log->getId()) {
                return $log;
            }
        }

        return null;
    }

    /**
     * @param string $filename
     */
    public function setGeneralLogFilename($filename)
    {
        $this->generalLogFilename = $filename;
    }

    /**
     * @param string $httpdLogDir
     */
    public function setHttpdLogDir($httpdLogDir)
    {
        $this->httpdLogDir = $httpdLogDir;
    }

    /**
     * @return \Symfony\Component\Translation\TranslatorInterface
     */
    protected function getTranslator()
    {
        return $this->translator;
    }

    /**
     * @param \Symfony\Component\Translation\TranslatorInterface $translator
     */
    public function setTranslator(\Symfony\Component\Translation\TranslatorInterface $translator)
    {
        $this->translator = $translator;
    }

    /**
     * Convenience function to call getTranslator()->trans().
     *
     * @param string $id The message id.
     * @param array $parameters An array of parameters for the message.
     * @param string $domain The domain for the message.
     * @param string $locale The locale.
     * @return string The translated string.
     */
    protected function tr($id, array $parameters = array(), $domain = 'messages', $locale = null)
    {
        return $this->getTranslator()->trans($id, $parameters, $domain, $locale);
    }
}
