<?php
namespace Topalis\MDM\Command;

use CFPropertyList\CFDictionary;
use CFPropertyList\CFPropertyList;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Helper\DialogHelper;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Topalis\Device\Entity\Device;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\MDM\MdmRequest;
use Topalis\Profile\ProfileFactory;
use Topalis\Profile\UUIDGenerator;

/**
 *
 *
 * @author Florian Pfitzer <pfitzer@w3p.cc>
 */
class SendMdmCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName('mdm:send')
            ->setDefinition(array(
                new InputArgument("input_file", InputArgument::REQUIRED, "plist file with mdm command"),
                new InputOption("device", "", InputOption::VALUE_OPTIONAL, "Device id"),
                new InputOption("profile", "", InputOption::VALUE_NONE,
                    "file is a profile. A InstallProfile will be generated"),
                new InputOption("output", "", InputOption::VALUE_OPTIONAL,
                    "output response in this file. Can be used with --response option"),
                new InputOption("response", "", InputOption::VALUE_OPTIONAL,
                    "file with a response from --output option"),
            ))
            ->setDescription('Send mdm command to a device and wait for response');
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $deviceEntityManager = $this->getContainer()->get("device.entity.manager.device");
        if (null === $input->getOption("device")) {
            /** @var DialogHelper $dialog */
            $dialog = $this->getHelper("dialog");
            /** @var Device[] $devices */
            $devices = $deviceEntityManager->createQueryBuilder("d")
                ->select("d")
                ->where("d.mdmtoken IS NOT NULL")
                ->getQuery()
                ->getResult()
            ;
            $deviceList = array();
            foreach ($devices as $device) {
                $deviceList[$device->getId()] = $device->getName();
            }
            $device = $dialog->select($output, "Choose a device", $deviceList);
        } else {
            $device = $input->getOption("device");
        }
        /** @var Device $device */
        $device = $deviceEntityManager->find($device);
        $admin = $this->getContainer()->get("admin.entity.manager.admin")->find(0);
        $commandEntityManager = $this->getContainer()->get("mdm.entity.manager.mdm_command");

        $uuidGenerator = new UUIDGenerator();
        $uuid = $uuidGenerator->generateUUID();
        $plist = new CFPropertyList($input->getArgument("input_file"));

        if ($input->getOption("profile")) {
            $output->writeln("<info>Creating InstallProfile command </info>");
            $profile = ProfileFactory::fromDictionary($plist->getValue());
            $command = $this->getContainer()->get("mdm.admin")->installProfile($device, $profile);
        } else {
            $output->writeln(sprintf("<info>Sending Command %s</info>", $uuid));
            /** @var CFDictionary $dict */
            $dict = $plist->getValue();
            $dict->get("CommandUUID")->setValue($uuid);
            $command = new MdmCommand();
            $command
                ->setDevice($device)
                ->setStatus(MdmCommand::STATUS_NEW)
                ->setAdmin($admin)
                ->setUuid($uuid)
                ->setRequestType($dict->get("Command")->get("RequestType")->getValue())
                ->setRequest($plist->toXML())
            ;
            $commandEntityManager->save($command);
        }
        if (null !== $responseFile = $input->getOption("response")) {
            $response = include $responseFile;
            $response['CommandUUID'] = $command->getUuid();
            $response['UDID'] = $device->getDeviceid();
            $request = new MdmRequest($response);
            $this->getContainer()->get("mdm.service")->handleMdmRequest($request);
        } else {
            $output->writeln("<info>waiting for response</info>");
            while (true) {
                $commandEntityManager->refresh($command);
                if (null !== $command->getResponseTime()) {
                    ladybug_set_option("general.show_backtrace", false);
                    ld($command->getResponse());
                    if (null !== $outputFile = $input->getOption("output")) {
                        file_put_contents($outputFile, "<?php return ".var_export($command->getResponse(), true).";");
                    }
                    break;
                }
                sleep(3);
            }
        }

    }
}
