<?php

namespace Topalis\MDM\Entity;

use Doctrine\ORM\Mapping as ORM;
use Topalis\Device\Entity\Device;
use Topalis\Device\MDM\MdmCommandInterface;

/**
 * MdmCommand
 *
 * @ORM\Table(name="mdmcommands", indexes={@ORM\Index(name="idx_mdmcommands_uuid", columns={"uuid"})})
 * @ORM\Entity
 */
class MdmCommand implements MdmCommandInterface
{
    const STATUS_NEW = "New";
    const STATUS_QUEUED = "Queued";
    const STATUS_CANCELED = "Canceled";
    const STATUS_ACKNOWLEDGED = "Acknowledged";
    const STATUS_IDLE = "Idle";
    const STATUS_NOT_NOW = "NotNow";
    const STATUS_ERROR = "Error";
    const STATUS_COMMAND_FORMAT_ERROR = "CommandFormatError";

    /**
     * @var string[] Commands that cannot be blocked.
     */
    public static $NON_BLOCKABLE = array(
        "DeviceInformation",
        "ProfileList",
        "DeviceLock",
        "EraseDevice",
        "ClearPasscode",
        "CertificateList",
        "ProvisioningProfileList",
        "InstalledApplicationList",
        "Restrictions"
    );

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="bigint", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    private $id;

    /**
     * @var string
     *
     * @ORM\Column(name="uuid", type="string", length=255, nullable=false,
     *     options={"comment"="UUID of the MDM command"})
     */
    private $uuid;

    /**
     * The administrator that generated the MDM command.
     * @var \Topalis\Admin\Entity\Admin
     *
     * @ORM\ManyToOne(targetEntity="Topalis\Admin\Entity\Admin")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="admin_id", referencedColumnName="id", onDelete="SET NULL")
     * })
     */
    private $admin;

    /**
     * The device the MDM command is bound for.
     * @var Device
     * @ORM\ManyToOne(targetEntity="Topalis\Device\Entity\Device", inversedBy="mdmCommands")
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="device_id", referencedColumnName="id", onDelete="CASCADE")
     * })
     */
    private $device;

    /**
     * @var integer
     *
     * @ORM\Column(name="createtime", type="integer", nullable=false,
     *     options={"unsigned"=true, "comment"="The creation time of the command"})
     */
    private $createTime;

    /**
     * @var string
     *
     * @ORM\Column(name="request", type="text", length=16777215, nullable=true,
     *     options={"comment"="The MDM command request"})
     */
    private $request;

    /**
     * @var string
     *
     * @ORM\Column(name="requesttype", type="string", length=64, nullable=false,
     *     options={"comment"="The type of the MDM command request"})
     */
    private $requestType;

    /**
     * @var boolean
     *
     * @ORM\Column(name="blockable", type="boolean", nullable=false,
     *     options={"comment"="Indicates if the command can be blocked by the device"})
     */
    private $blockable;

    /**
     * @var string
     *
     * @ORM\Column(name="status", type="string", length=64, nullable=false,
     *     options={"default"=MdmCommand::STATUS_NEW, "comment"="The current status of the command"})
     */
    private $status = self::STATUS_NEW;

    /**
     * @var integer
     *
     * @ORM\Column(name="copy_count", type="smallint", nullable=false, options={"unsigned"=true})
     */
    protected $copyCount = 0;

    /**
     * @var string
     *
     * @ORM\Column(name="response", type="text", length=65532, nullable=true,
     *     options={"comment"="The device response to the MDM command"})
     */
    private $response;

    /**
     * @var integer
     *
     * @ORM\Column(name="responsetime", type="integer", nullable=true,
     *     options={"unsigned"=true, "comment"="The time of the response"})
     */
    private $responseTime;

    /**
     * @var \Topalis\Mdm\Entity\MdmQueueEntry
     *
     * @ORM\OneToOne(targetEntity="Topalis\MDM\Entity\MdmQueueEntry", mappedBy="command", cascade={"all"})
     */
    private $queueEntry;

    /**
     * @var string
     *
     * @ORM\Column(name="info", type="string", length=255, nullable=true)
     */
    protected $info;

    public function __construct()
    {
        $this->createTime = time();
    }

    /**
     * @return int
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * @return string The UUID of the command.
     */
    public function getUuid()
    {
        return $this->uuid;
    }

    /**
     * Sets the UUID.
     *
     * @param string $uuid The UUID to set.
     *
     * @return $this
     */
    public function setUuid($uuid)
    {
        $this->uuid = $uuid;

        return $this;
    }

    public function setDevice(Device $device)
    {
        $this->device = $device;

        return $this;
    }

    /**
     * @return Device
     */
    public function getDevice()
    {
        return $this->device;
    }

    /**
     * @return string The command request as XML.
     */
    public function getRequest()
    {
        return $this->request;
    }

    /**
     * Sets the command request.
     *
     * @param string $request The command request to set as XML.
     */
    public function setRequest($request)
    {
        $this->request = $request;

        return $this;
    }

    /**
     * @return string The request type.
     */
    public function getRequestType()
    {
        return $this->requestType;
    }

    /**
     *
     * @return string The request type.
     */
    public function getType()
    {
        return $this->getRequestType();
    }

    /**
     * Sets the request type and sets the status of blockable accordingly.
     *
     * @param string $requestType The request type to set.
     *
     * @return $this
     * @see getBlockable()
     */
    public function setRequestType($requestType)
    {
        $this->requestType = $requestType;
        $this->setBlockable(array_search($requestType, self::$NON_BLOCKABLE) === false);

        return $this;
    }

    /**
     * @return \Topalis\Admin\Entity\Admin The administrator that generated the command.
     */
    public function getAdmin()
    {
        return $this->admin;
    }

    /**
     * Sets the administrator that generated the command.
     *
     * @param \Topalis\Admin\Entity\Admin $admin
     */
    public function setAdmin(\Topalis\Admin\Entity\Admin $admin)
    {
        $this->admin = $admin;

        return $this;
    }

    /**
     * @return int The creation time of the command. Is set to the current time
     *     in the constructor.
     */
    public function getCreateTime()
    {
        return $this->createTime;
    }

    /**
     * @param int $createTime The creation time of the command. Is set to the
     *     current time in the constructor.
     */
    public function setCreateTime($createTime)
    {
        return $this->createTime = $createTime;
    }

    /**
     * @return boolean true, if the command may be blocked by the device, false
     *     otherwise.
     */
    public function isBlockable()
    {
        return (boolean) $this->blockable;
    }

    /**
     * Sets if the command may be blocked by the device.
     *
     * @param boolean $blockable true, if the command bay be blocked by the
     *     device, false otherwise.
     */
    public function setBlockable($blockable)
    {
        $this->blockable = $blockable;

        return $this;
    }

    public function getCopyCount()
    {
        return $this->copyCount;
    }

    public function incCopyCount()
    {
        $this->copyCount++;

        return $this;
    }

    public function setCopyCount($copyCount)
    {
        $this->copyCount = $copyCount;

        return $this;
    }

    /**
     * @return array The command response of the device.
     */
    public function getResponse()
    {
        if ($this->response === null) {
            return null;
        }
        return unserialize($this->response);
    }

    /**
     * Sets the command response of the device.
     * @param array $response
     */
    public function setResponse(array $response)
    {
        $this->response = serialize($response);

        return $this;
    }

    /**
     * @return int UNIX timestamp when the command was answered by the client.
     */
    public function getResponseTime()
    {
        return $this->responseTime;
    }

    /**
     * Set the time when the command was answered by the client.
     *
     * @param int $responseTime UNIX timestamp to set.
     */
    public function setResponseTime($responseTime)
    {
        $this->responseTime = $responseTime;

        return $this;
    }

    /**
     * @return string Status of the command. See class constants for possible
     *     statuses.
     */
    public function getStatus()
    {
        return $this->status;
    }

    /**
     * Sets the status of the command. See class constants for possible statuses.
     *
     * @param string $status The status to set.
     */
    public function setStatus($status)
    {
        $this->status = $status;

        return $this;
    }

    /**
     * @param \Topalis\Mdm\Entity\MdmQueueEntry $queueEntry
     *
     * @return self
     */
    public function setQueueEntry(\Topalis\MDM\Entity\MdmQueueEntry $queueEntry = null)
    {
        if ($this->queueEntry != $queueEntry) {
            $this->queueEntry = $queueEntry;
            if ($queueEntry !== null) {
                $queueEntry->setCommand($this);
            }
        }

        return $this;
    }

    /**
     * @return \Topalis\Mdm\Entity\MdmQueueEntry
     */
    public function getQueueEntry()
    {
        return $this->queueEntry;
    }

    /**
     * @return string Information about the command.
     */
    public function getInfo()
    {
        return $this->info;
    }

    /**
     * Sets the information about the command.
     *
     * @param string $info The command information to set
     *
     * @return MdmCommand self
     */
    public function setInfo($info)
    {
        $this->info = $info;

        return $this;
    }
}
