<?php

namespace Topalis\MDM\Entity;

use Doctrine\ORM\Mapping as ORM;
use Topalis\Device\Entity\Device;

/**
 * MdmQueue
 *
 * @ORM\Table(name="mdmqueue")
 * @ORM\Entity
 * @ORM\HasLifecycleCallbacks
 */
class MdmQueueEntry
{
    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="bigint", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    private $id;

    /**
     * The device the MDM command is bound for.
     * @var \Topalis\Device\Entity\Device
     * @ORM\ManyToOne(targetEntity="Topalis\Device\Entity\Device", inversedBy="mdmQueue", cascade={"persist"})
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="device_id", referencedColumnName="id", nullable=false, onDelete="CASCADE")
     * })
     */
    private $device;

    /**
     * The command in the queue.
     * @var MdmCommand
     *
     * @ORM\OneToOne(targetEntity="Topalis\MDM\Entity\MdmCommand", inversedBy="queueEntry", cascade={"persist"})
     * @ORM\JoinColumns({
     *   @ORM\JoinColumn(name="command_id", referencedColumnName="id", nullable=false, onDelete="CASCADE")
     * })
     */
    private $command;

    /**
     * @var boolean
     *
     * @ORM\Column(name="blockable", type="boolean", nullable=false,
     *     options={"comment"="Indicates if the command can be blocked by the device"})
     */
    private $blockable;

    /**
     * @var boolean
     *
     * @ORM\Column(name="waiting", type="boolean", nullable=false,
     *     options={"comment"="Inidcates if the system is waiting for a response i.e. push has been sent"})
     */
    private $waiting = false;

    /**
     * @var integer
     *
     * @ORM\Column(name="pushtime", type="integer", nullable=true,
     *     options={"unsigned"=true,
     *         "comment"="The time the last push has been sent for the device referenced by the queue entry"
     *     })
     */
    private $pushTime;

    public function getId()
    {
        return $this->id;
    }

    /**
     * @return int The command id.
     */
    public function getCommandId()
    {
        return $this->getCommand()->getId();
    }

    /**
     * @param \Topalis\MDM\Entity\MdmCommand $command
     *
     * @return self
     */
    public function setCommand(MdmCommand $command)
    {
        if ($this->command != $command) {
            $this->command = $command;
            $command->setQueueEntry($this);
        }

        return $this;
    }

    /**
     * @return \Topalis\MDM\Entity\MdmCommand
     */
    public function getCommand()
    {
        return $this->command;
    }

    public function setDevice(Device $device)
    {
        $this->device = $device;
        if (!$device->getMdmQueue()->contains($this)) {
            $device->getMdmQueue()->add($this);
        }

        return $this;
    }

    /**
     * @return \Topalis\Device\Entity\Device
     */
    public function getDevice()
    {
        return $this->device;
    }

    /**
     * @return boolean true if the command can be blocked, false otherwise.
     */
    public function isBlockable()
    {
        return (boolean) $this->blockable;
    }

    /**
     * Sets if the command can be blocked.
     *
     * @param boolean $blockable true, if the command can be blocked, false
     *     otherwise.
     */
    public function setBlockable($blockable)
    {
        $this->blockable = $blockable;
    }

    /**
     * @return int The UNIX timestamp when the last push has been sent for the
     *     device referenced by this queue entry.
     */
    public function getPushTime()
    {
        return $this->pushTime;
    }

    /**
     * Sets the time the last push has been sent for the device referenced by
     *     this queue entry.
     * @param int $pushTime The UNIX timestamp to set.
     */
    public function setPushTime($pushTime = null)
    {
        $this->pushTime = $pushTime;
    }

    /**
     * @return boolean true if a push has been sent and we are waiting for the
     *     device's request, false otherwise.
     */
    public function isWaiting()
    {
        return (boolean) $this->waiting;
    }

    /**
     * Sets the waiting status.
     *
     * @param boolean $waiting true if a push has been sent and we are waiting
     *     for the device's request, false otherwise.
     */
    public function setWaiting($waiting)
    {
        $this->waiting = $waiting;
    }

    /**
     * @ORM\PreRemove
     */
    public function preRemove()
    {
        $this->getDevice()->getMdmQueue()->removeElement($this);
        $this->getCommand()->setQueueEntry(null);
    }

}
