<?php
namespace Topalis\MDM;

use Topalis\Admin\Entity\Admin;
use Topalis\App\Entity\App;
use Topalis\App\Entity\AppState;
use Topalis\App\Entity\IosAppState;
use Topalis\Profile\Payload\DummyPayload;
use Topalis\Kernel\AbstractAuralisService;
use Topalis\Profile\AbstractProfile;
use Topalis\Device\Entity\MailAccount;
use Topalis\Device\Entity\Device;
use Topalis\Profile\ProfileFactory;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\MDM\Exception\MdmException;
use Topalis\MDM\Request\ApplyRedemptionCodeRequest;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\MDM\Request\InviteToProgramRequest;
use Topalis\User\Entity\User;
use Topalis\MDM\Event\MdmCommandEvents;
use Topalis\MDM\Event\MdmCommandResendEvent;

/**
 * This class contains methods to manage MDM devices.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class MdmAdmin extends AbstractAuralisService implements MdmAdminInterface
{
    /**
     * @var \Topalis\Admin\Entity\Admin
     */
    private $admin;

    /**
     * @return \Topalis\Admin\Entity\AdminEntityManager
     */
    private function getAdminEntityManager()
    {
        return $this->container->get("admin.entity.manager.admin");
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Profile\ProfileFactory
     */
    private function getProfileFactory()
    {
        return $this->container->get("profile.factory");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    private function getDeviceOs($typeName)
    {
        $deviceTypeFactory = $this->container->get("device.type.factory");
        return $deviceTypeFactory->getType($typeName)->getId();
    }

    /**
     * @return \Symfony\Component\EventDispatcher\EventDispatcherInterface
     */
    private function getEventDispatcher()
    {
        return $this->container->get("event_dispatcher");
    }

    /**
     * @return \Topalis\App\Entity\AppEntityManager
     */
    public function getIosAppEntityManager()
    {
        return $this->container->get("app.entity.manager.ios_app");
    }

    /**
     * @return \Topalis\App\Entity\AppStateEntityManager
     */
    private function getIosAppStateEntityManager()
    {
        return $this->container->get("app.entity.manager.ios_app_state");
    }

    /**
     * @return \Topalis\MDM\Entity\MdmCommandEntityManager
     */
    private function getMdmCommandEntityManager()
    {
        return $this->container->get("mdm.entity.manager.mdm_command");
    }

    /**
     * @return \Topalis\MDM\MdmQueue
     */
    private function getMdmQueue()
    {
        return $this->container->get("mdm.queue");
    }

    /**
     * @return \Symfony\Component\Security\Core\SecurityContext
     */
    private function getSecurityContext()
    {
        return $this->container->get("security.context");
    }

    /**
     *
     * @return Device[] Array of active (i.e. not disabled, not deleted, no
     *     wipe pending) managed devices.
     */
    public function getActiveManagedDevices()
    {
        $devices = $this->getDeviceEntityManager()->createQueryBuilder("d")
            ->select("d")
            ->where("d.os = :ios OR d.os = :android")
            ->andWhere("d.disabled = 0")
            ->andWhere("d.deleted = 0")
            ->andWhere("d.wipestatus = :wipestatus")
            ->leftJoin("d.mdmCert", "c")
            ->andWhere("c is NOT NULL")
            ->setParameter("ios", $this->getDeviceOs("ios"))
            ->setParameter("android", $this->getDeviceOs("android"))
            ->setParameter("wipestatus", Device::WIPE_NONE)
            ->getQuery()
            ->getResult()
        ;

        return $devices;
    }

    /**
     *
     * @return \Topalis\Admin\Entity\Admin
     */
    public function getAdmin()
    {
        if ($this->admin === null) {
            $token = $this->getSecurityContext()->getToken();
            if (null !== $token && $token->getUser() instanceof Admin) {
                $this->admin = $token->getUser();
            } else {
                $this->admin = $this->getAdminEntityManager()->find(0);
            }
        } else if (!$this->getAdminEntityManager()->contains($this->admin)) {
            // if the admin is not managed anymore because the enity manager was
            // cleared it has to be reloaded to prevent an exception
            $this->admin = $this->getAdminEntityManager()->find($this->admin->getId());
        }

        return $this->admin;
    }

    public function removeApp(AppState $state)
    {
        $this->getAppAdminForApp($state->getApp())->removeApp($state);
    }

    public function removeApps(App $app)
    {
        $this->getAppAdminForApp($app)->removeApps($app);
    }

    public function removeUnassignedApps(App $app)
    {
        $this->getAppAdminForApp($app)->removeUnassignedApps($app);
    }

    public function removeUnassignedAppsFromDevice(Device $device)
    {
        $this->getAppAdmin($device)->removeUnassignedAppsFromDevice($device);
    }

    /**
     * Removes the profile with the specified identifier from the device with
     * the specified id.
     *
     * @param Device $device     The device id.
     * @param string $identifier The profile identifier.
     * @param string $info       Additional information about the command. If
     *     not defined the specified identifier is used.
     *
     * @throws \Exception
     */
    public function removeProfile(Device $device, $identifier, $info = null)
    {
        if (!$device->isManaged()) {
            throw new \Exception("Profiles can only be removed through MDM from managed devices");
        }
        $this->getLogger()->debug("MdmAdmin: Remove profile '$identifier' on device ".$device->getId());
        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->removeProfile($identifier, $info);
        $this->getMdmCommandEntityManager()->save($mdmCommand);
    }

    /**
     * Removes the profile belonging to the specified mail account.
     *
     * @param MailAccount $mailAccount
     */
    public function removeMailProfile(MailAccount $mailAccount)
    {
        $this->removeProfile(
            $mailAccount->getDevice(),
            ProfileFactory::getMailProfileIdentifier($mailAccount),
            "Groupware " . $mailAccount->getEmail()
        );
    }

    /**
     * Creates an MDM command for every device to request its device information.
     */
    public function requestDeviceInfos()
    {
        $devices = $this->getActiveManagedDevices();
        foreach ($devices as $device) {
            $this->requestDeviceInfo($device);
        }
    }

    /**
     * Requests to send another push message to the APNS for the specified
     * device. This method does nothing for devices having no MDM command in the
     * queue.
     *
     * @param Device $device The id of the device to send another push.
     */
    public function repush(Device $device)
    {
        $this->getMdmQueue()->setWaiting($device, false);
    }

    /**
     * Creates an MDM command to request device information.
     *
     * @param Device $device The id of the device to request the device
     *     information.
     */
    public function requestDeviceInfo(Device $device)
    {
        if ($this->getMdmQueue()->existsCommand("DeviceInformation", $device)) {
            $this->getLogger()->debug("A request for device information of device " . $device->getId()
                . " is already in the queue");
            return;
        }

        $this->getLogger()->debug("MdmAdmin: Request device information of device " . $device->getId());
        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->deviceInfo();

        $this->getMdmCommandEntityManager()->save($mdmCommand);
        if ($device->getType()->hasSupport("apps", $device)) {
            $this->requestManagedApplicationList($device);
            $this->requestApplicationList($device);
        }
    }

    public function requestManagedApplicationList(Device $device, array $apps = array())
    {
        // TODO if $apps was not empty on a previous call we might still need to create a request
        if ($this->getMdmQueue()->existsCommand("ManagedApplicationList", $device)) {
            $this->getLogger()->debug("A request for managed application list of device " . $device->getId()
                . " is already in the queue");
            return null;
        }

        $this->getLogger()->debug("MdmAdmin: Request managed applications of device " . $device->getId());

        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->managedApplicationList($apps);
        $this->getMdmCommandEntityManager()->save($mdmCommand);

        return $mdmCommand;
    }

    /**
     * Sends the specified MDM command again. A new command will be created so
     * that the old command stays in the history.
     *
     * @param \Topalis\Device\Entity\Device $device
     * @param \Topalis\MDM\Entity\MdmCommand $command
     */
    public function resendCommand(Device $device, MdmCommand $command)
    {
        if ($command->getDevice() != $device) {
            throw new \InvalidArgumentException("The MDM command with the id " . $command->getId()
                . " is not destined for the specified device");
        }

        $newCommand = clone $command;
        $newCommand->setCreateTime($this->getTimeProvider()->getTime());
        $newCommand->setAdmin($this->getAdmin());
        $newCommand->setStatus(MdmCommand::STATUS_NEW);
        $newCommand->setQueueEntry(null);
        $newCommand->setResponse(array());
        $newCommand->setResponseTime(null);

        $command->incCopyCount();

        $em = $this->getMdmCommandEntityManager();
        $em->save($command, false);
        $em->save($newCommand);
        $this->getEventDispatcher()->dispatch(MdmCommandEvents::RESEND,
            new MdmCommandResendEvent($command, $newCommand));
    }

    /**
     * Applies the specified redemption code for the device and the iOS app
     * specified by $iosManagedAppState.
     *
     * @param IosAppState $state          Contains the device and
     *                                    iOS app to apply the redemption code.
     * @param string      $redemptionCode The redemption code to apply.
     *
     * @throws MdmException If the iOS app on the device doesn't need a
     *     redemption code.
     */
    public function applyRedemptionCode(IosAppState $state, $redemptionCode)
    {
        $device = $state->getDevice();
        $app = $state->getApp();
        if ($state->getState() != "NeedsRedemption") {
            throw new MdmException("Device " . $device->getId() . " does not need a redemption code for iOS app "
                . $app->getIdentifier() . "(" . $app->getId() . ")");
        }

        $this->getLogger()->debug("Apply redemption code for iOS app "
            . $app->getIdentifier() . " on device " . $device->getId());

        if ($state->getApplyRedemptionCodeCommand() !== null) {
            // TODO is this the correct approach? Or shall the existing command be canceled?
            switch ($state->getApplyRedemptionCodeCommand()->getStatus()) {
                case MdmCommand::STATUS_NEW:
                    /* fall through */
                case MdmCommand::STATUS_QUEUED:
                    return;
            }
        }

        $applyRedemptionCodeRequest = new ApplyRedemptionCodeRequest();
        $applyRedemptionCodeRequest->setIdentifier($app->getIdentifier());
        $applyRedemptionCodeRequest->setRedemptionCode($redemptionCode);
        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->createFromRequest($applyRedemptionCodeRequest, $app->getIdentifier());
        $state->setApplyRedemptionCodeCommand($mdmCommand);
        $this->getIosAppStateEntityManager()->save($state);
    }

    public function clearPasscode(Device $device)
    {
        $this->getLogger()->debug("MdmAdmin: Request clear passcode of device " . $device->getId());

        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->clearPasscode();
        $this->getMdmCommandEntityManager()->save($mdmCommand);
    }

    public function setPasscode(Device $device, $passcode)
    {
        $this->getLogger()->debug("MdmAdmin: Request set passcode of device " . $device->getId());

        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->setPasscode($passcode);
        $this->getMdmCommandEntityManager()->save($mdmCommand);
    }

    public function lockDevice(Device $device, $message = null, $phoneNumber = null)
    {
        $this->getLogger()->debug("MdmAdmin: Request lock of device " . $device->getId());
        $this->getDeviceEntityManager()->save($device);

        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->lockDevice($message, $phoneNumber);
        $this->getMdmCommandEntityManager()->save($mdmCommand);
    }

    /**
     * Get installed applications on device
     *
     * @param \Topalis\Device\Entity\Device $device
     */
    protected function requestApplicationList(\Topalis\Device\Entity\Device $device)
    {
        $this->getLogger()->debug("MdmAdmin: Rquesting application list of device ". $device->getId());
        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->installedApplicationList();
        $this->getMdmCommandEntityManager()->save($mdmCommand);
    }


    /**
     * Erases the device with the specified id. The device needs to be
     * reinitialized afterwards (full wipe).
     * The device attempts to send a response to the server, but unlike other commands, the response cannot be
     * resent if initial transmission fails. Even if the acknowledgement did not make it to the server (due to network
     * conditions), the device will still be erased.
     *
     * @param \Topalis\Device\Entity\Device $device The device to erase.
     * @see wipeDevice()
     */
    public function eraseDevice(Device $device)
    {
        $this->getLogger()->info("Erase device " . $device->getId());
        $this->cancelRemainingCommandsForDevice($device);
        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->eraseDevice();
        $this->getMdmCommandEntityManager()->save($mdmCommand);
    }

    /**
     * Creates MDM commands for the initialization.
     *
     * @param Device $device The device to initialize.
     */
    public function initDevice(Device $device)
    {
        $this->getLogger()->debug("MdmAdmin: Init device " . $device->getId());
        $this->requestDeviceInfo($device);
        $this->installDeviceProfile($device);
        $this->installMailProfiles($device, true);
        $this->installSecureIntranetProfile($device);
        $this->installWebClipProfile($device, false);
        $this->installWifiProfile($device);
        // invite to vpp, when user is not already associated
        if (User::VPP_STATUS_REGISTERED === $device->getUser()->getVppStatus()
            && $device->getOs() == $this->getDeviceOs("ios")) {
            $this->inviteToVolumePurchaseProgram($device, $device->getUser()->getVppInviteLink());
        }
    }

    /**
     * Installs the profile assigned to the device without any changes.
     *
     * @param Device $device The device to install its profile.
     */
    public function installDeviceProfile(Device $device)
    {
        $this->getLogger()->debug("MdmAdmin: Install device profile on device " . $device->getId());
        $profile = $this->getProfileFactory()->createDeviceProfile($device);

        $restrictions = null;
        if ($device->isFlag(Device::FLAG_APP_STORE_TEMPORARY_ENABLED)) {
            $restrictions = $profile->getRestrictions();
            $restrictions = reset($restrictions);
            if ($restrictions) {
                $allowAppInstallation = $restrictions->isAllowAppInstallation();
                $restrictions->setAllowAppInstallation(true);
            }
        }

        if (!$profile->hasPayloads()) {
            $profile->addPayload(new DummyPayload());
        }
        $this->installProfile($device, $profile);

        if ($restrictions) {
            // the profile is cached so we have to reset its restrictions
            $restrictions->setAllowAppInstallation($allowAppInstallation);
        }
    }

    /**
     * Installs device profiles on every device in the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config
     *                                                             group for which to install the device profiles.
     * @param string                                  $type device type name or null for all types
     */
    public function installDeviceProfiles(ConfigGroup $configGroup, $type = null)
    {
        $this->getLogger()->debug("Install device profiles for config group " . $configGroup->getId());
        foreach ($configGroup->getDevices() as $device) {
            if (null !== $type && $device->getType()->getName() != $type) {
                continue;
            }
            if ($device->isManaged() && $device->isActive()) {
                $this->installDeviceProfile($device);
            }
        }
    }

    public function getAppAdmin(Device $device)
    {
        return $this->container->get("app.mdm_admin.registry")->getForDevice($device);
    }

    public function getAppAdminForApp(App $app)
    {
        return $this->container->get("app.mdm_admin.registry")->getForApp($app);
    }

    public function installApp(Device $device, App $app, $force = false, $removeBeforeInstall = false)
    {
        $this->getAppAdmin($device)->installApp($device, $app, $force, $removeBeforeInstall);
    }

    public function installApps(App $app, $force = false)
    {
        $this->getAppAdminForApp($app)->installApps($app, $force);
    }

    public function installAppsForDevice(Device $device)
    {
        $this->getAppAdmin($device)->installAppsForDevice($device);
    }

    /**
     * Installs a profile for the specified mail account on the device the mail
     * account belongs to.
     *
     * @param MailAccount $mailAccount The mail account to install a profile for.
     */
    public function installMailProfile(MailAccount $mailAccount)
    {
        $this->getLogger()->debug("MdmAdmin: Install mail profile '" . $mailAccount->getEmail()
            . "' (id " . $mailAccount->getId() . ") on device " . $mailAccount->getDevice()->getId());
        $profile = $this->getProfileFactory()->createMailProfile($mailAccount);
        $this->installProfile($mailAccount->getDevice(), $profile);
    }

    /**
     * Installs profiles for all mail accounts of the specified device that are
     * not installed, yet, i.e. they have no referenced certificate.
     *
     * @param Device $device The device to install profiles for its mail
     *     accounts.
     * @param boolean $force If false the profile for each mail account is only
     *     installed if no certificate is related to it.
     */
    public function installMailProfiles(Device $device, $force = false)
    {
        foreach ($device->getMailAccounts() as $mailAccount) {
            if ($force || $mailAccount->getCert() === null) {
                $this->installMailProfile($mailAccount);
            }
        }
    }

    /**
     * Installs mail profiles on every device in the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup
     */
    public function installMailProfilesForGroup(ConfigGroup $configGroup)
    {
        $this->getLogger()->debug("Install mail profiles for config group " . $configGroup->getId());
        foreach ($configGroup->getDevices() as $device) {
            if ($device->isManaged() && $device->isActive() && ($device->getOs() == $this->getDeviceOs("ios")
                || $device->getOs() == $this->getDeviceOs("android"))) {
                $this->installMailProfiles($device, true);
            }
        }
    }

    /**
     * Installs the specified profile on the device with the specified id.
     *
     * @param Device          $device  The id of the device to install the profile on.
     * @param AbstractProfile $profile The profile to install.
     * @param string          $info    Additional information about the command.
     *     If not defined the profile's display name is used, or the profile's
     *     type if the first one is empty.
     * @throws \Exception
     */
    public function installProfile(Device $device, AbstractProfile $profile, $info = null)
    {
        if (!$device->isManaged() || !$device->isActive()) {
            throw new \Exception("Profiles can only be installed through MDM on active managed devices");
        }

        $profile->setRemovalDisallowed(!$device->getConfigGroup()->isAllowProfileRemoval());
        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $serverCert = $this->getCertificateManager()->getServerCert();
        $mdmCommand = $mdmCommandGenerator->installProfile($profile, $serverCert->getResource(),
            $serverCert->getPrivateKey(), $info);
        $this->getMdmCommandEntityManager()->save($mdmCommand);

        return $mdmCommand;
    }

    /**
     * Installs the profile with the SecureIntranet CA and SCEP payload on the
     * specified device.
     *
     * @param Device $device The device to install the profile.
     */
    public function installSecureIntranetProfile(Device $device)
    {
        $this->getLogger()->debug("MdmAdmin: Install SecureIntranet profile on device " . $device->getId());
        $profile = $this->getProfileFactory()->createSecureIntranetProfile($device);
        $this->installProfile($device, $profile, "SecureIntranet");
    }

    /**
     * Installs a profile with web clips for the specified device.
     *
     * @param Device $device The device to install the profile with web clips.
     * @param boolean $remove If true a previously installed web clips profile will be
     *     removed first. Normally previously installed profiles are replaced
     *     by new ones with the same identifier. However if we don't remove the
     *     old one previous web clips might stay on the home screen.
     */
    public function installWebClipProfile(Device $device, $remove = true)
    {
        $this->getLogger()->debug("MdmAdmin: Install WebClip profile on device " . $device->getId());
        $profile = $this->getProfileFactory()->createWebClipProfile($device);
        if ($remove) {
            $this->removeProfile($device, $profile->getIdentifier(), "Web Clips");
        }
        if (!$profile->hasPayloads()) {
            $profile->addPayload(new DummyPayload());
        }
        $this->installProfile($device, $profile);
    }

    /**
     * Installs web clip profile on every device in the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config
     *     group for which to install the web clip profiles.
     */
    public function installWebClipProfiles(ConfigGroup $configGroup)
    {
        $this->getLogger()->debug("Install web clip profiles for config group " . $configGroup->getId());
        foreach ($configGroup->getDevices() as $device) {
            if ($device->isManaged() && $device->isActive() && ($device->getOs() == $this->getDeviceOs("ios")
                || $device->getOs() == $this->getDeviceOs("android"))) {
                $this->installWebClipProfile($device);
            }
        }
    }

    public function installWebClipProfilesForWebClip(\Topalis\WebClip\Entity\WebClipInterface $webClip)
    {
        foreach ($webClip->getConfigGroups() as $group) {
            $this->installWebClipProfiles($group);
        }
    }

    /**
     * Invites the specified device to join the Volume Purchasing Program with
     * the specified URL.
     *
     * @param Device $device The device to invite.
     * @param string $invitationURL The invitation URL.
     */
    public function inviteToVolumePurchaseProgram(Device $device, $invitationURL)
    {
        $query = parse_url($invitationURL, PHP_URL_QUERY);
        $parsedQuery = array();
        parse_str($query, $parsedQuery);
        if (isset($parsedQuery['inviteCode'])) {
            $info = "Invite code: " . $parsedQuery['inviteCode'];
        } else {
            $info = $query;
        }

        $inviteCommands = $this->getMdmQueue()->getCommandsForDevice($device, InviteToProgramRequest::REQUEST_TYPE);
        if (!empty($inviteCommands)) {
            $previousInvitation = $inviteCommands[0];
            if ($previousInvitation->getInfo() == $info) {
                // cancel previous invitation with different invite code
                $this->getMdmQueue()->cancelCommand($previousInvitation);
            } else {
                // there's already a invitation with the same invite code
                return;
            }
        }

        $this->getLogger()->debug(sprintf("Invite device %d to vpp '%s'", $device->getId(), $invitationURL));

        $request = new InviteToProgramRequest();
        $request->setInvitationURL($invitationURL);
        $request->setProgramID(InviteToProgramRequest::VOLUME_PURCHASING_PROGRAM_ID);

        $mdmCommandGenerator = new MdmCommandGenerator($this->getAdmin(), $device);
        $mdmCommand = $mdmCommandGenerator->createFromRequest($request, $info);
        $this->getMdmCommandEntityManager()->save($mdmCommand);
    }

    /**
     * Installs the profile containing all WiFi configurations associated with
     * the specified device.
     *
     * @param \Topalis\Device\Entity\Device $device The device to install the
     *     WiFi configurations.
     */
    public function installWifiProfile(\Topalis\Device\Entity\Device $device)
    {
        $this->getLogger()->debug("Install WiFi profile on device " . $device->getId());
        $profile = $this->getProfileFactory()->createWifiProfile($device);
        if (!$profile->hasPayloads()) {
            $profile->addPayload(new DummyPayload());
        }
        $this->installProfile($device, $profile);
    }

    /**
     * Installs WiFi profile on every device in the specified config group.
     *
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup The config
     *     group for which to install the WiFi profiles.
     */
    public function installWifiProfiles(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup)
    {
        $this->getLogger()->debug("Install WiFi profiles for config group " . $configGroup->getId());
        foreach ($configGroup->getDevices() as $device) {
            if ($device->isManaged() && $device->isActive() && ($device->getOs() == $this->getDeviceOs("ios")
                || $device->getOs() == $this->getDeviceOs("android"))) {
                $this->installWifiProfile($device);
            }
        }
    }

    public function cancelCommand(MdmCommand $command)
    {
        $this->getMdmQueue()->cancelCommand($command);
    }

    /**
     * Cancels the remaining commands for the device with the specified id.
     *
     * @param Device $device The id of the device to cancel the remaining
     *     commands.
     */
    public function cancelRemainingCommandsForDevice(Device $device)
    {
        $this->getMdmQueue()->cancelCommandsForDevice($device);
    }

    /**
     * Makes an enterprise wipe by deleting the MDM profile.
     *
     * @param Device $device of the device to wipe.
     * @see eraseDevice()
     */
    public function wipeDevice(Device $device)
    {
        $this->getLogger()->info("Wipe device " . $device->getId());
        $this->cancelRemainingCommandsForDevice($device);
        $this->removeProfile($device, ProfileFactory::getRolloutProfileIdentifier());
    }
}
