<?php
namespace Topalis\MDM;

use CFPropertyList\CFData;
use CFPropertyList\CFTypeDetector;
use CFPropertyList\CFPropertyList;
use Topalis\App\Entity\IosApp;
use Topalis\Device\Entity\Device;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\Profile\UUIDGenerator;
use Topalis\Certificate\Util\CertUtil;
use Topalis\Profile\AbstractProfile;
use Topalis\Admin\Entity\Admin;
use Topalis\MDM\Request\AbstractMdmRequest;
use CFPropertyList\CFDictionary;

/**
 * This class is used to generate objects of type MdmCommand for different MDM
 * request types.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class MdmCommandGenerator
{
    private $admin;
    private $device;
    private $uuidGenerator;

    /**
     * Constructor.
     *
     * @param Admin $admin The administrator that generates the command,
     *     0 if the command is generated by the system.
     * @param Device $device The id of the device the command is intended for.
     */
    public function __construct(Admin $admin, Device $device)
    {
        $this->admin = $admin;
        $this->device = $device;
        $this->uuidGenerator = new UUIDGenerator();
    }

    /**
     * Generates an MDM command from the specified request.
     *
     * @param AbstractMdmRequest $request The request to generate the MDM
     *     command for.
     * @param string $info Additional information about the command.
     */
    public function createFromRequest(AbstractMdmRequest $request, $info = null)
    {
        $request = $this->toGenericRequest($request->getDictionary());
        return $this->generate($request, $info);
    }

    /**
     * Generate a DeviceInformation command.
     *
     * @param array $queries The values to request, or null to request all
     *     possible values..
     * @return MdmCommand
     */
    public function deviceInfo(array $queries = null)
    {
        $request = array("RequestType" => "DeviceInformation");
        if ($queries === null) {
            // request everything
            $request['Queries'] = array(
                    // General
                    "UDID",
                    // Device information
                    "DeviceName",
                    "Vendor",
                    "OSVersion",
                    "BuildVersion",
                    "ModelName",
                    "Model",
                    "ProductName",
                    "SerialNumber",
                    "DeviceCapacity",
                    "AvailableDeviceCapacity",
                    "BatteryLevel",
                    "CellularTechnology",
                    "IMEI",
                    "MEID",
                    "ModemFirmwareVersion",
                    // Network information
                    "ICCID",
                    "BluetoothMAC",
                    "WiFiMAC",
                    "CurrentCarrierNetwork",
                    "SIMCarrierNetwork",
                    "SubscriberCarrierNetwork",
                    "CarrierSettingsVersion",
                    "PhoneNumber",
                    "VoiceRoamingEnabled",
                    "DataRoamingEnabled",
                    "IsRoaming",
                    "SubscriberMCC",
                    "SubscriberMNC",
                    "CurrentMCC",
                    "CurrentMNC",
                    // Android
                    "WiFiIp",
                    "WiFiSSID",
                    "ExternalCapacity",
                    "AvailableExternalCapacity",
                    "VoiceMailNumber",
                    "AdbEnabled",
                    "InstallNonMarketApps",
                    "AppVersion",
                    "PasscodeSufficient",
                    "DeviceEncrypted",
                    "PasscodePolicy"
            );
        } else {
            $request['Queries'] = $queries;
        }

        $request = $this->toGenericRequest($request);
        return $this->generate($request);
    }

    /**
     * Generate an EraseDevice command.
     *
     * @return \Topalis\MDM\Entity\MdmCommand
     */
    public function eraseDevice()
    {
        $request = array("RequestType" => "EraseDevice");
        $request = $this->toGenericRequest($request);
        return $this->generate($request);
    }

    /**
     * Generate a InstallProfile command. The payload will be signed if a
     * certificate and a key is specified.
     *
     * @param AbstractProfile $profile The profile to install.
     * @param mixed $cert The certificate to sign with.
     * @param mixed $key The certificate to sign with.
     * @param string $info Additional information about the command. If not
     *     defined the profile's display name is used, or the profile's type if
     *     the first one is empty.
     * @return \Topalis\MDM\Entity\MdmCommand
     */
    public function installProfile(AbstractProfile $profile, $cert = null, $key = null, $info = null)
    {
        $payload = $profile->toXML();
        if ($cert !== null && $key !== null) {
            $payload = CertUtil::signPKCS7($payload, $cert, $key);
        }

        $request = array(
            "RequestType" => "InstallProfile",
            "Payload" => new CFData($payload)
        );
        $request = $this->toGenericRequest($request);
        if ($info === null) {
            // remove "auralis " at the beginning of the string
            $info = preg_replace("/^auralis /", "", $profile->getDisplayName());
            if (!$info) {
                $info = $profile->getType();
            }
        }
        return $this->generate($request, $info);
    }

    /**
     * Generate a RemoveProfile command.
     *
     * @param string $identifier The identifier of the profile to remove.
     * @param string $info Additional information about the command. If not
     *     defined the specified identifier is used.
     * @return \Topalis\MDM\Entity\MdmCommand
     */
    public function removeProfile($identifier, $info = null)
    {
        $request = array(
            "RequestType" => "RemoveProfile",
            "Identifier" => $identifier
        );
        $request = $this->toGenericRequest($request);
        if ($info === null) {
            $info = $identifier;
        }
        return $this->generate($request, $info);
    }

    public function clearPasscode()
    {
        $request = array("RequestType" => "ClearPasscode");
        if ($this->device->getType()->getName() == "ios") {
            // unlock token is only available in iOS
            $tokenStream = $this->device->getMdmunlocktoken();
            $token = null;
            while (!feof($tokenStream)) {
                $token .= fgets($tokenStream);
            }
            new CFPropertyList(); // autoloading bug...
            $request['UnlockToken'] = new CFData($token);
        }
        $request = $this->toGenericRequest($request);

        return $this->generate($request);
    }

    public function setPasscode($passcode)
    {
        $request = $this->toGenericRequest(array(
            "RequestType" => "SetPasscode",
            "Passcode"    => $passcode,
        ));

        return $this->generate($request);
    }

    public function lockDevice($message = null, $phoneNumber = null)
    {
        $request = array("RequestType" => "DeviceLock");
        if (null !== $message) {
            $request['Message'] = $message;
        }
        if (null !== $phoneNumber) {
            $request['PhoneNumber'] = $phoneNumber;
        }

        $request = $this->toGenericRequest($request);

        return $this->generate($request);
    }

    public function installedApplicationList()
    {
        $request = array("RequestType" => "InstalledApplicationList");

        return $this->generate($this->toGenericRequest($request));
    }

    /**
     * @param IosApp[] $apps
     *
     * @return MdmCommand
     */
    public function managedApplicationList(array $apps = array())
    {
        $request = array("RequestType" => "ManagedApplicationList");
        if (!empty($apps)) {
            $identifiers = array();
            foreach ($apps as $app) {
                if (!$app->getIdentifier()) {
                    continue;
                }
                $identifiers[] = $app->getIdentifier();
            }
            $request['Identifiers'] = $identifiers;
        }

        return $this->generate($this->toGenericRequest($request));
    }

    /**
     *
     * @param array|CFDictionary $commandRequest
     * @return array
     */
    protected function toGenericRequest($commandRequest)
    {
        return array(
            "CommandUUID" => $this->getUUIDGenerator()->generateUUID(),
            "Command" => $commandRequest
        );
    }

    /**
     *
     * @param array $request
     * @param string $info Additional information about the command.
     * @return \Topalis\MDM\Entity\MdmCommand
     */
    protected function generate(array $request, $info = null)
    {
        $plist = new CFPropertyList();
        $td = new CFTypeDetector(array("castNumericStrings" => false));
        $cfPayload = $td->toCFType($request);
        $plist->add($cfPayload);
        $command = new MdmCommand();
        $command->setDevice($this->device);
        $command->setAdmin($this->admin);
        $command->setRequest($plist->toXML());
        $command->setUuid($request['CommandUUID']);
        if (is_array($request['Command'])) {
            $command->setRequestType($request['Command']['RequestType']);
        } else {
            $command->setRequestType($request['Command']->get("RequestType")->getValue());
        }
        $command->setInfo($info);

        return $command;
    }

    /**
     * @return \Topalis\Profile\UUIDGenerator
     */
    protected function getUUIDGenerator()
    {
        return $this->uuidGenerator;
    }
}
