<?php
namespace Topalis\MDM;

use Doctrine\ORM\NoResultException;
use Topalis\Device\Entity\Device;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\MDM\Entity\MdmQueueEntry;
use Symfony\Component\DependencyInjection\ContainerAware;

/**
 * The MdmQueue manages the queued MdmCommands.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class MdmQueue extends ContainerAware implements MdmQueueInterface
{
    /**
     * @var integer
     */
    private $repushTimeout = 43200; // 12 * 60 * 60;

    /**
     * @var integer
     */
    private $rolloutRepushTimeout = 600; // 10 * 60

    /**
     * Constructor.
     *
     * @param \Symfony\Component\DependencyInjection\ContainerInterface $container
     */
    public function __construct(\Symfony\Component\DependencyInjection\ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * @return MySqlDb
     */
    private function getDB()
    {
        return $this->container->get("db");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\MDM\Entity\MdmCommandEntityManager
     */
    private function getMdmCommandEntityManager()
    {
        return $this->container->get("mdm.entity.manager.mdm_command");
    }

    /**
     * @return \Topalis\MDM\Entity\MdmQueueEntityManager
     */
    private function getMdmQueueEntityManager()
    {
        return $this->container->get("mdm.entity.manager.mdm_queue");
    }

    /**
     * @return integer The number in seconds until a new push message is sent
     *     if the device hasn't connected to the MDM service.
     */
    protected function getRepushTimeout()
    {
        return $this->repushTimeout;
    }

    public function setRepushTimeout($repushTimeout)
    {
        $this->repushTimeout = $repushTimeout;
    }

    /**
     * @return integer The number in seconds until a new push message is sent
     *     if the device hasn't connected to the MDM service and is in rollout
     *     phase i.e. the first MDM command hasn't been processed.
     */
    protected function getRolloutRepushTimeout()
    {
        return $this->rolloutRepushTimeout;
    }

    public function setRolloutRepushTimeout($rolloutRepushTimeout)
    {
        $this->rolloutRepushTimeout = $rolloutRepushTimeout;
    }

    /**
     * Adds the specified command to the queue.
     *
     * @param MdmCommand $mdmCommand The command to add.
     * @return MdmQueueEntry The entry added to the queue.
     */
    public function addCommand(MdmCommand $mdmCommand)
    {
        $queueEntry = $mdmCommand->getQueueEntry();
        if ($queueEntry !== null) {
            $queueEntry->setPushtime(null);
            $queueEntry->setWaiting(false);
        } else {
            $queueEntry = new MdmQueueEntry();
        }

        $queueEntry->setBlockable($mdmCommand->isBlockable());
        $queueEntry->setCommand($mdmCommand);
        $queueEntry->setDevice($mdmCommand->getDevice());

        /*
         * first save the queue entry before we set wait status false
         * in case the mdm service is handling the previously last command
         * at the same time to ensure that we always send a push if required.
         */
        $this->getMdmQueueEntityManager()->save($queueEntry);

        $query = $this->getMdmQueueEntityManager()->createQueryBuilder("q")
            ->select("q.pushTime")
            ->where("q.device = :device")
            ->andWhere("q.waiting = true")
            ->setParameter("device", $mdmCommand->getDevice())
            ->setMaxResults(1)
        ;

        if (!$mdmCommand->isBlockable()) {
            $query->andWhere("q.blockable = false");
        }
        $pushTime = null;
        try {
            $pushTime = $query->getQuery()->getSingleScalarResult();
        } catch (NoResultException $e) {
             // doctrine will throw an exception if result is empty
        }

        if ($pushTime !== null) {
            $queueEntry->setWaiting(true);
            $queueEntry->setPushTime($pushTime);
            $this->getMdmQueueEntityManager()->save($queueEntry);
        }

        return $queueEntry;
    }

    /**
     * Cancels the specified MDM command. If it is neither new nor queued,
     * nothing happens.
     *
     * @param MdmCommand $command The MDM command to cancel.
     */
    public function cancelCommand(MdmCommand $command)
    {
        if ($command->getStatus() != MdmCommand::STATUS_NEW
            && $command->getStatus() != MdmCommand::STATUS_QUEUED
            && $command->getStatus() != MdmCommand::STATUS_NOT_NOW) {
            return;
        }

        $this->getMdmQueueEntityManager()->beginTransaction();
        try {
            $this->removeCommand($command);
            $device = $command->getDevice();
            $command->setStatus(MdmCommand::STATUS_CANCELED);
            $this->getDeviceEntityManager()->save($device);
            $this->getMdmQueueEntityManager()->commit();
        } catch (\Exception $e) {
            $this->getMdmQueueEntityManager()->rollback();
        }
    }

    public function cancelCommandsBy(array $criteria)
    {
        $criteria['status'] = array(
            MdmCommand::STATUS_NEW,
            MdmCommand::STATUS_QUEUED,
            MdmCommand::STATUS_NOT_NOW,
        );

        $commandEm = $this->getMdmCommandEntityManager();
        $commandEm->beginTransaction();
        try {
            $commands = $this->getMdmCommandEntityManager()->findBy($criteria);
            foreach ($commands as $command) {
                $this->cancelCommand($command);
            }

            $commandEm->commit();
            return count($commands);
        } catch (\Exception $e) {
            $commandEm->rollback();
            throw $e;
        }
    }

    /**
     * Cancel all remaining commands (status "Queued" and "New") for the device
     * with the specified id. All queue entries for the specified device are
     * removed and the status of the remaining command are set to "Canceled".
     *
     * @param Device $device The id of the device to remove the commands for.
     * @throws \Exception On error
     */
    public function cancelCommandsForDevice(Device $device)
    {
        $this->getDB()->beginTransaction();
        try {
            $entries = $device->getMdmQueue();
            foreach ($entries as $entry) {
                $this->getMdmQueueEntityManager()->delete($entry, false);
            }
            $device->getMdmQueue()->clear();
            $this->getMdmQueueEntityManager()->flush();
            /** @var $commands MdmCommand[] */
            $this->getMdmCommandEntityManager()->createQueryBuilder("c")
                ->update()
                ->set("c.status", ":canceled")
                ->where("c.device = :device AND (c.status = :queued OR c.status = :new OR c.status = :notNow)")
                ->getQuery()->execute(array(
                    "canceled" => MdmCommand::STATUS_CANCELED,
                    "device" => $device,
                    "queued" => MdmCommand::STATUS_QUEUED,
                    "new" => MdmCommand::STATUS_NEW,
                    "notNow" => MdmCommand::STATUS_NOT_NOW,
                ));
            $this->getMdmCommandEntityManager()->flush();
            $this->getDB()->commitTransaction();
        } catch (\Exception $e) {
            $this->getDB()->rollbackTransaction();
            throw $e;
        }
    }

    /**
     * @param string $requestType
     * @param Device $device
     * @return boolean true if a command for the specified device with the
     *     specified request type is either new or queued (this includes
     *     commands in status "NotNow"), false otherwise.
     */
    public function existsCommand($requestType, Device $device)
    {
        return $this->getMdmCommandEntityManager()->createQueryBuilder("c")
            ->select("1")->where("c.device = :device")
            ->andWhere("c.requestType = :requestType")
            ->andWhere("c.status = :new OR c.status = :queued OR c.status = :notNow")
            ->setParameter("device", $device)
            ->setParameter("requestType", $requestType)
            ->setParameter("new", MdmCommand::STATUS_NEW)
            ->setParameter("queued", MdmCommand::STATUS_QUEUED)
            ->setParameter("notNow", MdmCommand::STATUS_NOT_NOW)
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult() !== null;
    }

    public function getCommandsForDevice(Device $device, $requestType)
    {
        return $this->getMdmCommandEntityManager()->createQueryBuilder("c")
            ->where("c.device = :device")
            ->andWhere("c.requestType = :requestType")
            ->andWhere("c.status = :new OR c.status = :queued OR c.status = :notNow")
            ->setParameter("device", $device)
            ->setParameter("requestType", $requestType)
            ->setParameter("new", MdmCommand::STATUS_NEW)
            ->setParameter("queued", MdmCommand::STATUS_QUEUED)
            ->setParameter("notNow", MdmCommand::STATUS_NOT_NOW)
            ->getQuery()->getResult()
        ;
    }

    /**
     * @param Device $device The device to get the next command for.
     * @return MdmCommand The next command in the queue for the specified device.
     */
    public function getNextCommand(Device $device)
    {
        /** @var $entry MdmQueueEntry */
        $qb = $this->getMdmQueueEntityManager()->createQueryBuilder("q")
            ->select("q")
            ->where("q.device = :device")
            ->setParameter("device", $device);
        if ($device->isBlocked()) {
            $qb->andWhere("q.blockable = 0");
        }
        $entry = $qb->orderBy("q.command", "ASC")
            ->setMaxResults(1)
            ->getQuery()
            ->getOneOrNullResult()
        ;

        if ($entry === null) {
            return null;
        }

        return $entry->getCommand();
    }

    /**
     * @param Device $device
     * @return boolean true if there is at least one command left in the queue
     *     for the specified device, false otherwise.
     */
    public function hasCommand(Device $device)
    {
        return $device->getMdmQueue()->count() > 0;
    }

    /**
     * Remove the command with the specified id from the queue.
     *
     * @param MdmCommand $command The command to remove.
     */
    public function removeCommand(MdmCommand $command)
    {
        if ($command->getQueueEntry() !== null) {
            $this->getMdmQueueEntityManager()->delete($command->getQueueEntry());
        }
    }

    /**
     * Sets the waiting status of the device with the specified id. True means
     * push has been sent and we're waiting for the device's request. If it is
     * set to false and there are commands in the queue a push is going to be
     * sent.
     *
     * @param Device $device The id of the device to set the waiting status for.
     * @param boolean $waiting The waiting status to set.
     */
    public function setWaiting(Device $device, $waiting)
    {
        $this->getMdmQueueEntityManager()->createQueryBuilder("q")
            ->update("TopalisMdmBundle:MdmQueueEntry", "q")->set("q.waiting", $waiting ? 1 : 0)
            ->where("q.device = :device")
            ->setParameter("device", $device)
            ->getQuery()->execute();
        $this->getMdmQueueEntityManager()->flush();
    }

    /**
     * @param integer $os The OS id to get the devices for.
     * @return Device[] Devices with queue entries marked as non waiting.
     */
    public function getNonWaiting($os)
    {
        return $this->getDeviceEntityManager()->createQueryBuilder("d")
            ->select("d")
            ->where("q.waiting = false")
            ->andWhere("d.os = :os")
            ->join("d.mdmQueue", "q")
            ->groupBy("d.id")
            ->setParameter("os", $os)
            ->getQuery()
            ->getResult()
        ;
    }

    public function setPushTime(Device $device, $pushTime)
    {
        foreach ($device->getMdmQueue() as $entry) {
            $entry->setPushTime($pushTime);
            $this->getMdmQueueEntityManager()->save($entry, false);
        }
        $this->getMdmQueueEntityManager()->flush();
    }

    /**
     * @param integer $os The OS id to get the devices for.
     * @return \Topalis\Device\Entity\Device[]
     */
    public function getPushRequired($os = 0)
    {
        $repushTimeout = $this->getRepushTimeout();
        $rolloutRepushTimeout = $this->getRolloutRepushTimeout();

        if ($repushTimeout <= 0 && $rolloutRepushTimeout <= 0) {
            return $this->getNonWaiting($os);
        }

        $qb = $this->getDeviceEntityManager()->createQueryBuilder("d")
            ->select("d")
            ->join("d.mdmQueue", "q")
            ->groupBy("d.id, q.waiting")
            ->where("d.os = :os")
            ->having("q.waiting = 0")
            ->setParameter("os", $os);
        if ($repushTimeout > 0) {
            $qb->orHaving(":now - MIN(q.pushTime) >= :repushtimeout")
            ->setParameter("now", time())
            ->setParameter("repushtimeout", $repushTimeout);
        }
        if ($rolloutRepushTimeout > 0) {
            $qb->join("d.mdmCommands", "dc")
            ->join("q.command", "c")
            ->orHaving("MIN(c.id) = MIN(dc.id) AND :now - MIN(q.pushTime) >= :rolloutRepushTimeout")
            ->setParameter("now", time())
            ->setParameter("rolloutRepushTimeout", $rolloutRepushTimeout);
        }
        return $qb->getQuery()->getResult();
    }
}
