<?php
namespace Topalis\MDM;

use Doctrine\ORM\NoResultException;
use Topalis\Device\Entity\Device;
use Topalis\MDM\Entity\MdmCommand;
use Topalis\MDM\Entity\MdmQueueEntry;
use Symfony\Component\DependencyInjection\ContainerAware;

/**
 * The MdmQueue manages the queued MdmCommands.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
interface MdmQueueInterface
{
    /**
     * Adds the specified command to the queue.
     *
     * @param MdmCommand $mdmCommand The command to add.
     * @return MdmQueueEntry The entry added to the queue.
     */
    public function addCommand(MdmCommand $mdmCommand);

    /**
     * Cancels the specified MDM command. If it is neither new nor queued,
     * nothing happens.
     *
     * @param MdmCommand $command The MDM command to cancel.
     */
    public function cancelCommand(MdmCommand $command);

    /**
     * Cancels the commands matching the specified criteria. The criteria may
     * be any criteria that is excepted by
     * {@link \Topalis\MDM\Entity\MdmCommandEntityManager::findBy()}.
     *
     * @param array $criteria
     *
     * @return int The number of canceled commands.
     */
    public function cancelCommandsBy(array $criteria);

    /**
     * Cancel all remaining commands (status "Queued" and "New") for the device
     * with the specified id. All queue entries for the specified device are
     * removed and the status of the remaining command are set to "Canceled".
     *
     * @param Device $device The id of the device to remove the commands for.
     * @throws \Exception On error
     */
    public function cancelCommandsForDevice(Device $device);

    /**
     * @param string $requestType
     * @param Device $device
     * @return boolean true if a command for the specified device with the
     *     specified request type is either new or queued , false otherwise.
     */
    public function existsCommand($requestType, Device $device);

    /**
     * @param Device $device
     * @param string $requestType
     *
     * @return MdmCommand[] The commands for the specified device with the
     *    specified request type.
     */
    public function getCommandsForDevice(Device $device, $requestType);

    /**
     * @param Device $device The device to get the next command for.
     * @return MdmCommand The next command in the queue for the specified device.
     */
    public function getNextCommand(Device $device);

    /**
     * @param Device $device
     * @return boolean true if there is at least one command left in the queue
     *     for the specified device, false otherwise.
     */
    public function hasCommand(Device $device);

    /**
     * Remove the command with the specified id from the queue.
     *
     * @param MdmCommand $command The command to remove.
     */
    public function removeCommand(MdmCommand $command);

    /**
     * Sets the waiting status of the device with the specified id. True means
     * push has been sent and we're waiting for the device's request. If it is
     * set to false and there are commands in the queue a push is going to be
     * sent.
     *
     * @param Device $device The id of the device to set the waiting status for.
     * @param boolean $waiting The waiting status to set.
     */
    public function setWaiting(Device $device, $waiting);

    /**
     * @param integer $os The OS id to get the devices for.
     * @return Device[] Devices with queue entries marked as non waiting.
     */
    public function getNonWaiting($os);

    /**
     * Sets the time the last push for the specified device was sent.
     *
     * @param Device $device The device to set the last push time.
     * @param int $pushTime The UNIX timestamp to set.
     */
    public function setPushTime(Device $device, $pushTime);

    /**
     * @param integer $os The OS id to get the devices for.
     * @return \Topalis\Device\Entity\Device[]
     */
    public function getPushRequired($os = 0);
}
