<?php
namespace Topalis\Mail;

use Topalis\Util\ArrayUtil;
use Swift_SmtpTransport;
use Topalis\Kernel\AbstractAuralisService;

/**
 * creates mails and sends them
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class Mailer extends AbstractAuralisService implements MailerInterface
{
    /**
     * @var boolean
     */
    protected $alwaysLoadSettings = false;

    /**
     * @var \Swift_Mailer
     */
    protected $mailer;

    /**
     * array with mail settings
     * @var array
     */
    protected $settings = array();

    /**
     * @return boolen true if the mailer is configured such that mails can be
     *     send.
     */
    public function isConfigured()
    {
        $settingsEmpty = empty($this->settings);
        $configured = false;
        if (!$settingsEmpty) {
            $this->loadSettingsFromConfig();
            $settings = $this->settings;
            $configured = strlen(ArrayUtil::get($settings, "server")) > 0
                && ArrayUtil::get($settings, "port") > 0
                && strlen(ArrayUtil::get($this->settings, "sender")) > 0
                && strlen(ArrayUtil::get($this->settings, "contact")) > 0;
        }

        if ($settingsEmpty || (!$configured && $this->alwaysLoadSettings)) {
            $this->loadSettingsFromConfig();
            $settings = $this->settings;
            $configured = strlen(ArrayUtil::get($settings, "server")) > 0
                && ArrayUtil::get($settings, "port") > 0
                && strlen(ArrayUtil::get($this->settings, "sender")) > 0
                && strlen(ArrayUtil::get($this->settings, "contact")) > 0;
        }

        return $configured;
    }

    public function loadSettingsFromConfig()
    {
        $config = $this->getConfig();
        if ($config === null) {
            return false;
        }

        $settings = array(
                "server" => $config->getMailServer(),
                "port" => $config->getMailPort(),
                "encryption" => $config->getMailEncryption(),
                "sender" => $config->getMailSender(),
                "contact" => $config->getMailContact(),
        );

        $this->setMailSettings($settings);
        return true;
    }

    /**
     * @return boolean true if settings are always loaded from auralis config
     *     before sending a message, false otherwise. Default: false
     */
    public function isAlwaysLoadSettings()
    {
        return $this->alwaysLoadSettings;
    }

    /**
     * Enables or disables the loading of settings from auralis config always
     * before sending a message. Default: false
     *
     * @param boolean $alwaysLoadSettings true to enable, false to disable.
     */
    public function setAlwaysLoadSettings($alwaysLoadSettings)
    {
        $this->alwaysLoadSettings = $alwaysLoadSettings;
    }

    public function setMailSettings(array $settings)
    {
        $this->settings = $settings;
        $transport = Swift_SmtpTransport::newInstance()
            ->setHost(ArrayUtil::get($settings, "server"))
            ->setPort(ArrayUtil::get($settings, "port"))
        ;
        $encryption = ArrayUtil::get($settings, "encryption");
        if (strlen($encryption) > 0 && $encryption != "none") {
            $transport->setEncryption($encryption);
        }

        $username = ArrayUtil::get($this->settings, "username");
        if (!empty($username)) {
            $transport->setUsername($username);
        }

        $password = ArrayUtil::get($this->settings, "password");
        if (!empty($password)) {
            $transport->setPassword($password);
        }

        $this->mailer = \Swift_Mailer::newInstance($transport);
    }

    /**
     * @param string $subject Subject of the message
     *
     * @return \Swift_Message
     */
    public function createMessage($subject)
    {
        $message = \Swift_Message::newInstance($subject);
        $message->setFrom(ArrayUtil::get($this->settings, "sender"), "auralis");

        return $message;
    }

    /**
     * Like createMessage, but sets recipient to configured admins
     * @param string $subject
     *
     * @return \Swift_Message
     */
    public function createAdminMessage($subject)
    {
        $message = $this->createMessage("[auralis] " . $subject);

        $addresses = explode(",", ArrayUtil::get($this->settings, "contact"));
        foreach ($addresses as $key => $address) {
            $addresses[$key] = trim($address);
            if (empty($addresses[$key])) {
                unset($addresses[$key]);
            }
        }

        $message->setTo($addresses);

        return $message;
    }

    /**
     * @param string $text
     * @return string The specified text with a salutation and a footer.
     */
    public function prepareAdminText($text)
    {
        $hostname = $this->getConfig()->getHostname();
        $secondaryPort = $this->container->get("apache")->getDefine("auralis_secondary_port");
        $salutation = $this->getTranslator()->trans("admin.mailer.salutation");
        $footer = $this->getTranslator()->trans("admin.mailer.footer", array(
            "%url%" => "https://" . $hostname . ":" . $secondaryPort . "/admin",
            "%hostname%" => $hostname,
            "%version%" => $this->getAuralisVersion(),
            "%time%" => $this->getTimeProvider()->getDate($this->tr("Y-m-d H:i:s")),
            "%auralisid%" => $this->getLicense()->getAuralisId(),
            "%expiration_date%" => date($this->tr("Y-m-d"), $this->getLicense()->getValidTo()),
            "%users%" => $this->getDB()->count("users"),
            "%users_allowed%" => $this->getLicense()->getAllowedUsers(),
        ));
        return $salutation . "\n\n" . $text . "\n\n" . $footer;
    }

    /**
     * Sends message to configured admins with specified subject and text.
     *
     * @param string $subject The mail subject.
     * @param string $text The mail text.
     * @return int The number of recipients who were accepted for delivery.
     */
    public function sendAdminMessage($subject, $text)
    {
        $text = $this->prepareAdminText($text);
        $message = $this->createAdminMessage($subject);
        $message->setBody($text);
        return $this->sendMessage($message);
    }

    /**
     * sends a message
     * @param \Swift_Message $message
     * @return int The number of recipients who were accepted for delivery.
     */
    public function sendMessage(\Swift_Message $message)
    {
        if (empty($this->settings) || $this->alwaysLoadSettings) {
            $this->loadSettingsFromConfig();
        }

        return $this->mailer->send($message);
    }

}
