<?php
namespace Topalis\Profile;

use Topalis\Util\CryptUtil;
use Topalis\Profile\Payload\PayloadFactory;
use Topalis\Profile\Payload\AbstractPayload;
use CFPropertyList\CFData;
use CFPropertyList\CFDate;
use CFPropertyList\CFBoolean;
use CFPropertyList\CFNumber;
use CFPropertyList\CFDictionary;
use CFPropertyList\CFArray;
use CFPropertyList\CFPropertyList;

/**
 * Abstract class representing a client profile
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */

abstract class AbstractProfile extends AbstractPayload
{
    /**
     * @var UUIDGenerator
     */
    private $uuidGenerator;

    private $encryptCert;

    /**
     * @var AbstractPayload[]
     */
    protected $payloads = array();

    public function set($key, $value)
    {
        if ($key == "PayloadContent") {
            throw new \InvalidArgumentException("Setting 'PayloadContent' directly is not allowed");
        }
        if ($key == "EncryptedPayloadContent") {
            throw new \InvalidArgumentException("Setting 'EncryptedPayloadContent' directly is not allowed");
        }
        parent::set($key, $value);
    }

    public function del($key)
    {
        if ($key == "PayloadContent") {
            throw new \InvalidArgumentException("Deleting 'PayloadContent' directly is not allowed");
        }
        if ($key == "EncryptedPayloadContent") {
            throw new \InvalidArgumentException("Deleting 'EncryptedPayloadContent' directly is not allowed");
        }
        parent::del($key);
    }

    /**
     * @param string $content profile content (plist)
     *
     * @throws \InvalidArgumentException
     */
    public function __construct($content = null)
    {
        if (null !== $content) {
            if (is_string($content)) {
                $plist = new CFPropertyList();
                $plist->parse($content);
                $dictionary = $plist->getValue();
            } else if ($content instanceof CFPropertyList) {
                $plist = $content;
                $dictionary = $plist->getValue();
            } else if ($content instanceof CFDictionary) {
                $dictionary = $content;
            } else {
                throw new \InvalidArgumentException("String, CFPropertyList, or CFDictionary expcected");
            }
            parent::__construct($dictionary);
            $this->payloads = $this->getPayloadsFromDictionary();
        } else {
            parent::__construct();
        }
        if ($this->getUUID() === null) {
            $this->setUUID("");
        }
        $this->setUUIDGenerator(new UUIDGenerator());
        $this->regenerateUUIDs(null, true);
    }

    public function hasRemovalPasscode()
    {
        return $this->getValue("HasRemovalPasscode", false);
    }

    public function isEncrypted()
    {
        return $this->getValue("IsEncrypted", false);
    }

    public function decrypt($cert, $key)
    {
        $currentPayloadContent = $this->get("PayloadContent");
        $encryptedPayloadContent = $this->getValue("EncryptedPayloadContent");
        if ($encryptedPayloadContent !== null) {
            $payloadContent = CryptUtil::decryptPKCS7($encryptedPayloadContent, $cert, $key);
            $plist = new CFPropertyList();
            $plist->parse($payloadContent);
            $payloadContent = $plist->getValue(true);
            if ($currentPayloadContent !== null) {
                if ($payloadContent !== null) {
                    foreach ($currentPayloadContent as $payload) {
                        $payloadContent->add($payload);
                    }
                } else {
                    $payloadContent = $currentPayloadContent;
                }
            }
            $this->setPayloadContent($payloadContent);
            $this->setEncryptedPayloadContent(null);
            $this->payloads = $this->getPayloadsFromDictionary();
            $this->findUUIDs();
        }
        $this->set("IsEncrypted", new CFBoolean(false));
    }

    public function encrypt($cert)
    {
        // encryption will be done when toPlist() is called
        // so it is still possible to add payloads
        $this->encryptCert = $cert;
        $this->set("IsEncrypted", new CFBoolean(true));
    }

    public function isRemovalDisallowed()
    {
        return $this->getValue("PayloadRemovalDisallowed", false);
    }

    public function setRemovalDisallowed($removalDisallowed)
    {
        if ($removalDisallowed === null) {
            $this->del("PayloadRemovalDisallowed");
        } else {
            $this->set("PayloadRemovalDisallowed", new CFBoolean($removalDisallowed));
        }
        return $this;
    }

    public function getRemovalDate($format = CFDate::TIMESTAMP_UNIX)
    {
        $date = $this->get("RemovalDate");
        return $date === null ? null : $date->getValue($format);
    }

    public function setRemovalDate($date, $format = CFDate::TIMESTAMP_UNIX)
    {
        if ($date === null) {
            $this->del("RemovalDate");
        } else {
            $this->set("RemovalDate", new CFDate($date, $format));
        }
        return $this;
    }

    public function getDurationUntilRemoval()
    {
        $this->getValue("DurationUntilRemoval");
    }

    public function setDurationUntilRemoval($durationUntilRemoval)
    {
        if ($durationUntilRemoval === null) {
            $this->del("DurationUntilRemoval");
        } else {
            $this->set("DurationUntilRemoval", new CFNumber((float) $durationUntilRemoval));
        }
        return $this;
    }

    public function addPayload(AbstractPayload $payload)
    {
        if ($payload->getUUID() === null) {
            $payload->setUUID("");
        }
        $this->regenerateUUIDs($payload->getDictionary(), true);

        $identifier = $payload->getIdentifier();
        if (empty($identifier)) {
            $identifier = $this->getIdentifier() . "." . $payload->getType() . "." . $payload->getUUID();
            $payload->setIdentifier($identifier);
        }
        if (null === $payload->getOrganization()) {
            $payload->setOrganization($this->getOrganization());
        }

        $payloadContent = $this->get("PayloadContent");
        if ($payloadContent === null) {
            $payloadContent = new CFArray();
            $this->setPayloadContent($payloadContent);
        }
        $payloadContent->add($payload->getDictionary());
        $this->payloads[] = $payload;
    }

    public function clearPayloads()
    {
        $this->payloads = array();
        $this->setPayloadContent(null);
    }

    public function clearPayloadWithType($type)
    {
        $count = count($this->payloads);
        // start with last payload because removePayload() updates indexes
        for ($i = $count - 1; $i >= 0; $i--) {
            $payload = $this->payloads[$i];
            if ($payload->getType() == $type) {
                $this->removePayload($i);
            }
        }
    }

    public function hasPayloads()
    {
        return count($this->getPayloads()) > 0
            || ($this->isEncrypted() && $this->get("EncryptedPayloadContent") !== null);
    }

    public function getPayload($index)
    {
        return $this->payloads[$index];
    }

    public function getPayloads($type = null)
    {
        if ($type === null) {
            return $this->payloads;
        }
        $payloads = array();
        foreach ($this->payloads as $payload) {
            if ($payload->getType() == $type) {
                $payloads[] = $payload;
            }
        }
        return $payloads;
    }

    public function removePayload($index)
    {
        array_splice($this->payloads, $index, 1);
        /*
         * we cant use del on PayloadContent because CFArray uses the php method
         * unset that keeps the other keys unchanged, so we create a new one
         */
        $payloadContent = new CFArray();
        foreach ($this->payloads as $payload) {
            $payloadContent->add($payload->getDictionary());
        }
        $this->setPayloadContent($payloadContent);
    }

    public function setPayloads(array $payloads)
    {
        $this->clearPayloads();
        foreach ($payloads as $payload) {
            $this->addPayload($payload);
        }
    }

    /**
     * @return \Topalis\Profile\UUIDGenerator
     */
    public function getUUIDGenerator()
    {
        // TODO return UUIDGenerator from parent profile if there is one
        return $this->uuidGenerator;
    }

    public function setUUIDGenerator($uuidGenerator)
    {
        $this->uuidGenerator = $uuidGenerator;
        $this->findUUIDs();
    }

    /**
     * Generate and set new UUIDs in this profile.
     *
     * @param \Iterator $it Set all payload UUIDs below this iterator. If null
     *    the profile's dictionary is used.
     * @param boolean $onlyIfUsed Specifies whether an existing UUID is
     *    replaced. true to replace only UUIDs that exists multiple times, or
     *    false to set all.
     */
    public function regenerateUUIDs(\Iterator $it = null, $onlyIfUsed = false)
    {
        $uuidGenerator =  $this->getUUIDGenerator();

        $queue = array($it === null ? $this->getDictionary() : $it);
        do {
            $it = array_shift($queue);
            foreach ($it as $key => $value) {
                /** @var $value CFType */
                if ($value instanceof \Iterator) {
                    $queue[] = $value;
                }

                if ("PayloadUUID" !== $key) {
                    continue;
                }

                $uuid = $value->getValue();
                if (!$onlyIfUsed || !is_string($uuid) || strlen($uuid) == 0 || $uuidGenerator->isUsed($uuid)) {
                    $value->setValue($uuidGenerator->generateUUID());
                } else {
                    $uuidGenerator->addUUID($uuid);
                }
            }
        } while (!empty($queue));
    }

    /**
     * In contrast to the overridden method this one always returns a cloned
     * plist, no matter if $clone is set to true or false.
     * @param boolean $clone ignored.
     *
     * @see \Topalis\Profile\Payload\AbstractPayload::toPlist()
     */
    public function toPlist($clone = true)
    {
        $clone; // unused
        if ($this->isEncrypted()) {
            if ($this->encryptCert === null) {
                return parent::toPlist();
            }
            $payloadContent = $this->get("PayloadContent");
            if ($payloadContent === null) {
                $this->setEncryptedPayloadContent(null);
            } else {
                $payloadContentPlist = new CFPropertyList();
                $payloadContentPlist->add($payloadContent);
                $payloadContentXml = $payloadContentPlist->toXML();
                $encryptedPayloadContent = CryptUtil::encryptPKCS7($payloadContentXml, $this->encryptCert);
                $this->setEncryptedPayloadContent(new CFData($encryptedPayloadContent));
            }
            $this->setPayloadContent(null);
            $plist = parent::toPlist();
            $this->setPayloadContent($payloadContent);
            return $plist;
        } else {
            $encryptedPayloadContent = $this->get("EncryptedPayloadContent");
            $this->setEncryptedPayloadContent(null);
            $plist = parent::toPlist();
            $this->setEncryptedPayloadContent($encryptedPayloadContent);
            return $plist;
        }
    }

    protected function setPayloadContent($payloadContent)
    {
        if ($payloadContent === null) {
            parent::del("PayloadContent");
        } else {
            parent::set("PayloadContent", $payloadContent);
        }
    }

    protected function setEncryptedPayloadContent($encryptedPayloadContent)
    {
        if ($encryptedPayloadContent === null) {
            parent::del("EncryptedPayloadContent");
        } else {
            parent::set("EncryptedPayloadContent", $encryptedPayloadContent);
        }
    }

    public function getConsentText()
    {
        return $this->getValue("ConsentText", array());
    }

    public function setConsentText(array $texts)
    {
        $this->setValue("ConsentText", $texts, self::TYPE_DICTIONARY);
    }

    protected function getPayloadsFromDictionary()
    {
        $payloads = array();
        $payloadContent = $this->get("PayloadContent");
        if ($payloadContent !== null) {
            if ($payloadContent instanceof CFDictionary) {
                $payloadContent = array($payloadContent);
            }
            foreach ($payloadContent as $dictionary) {
                $payload = PayloadFactory::fromDictionary($dictionary);
                $payloads[] = $payload;
            }
        }
        return $payloads;
    }

    protected function findUUIDs(\Iterator $it = null)
    {
        $uuidGenerator =  $this->getUUIDGenerator();
        $queue = array($it === null ? $this->getDictionary() : $it);
        do {
            $it = array_shift($queue);
            foreach ($it as $key => $value) {
                /** @var $value CFType */
                if ($value instanceof \Iterator) {
                    $queue[] = $value;
                }

                if ("PayloadUUID" != $key) {
                    continue;
                }

                $uuidGenerator->addUUID($value->getValue());
            }
        } while (!empty($queue));
    }

    /**
     * Clears existing payloads of this type and adds all new ones
     * @param string $type
     * @param array  $payloads
     */
    protected function clearAndSetPayloads($type, array $payloads)
    {
        $this->clearPayloadWithType($type);
        foreach ($payloads as $payload) {
            if (null === $payload) {
                continue;
            }
            $this->addPayload($payload);
        }
    }
}
