<?php
namespace Topalis\Profile\Plist;

use CFPropertyList\CFDictionary;
use CFPropertyList\CFPropertyList;
use CFPropertyList\CFBoolean;
use CFPropertyList\CFData;
use CFPropertyList\CFDate;
use CFPropertyList\CFNumber;
use CFPropertyList\CFString;
use CFPropertyList\CFTypeDetector;
use CFPropertyList\CFArray;

/**
 * A wrapper class around CFDictionary.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 * @see CFDictionary
 */
class CFDictionaryWrapper
{
    const TYPE_DETECT = 0;
    const TYPE_BOOLEAN = 1;
    const TYPE_DATA = 2;
    const TYPE_DATE = 3;
    const TYPE_FLOAT = 4;
    const TYPE_INTEGER = 5;
    const TYPE_NUMBER = 6;
    const TYPE_STRING = 7;
    const TYPE_ARRAY = 8;
    const TYPE_DICTIONARY = 9;

    /**
     *
     * @var CFDictionary
     */
    private $dictionary;

    /**
     * Constructor.
     *
     * @param CFDictionary $dictionary The initial dictionary. If null a new
     *        dictionary will be created.
     */
    public function __construct(CFDictionary $dictionary = null)
    {
        // We MUST access CFPropertyList once to have the different sub classes
        // of CFType available (why?)
        CFPropertyList::FORMAT_AUTO;
        if ($dictionary === null) {
            $this->dictionary = new CFDictionary();
        } else {
            $this->dictionary = $dictionary;
        }
    }

    /**
     * Get CFType.
     *
     * @param string $key Key of CFType to retrieve.
     * @return CFType CFType found at $key, null otherwise.
     */
    public function get($key)
    {
        return $this->dictionary->get($key);
    }

    /**
     * Get value.
     *
     * @param string $key Key of value to retrieve.
     * @param mixed $default The value returned if specified key does not exist.
     * @return mixed The value in the CFType found at $key, or $default if it
     *         does not exist.
     */
    public function getValue($key, $default = null)
    {
        $value = $this->get($key);
        return $value === null ? $default : $value->toArray();
    }

    /**
     * Deletes CFType.
     *
     * @param string $key Key of CFType to remove.
     */
    public function del($key)
    {
        $this->dictionary->del($key);
    }

    /**
     * Add CFType.
     *
     * @param string $key Key to add.
     * @param CFType|NULL $value CFType to add. If null the key will be
     *        removed.
     */
    public function set($key, $value)
    {
        if ($value === null) {
            $this->del($key);
        } else {
            $this->dictionary->add($key, $value);
        }
    }

    /**
     * Add value.
     *
     * @param string $key Key to add.
     * @param mixed $value value to add. Will be converted to CFType based on
     *        the specified type. If null the key will be removed. If the value
     *        is already a CFType, no conversion will be done.
     * @param int $type The type of the value. If TYPE_DETECT CFTypeDetector is
     *        used.
     * @throws \InvalidArgumentException
     */
    public function setValue($key, $value, $type)
    {
        if ($value === null) {
            $this->del($key);
            return;
        }

        $this->set($key, $this->convertToCFType($value, $type));
    }

    /**
     * Convert the CFTypes in the array at the specified key to objects of the
     * type $class. The respective CFType is used as constructor argument of
     * type $class.
     *
     * @param string $key The key of the array.
     * @param string $class The type of the objects to return.
     *
     * @return array|NULL Array of objects of type $class, or null if no value
     *         is set at the specified key.
     */
    public function getSubTypeArray($key, $class)
    {
        $valueList = array();
        $values = $this->get($key);
        if (null === $values) {
            return null;
        }

        foreach ($values as $value) {
            $valueList[] = new $class($value);
        }

        return $valueList;
    }

    /**
     * Sets an array at the specified key containing the specified values
     * converted to a CFType. If a value is already a CFType no conversion is
     * done. Otherwise the method toCFType(), or getDictionary() is called, if
     * existant, else CFTypeDetector is used for conversion.
     *
     * @param string $key The key where the array will be set.
     * @param array|NULL $values The values to convert and set. If null the
     *        array at the specified key will be removed.
     */
    public function setSubTypeArray($key, array $values = null)
    {
        if (null === $values) {
            $this->del($key);
            return;
        }

        $valueList = array();
        foreach ($values as $value) {
            if ($value instanceof CFType) {
                $valueList[] = $value;
            } else if (method_exists($value, "toCFType")) {
                $valueList[] = $value->toCFType();
            } else if (method_exists($value, "getDictionary")) {
                $valueList[] = $value->getDictionary();
            } else {
                $td = new CFTypeDetector();
                $valueList[] = $td->toCFType($value);
            }
        }

        $this->setValue($key, $valueList, self::TYPE_ARRAY);
    }

    /**
     * @param boolean $clone true to return a cloned plist, otherwise it
     *     contains the CFDictionary of this payload, i.e. changes to that plist
     *     affects the payload. Defaults to true.
     * @return \CFPropertyList\CFPropertyList A CFPropertyList containing the
     *     payload's dictionary, or a cloned dictionary, resp.
     */
    public function toPlist($clone = true)
    {
        $plist = new CFPropertyList();
        $plist->add($this->getDictionary());
        if ($clone) {
            $plistClone = new CFPropertyList();
            $plistClone->parse($plist->toXML());
            $plist = $plistClone;
        }
        return $plist;
    }

    /**
     * @return array The dictionary as array.
     */
    public function toArray()
    {
        return $this->toPlist(false)->toArray();
    }

    /**
     * @param boolean $formatted Print plist formatted (i.e. with newlines and
     *     whitespace indention) if true; defaults to false.
     * @return string The dictionary as XML embedded in a plist.
     */
    public function toXML($formatted = false)
    {
        return $this->toPlist(false)->toXML($formatted);
    }

    /**
     * @return \CFPropertyList\CFDictionary The dictionary.
     */
    protected function getDictionary()
    {
        return $this->dictionary;
    }

    /**
     * Converts the specified value to a CFType. If the value is already a
     * CFType, no conversion will be done.
     *
     * @param mixed $value The value to convert.
     * @param int $type The type of the value. If TYPE_DETECT CFTypeDetector is
     *        used.
     * @throws \InvalidArgumentException
     * @return \CFPropertyList\CFType
     */
    protected function convertToCFType($value, $type)
    {
        if ($value instanceof CFType) {
            return $value;
        }

        switch ($type) {
            case self::TYPE_BOOLEAN:
                return new CFBoolean($value);
            case self::TYPE_DATA:
                return new CFData($value);
            case self::TYPE_DATE:
                return new CFDate($value);
            case self::TYPE_FLOAT:
                return new CFNumber((float) $value);
            case self::TYPE_INTEGER:
                return new CFNumber((int) $value);
            case self::TYPE_NUMBER:
                return new CFNumber($value);
            case self::TYPE_STRING:
                return new CFString($value);
            case self::TYPE_ARRAY:
                $td = new CFTypeDetector();
                $cfValue = $td->toCFType($value);
                if (! ($cfValue instanceof CFArray)) {
                    throw new \InvalidArgumentException("Expected non-associative array");
                }
                return $cfValue;
            case self::TYPE_DICTIONARY:
                $td = new CFTypeDetector();
                $cfValue = $td->toCFType($value);
                if (! ($cfValue instanceof CFDictionary)) {
                    throw new \InvalidArgumentException("Expected associative array");
                }
                return $cfValue;
            case self::TYPE_DETECT:
                $td = new CFTypeDetector();
                return $td->toCFType($value);
            default:
                throw new \InvalidArgumentException("Invalid type specification");
        }
    }
}
