<?php
namespace Topalis\Profile;

use CFPropertyList\CFDictionary;
use CFPropertyList\CFPropertyList;
use CFPropertyList\CFString;
use CFPropertyList\CFTypeDetector;
use Symfony\Component\DependencyInjection\ContainerAware;
use Topalis\Certificate\Certificate;
use Topalis\Certificate\Util\CertUtil;
use Topalis\Device\Entity\Device;
use Topalis\Device\Entity\MailAccount;
use Topalis\Device\Type\AndroidDeviceType;
use Topalis\Profile\Payload\Android\AndroidPayloadFactory;
use Topalis\Profile\Payload\Data\Ipv4;
use Topalis\Profile\Payload\Data\OnDemandRule;
use Topalis\Profile\Payload\Data\OpenVpnConfig;
use Topalis\Profile\Payload\PayloadFactory;
use Topalis\Profile\PayloadEvent;
use Topalis\Profile\PayloadEvents;
use Topalis\Util\Random;

/**
 * This class is used to create the different types of iOS profiles.
 * Note: The profile identifiers MUST NOT be changed to do profile updates or
 *     removals!
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ProfileFactory extends ContainerAware
{
    /**
     * @var string
     */
    private $uploadDir;

    /**
     * @var boolean
     */
    private $userDevelopmentApns = false;

    public static function fromArray(array $payload)
    {
        $td = new CFTypeDetector();
        $dictionary = $td->toCFType($payload);
        return self::fromDictionary($dictionary);
    }

    public static function fromDictionary(CFDictionary $dictionary)
    {
        $type = $dictionary->get("PayloadType");
        if ($type !== null) {
            $type = $type->getValue();
        }
        switch ($type) {
            case ConfigurationProfile::TYPE:
                $profile = new ConfigurationProfile($dictionary);
                break;
            case ProfileServiceProfile::TYPE:
                $profile = new ProfileServiceProfile($dictionary);
                break;
            case AndroidConfigurationProfile::TYPE:
                $profile = new AndroidConfigurationProfile($dictionary);
                break;
            default:
                $profile = null;
                break;
        }

        return $profile;
    }

    public static function fromXML($xml)
    {
        $plist = new CFPropertyList();
        $plist->parse($xml);

        return self::fromDictionary($plist->getValue());
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    public function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Doctrine\Common\Persistence\ObjectManager
     */
    private function getEntityManager()
    {
        return $this->container->get("doctrine.orm.default_entity_manager");
    }

    /**
     * @return \Topalis\Device\Entity\MailAccountEntityManager
     */
    private function getMailAccountEntityManager()
    {
        return $this->container->get("device.entity.manager.mail_account");
    }

    private function getEventDispatcher()
    {
        return $this->container->get("event_dispatcher");
    }

    /**
     * @return \Symfony\Bundle\FrameworkBundle\Routing\Router
     */
    public function getRouter()
    {
        return $this->container->get("router");
    }

    /**
     * @return \Symfony\Bundle\FrameworkBundle\Translation\Translator
     */
    public function getTranslator()
    {
        return $this->container->get("translator");
    }

    /**
     * @param mixed $uploadDir
     *
     * @return self
     */
    public function setUploadDir($uploadDir)
    {
        $this->uploadDir = $uploadDir;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getUploadDir()
    {
        return $this->uploadDir;
    }

    /**
     * @return boolean
     */
    protected function isUseDevelopmentApns()
    {
        return $this->userDevelopmentApns;
    }

    /**
     * @param boolean $useDevelopmentApns
     */
    public function setUseDevelopmentApns($useDevelopmentApns)
    {
        $this->userDevelopmentApns = $useDevelopmentApns;
    }

    private function getPayloadFactory(Device $device, AbstractProfile $profile)
    {
        if ($device->getType() instanceof AndroidDeviceType) {
            return new AndroidPayloadFactory($profile, $this->getConfig());
        } else {
            return new PayloadFactory($profile, $this->getConfig());
        }
    }

    /**
     * Creates a profiles for the specified device based on the settings of its
     * config group.
     *
     * @param Device $device The device to create the profile for.
     * @return \Topalis\Profile\ConfigurationProfile
     */
    public function createDeviceProfile(Device $device)
    {
        $profile = $device->getType()->getDeviceProfile($device);

        $caCert = $this->getCertificateManager()->getCACertificate();
        $payloadFactory = new PayloadFactory($profile, $this->getConfig());
        $caCertPayload = $payloadFactory->createRootCertificatePayload(CertUtil::pem2der($caCert->getPem()));
        $profile->addPayload($caCertPayload);

        $profile->setIdentifier(self::getDeviceProfileIdentifier());
        $profile->setDisplayName("auralis Device profile");
        return $profile;
    }

    /**
     * @param \Topalis\Device\Entity\Device $device
     * @param string                        $scepChallenge
     *
     * @return ConfigurationProfile The created profile.
     */
    public function createEnrollProfile(Device $device, $scepChallenge)
    {
        $profile = new ConfigurationProfile();
        $profile->setIdentifier(self::getRolloutProfileIdentifier());
        $profile->setOrganization($this->getCertificateManager()->getCAOrg());
        $profile->setDisplayName("auralis Enroll");
        $profile->setDescription("Enrolls Device Encryption Identification");

        $payloadFactory = new PayloadFactory($profile, $this->getConfig());
        $caCert = $this->getCertificateManager()->getCACertificate();
        if (null !== $scepChallenge) {
            $scepPayload = $payloadFactory->createScepPayload("Profile Service", $caCert, $scepChallenge);
            $profile->addPayload($scepPayload);
        } else {
            $cert = $this->getCertificateManager()->createClientCert($device, "Profile Service", '');
            $pkcs12 = $this->getCertificateManager()->createClientP12($cert, "install");
            $pkcs12Payload = $payloadFactory->createPkcs12Payload($pkcs12, "install");
            $pkcs12Payload->setDisplayName("device-".$device->getId().".p12");
            $pkcs12Payload->setPayloadCertificateFileName("device-".$device->getId().".p12");
            $profile->addPayload($pkcs12Payload);
        }
        $this->getEventDispatcher()->dispatch(PayloadEvents::CREATE_ENROLL_PROFILE,
            new PayloadEvent($device, $payloadFactory, $profile)
        );

        return $profile;
    }

    /**
     * @param \Topalis\Device\Entity\Device $device
     * @param string                        $scepChallenge
     *
     * @return ConfigurationProfile The created profile.
     */
    public function createMdmProfile(Device $device, $scepChallenge)
    {
        $profile = new ConfigurationProfile();
        $profile->setDisplayName("auralis MDM");
        $profile->setDescription("auralis Management Connection");
        $profile->setIdentifier(self::getRolloutProfileIdentifier());
        $profile->setOrganization($this->getCertificateManager()->getCAOrg());

        $payloadFactory = new PayloadFactory($profile, $this->getConfig());
        $caCert = $this->getCertificateManager()->getCACertificate();
        $caCertPayload = $payloadFactory->createRootCertificatePayload($caCert->getDer());
        $profile->addPayload($caCertPayload);
        $scepPayload = $payloadFactory->createScepPayload("MDM Authentication", $caCert, $scepChallenge);
        $profile->addPayload($scepPayload);
        $mdmTopic = $device->getType()->getPushTopic();
        $useDevelopmentApns = $this->isUseDevelopmentApns();
        $mdmPayload = $payloadFactory->createMdmPayload($mdmTopic, $scepPayload->getUUID(), $useDevelopmentApns);
        $profile->addPayload($mdmPayload);

        return $profile;
    }

    /**
     *
     * @param Device $device
     * @return \Topalis\Profile\ProfileServiceProfile
     */
    public function createInitialProfile(Device $device)
    {
        // Register a device. Provide a new profile and assign this device to the requesting user.
        do {
            // find unique challenge
            $challenge = Random::getString(20);
        } while (null !== $this->getDeviceEntityManager()->findOneBy(array("challenge" => sha1($challenge))));

        $device->setPlainChallenge($challenge);

        $profile = new ProfileServiceProfile();
        $profile->setIdentifier(self::getRolloutProfileIdentifier());
        $profile->setOrganization($this->getCertificateManager()->getCAOrg());
        $profile->setDisplayName("auralis OTA RollOut");
        $profile->setDescription("Install this profile to get access through auralis");

        $payloadFactory = new PayloadFactory($profile, $this->getConfig());
        $payload = $payloadFactory->createProfileServicePayload($challenge);
        $profile->setPayload($payload);

        $this->getDeviceEntityManager()->save($device);

        return $profile;
    }

    /**
     *
     * @param MailAccount $mailAccount
     * @throws \Exception
     * @return \Topalis\Profile\ConfigurationProfile
     */
    public function createMailProfile(MailAccount $mailAccount)
    {
        $device = $mailAccount->getDevice();

        $profile = new ConfigurationProfile();
        $profile->setIdentifier(self::getMailProfileIdentifier($mailAccount));
        $profile->setOrganization($this->getCertificateManager()->getCAOrg());
        $payloadFactory = new PayloadFactory($profile, $this->getConfig());

        $exchangeCert = $this->getCertificateManager()->initClientCert($device);
        $caCert = $this->getCertificateManager()->getCACertificate();
        $scepPayload = $payloadFactory->createScepPayload(
            $mailAccount->getEmail(), $caCert, $exchangeCert->getPlainChallenge(), $mailAccount->getEmail()
        );
        $profile->addPayload($scepPayload);

        $exchangePayload = $payloadFactory->createExchangePayload(
            $mailAccount->getEmail(), $mailAccount->getUser(), $device->getConfigGroup());
        $exchangePayload->setCertificateUUID($scepPayload->getUUID());
        $profile->addPayload($exchangePayload);
        $profile->setDisplayName("auralis Groupware " . $mailAccount->getEmail());
        $profile->setDescription("auralis Groupware Account for " . $mailAccount->getEmail());

        $cert = $mailAccount->getCert();
        if ($cert !== null) {
            $cert->setUnused(true);
            $this->getEntityManager()->persist($cert);
        }

        $mailAccount->setCert($exchangeCert);
        $this->getMailAccountEntityManager()->save($mailAccount);

        return $profile;
    }

    public function createSecureIntranetProfile(\Topalis\Device\Entity\Device $device)
    {
        $profile = new ConfigurationProfile();
        $profile->setIdentifier(self::getProfileIdentifierPrefix() . ".secure_intranet");
        $profile->setOrganization($this->getCertificateManager()->getCAOrg());
        $profile->setDisplayName("auralis SecureIntranet");
        $profile->setDescription("Certificate to access the auralis SecureIntranet");

        $cert = $this->getCertificateManager()->initClientCert($device);
        $caCert = $this->getCertificateManager()->getReverseProxyCACert();

        $payloadFactory = new PayloadFactory($profile, $this->getConfig());
        $scepPayload = $payloadFactory->createScepPayload(
            "SecureIntranet", $caCert, $cert->getPlainChallenge(), $device->getUser()->getEmail()
        );
        $profile->addPayload($scepPayload);

        $caCertPayload = $payloadFactory->createRootCertificatePayload(CertUtil::pem2der($caCert->getPem()));
        $profile->addPayload($caCertPayload);

        return $profile;
    }

    public function createWebClipProfile(Device $device)
    {
        $config = $this->getConfig();

        $profile = new ConfigurationProfile();
        $profile->setIdentifier(self::getWebClipsProfileIdentifier());
        $profile->setOrganization($this->getCertificateManager()->getCAOrg());
        $profile->setDisplayName("auralis Web Clips");

        $payloadFactory = new PayloadFactory($profile, $this->getConfig());

        $urls = $device->getConfigGroup()->getReverseProxyUrls();
        foreach ($urls as $url) {
            if (!$url->isVisible()) {
                continue;
            }

            $externalUrl = "https://". $config->getHostname() . ":" . $config->getPrimaryPort()
                . "/proxy/" . $url->getExternalPath();
            $label = $url->getName();
            $webClipPayload = $payloadFactory->createWebClipPayload($externalUrl, $label, $url->getId());

            $filename = $this->getUploadDir() . "/reverse_proxy/" . $url->getIcon();
            if (is_file($filename)) {
                $icon = file_get_contents($filename);
                $webClipPayload->setIcon($icon);
            }

            $webClipPayload->setFullScreen($url->isFullscreen());
            $webClipPayload->setRemovable($url->isRemovable());
            $webClipPayload->setPreComposed($url->isPreComposed());

            $profile->addPayload($webClipPayload);
        }

        // normal webclips
        $webClips = $device->getConfigGroup()->getWebClips();
        foreach ($webClips as $webClip) {
            $webClipPayload = $payloadFactory->createWebClipPayload($webClip->getUrl(),
                $webClip->getName(), $webClip->getId());
            $filename = $this->getUploadDir() . "/". $webClip->getUploadDir() ."/" . $webClip->getIcon();
            if (is_file($filename)) {
                $icon = file_get_contents($filename);
                $webClipPayload->setIcon($icon);
            }

            $webClipPayload
                ->setFullScreen($webClip->isFullscreen())
                ->setRemovable($webClip->isRemovable())
                ->setPreComposed($webClip->isPrecomposed())
            ;

            $profile->addPayload($webClipPayload);
        }

        return $profile;
    }

    public function createWifiProfile(Device $device)
    {
        $profile = new ConfigurationProfile();
        $profile->setIdentifier(self::getWifiProfileIdentifier());
        $profile->setOrganization($this->getCertificateManager()->getCAOrg());
        $profile->setDisplayName("auralis WiFi");

        $payloadFactory = $this->getPayloadFactory($device, $profile);

        $wifiConfigs = $device->getConfigGroup()->getWifiConfigs();
        foreach ($wifiConfigs as $wifiConfig) {
            $wifiPayload = $payloadFactory->createWifiPayload($wifiConfig);
            $profile->addPayload($wifiPayload);
        }

        return $profile;
    }

    /**
     * @param Device $device
     * @param string $cn CN for the certificate
     * @param bool   $wifi also route wifi traffic over vpn
     * @param bool   $scep use scep for client certificate. PKCS#12 will be added otherwise.
     *
     * @return \Topalis\Profile\ConfigurationProfile
     */
    public function createMobileControlProfile(Device $device, $cn, $wifi = false, $scep = true)
    {
        $caCert = $this->getCertificateManager()->getCACertificate();

        $profile = new ConfigurationProfile();
        $profile->setIdentifier(self::getProfileIdentifierPrefix() . ".mobile_control");
        $profile->setOrganization($this->getCertificateManager()->getCAOrg());
        $profile->setDisplayName("auralis Mobile-Control");
        $payloadFactory = new PayloadFactory($profile, $this->getConfig());

        $vpnPayload = $payloadFactory->createVpnPayload();
        $vpnPayload->setDisplayName("VPN (auralis-TrafficControl)");
        $profile->addPayload($vpnPayload);
        $ipv4 = new Ipv4();
        $ipv4->setOverridePrimary(true);
        $vpnPayload->setIpv4($ipv4);
        $openVpnConfig = new OpenVpnConfig();
        $openVpnConfig->setAuthName("device-".$device->getId());
        $openVpnConfig->setAuthenticationMethod("Certificate");
        $openVpnConfig->setOnDemandEnabled(true);
        $openVpnConfig->setRemoteAddress($this->getConfig()->getOpenVpnServer());

        $onDemandRules = array();
        // always connect when on cellular
        $cellularRule = new OnDemandRule();
        $cellularRule->setAction("Connect");
        $cellularRule->setInterfaceTypeMatch("Cellular");
        $onDemandRules[] = $cellularRule;

        // connect when on wifi
        if ($wifi) {
            $wifiRule = new OnDemandRule();
            $wifiRule->setAction("Connect");
            $wifiRule->setInterfaceTypeMatch("WiFi");
            $onDemandRules[] = $wifiRule;
        }
        $openVpnConfig->setOnDemandRules($onDemandRules);

        $vpnPayload->setUserDefinedName("auralis-TrafficControl");
        $vpnPayload->setVendorConfig(array(
            "ca"       => str_replace("\n", "\\n", trim($caCert->getPem())),
            "cipher"   => "none",
            "auth"     => "none",
            "comp-lzo" => "value",
            "dev"      => "tun",
            "port"     => new CFString($this->getConfig()->getOpenVpnPort()), // must be casted to string
            "proto"    => "udp",
            "remote"   => $this->getConfig()->getOpenVpnServer().":".$this->getConfig()->getOpenVpnPort(),
            "verb"     => new CFString(3), // must be casted to string
        ));

        if ($scep) {
            $cert = $this->getCertificateManager()->initClientCert($device);
            $scepPayload = $payloadFactory->createScepPayload($cn, $caCert, $cert->getChallenge());
            $profile->addPayload($scepPayload);
            $openVpnConfig->setPayloadCertificateUUID($scepPayload->getUUID());
        } else {
            $cert = $this->getCertificateManager()->createClientCert($device, $cn, '');
            $pkcs12 = $this->getCertificateManager()->createClientP12($cert, "install");
            $pkcs12Payload = $payloadFactory->createPkcs12Payload($pkcs12, "install");
            $pkcs12Payload->setDisplayName("device-".$device->getId().".p12");
            $pkcs12Payload->setPayloadCertificateFileName("device-".$device->getId().".p12");
            $profile->addPayload($pkcs12Payload);
            $openVpnConfig->setPayloadCertificateUUID($pkcs12Payload->getUUID());
        }
        $vpnPayload->setOpenVpn($openVpnConfig);

        return $profile;
    }

    public static function getDeviceProfileIdentifier()
    {
        return self::getProfileIdentifierPrefix() . ".device";
    }

    public static function getMailProfileIdentifier(MailAccount $mailAccount)
    {
        // The identifier of mail profiles MUST NOT be changed!
        return self::getProfileIdentifierPrefix() . ".exchange." . $mailAccount->getId();
    }

    public static function getRolloutProfileIdentifier()
    {
        // The identifier of the MDM profile MUST NOT be changed!
        return self::getProfileIdentifierPrefix() . ".ota";
    }

    public static function getWebClipsProfileIdentifier()
    {
        // The identifier of the WebClip profile MUST NOT be changed!
        return self::getProfileIdentifierPrefix() . ".webclips";
    }

    public static function getWifiProfileIdentifier()
    {
        // The identifier of the WebClip profile MUST NOT be changed!
        return self::getProfileIdentifierPrefix() . ".wifi";
    }

    public static function getProfileIdentifierPrefix()
    {
        // The profile identifier prefix MUST NOT be changed!
        return "com.to.auralis";
    }
}
