<?php
namespace Topalis\ReverseProxy\Validator;

use Symfony\Component\Validator\Constraint;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Exception\UnexpectedTypeException;
use Symfony\Component\HttpFoundation\Request;
use Topalis\Configuration\AuralisConfig;
use Topalis\ReverseProxy\Entity\ReverseProxyUrl;

/**
 * If the configured web proxy is used as remote proxy in Apache, the host of
 * the internal URL must not refer to the auralis host itself because the
 * configuration in Apache is applied to the whole virtual host. That means
 * that any request to that URL would be passed to the proxy and thus it would
 * not be reachable, anymore.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class InternalHostValidator extends ConstraintValidator
{
    /**
     * @var AuralisConfig
     */
    private $config;

    /**
     * @var Request
     */
    private $request;

    public function __construct(Request $request, AuralisConfig $config)
    {
        $this->request = $request;
        $this->config = $config;
    }

    public function validate($value, Constraint $constraint)
    {
        if (!$value instanceof ReverseProxyUrl) {
            throw new UnexpectedTypeException($value, "Topalis\ReverseProxy\Entity\ReverseProxyUrl");
        }

        if (!$value->isUseRemoteProxy()) {
            // if no remote proxy is used any host is allowed
            return;
        }

        $forbiddenHosts = array(
            "127.0.0.1",
            "localhost",
            strtolower($this->config->getHostname()),
            strtolower($this->request->server->get("SERVER_NAME")),
            strtolower($this->request->server->get("SERVER_ADDR")),
            strtolower($this->request->headers->get("HOST")),
        );

        $host = strtolower(parse_url($value->getInternalPath(), PHP_URL_HOST));
        if (in_array($host, $forbiddenHosts)) {
            $this->context->addViolationAt("internalPath", $constraint->message);
        }
    }
}
