<?php
namespace Topalis\Rollout\Controller;

use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Topalis\Device\Entity\Device;
use Topalis\Device\Type\AndroidDeviceType;
use Topalis\Device\Type\IosDeviceType;
use Topalis\Kernel\Controller;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;

/**
 * Class to deploy the initial profiles.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Frederick Roth <frederick.roth@topalis.com>
 * @author Fritz Elfert <fritz.elfert@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("", service="rollout.controller.deploy")
 */
class DeployController extends Controller
{
    /**
     * @Template()
     * @Route("/", name="rollout_index")
     * @Route("/rollout", name="rollout_deploy")
     *
     * @return array
     */
    public function indexAction()
    {
        // Handle GET request.
        $this->getLogger()->debug("DeployController: handle request");

        $session = $this->getSession();
        $user = $session->get("user");

        $device = $this->getDevice();
        if ($device === null) {
            $errorMsg = $this->tr("rollout.error.noProfile", array("%user%" => $user));
            $this->logout();
            return array("errorMessage" => $errorMsg);
        }

        if ($device->isIPhone()) {
            $respTxt = $this->tr("rollout.success");
        } else if ($device->getType()->getName() == "other") {
            $respTxt = $this->tr("rollout.otherSuccess");
        } else {
            $errorMsg = $this->tr("rollout.error.deviceTypeNotSupported");
            $this->logout();
            return array("errorMessage" => $errorMsg);
        }

        return array(
            "response" => $respTxt,
            "download" => true,
            "iPhone" => $device->isIPhone()
        );
    }

    /**
     * For Android devices.
     * Note: We could do the rollout without the initial signing on the Android
     * device because we cannot verify it, anyway.
     *
     * @Template()
     * @Route("/profile/android", name="rollout_profile_android")
     *
     * @return \Symfony\Component\HttpFoundation\Response The response.
     */
    public function profileAndroidAction()
    {
        return $this->getProfileService()->handleRequest($this->getRequest());
    }

    /**
     * For iOS devices.
     *
     * @Template()
     * @Route("/profile/ios", name="rollout_profile_ios")
     *
     * @return \Symfony\Component\HttpFoundation\Response The response.
     */
    public function profileIosAction()
    {
        return $this->getProfileService()->handleRequest($this->getRequest());
    }

    /**
     * @Route("/license", name="rollout_license")
     */
    public function licenseAction()
    {
        $device = $this->getDevice();
        if ($device === null) {
            throw new AccessDeniedHttpException();
        }
        $type = $device->getType();
        if (!$type instanceof AndroidDeviceType) {
            throw new NotFoundHttpException();
        }

        return new Response($this->container->get("config.auralis")->getSamsungLicense());
    }

    /**
     * @return Device
     */
    protected function getDevice()
    {
        return $this->getSecurityContext()->getToken()->getUser();
    }

    /**
     * @Route("/download", name="rollout_download")
     * @return Response
     * @throws \Exception
     */
    public function downloadAction()
    {
        $device = $this->getDevice();
        if ($device === null) {
            throw new \Exception("No device");
        }
        // reset the device password to make an additional rollout impossible
        $device->setPassword(null);
        $this->getDeviceEntityManager()->save($device);
        $user = $device->getUser();
        $this->getLogger()->debug("DeployController: Send initial profile for user '".$user->getLogin()."'.");
        if ($device->getType() instanceof IosDeviceType || $device->getType() instanceof AndroidDeviceType) {
            $profile = $this->createInitialProfile($device);
            $response = $this->createProfileResponse($profile);
        } else {
            $p12 = $this->createClientP12($device);
            $response = $this->createP12Response($p12);
        }

        $this->logout();
        return $response;
    }

    protected function createInitialProfile(Device $device)
    {
        return $this->getProfileFactory()->createInitialProfile($device)->toXML();
    }

    protected function createProfileResponse($content)
    {
        $serverCert = $this->getCertificateManager()->getServerCert();
        $content = $this->getCertificateManager()->sign($content, $serverCert->getResource(),
            $serverCert->getPrivateKey());
        $response = new Response($content);
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            "client.mobileconfig"
        );
        $response->headers->set("Content-type", "application/x-apple-aspen-config; charset=utf-8");
        $response->headers->set("Content-Disposition", $disposition);

        return $response;
    }

    protected function createClientP12(Device $device)
    {
        if ($device->getCert() === null) {
            $cert = $this->getCertificateManager()->createClientCertForDevice($device);
            $device->setCert($cert);
            $this->getDeviceEntityManager()->save($device);
        }

        $password = $this->getSecurityContext()->getToken()->getAttribute("password");
        return $this->getCertificateManager()->createClientP12($device->getCert(), $password);
    }

    protected function createP12Response($content)
    {
        $response = new Response($content);
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            "client.P12"
        );
        $response->headers->set("Content-type", "application/x-pkcs12");
        $response->headers->set("Content-Disposition", $disposition);
        $response->headers->set("Content-Length", strlen($content));
        return $response;
    }

    protected function logout()
    {
        $this->getSecurityContext()->setToken(null);
        $this->getSession()->invalidate(1);
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Topalis\Profile\ProfileFactory
     */
    private function getProfileFactory()
    {
        return $this->container->get("profile.factory");
    }

    /**
     * @return \Topalis\Rollout\ProfileService
     */
    private function getProfileService()
    {
        return $this->container->get("rollout.profile_service");
    }

    /**
     * @return \Symfony\Component\Security\Core\SecurityContext
     */
    public function getSecurityContext()
    {
        return $this->container->get("security.context");
    }
}
