<?php
namespace Topalis\Rollout;

use Topalis\Kernel\AbstractAuralisService;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;
use Symfony\Component\HttpFoundation\Response;
use Topalis\Device\Entity\Device;
use CFPropertyList\CFPropertyList;
use Topalis\HttpError;
use Topalis\Certificate\Util\CertUtil;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Security\Core\Authentication\Token\UsernamePasswordToken;
use Symfony\Component\Security\Core\Exception\AuthenticationException;

/**
 * The ProfileService is used in the rollout process (phase 2 and 3).
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 * @see Apple: Over-the-Air Profile Delivery and Configuration
 * https://developer.apple.com/library/ios/#documentation/NetworkingInternet/Conceptual/iPhoneOTAConfiguration/
 */
class ProfileService extends AbstractAuralisService
{
    protected $requestCert;
    protected $requestSignerName;

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\MDM\MdmAdmin
     */
    private function getMdmAdmin()
    {
        return $this->container->get("mdm.admin");
    }

    /**
     * @return \Topalis\Profile\ProfileFactory
     */
    private function getProfileFactory()
    {
        return $this->container->get("profile.factory");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    public function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * Handles the specified request and returns a fitting response.
     *
     * @param Request $request The request to handle.
     * @return \Symfony\Component\HttpFoundation\Response The response.
     */
    public function handleRequest(Request $request)
    {
        $this->getLogger()->debug("ProfileService: handleRequest");

        try {
            $content = $request->getContent();
            $chainedCA = $this->getCertificateManager()->getChainedCACert();
            $pkcs7 = CertUtil::decodePKCS7($content, PKCS7_NOCHAIN | PKCS7_BINARY, $chainedCA);

            $parsedSignerCert = openssl_x509_parse($pkcs7['signerCert']);
            $this->requestSignerName = $parsedSignerCert['name'];
            $this->requestCert = $pkcs7['signerCert'];
            if (isset($parsedSignerCert['extensions']['authorityKeyIdentifier'])) {
                $subjectKeyId = isset($parsedSignerCert['extensions']['subjectKeyIdentifier']) ?
                    $parsedSignerCert['extensions']['subjectKeyIdentifier'] : null;
                $this->getLogger()->debug("ProfileService: authorityKeyIdentifier: "
                    . $parsedSignerCert['extensions']['authorityKeyIdentifier']
                    . "subjectKeyIdentifier: " . $subjectKeyId);
            }

            $plistData = $pkcs7['data'];
            if (strlen($plistData) == 0) {
                $this->getLogger()->info("ProfileService: Invalid PKCS7 container");
                return new Response("Invalid PKCS7 container", 400);
            }

            try {
                $plist = new CFPropertyList();
                $plist->parse($plistData);
                $plistProperties = $plist->toArray();
            } catch (\Exception $e) {
                $this->getLogger()->warn("ProfileService: Invalid plist", array("exception" => $e));
                return new Response("Invalid plist", 400);
            }

            return $this->handlePlist($plistProperties, $pkcs7, $request);
        } catch (\Exception $e) {
            $this->getLogger()->error("ProfileService: handling request failed", array("exception" => $e));
            return new Response(Response::$statusTexts[500], 500);
        }
    }

    /**
     * @param array $plistProperties
     * @param array $pkcs7
     * @param Request $request The request.
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function handlePlist(array $plistProperties, array $pkcs7, Request $request)
    {
        // indicate if we are using Device Enrollment Program
        $dep = false;
        if (isset($plistProperties['CHALLENGE'])) {
            // First seen... Check for CHALLENGE in device table
            $seenPreviously = false;
            $deviceArr = $this->getDeviceEntityManager()
                ->findBy(array("challenge" => sha1($plistProperties['CHALLENGE']), "deleted" => 0));
        } else if ($request->headers->has("PHP_AUTH_USER") && $request->headers->has("PHP_AUTH_PW")) {
            // Device Enrollment Program using ConfigurationURL #AUR-525
            $seenPreviously = false;
            $dep = true;
            $authProvider = $this->container->get("kernel.security.authentication.provider.device");
            $user = $request->headers->get("PHP_AUTH_USER");
            $credentials = $request->headers->get("PHP_AUTH_PW");
            try {
                $this->getLogger()->debug("Authenticate through basic auth");
                $token = $authProvider->authenticate(new UsernamePasswordToken($user, $credentials, "device"));
            } catch (AuthenticationException $e) {
                $this->getLogger()->info("Authentication failed (" . $e->getMessage() . ": "
                    . $e->getMessageKey() . ")");
                return new Response("", 401, array("WWW-Authenticate" => "Basic realm=\"auralis ProfileService\""));
            }
            $deviceArr = array($token->getUser());
        } else if (isset($plistProperties['UDID'])) {
            if (!$pkcs7['verified']) {
                // the device has no certificate rolled out through SCEP, so the
                // PKCS#7 couldn't be verified.
                // Device Enrollment Program using ConfigurationURL #AUR-525
                $this->getLogger()->debug(
                    "PKCS#7 couldn't be verified. Propably Device Enrollment Program using ConfigurationURL."
                );
                return new Response("", 401, array("WWW-Authenticate" => "Basic realm=\"auralis ProfileService\""));
            }
            $seenPreviously = true;
            $deviceArr = $this->getDeviceEntityManager()
                ->findBy(array(
                    "deviceid" => $plistProperties['UDID'],
                    "disabled" => 0,
                    "challenge" => null,
                    "mdmCert" => null
                ));
        }

        if (empty($deviceArr)) {
            $this->getLogger()->info("ProfileService: Device not found");
            return new Response("Device not found", 403);
        }

        /** @var $device Device */
        $device = $deviceArr[0];

        if (!$seenPreviously) {
            // Phase 2, before SCEP
            $this->getLogger()->debug("ProfileService: Phase 2");

            if ($device->getType()->getName() == "ios"
                && $this->container->getParameter("kernel.environment") != "test") {
                // the signing certificate must be signed by the Apple CA
                // in tests we don't have a certificate signed by the Apple CA
                // use PKCS7_NOVERIFY because the Apple CA is not valid after
                // Apr 16 22:54:46 2014 GMT
                $appleCA = __DIR__ . "/Resources/apple-cert-chain.pem";
                $content = $request->getContent();
                $pkcs7 = CertUtil::decodePKCS7($content, PKCS7_NOCHAIN | PKCS7_BINARY | PKCS7_NOVERIFY, $appleCA);
                if (!$pkcs7['verified']) {
                    $this->getLogger()->debug("ProfileService: Invalid signature");
                    $device->setDisabled(true);
                    $this->getDeviceEntityManager()->save($device);
                    return HttpError::forbiddenResponse("Invalid signature");
                }
            }

            $device->setImei($plistProperties['IMEI']);
            $device->setProduct($plistProperties['PRODUCT']);
            $device->setVersion($plistProperties['VERSION']);

            // if there is a device with the same UDID that's not disabled a
            // rollout is not possible
            $sameUdidDevice = $this->getDeviceEntityManager()->findOneBy(
                array("deviceid" => $plistProperties['UDID'], "disabled" => 0));
            if (null !== $sameUdidDevice) {
                $this->handleSameUdidDevice($device, $sameUdidDevice);
                return new Response("A device with your device id already exists", 403);
            }

            /*
             * Delete previously rolled out device with same UDID that is
             * disabled. There can be only one such device. Nevertheless we use
             * foreach to be on the safe side.
             */
            $disabledDevices = $this->getDeviceEntityManager()->createQueryBuilder("d")
                ->where("d.deviceid = :udid")
                ->andWhere("d.disabled = 1")
                ->setParameter("udid", $plistProperties['UDID'])
                ->getQuery()->getResult()
            ;
            foreach ($disabledDevices as $disabledDevice) {
                if ($disabledDevice->getWipestatus() == Device::WIPE_PENDING) {
                    $disabledDevice->setWipestatus(Device::WIPE_UNKNOWN);
                }
                $this->getMdmAdmin()->cancelRemainingCommandsForDevice($disabledDevice);
                $disabledDevice->setDeleted(true);
                $this->getDeviceEntityManager()->save($disabledDevice);
            }

            if ($dep) {
                // reset the device password to make an additional rollout impossible
                $device->setPassword(null);
            } else {
                $cert = $this->getCertificateManager()->initClientCert($device);
                $device->setCert($cert);
                $device->setChallenge(null);
                $profile = $this->createEnrollProfile($device, $cert->getPlainChallenge());
            }

            // the UDID has to set at last to avoid an exception because of a duplicate device
            $device->setDeviceid($plistProperties['UDID']);
            $this->getDeviceEntityManager()->save($device);
        }

        if ($seenPreviously || $dep) {
            /*
             * Phase 3, after SCEP, or Device Enrollment Program.
             * If we are not using DEP PKCS#7 must be signed with scep cert,
             * i.e. the signing cert is signed by the aurlis CA. The
             * verification is done above.
             */
            $this->getLogger()->debug("ProfileService: Phase 3");
            $cert = $this->getCertificateManager()->initClientCert($device);
            $device->setMdmCert($cert);
            $device->setWipeStatus(Device::WIPE_NONE);
            if (!$dep) {
                // certificate created in step 2 is now unused
                $device->getCert()->setUnused(true);
            }
            $this->getDeviceEntityManager()->save($device);
            $profile = $this->createMDMProfile($device, $cert->getPlainChallenge());
            $this->getMdmAdmin()->initDevice($device);
        }

        return $this->createProfileResponse($profile);
    }

    /**
     * @param \Topalis\Device\Entity\Device $device
     * @param string                        $scepChallenge
     *
     * @return string The created profile as PList.
     */
    protected function createEnrollProfile(Device $device, $scepChallenge)
    {
        return $this->getProfileFactory()->createEnrollProfile($device, $scepChallenge)->toXML();
    }

    /**
     * @param \Topalis\Device\Entity\Device $device
     * @param string                        $scepChallenge
     *
     * @internal param array $plistProperties
     * @return string The created profile as PList.
     */
    protected function createMDMProfile(Device $device, $scepChallenge)
    {
        $profile = $this->getProfileFactory()->createMdmProfile($device, $scepChallenge);
        $profile->encrypt($this->requestCert);
        return $profile->toXML();
    }

    /**
     * Signs the specified profile and creates a response with fitting headers.
     *
     * @param string $content The profile as PList.
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function createProfileResponse($content)
    {
        $serverCert = $this->getCertificateManager()->getServerCert();
        $content = CertUtil::signPKCS7($content, $serverCert->getResource(), $serverCert->getPrivateKey());
        $response = new Response($content);
        $disposition = $response->headers->makeDisposition(
            ResponseHeaderBag::DISPOSITION_ATTACHMENT,
            "client.mobileconfig"
        );
        $response->headers->set("Content-type", "application/x-apple-aspen-config; charset=utf-8");
        $response->headers->set("Content-Disposition", $disposition);

        return $response;
    }

    private function handleSameUdidDevice(Device $device, Device $rolledOutDevice)
    {
        $logMsg = "ProfileService: A device with the UDID "
            . $rolledOutDevice->getDeviceid() . " is already rolled out";
        $this->getLogger()->warn($logMsg,
            array("device" => $device->getId(), "sameUdidDevice" => $rolledOutDevice->getId()));
        $device->setDisabled(true);
        $this->getDeviceEntityManager()->save($device);

        // inform the admin by mail
        $mailer = $this->container->get("mailer");
        if (!$mailer->isConfigured()) {
            return;
        }

        $translator = $this->container->get("translator");
        $subject = $translator->trans("admin.rollout.mail.udidRolledOut.subject",
            array("%udid%" => $rolledOutDevice->getDeviceid()));
        $text = $translator->trans("admin.rollout.mail.udidRolledOut.text", array(
            "%udid%" => $rolledOutDevice->getDeviceid(),
            "%user%" => $device->getUser()->getLogin(),
            "%rolledOutUser%" => $rolledOutDevice->getUser() ? $rolledOutDevice->getUser()->getLogin() : null,
            "%rolledOutDevice%" => $rolledOutDevice->getName(),
        ));
        $mailer->sendAdminMessage($subject, $text);
    }
}
