<?php
namespace Topalis\Scep;

use Topalis\Kernel\AbstractAuralisService;
use Topalis\Util\CryptUtil;
use Topalis\Certificate\Util\CertUtil;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;
use Topalis\Util\Random;
use Topalis\Scep\Util\ScepUtil;

/**
 * This class contains a limited implementation of the server side of SCEP.
 * @see https://tools.ietf.org/html/draft-nourse-scep-23
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class ScepService extends AbstractAuralisService
{
    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    protected function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function getCACaps()
    {
        return new Response("POSTPKIOperation\nSHA-1\nDES3", 200, array("Content-Type" => "text/plain"));
    }

    /**
     *
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function getCACert()
    {
        $pkcs7 = CertUtil::createDegneratePKCS7(array(
            $this->getCertificateManager()->getRASignCert(),
            $this->getCertificateManager()->getRACryptCert(),
            $this->getConfig()->getReverseProxyCACert(),
            $this->getConfig()->getCACert()
        ), "DER");
        $response = new Response($pkcs7, 200, array("Content-Type" => "application/x-x509-ca-ra-cert"));
        return $response;
    }

    /**
     * Signs the certificate in the certificate signing request in the specified
     * request.
     *
     * @param Request $request The request containing the CSR.
     * @return \Symfony\Component\HttpFoundation\Response
     */
    protected function pkiOperation(Request $request)
    {
        $this->getLogger()->debug("ScepService: PKIOperation");
        // Get the POST body or GET parameter and handle the PKIOperation
        if ($request->isMethod("GET")) {
            // "+" is replaced by space by PHP's urldecode, so we have to revert it
            $content = str_replace(" ", "+", $request->query->get("message"));
            $content = base64_decode($content);
        } else {
            $content = $request->getContent();
        }

        if (strlen($content) == 0) {
            return new Response("No content set", 400);
        }

        $this->getLogger()->debug("Get signed attributes");
        $signedAttributes = ScepUtil::getSignedAttributes($content, array(
            ScepConstants::$oids['transactionID'],
            ScepConstants::$oids['messageType'],
            ScepConstants::$oids['senderNonce'],
        ));

        $outSignedAttributes = array(
            ScepConstants::$oids['transactionID'] => $signedAttributes[ScepConstants::$oids['transactionID']],
            ScepConstants::$oids['messageType'] => ScepConstants::MESSAGE_TYPE_CERTREP,
            ScepConstants::$oids['senderNonce'] => Random::getString(16),
            ScepConstants::$oids['recipientNonce'] => $signedAttributes[ScepConstants::$oids['senderNonce']],
        );

        $messageType = $signedAttributes[ScepConstants::$oids['messageType']];
        if ($messageType != ScepConstants::MESSAGE_TYPE_PKCSREQ) {
            $this->getLogger()->warning("The message Type " . $messageType . " is not supported");
            return $this->createPkiOperationResponse(null, $outSignedAttributes,
                null, ScepConstants::FAILINFO_BAD_REQUEST);
        }

        try {
            $pkcs7 = CertUtil::decodePKCS7($content, PKCS7_BINARY);
        } catch (\Exception $e) {
            $this->getLogger()->error("Failed to decode PKCS#7", array("exception" => $e));
            return $this->createPkiOperationResponse(null, $outSignedAttributes,
                null, ScepConstants::FAILINFO_BAD_MESSAGE_CHECK);
        }

        $raCert = $this->getCertificateManager()->getRACryptCert();
        try {
            $csrDer = CryptUtil::decryptPKCS7($pkcs7['data'], $raCert->getResource(), $raCert->getPrivateKey());
        } catch (\Exception $e) {
            $this->getLogger()->error("Failed to decrypt PKCS#7", array("exception" => $e));
            return $this->createPkiOperationResponse(null, $outSignedAttributes,
                $pkcs7['signerCert'], ScepConstants::FAILINFO_BAD_MESSAGE_CHECK);
        }

        $csr = CertUtil::der2pem($csrDer, "CERTIFICATE REQUEST");
        $challenge = ScepUtil::getChallenge($csr);
        $cert = $this->getCertificateManager()->getChallengedClientCert($challenge);
        if ($cert === null) {
            $this->getLogger()->info("ScepService: Invalid challenge '$challenge'");
            return $this->createPkiOperationResponse(null, $outSignedAttributes,
                $pkcs7['signerCert'], ScepConstants::FAILINFO_BAD_MESSAGE_CHECK);
        }

        $caCert = null;
        if ($cert->getDevice()->getType()->getName() == "winphone") {
            // all WP certs must be signed with sub CA, because it is
            // required by WP's SCEP implementation. Otherwise the CA is
            // choosen by the certificate manager.
            $caCert = $this->getCertificateManager()->getReverseProxyCACert();
        }
        $certPem = $this->getCertificateManager()->signClientCert($cert, $csr, $caCert);
        $messageData = CertUtil::createDegneratePKCS7($certPem, "DER");
        $this->getLogger()->debug("CSR signed. Create PKIOperation response");
        return $this->createPkiOperationResponse($messageData, $outSignedAttributes, $pkcs7['signerCert']);
    }

    /**
     * Handles the specified request and returns a fitting response.
     *
     * @param Request $request The request to handle.
     * @return \Symfony\Component\HttpFoundation\Response The response.
     */
    public function handleRequest(Request $request)
    {
        try {
            $operation = $request->get("operation");
            switch ($operation) {
                case "GetCACaps":
                    return $this->getCACaps();
                case "GetCACert":
                    return $this->getCACert();
                case "PKIOperation":
                    $response = $this->pkiOperation($request);
                    return $response;
            }

            return new Response(Response::$statusTexts[400], 400);
        } catch (\Exception $e) {
            $this->getLogger()->error("ScepService: error handling request", array("exception" => $e));
            return new Response(Response::$statusTexts[500], 500);
        }
    }

    /**
     * Creates a SCEP PKCS#7 as response to a PKIOperation.
     * @param string $content The content to encrypt and sign.
     * @param array $signedAttributes The attributes to add to the signed
     *     attributes in the PKCS#7
     * @param string|NULL $encCert The certificate to encrypt the content with,
     *     or null to not encrypt the content.
     * @param integer $failInfo The failInfo in case of errors. If specified the
     *     pkiStatus will be set to FAILURE, otherwise to SUCCESS.
     * @return Response The response containing the created PKCS#7.
     */
    protected function createPkiOperationResponse($content, array $signedAttributes, $encCert = null, $failInfo = null)
    {
        $raCert = $this->getCertificateManager()->getRASignCert();
        $pkiMessage = ScepUtil::createPkiMessage($content, $signedAttributes, $raCert, $encCert, $failInfo);
        return new Response($pkiMessage, 200, array("Content-Type" => "application/x-pki-message"));
    }
}
