<?php
namespace Topalis\System;

use Topalis\Util\FileSystemUtil;
use Topalis\Util\Process;

/**
 * Class to access the configuration of a network interface of a RedHat
 * Enterprise Linux or a CentOS system.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class InterfaceConfig
{
    /**
     * @var string
     */
    private static $helperCommand = "/usr/sbin/aurnetwork";

    /**
     * @var string
     */
    private static $networkScriptsDir = "/etc/sysconfig/network-scripts";

    private $ifConfigFile;
    private $ifConfig = array();
    private $interface;

    /**
     * @return boolean true if the interface configuration is supported, false
     *     otherwise. It checks if the related directory and file exist.
     */
    public static function isSystemSupported()
    {
        return is_dir(self::$networkScriptsDir) && Process::commandExists(self::$helperCommand);
    }

    /**
     * @var boolean indicates whether the shutdown function to change the
     *     network configuration was registered.
     * @see registerShutdownFunction()
     */
    private $ifShutdownFunctionRegistered = false;

    /**
     * Sets the command used to store the interface configuration.
     *
     * @param string $helperCommand The command to set.
     */
    public static function setHelperCommand($helperCommand)
    {
        self::$helperCommand = $helperCommand;
    }

    /**
     * @return string The directory which contains the network configuration
     *     files.
     */
    protected static function getNetworkScriptsDir()
    {
        return self::$networkScriptsDir;
    }

    /**
     * Sets the directory which contains the network configuration files.
     *
     * @param string $networkScriptsDir The directory to set.
     */
    public static function setNetworkScriptsDir($networkScriptsDir)
    {
        self::$networkScriptsDir = $networkScriptsDir;
    }

    /**
     * @param string $interface The interface name to check the existance of.
     * @return boolean true if the specified interface exists (i.e. an
     *     appropriate config file exists).
     */
    public static function existsInterface($interface)
    {
        return file_exists(self::createIfConfigFilename($interface));
    }

    /**
     * @param string $interface Interface name.
     * @return string The file name of the configuration file for the specified
     *     interface.
     */
    protected static function createIfConfigFilename($interface)
    {
        return self::getNetworkScriptsDir() . "/ifcfg-" . $interface;
    }

    /**
     * Constructor.
     *
     * @param string $interface The interface name. If null the first ethX
     *     interface is used. If it does not exist the first wlanX interface is
     *     used. An interface is detected as existing if an appropriate config
     *     file is found (CentOS).
     * @throws \Exception If interface is not specified and no interface is
     *     found.
     */
    public function __construct($interface = null)
    {
        if ($interface === null) {
            for ($i = 0; $i < 10; $i++) {
                if (self::existsInterface("eth" . $i)) {
                    $interface = "eth" . $i;
                    break;
                }
            }
        }
        if ($interface === null) {
            for ($i = 0; $i < 10; $i++) {
                if (self::existsInterface("wlan" . $i)) {
                    $interface = "wlan" . $i;
                    break;
                }
            }
        }
        if ($interface === null) {
            throw new \Exception("No network interface found");
        }

        $this->interface = $interface;
        $this->ifConfigFile = self::createIfConfigFilename($interface);
        if (file_exists($this->ifConfigFile)) {
            $this->ifConfig = parse_ini_file($this->ifConfigFile, false, INI_SCANNER_RAW);
        }
    }

    /**
     *
     * @return string The boot protocol ('static' or 'dhcp').
     */
    public function getBootProto()
    {
        $bootproto = strtolower($this->getIfConfigValue('BOOTPROTO'));
        if ($bootproto == "none") {
            return "static";
        } else if ($bootproto == "bootp") {
            return "dhcp";
        }
        return $bootproto;
    }

    /**
     * Sets the boot protocol.
     *
     * @param string $bootProto The boot protocol to set ('static' or 'dhcp').
     * @throws \InvalidArgumentException
     */
    public function setBootProto($bootProto)
    {
        if ($bootProto == "static") {
            $bootProto = "none";
        } else if ($bootProto == "dhcp") {
            $this->setIpAddress(null);
            $this->setNetmask(null);
            $this->setGateway(null);
            $this->setDNS1(null);
            $this->setDNS2(null);
        } else if ($bootProto != "none") {
            throw new \InvalidArgumentException("either 'dhcp', 'none' or 'static'");
        }
        $this->setIfConfigValue('BOOTPROTO', $bootProto);
    }

    /**
     * @return string|null The IP address or null if DHCP is enabled.
     */
    public function getIpAddress()
    {
        return $this->getIfConfigValue('IPADDR');
    }

    /**
     * Sets the IP address. Requires boot protocol 'static'.
     *
     * @param string $ipAddress The IP address to set.
     */
    public function setIpAddress($ipAddress)
    {
        $this->setIfConfigValue('IPADDR', $ipAddress);
    }

    /**
     * @return string|null The primary DNS server, or null if DHCP is enabled.
     */
    public function getDNS1()
    {
        return $this->getIfConfigValue('DNS1');
    }

    /**
     * Sets the primary DNS server. Requires boot protocol 'static'.
     *
     * @param string $dns1 The primary DNS server to set.
     */
    public function setDNS1($dns1)
    {
        $this->setIfConfigValue('DNS1', $dns1);
    }

    /**
     * @return string|null The secondary DNS server, or null if DHCP is enabled.
     */
    public function getDNS2()
    {
        return $this->getIfConfigValue('DNS2');
    }

    /**
     * Sets the secondary DNS server. Requires boot protocol 'static'.
     *
     * @param string $dns2 The primary DNS server to set.
     */
    public function setDNS2($dns2)
    {
        $this->setIfConfigValue('DNS2', $dns2);
    }

    /**
     * @return string|null The netmask, or null if DHCP is enabled.
     */
    public function getNetmask()
    {
        return $this->getIfConfigValue('NETMASK');
    }

    /**
     * Sets the netmask. Requires boot protocol 'static'.
     *
     * @param string $netmask The netmask to set.
     */
    public function setNetmask($netmask)
    {
        $this->setIfConfigValue('NETMASK', $netmask);
    }

    /**
     * @return string|null The default gateway, or null if DHCP is enabled.
     */
    public function getGateway()
    {
        return $this->getIfConfigValue('GATEWAY');
    }

    /**
     * Sets the default gateway. Requires boot protocol 'static'.
     *
     * @param string $gateway The default gateway to set.
     */
    public function setGateway($gateway)
    {
        $this->setIfConfigValue('GATEWAY', $gateway);
    }

    protected function getIfConfigValue($key)
    {
        return isset($this->ifConfig[$key]) ? $this->ifConfig[$key] : null;
    }

    protected function setIfConfigValue($key, $value)
    {
        if ($this->getIfConfigValue($key) != $value) {
            $this->ifConfig[$key] = $value;
            $this->registerShutdownFunction();
        }
    }

    /**
     * Registers the shutdown function to change the network configuration.
     */
    private function registerShutdownFunction()
    {
        if (!$this->ifShutdownFunctionRegistered) {
            register_shutdown_function(array($this, 'store'));
            $this->ifShutdownFunctionRegistered = true;
        }
    }

    public function store()
    {
        $curIfConfig = parse_ini_file($this->ifConfigFile, false, INI_SCANNER_RAW);
        if ($this->ifConfig == $curIfConfig) {
            // nothing changed
            return;
        }

        $tmpIfFilename = tempnam(sys_get_temp_dir(), "auralis" . $this->interface);
        try {
            $file = fopen($tmpIfFilename, 'w');
            foreach ($this->ifConfig as $key => $value) {
                if (strlen($value) > 0) {
                    fwrite($file, $key . "=\"" . $value . "\"\n");
                }
            }
            fclose($file);
            $cmd = self::$helperCommand . " --interface $this->interface $tmpIfFilename";
            $out = "";
            Process::execThrow($cmd, $out);
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpIfFilename);
            throw $e;
        }
    }

}
