<?php
namespace Topalis\System;

use Topalis\Util\FileSystemUtil;
use Topalis\Util\StringUtil;
use Topalis\Util\Process;

/**
 * Class to access different parts of the system configuration of a RedHat
 * Enterprise Linux or a CentOS system.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class SystemConfig
{
    /**
     * @var string
     */
    private $clockConfigFilename = "/etc/sysconfig/clock";

    /**
     * @var InterfaceConfig
     */
    private $interface;

    /**
     * @var string
     */
    private $ntpConfigCommand = "/usr/sbin/aurntp";

    /**
     * @var string
     */
    private $ntpConfigFilename = "/etc/ntp.conf";

    /**
     * @var string
     */
    private $timeZoneCommand = "/usr/sbin/aurtimezone";

    /**
     * @var string
     */
    private $timeZoneDir = "/usr/share/zoneinfo";

    /**
     * @return boolean true if the network configuration is supported, false
     *     otherwise.
     * @see InterfaceConfig::isSystemSupported()
     */
    public function isNetworkConfigSupported()
    {
        return InterfaceConfig::isSystemSupported();
    }

    /**
     * @return boolean true if the configuration of time related things is
     *     supported, false otherwise. It checks if the related files and
     *     directories exist.
     */
    public function isTimeConfigSupported()
    {
        return is_file($this->clockConfigFilename) && is_file($this->ntpConfigFilename)
            && is_dir($this->timeZoneDir) && Process::commandExists($this->ntpConfigCommand);
    }

    /**
     * @return string The name of the file containing the clock (i.e. time zone)
     *     configuration.
     */
    protected function getClockConfigFilename()
    {
        return $this->clockConfigFilename;
    }

    /**
     * Sets the name of the file containing the clock (i.e. time zone)
     *     configuration.
     * @param string $clockConfigFilename The file name to set.
     */
    public function setClockConfigFilename($clockConfigFilename)
    {
        $this->clockConfigFilename = $clockConfigFilename;
    }

    /**
     * Sets the command used to store the NTP configuration.
     *
     * @param string $ntpConfigCommand The command to set.
     */
    public function setNtpConfigCommand($ntpConfigCommand)
    {
        $this->ntpConfigCommand = $ntpConfigCommand;
    }

    /**
     * @return string The name of the file containing the NTP configuration.
     */
    protected function getNtpConfigFilename()
    {
        return $this->ntpConfigFilenamename;
    }

    /**
     * Sets the name of the file containing the NTP configuration.
     *
     * @param string $ntpConfigFilename The file name to set.
     */
    public function setNtpConfigFilename($ntpConfigFilename)
    {
        $this->ntpConfigFilename = $ntpConfigFilename;
    }

    /**
     * Sets the command used to store the time zone.
     *
     * @param string $timeZoneCommand
     */
    public function setTimeZoneCommand($timeZoneCommand)
    {
        $this->timeZoneCommand = $timeZoneCommand;
    }

    /**
     * @return string The directory containing the time zone files.
     */
    protected function getTimeZoneDir()
    {
        return $this->timeZoneDir;
    }

    /**
     * Sets the directory containing the time zones.
     *
     * @param string $timeZoneDir The directory to set.
     */
    public function setTimeZoneDir($timeZoneDir)
    {
        $this->timeZoneDir = $timeZoneDir;
    }

    /**
     * @return string[] List of available time zones.
     */
    public function getAvailableTimeZones()
    {
        static $tzlist = null;
        static $regions = array(
                'Africa' => \DateTimeZone::AFRICA,
                'America' => \DateTimeZone::AMERICA,
                'Antarctica' => \DateTimeZone::ANTARCTICA,
                'Asia' => \DateTimeZone::ASIA,
                'Atlantic' => \DateTimeZone::ATLANTIC,
                'Australia' => \DateTimeZone::AUSTRALIA,
                'Europe' => \DateTimeZone::EUROPE,
                'Indian' => \DateTimeZone::INDIAN,
                'Pacific' => \DateTimeZone::PACIFIC
        );

        if (!isset($tzlist)) {
            foreach ($regions as $mask) {
                foreach (\DateTimeZone::listIdentifiers($mask) as $tz) {
                    if (file_exists("/usr/share/zoneinfo/" . $tz)) {
                        $tzlist[] = $tz;
                    }
                }
            }
        }
        return $tzlist;
    }

    /**
     * @return InterfaceConfig Configuration for ethX, where X is the smallest
     *     number of an existing interface. An interface is detected as existing
     *     if an appropriate config file is found (CentOS).
     * @throws \Exception If no interface is found.
     */
    public function getNetwork()
    {
        if ($this->interface === null) {
            $this->interface = new InterfaceConfig();
        }
        return $this->interface;
    }

    /**
     * @return string[] The NTP servers.
     */
    public function getNTPServers()
    {
        static $servers = null;

        if ($servers === null) {
            $servers = array();
            $configLines = file($this->ntpConfigFilename, FILE_SKIP_EMPTY_LINES);
            foreach ($configLines as $line) {
                $line = trim($line);
                if (StringUtil::startsWith($line, "server")) {
                    $split = preg_split("/\s+/", $line);
                    if (isset($split[1])) {
                        $servers[] = $split[1];
                    }
                }
            }
        }
        return $servers;
    }

    /**
     * Sets the NTP server.
     *
     * @param string[] $ntpServers The NTP servers to use as array.
     * @throws \Topalis\Util\Exception\Process\ExecException
     * @throws \Exception
     */
    public function setNTPServers(array $ntpServers)
    {
        if ($this->getNTPServers() == $ntpServers || count($ntpServers) == 0) {
            return;
        }

        $configLines = file($this->ntpConfigFilename);
        $newConfigLines = array();
        foreach ($configLines as $line) {
            if (!StringUtil::startsWith(trim($line), "server")) {
                $newConfigLines[] = $line;
            }
        }
        $newConfigLines[] = "\n"; // add empty line before list of servers
        foreach ($ntpServers as $ntpServer) {
            $newConfigLines[] = "server " . trim($ntpServer) . "\n";
        }
        $tmpFilename = tempnam(sys_get_temp_dir(), "auralisNTP");
        try {
            file_put_contents($tmpFilename, $newConfigLines);

            $cmd = $this->ntpConfigCommand . " " . $tmpFilename;
            $out = "";
            Process::execThrow($cmd, $out);
        } catch (\Exception $e) {
            FileSystemUtil::tryUnlinkFile($tmpFilename);
            throw $e;
        }
    }

    /**
     * @return string The time zone.
     */
    public function getTimeZone()
    {
        static $timeZone = null;

        if ($timeZone === null) {
            // @codingStandardsIgnoreStart
            $clockConfig = file_exists($this->clockConfigFilename) ?
                @parse_ini_file($this->clockConfigFilename, false, INI_SCANNER_RAW) : array();
            $timeZone = isset($clockConfig['ZONE']) ? $clockConfig['ZONE'] : @date_default_timezone_get();
            // @codingStandardsIgnoreEnd
        }
        return $timeZone;
    }

    /**
     * Sets the time zone.
     *
     * @param string $timeZone The time zone to set.
     * @throws \InvalidArgumentException
     * @throws \Topalis\Util\Exception\Process\ExecException
     */
    public function setTimeZone($timeZone)
    {
        if ($timeZone == $this->getTimeZone()) {
            return;
        }

        if (!file_exists($this->getTimeZoneDir() ."/" . $timeZone)) {
            throw new \InvalidArgumentException("invalid time zone: '$timeZone'");
        }

        $cmd = $this->timeZoneCommand . " set " . $timeZone;
        Process::execThrow($cmd);
    }
}
