<?php
namespace Topalis\Test;

use Symfony\Bundle\FrameworkBundle\Test\WebTestCase;
use Topalis\Device\EventListener\DeviceTypeListener;
use Topalis\Util\FileSystemUtil;

abstract class TestCase extends WebTestCase
{
    /**
     * Event listeners that are disabled.
     *
     * @var array
     */
    private $disabledEventListeners = array();

    /**
     * Defines whether a DB transaction shall be started in setUp()
     * @var boolean
     */
    protected $beginTransaction = true;

    /**
     * @var \Symfony\Bundle\FrameworkBundle\Client $client
     */
    protected $client;

    /**
     * @var \Topalis\Configuration\AuralisConfig
     */
    protected $config;

    /**
     * @var \PSS\SymfonyMockerContainer\DependencyInjection\MockerContainer
     */
    protected $container;

    /**
     * @var \Topalis\Database\MySqlDb
     */
    protected $db;

    /**
     * Defines whether an Apache HTTPD reload shall be disabled in tearDown()
     *
     * @var boolean
     */
    protected $disableApacheReload = true;

    /**
     * @var \PSR\Log\LoggerInterface
     */
    protected $logger;

    /**
     * @var MockObjectFactory
     */
    protected $mockObjectFactory;

    /**
     * Defines whether a DB transaction shall be rolled back in tearDown()
     *
     * @var boolean
     */
    protected $rollbackTransaction = true;

    /**
     * Defines whether mocked services shall be unmocked and the mockery
     * container shall be closed in tearDown()
     *
     * @var boolean
     */
    protected $unmock = true;

    /**
     * If you want to set custom services you should first call #initContainer()
     * and set the services before calling setUp. Otherwise the services might
     * already be used.
     */
    protected function setUp()
    {
        parent::setUp();
        $this->initContainer();
        $this->config = $this->container->get("config.auralis");
        $this->db = $this->container->get("db");
        $this->logger = $this->container->get("logger");
        $this->mockObjectFactory = new MockObjectFactory($this->db);
        if ($this->beginTransaction) {
            $doctrine = $this->container->get("doctrine");
            foreach ($doctrine->getConnections() as $connection) {
                $connection->beginTransaction();
            }
        }

        // copy test config to %test.tmp.dir%/config
        $tmpDir = $this->container->getParameter("test.tmp.dir") . DIRECTORY_SEPARATOR . "config";
        if (!is_dir($tmpDir)) {
            mkdir($tmpDir, 0755, true);
        }
        $source = $this->container->getParameter("kernel.root_dir") . DIRECTORY_SEPARATOR
            . "config" . DIRECTORY_SEPARATOR . "test" . DIRECTORY_SEPARATOR;
        FileSystemUtil::copyRecursive($source, $tmpDir, 0755);
    }

    public function initContainer()
    {
        if ($this->client === null) {
            $this->client = static::createClient();
            $this->container = $this->client->getContainer();
        }
    }

    protected function tearDown()
    {
        $doctrine = $this->container->get("doctrine");
        foreach ($doctrine->getConnections() as $connection) {
            if ($this->rollbackTransaction) {
                $connection->rollback();
            }
            $connection->close();
        }

        if ($this->disableApacheReload) {
            $this->container->get("apache")->disableReload();
        }

        if ($this->unmock) {
            foreach (array_keys($this->container->getMockedServices()) as $id) {
                $this->container->unmock($id);
            }
            \Mockery::close();
        }

        $tmpDir = $this->container->getParameter("test.tmp.dir");
        if (is_dir($tmpDir)) {
            FileSystemUtil::rmdirRecursive($tmpDir);
        }
    }

    protected function disableDoctrineFilters()
    {
        $managers = $this->db->getDoctrine()->getManagers();
        foreach ($managers as $em) {
            $filterCollection = $em->getFilters();
            $enabledFilters = $filterCollection->getEnabledFilters();
            foreach (array_keys($enabledFilters) as $filterName) {
                echo "disable filter $filterName\n";
                $filterCollection->disable($filterName);
            }
        }
    }

    protected function removeSymfonySubscribers(array $subscribers)
    {
        $dispatcher = $this->container->get("event_dispatcher");
        foreach ($subscribers as $subscriber) {
            $dispatcher->removeSubscriber($subscriber);
        }
    }

    protected function disableEventListeners(array $disableListeners = null)
    {
        $em = $this->container->get("doctrine.orm.entity_manager");
        $eventManager = $em->getEventManager();
        $listeners = $eventManager->getListeners();
        foreach ($listeners as $event => $listenersHash) {
            foreach ($listenersHash as $listener) {
                if (null === $disableListeners || in_array(get_class($listener), $disableListeners)) {
                    if (null == $disableListeners && $listener instanceof DeviceTypeListener) {
                        // disable DeviceTypeListener only if it is explicitly requested
                        continue;
                    }

                    $eventManager->removeEventListener($event, $listener);
                    $this->disabledEventListeners[$event][] = $listener;
                }
            }
        }
    }

    protected function enableEventListeners(array $enableListeners = null)
    {
        $em = $this->container->get("doctrine.orm.entity_manager");
        $eventManager = $em->getEventManager();
        foreach ($this->disabledEventListeners as $event => $listeners) {
            foreach ($listeners as $key => $listener) {
                if (null === $enableListeners || in_array(get_class($listener), $enableListeners)) {
                    $eventManager->addEventListener($event, $listener);
                    unset($this->disabledEventListeners[$event][$key]);
                }
            }
        }
    }

    /**
     *
     * @return \Doctrine\ORM\EntityManager
     */
    protected function getEntityManager()
    {
        return $this->db->getDoctrine()->getManager();
    }

    /**
     * @param string $typeName The OS type name.
     * @return integer The OS id.
     */
    protected function getDeviceOs($typeName)
    {
        $deviceTypeFactory = $this->container->get("device.type.factory");
        return $deviceTypeFactory->getType($typeName)->getId();
    }

    /**
     * @return \Topalis\Kernel\TimeProviderInterface
     */
    protected function getTimeProvider()
    {
        return $this->container->get("kernel.time_provider");
    }

    /**
     * @return \Topalis\Test\MockObjectFactory
     */
    protected function getMockObjectFactory()
    {
        return $this->mockObjectFactory;
    }

    /**
     *
     * @param string $id #Service id
     * @param string $class #Class to mock
     * @param array $methods methods to mock
     * @param array $args Constructor arguments
     *
     * @return \Mockery\MockInterface The mocked object.
     */
    protected function mockService($id, $class, array $methods = null, array $args = null)
    {
        $typeHint = $class;
        if ($methods !== null) {
            $typeHint .= "[" . implode(", ", $methods) . "]";
        }

        if (null === $args) {
            $service = \Mockery::mock($typeHint);
        } else {
            $service = \Mockery::mock($typeHint, $args);
        }
        $this->container->set($id, $service);
        if (method_exists($service, "setContainer")) {
            $service->setContainer($this->container);
        }
        return $service;
    }
}
