<?php
namespace Topalis\Update\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;

/**
 * This command checks for updates and sends an email if there's a new update.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateCheckCommand extends ContainerAwareCommand
{
    const UPDATE_AVAILABLE_SUBJECT = "New update available";
    const UPDATE_AVAILABLE_TEXT = "There's a new update for your auralis appliance available: auralis %s\n\n%s";

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->getContainer()->get("config.auralis");
    }

    /**
     * @return \Topalis\License\License
     */
    private function &getLicense()
    {
        return $this->getLicenseManager()->getLicense();
    }

    /**
     * @return \Topalis\License\LicenseManager
     */
    private function getLicenseManager()
    {
        return $this->getContainer()->get("license.manager");
    }

    /**
     * @return \Topalis\Mail\Mailer
     */
    private function getMailer()
    {
        return $this->getContainer()->get("mailer");
    }

    /**
     * @return \Topalis\Update\UpdateManager
     */
    private function getUpdateManager()
    {
        return $this->getContainer()->get("update.manager");
    }

    protected function configure()
    {
        $this
        ->setName("update:check")
        ->setDescription("Checks if updates are available and sends an email if that's the case.
The email will be send, if the version wasn't available in a previous update check.")
        ->setHelp("Checks if updates are available and sends an email if that's the case.
The email will be send, if the version wasn't available in a previous update check.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        $mgr = $this->getUpdateManager();

        $license = $this->getLicense();
        if ($license->isExpired()) {
            $output->writeln("Your license is expired. Updates are not possible");
            return 1;
        }

        if (!$license->existsApnsCertificate()) {
            $output->writeln("You need a push certificate to get updates");
            return 1;
        }

        $config = $this->getConfig();
        $updatesAvailable = json_decode($config->getUpdatesAvailable(), true);

        $updatesAvailableNew = $mgr->checkUpdates();
        if (empty($updatesAvailableNew)) {
            $output->writeln("No new version is available");
            return;
        }

        $releaseNotes = $this->getUpdateManager()->getReleaseNotes();

        $maxUpdateVersion = isset($updatesAvailable[0]['version']) ?
            $updatesAvailable[0]['version'] : $mgr->getAuralisVersion();
        $maxUpdateVersionNew = $updatesAvailableNew[0]['version'];

        if (version_compare($maxUpdateVersionNew, $maxUpdateVersion, ">")) {
            $output->writeln("Update is available. The most recent version is " . $maxUpdateVersionNew);
            $mailer = $this->getMailer();
            if ($mailer->isConfigured()) {
                $subject = self::UPDATE_AVAILABLE_SUBJECT;
                $text = sprintf(self::UPDATE_AVAILABLE_TEXT, $maxUpdateVersionNew, $releaseNotes);
                $mailer->sendAdminMessage($subject, $text);
            }
        }
    }
}
