<?php
namespace Topalis\Update\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Topalis\Util\FileSystemUtil;

/**
 * This command performs the update tasks.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateCommand extends ContainerAwareCommand
{
    private function getUpdateCommandHelper()
    {
        static $updateCommandHelper = null;

        if ($updateCommandHelper === null) {
            $updateCommandHelper = new UpdateCommandHelper($this->getContainer());
        }
        return $updateCommandHelper;
    }

    /**
     * @return \Topalis\Apache\Apache
     */
    private function getApache()
    {
        return $this->getContainer()->get("apache");
    }

    /**
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->getContainer()->get("certificate.manager");
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->getContainer()->get("config.auralis");
    }

    protected function configure()
    {
        $this
        ->setName("update")
        ->setDescription("Performs the steps required to finish the update. This command must be run only once.")
        ->setHelp("Performs the steps required to finish the update. This command must be run only once.");
    }

    protected function execute(\Symfony\Component\Console\Input\InputInterface $input,
        \Symfony\Component\Console\Output\OutputInterface $output)
    {
        $input; // unused

        $config = $this->getConfig();
        $prevVersion = $config->getVersion();

        if (version_compare("2.1.0", $prevVersion, ">")) {
            // this check is used because $prevVersion exists not before 2.2.0
            if (!is_file("/etc/auralis/pki/CA/revproxy-cacert.pem") && $this->getCertificateManager()->existsCACert()) {
                $output->writeln("Create auralis sub CA certificate for SecureIntranet");
                $this->getCertificateManager()->exportCACert();

                // ensure that reverse proxy is enabled
                $this->getApache()->setReverseProxyEnabled(true);

                $output->writeln("Install SecureIntranet device profiles");
                $this->getUpdateCommandHelper()->installSecureIntranetProfiles();

                $output->writeln("Remove old device profiles");
                $this->getUpdateCommandHelper()->removeOldDeviceProfiles();
            }
            $prevVersion = "2.1.0";
            $config->setVersion($prevVersion);
        }

        $serverCertRegenerated = false;
        if (version_compare("2.2.0", $prevVersion, ">")) {
            // fix Apache config
            $output->writeln("Update Apache HTTPD CA config");
            $this->getCertificateManager()->exportCACert();

            // revoke certificates with the signer CA
            $output->writeln("Create new revocation list");
            $updateHelper = new UpdateHelper020200($this->getContainer());
            $updateHelper->setClientCertIssuers();
            $this->getCertificateManager()->doCreateRevocationList();
            // remove old CRL
            FileSystemUtil::tryUnlinkFile("/etc/auralis/pki/CA/crl.pem");

            $output->writeln("Clear used device passwords");
            $updateHelper->clearDevicePasswords();

            $output->writeln("Regnerate server certificate");
            $serverCertRegenerated = $updateHelper->regenerateServerCert();

            $output->writeln("Fill new aggregation table");
            $updateHelper->aggregateYearMonth();

            $prevVersion = "2.2.0";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.2.1", $prevVersion, ">")) {
            if (!$serverCertRegenerated) {
                $updateHelper = new UpdateHelper020201($this->getContainer());
                $serverCertRegenerated = $updateHelper->regenerateServerCert();
            }

            $prevVersion = "2.2.1";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.2.2", $prevVersion, ">")) {
            if (!$serverCertRegenerated) {
                $updateHelper = new UpdateHelper020202($this->getContainer());
                $serverCertRegenerated = $updateHelper->cleanReverseProxyUrls();
            }

            $prevVersion = "2.2.2";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.3.0", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020300($this->getContainer());
            $updateHelper->disablePDFAppearanceStreamRemoval();
            $updateHelper->updateWinPhoneConfig();
            $updateHelper->updateCSPs();
            $updateHelper->updateReverseProxy();

            $prevVersion = "2.3.0";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.3.1", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020301($this->getContainer());
            $updateHelper->setEmptyAppTypeToUnknown();

            $prevVersion = "2.3.1";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.3.4", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020304($this->getContainer());
            $updateHelper->informAdminWrongCertSerial();

            $prevVersion = "2.3.4";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.3.5", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020305($this->getContainer());
            $updateHelper->fixWindowsPhonePolling();

            $prevVersion = "2.3.5";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.4.1", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020401($this->getContainer());
            $updateHelper->fixDeviceProfile();

            $prevVersion = "2.4.1";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.4.2", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020402($this->getContainer());
            $updateHelper->fixConfigGroupChange();

            $prevVersion = "2.4.2";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.5.0", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020500($this->getContainer());
            $updateHelper->resetASFPort();
            $updateHelper->setApacheRemoteProxyServer();
            $updateHelper->resignCerts();
            $updateHelper->updateLastDeviceAccess();

            $prevVersion = "2.5.0";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.5.3", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020503($this->getContainer());
            $updateHelper->fixUploadedIosAppIcons();

            $prevVersion = "2.5.3";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.5.4", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020504($this->getContainer());
            $updateHelper->fixReverseProxyCACert();

            $prevVersion = "2.5.4";
            $config->setVersion($prevVersion);
        }

        if (version_compare("2.5.7", $prevVersion, ">")) {
            $updateHelper = new UpdateHelper020507($this->getContainer());
            $updateHelper->installWifiProfiles();

            $prevVersion = "2.5.7";
            $config->setVersion($prevVersion);
        }
    }
}
