<?php
namespace Topalis\Update\Command;

use Topalis\Util\StringUtil;

/**
 * This class contains methods to perform the update to version 2.3.0
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateHelper020300
{
    private $disabledEventListeners = array();

    private static $restrictionKeys = array(
        "requireDeviceEncryption",
        "allowStore",
        "allowCamera",
        "allowScreenCapture",
        "allowVoiceRecording",
        "allowSaveAsOfOfficeFiles",
        "allowCortana",
        "allowCopyPaste",
        "allowSyncMySettings",
        "allowWiFi",
        "allowAutoConnectToWiFiSenseHotspots",
        "allowInternetSharing",
        "allowManualWiFiConfiguration",
        "allowWiFiHotspotReporting",
        "allowNFC",
        "allowBluetooth",
        "allowCellularDataRoaming",
        "allowVPNOverCellular",
        "allowVPNRoamingOverCellular",
        "allowBrowser",
        "allowUSBConnection",
        "allowStorageCard",
        "allowTelemetry" => 2,
        "allowLocation",
        "allowSearchToUseLocation",
        "safeSearchPermissions" => 1,
        "allowStoringImagesFromVisionSearch",
        "allowManualRootCertificateInstallation",
        "allowDeveloperUnlock",
        "allowMicrosoftAccountConnection",
        "allowAddingNonMicrosoftAccountsManually",
    );

    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(\Symfony\Component\DependencyInjection\ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * Disable the removal of PDF Appearance Streams #AUR-454.
     */
    public function disablePDFAppearanceStreamRemoval()
    {
        $filterServer = $this->container->get("asf.filter_server");
        $pdfCompressorPlugin = $filterServer->getPluginManager()->getPlugin("pdfcompressor");
        $groupEm = $this->container->get("config_group.entity.manager.config_group");
        foreach ($groupEm->findAll() as $group) {
            $pluginConfigEntry = $pdfCompressorPlugin->getConfig($group)->getEntry("RemoveAppearanceStreams");
            if (StringUtil::strtobool($pluginConfigEntry->getValue())) {
                $pluginConfigEntry->setValue(false);
            }
        }
    }

    public function updateWinPhoneConfig()
    {
        $this->disableEventListeners();
        $groupEm = $this->container->get("config_group.entity.manager.config_group");

        foreach ($groupEm->findAll() as $group) {
            $passCode = $group->getWinPhonePassCode();
            if ($passCode !== null) {
                $passCode['allowIdleReturnWithoutPassword'] = 1;
                if (isset($passCode['minDevicePasswordLength']) && $passCode['minDevicePasswordLength'] > 16) {
                    // WP 8.1: min password length is now limited to 16 (previously 18).
                    $passCode['minDevicePasswordLength'] = 16;
                }
                $group->setWinPhonePassCode($passCode);
            }

            $restrictions = $group->getWinPhoneRestrictions();
            if ($restrictions !== null) {
                if (!isset($restrictions['encryption']) || $restrictions['encryption']) {
                    $restrictions['requireDeviceEncryption'] = 1;
                } else {
                    $restrictions['requireDeviceEncryption'] = 0;
                }
                unset($restrictions['encryption']);

                if (!isset($restrictions['allowStorage']) || $restrictions['allowStorage']) {
                    $restrictions['allowStorageCard'] = 1;
                } else {
                    $restrictions['allowStorageCard'] = 0;
                }
                unset($restrictions['allowStorage']);

                foreach (self::$restrictionKeys as $key => $val) {
                    if (is_int($key)) {
                        $key = $val;
                        $val = 1;
                    }
                    if (!isset($restrictions[$key])) {
                        $restrictions[$key] = $val;
                    }
                }

                $group->setWinPhoneRestrictions($restrictions);
            }

            $groupEm->save($group, false);
        }

        $groupEm->flush();
        $this->enableEventListeners();
    }

    public function updateCSPs()
    {
        $syncMLAdmin = $this->container->get("winphone.oma.syncml.admin");

        $deviceEm = $this->container->get("device.entity.manager.device");
        foreach ($deviceEm->findAll() as $device) {
            if ($device->getOs() !== $this->getWinPhoneDeviceOs()) {
                continue;
            }

            if (version_compare($device->getVersion(), "8.1", "<")) {
                continue;
            }

            // use PolicyManager CSP instead of DeviceLock CSP and Storage CSP for WP 8.1
            $syncMLAdmin->installPolicyManagerCSP($device);
            $syncMLAdmin->removeDeviceLockCSP($device);
            $syncMLAdmin->removeStorageCSP($device);
        }
    }

    public function updateReverseProxy()
    {
        $apache = $this->container->get("apache");
        $revProxyEm = $this->container->get("reverse_proxy.entity.manager.url");
        foreach ($revProxyEm->findAll() as $url) {
            $apache->writeReverseProxy($url);
        }
    }

    private function disableEventListeners()
    {
        $em = $this->container->get("doctrine.orm.entity_manager");
        $eventManager = $em->getEventManager();
        $listeners = $eventManager->getListeners();
        foreach ($listeners as $event => $listenersHash) {
            foreach ($listenersHash as $listener) {
                $eventManager->removeEventListener($event, $listener);
                $this->disabledEventListeners[$event][] = $listener;
            }
        }
    }

    private function enableEventListeners()
    {
        $em = $this->container->get("doctrine.orm.entity_manager");
        $eventManager = $em->getEventManager();
        foreach ($this->disabledEventListeners as $event => $listeners) {
            foreach ($listeners as $listener) {
                $eventManager->addEventListener($event, $listener);
            }
            unset($this->disabledEventListeners[$event]);
        }
    }

    /**
     *
     * @return integer
     */
    private function getWinPhoneDeviceOs()
    {
        static $deviceOs = null;
        if ($deviceOs === null) {
            $deviceTypeFactory = $this->container->get("device.type.factory");
            $deviceOs = $deviceTypeFactory->getType("winphone")->getId();
        }

        return $deviceOs;
    }
}
