<?php
namespace Topalis\Update\Command;

use Topalis\Certificate\Certificate;

/**
 * This class contains methods to perform the update to version 2.3.4
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateHelper020304
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(\Symfony\Component\DependencyInjection\ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * Because of a bug in phpseclib certificates which should have a serial of
     * 48 actually have a serial of 0. #AUR-629
     * This method checks if such a certificate exists and asks the admin to
     * rerollout the affected device if it is still in use.
     */
    public function informAdminWrongCertSerial()
    {
        try {
            $certEm = $this->container->get("certificate.entity.manager.certificate");
            $cert = $certEm->find(48);
            if (null === $cert) {
                return;
            }

            $certificate = new Certificate($cert->getCert());
            if ($certificate->getSerial() == 48) {
                return;
            }

            $device = $cert->getDevice();
            if ($device->isDeleted()) {
                return;
            }

            $user = $device->getUser();
            $username = $user->getLogin();
            $deviceName = $device->getName();
            if (strlen($device->getComment() > 0)) {
                $deviceName .= " (" . $device->getComment() . ")";
            }

            $mailer = $this->container->get("mailer");
            if (!$mailer->isConfigured()) {
                $this->container->get("logger")->warning(
                    sprintf(
                        "The device '%s' of user '%s' has a certificate with serial 0 which should have serial 48",
                        $deviceName, $username
                    ), array("device id" => $device->getId())
                );
                return;
            }

            $translator = $this->container->get("translator");
            $subject = $translator->trans("admin.update.inform.wrong_cert_serial.subject");
            $text = $translator->trans("admin.update.inform.wrong_cert_serial.text", array(
                "%user%" => $username,
                "%device%" => $deviceName
            ));

            $mailer->sendAdminMessage($subject, $text);
        } catch (\Exception $e) {
            $this->container->get("logger")->error(
                "Informing admin about invalid certificate serial failed", array("exception" => $e));
        }
    }
}
