<?php
namespace Topalis\Update\Command;

use Topalis\Certificate\Certificate;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * This class contains methods to perform the update to version 2.5.0
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class UpdateHelper020500
{
    /**
     * @var ContainerInterface
     */
    private $container;

    public function __construct(ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     *
     * @return \Topalis\Certificate\CertificateManager
     */
    private function getCertificateManager()
    {
        return $this->container->get("certificate.manager");
    }

    /**
     * @return \Topalis\Configuration\AuralisConfig
     */
    private function getConfig()
    {
        return $this->container->get("config.auralis");
    }

    /**
     * In ModConfig::setFilterPort() the parameter was converted to int so if
     * the advanced settings where modified the port of the ASF server was not
     * set.
     */
    public function resetASFPort()
    {
        $apacheMod = $this->container->get("asf.apache_mod");
        $config = $apacheMod->getConfig();
        if ("\${activesyncfilter_port}" != $config->getFilterPort()) {
            $config->setFilterPort("\${activesyncfilter_port}");
            $apacheMod->enable($config);
        }
    }

    /**
     * Resign CA and server certificate with SHA256 to replace SHA1 signature.
     */
    public function resignCerts()
    {
        $logger = $this->container->get("logger");
        $config = $this->getConfig();

        $caCert = $this->getCertificateManager()->getCACert();
        if (null === $caCert) {
            // installation wizard wasn't performed
            return;
        }

        // signatureTypeLN is available since PHP 5.5
        $caParsed = $caCert->getParsed();
        if ($caCert->isSelfSigned()
            && (!isset($caParsed['signatureTypeLN']) || "sha256WithRSAEncryption" != $caParsed['signatureTypeLN'])) {
            // CA cert is (propably) created by auralis
            $logger->info("Resign CA certificate");
            $serial = $config->getAndDecrementCertNextSerial();
            $cert = $caCert->export($serial);
            $config->setCACert($cert);
            $this->getCertificateManager()->exportCACert();
        }

        $serverCert = $this->getCertificateManager()->getServerCert();
        // check if server cert is signed by auralis CA
        if ($serverCert->getSignerKeyId() != $caCert->getKeyId()) {
            return;
        }

        $serverParsed = $serverCert->getParsed();
        if (isset($serverParsed['signatureTypeLN']) && "sha256WithRSAEncryption" == $serverParsed['signatureTypeLN']) {
            return;
        }

        $logger->info("Resign server certificate");
        $serial = $config->getAndDecrementCertNextSerial();
        $cert = $serverCert->export($serial);
        $config->setServerCert($cert);

        $chainedCA = $this->getCertificateManager()->getChainedCACert();
        $config->setServerChain(serialize($chainedCA));
        $this->getCertificateManager()->exportServerCert();
    }

    /**
     * Sets the last access in the devices table to the last timestamp
     */
    public function updateLastDeviceAccess()
    {
        $connection = $this->container->get("doctrine")->getConnection();
        $connection->executeUpdate(
            "UPDATE devices d SET d.last_access = "
            . "(SELECT UNIX_TIMESTAMP(MAX(a.tstamp)) FROM accesslog a WHERE a.device_id = d.id)"
        );
    }

    /**
     * Sets the remote proxy server in the Apache config for reverse proxy
     * entries.
     */
    public function setApacheRemoteProxyServer()
    {
        $config = $this->getConfig();
        $apache = $this->container->get("apache");
        $apacheRemoteProxyServer = null;
        if ($config->getHttpProxy() && !$config->getHttpProxyUser()) {
            // proxy authentication is not supported by Apache
            // https://bz.apache.org/bugzilla/show_bug.cgi?id=37355
            $apacheRemoteProxyServer = "http://" . $config->getHttpProxy();
        }
        $apache->setDefine("auralis_remote_proxy_server", $apacheRemoteProxyServer);
    }
}
