<?php
namespace Topalis\User\Controller;

use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Symfony\Component\Form\FormError;
use Symfony\Component\HttpFoundation\Response;
use Topalis\Kernel\Controller;
use Topalis\User\Entity\User;
use Topalis\User\Form\UserForm;

use Sensio\Bundle\FrameworkExtraBundle\Configuration\Route;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Template;
use Topalis\ConfigGroup\Entity\ConfigGroup;

/**
 *  User Controller for managing users
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 * @Route("/admin/user", service="user.controller")
 */
class UserController extends Controller
{
    /**
     * @return \Topalis\User\Entity\UserEntityManager
     */
    private function getUserEntityManager()
    {
        return $this->container->get("user.entity.manager.user");
    }

    /**
     * @return \Topalis\Device\DeviceManager
     */
    private function getDeviceManager()
    {
        return $this->container->get("device.manager");
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroupEntityManager
     */
    private function getConfigGroupEntityManager()
    {
        return $this->container->get("config_group.entity.manager.config_group");
    }

    /**
     * @Route("", name="admin_user")
     * @Route("/group/{group}", name="admin_user_group")
     * @Template()
     * @param ConfigGroup $group
     *
     * @return Response
     */
    public function indexAction(ConfigGroup $group = null)
    {
        $query = $this->getUserEntityManager()->createQueryBuilder("u")
            ->select(
                "u AS user",
                "COUNTIF(d.disabled = 0) AS active",
                "COUNTIF(d.disabled = 1 AND d.deleted = 0) AS inactive",
                "COUNTIF(d.deleted = 0) AS total"
            )
            ->leftJoin("u.devices", "d")
            ->orderBy("u.login", "ASC")
        ;

        if ($group !== null) {
            $query
                ->join("u.configGroup", "c")
                ->andWhere("c = :group")
                ->setParameter("group", $group)
            ;
        }

        $users = $query
            ->groupBy("u.id")
            ->getQuery()
            ->getResult()
        ;

        $groupsExist = $this->getConfigGroupEntityManager()->exists();

        return array(
            "users" => $users,
            "group" => $group,
            "groupsExist" => $groupsExist
        );
    }

    /**
     * @Route("/admin/user/add", name="admin_user_add", options={"expose" = true})
     * @Template()
     *
     * @return string|Response
     */
    public function addAction()
    {
        $user = new User();

        return $this->forward("user.controller:editAction", array("user" => $user));
    }

    /**
     * @Route("/admin/user/{id}/edit", name="admin_user_edit", options={"expose" = true})
     * @Template()
     *
     * @param User $user
     *
     * @throws NotFoundHttpException
     * @return string|Response
     */
    public function editAction(User $user)
    {
        $form = $this->createForm(new UserForm());
        try {
            $request = $this->getRequest();
            $form->setData($user);

            if ($request->isMethod("POST")) {
                $form->submit($request);
                if ($form->isValid()) {
                    $this->getUserEntityManager()->save($user);
                    $this->addFlash("success", $this->tr("admin.users.saveSuccess"));
                    return new JsonResponse(array("status" => "OK", "user" => $user->getId()));
                }
            }
        } catch (\Exception $e) {
            if ($user->getId() === null) {
                $this->getLogger()->error("Error inserting user", array("exception" => $e));
                $msg = $this->tr(
                    "admin.users.error.addUser",
                    array("%errmsg%" => $e->getMessage())
                );
            } else {
                $this->getLogger()->error("Error updating user with id " . $user->getId(), array("exception" => $e));
                $msg = $this->tr(
                    "admin.users.error.changeUser",
                    array("%id%" => $user->getId(), "%errmsg%" => $e->getMessage())
                );
            }
            $form->addError(new FormError($msg));
        }

        return array(
            "form" => $form->createView(),
            "user" => $user,
        );
    }

    /**
     * @Route("/admin/user/{id}/delete", name="admin_user_delete")
     * @param User $user
     *
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function deleteAction(User $user)
    {
        try {
            $this->getUserEntityManager()->delete($user);
            $this->addFlash("success", $this->tr("admin.users.deleteSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error deleting user with id " . $user->getId(), array("exception" => $e));
            $msg = $this->tr("admin.users.error.delete",
                array("%id%" => $user->getId(), "%errmsg%" => $e->getMessage()));
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_user"));
    }

    /**
     * @Route("/admin/user/{id}/disable", name="admin_user_disable")
     * @return \Symfony\Component\HttpFoundation\RedirectResponse
     */
    public function disableAction()
    {
        $id = $this->getRequest()->query->getInt("id");
        if (null == $id) {
            return $this->redirect($this->generateUrl("admin_user"));
        }

        try {
            $devices = $this->getDB()->fetchObjects("devices", array("id"), array("user_id" => $id, "disabled" => 0));
            foreach ($devices as $device) {
                $this->getDeviceManager()->setDeviceEnabled($device->id, false);
            }
            $this->addFlash("success", $this->tr("admin.users.disableSuccess"));
        } catch (\Exception $e) {
            $this->getLogger()->error("Error disabling devices of user with id $id", array("exception" => $e));
            $msg = $this->tr(
                "admin.users.error.disableDevices",
                array("%id%" => $id, "%errmsg%" => $e->getMessage())
            );
            $this->addError($msg);
        }

        return $this->redirect($this->generateUrl("admin_user"));
    }

    /**
     * @Route("/admin/user/{id}/load_group", name="admin_user_load_group", options={"expose" = true},
     *  defaults={"_format"="json"}
     * )
     * @param User $user
     *
     * @return Response
     */
    public function loadGroup(User $user)
    {
        $group = $user->getConfigGroup();
        return new JsonResponse(array(
            "id" => $group->getId(),
            "name" => $group->getName()
        ));
    }

    /**
     * @Route("/search", name="admin_user_search", options={"expose" = true}, defaults={"_format"="json"})
     */
    public function searchUsersAction()
    {
        $request = $this->getRequest();
        if ($request->query->has("query")) {
            $query = $this->getRequest()->query->get("query");
            /** @var $users User[] */
            $users = $this->getUserEntityManager()->createQueryBuilder("u")
                ->select("u")
                ->where("u.login LIKE :query OR u.realname LIKE :query")
                ->setParameter("query", "%".$query."%")
                ->setMaxResults(10)
                ->getQuery()
                ->getResult()
            ;

            $result = array();
            foreach ($users as $user) {
                $result[] = array(
                    "id" => $user->getId(),
                    "text" => $user->getLogin()
                );
            }

            return new JsonResponse($result);
        }

        if ($request->query->has("id")) {
            $user = $this->getUserEntityManager()->find($request->query->getInt("id"));

            return new JsonResponse(array(
                "id" => $user->getId(),
                "text" => $user->getLogin()
            ));
        }

    }

}
