<?php
namespace Topalis\User\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\ORM\Mapping as ORM;
use Topalis\App\Entity\VppLicense;
use Topalis\ConfigGroup\Entity\ConfigGroup;
use Topalis\Device\Entity\Device;
use Symfony\Bridge\Doctrine\Validator\Constraints\UniqueEntity;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Mapping\ClassMetadata;

/**
 * User
 *
 * @ORM\Table(name="users")
 * @ORM\Entity
 * @UniqueEntity(fields={"login"}, message="users.loginTaken")
 */
class User
{
    /**
     * default status, when user is not registered with vpp
     */
    const VPP_STATUS_UNREGISTERED = "unregistered";

    /**
     * status, when user got inivited to vpp
     */
    const VPP_STATUS_REGISTERED = "registered";

    /**
     * status, when user accepted invitation to vpp
     */
    const VPP_STATUS_ASSOCIATED = "associated";

    /**
     * @var integer
     *
     * @ORM\Column(name="id", type="integer", nullable=false, options={"unsigned"=true})
     * @ORM\Id
     * @ORM\GeneratedValue(strategy="IDENTITY")
     */
    protected $id;

    /**
     * @var string
     *
     * @ORM\Column(name="login", type="string", length=50, nullable=false, unique=true,
     *     options={"comment"="The unique login name"})
     */
    protected $login;

    /**
     * @var string
     *
     * @ORM\Column(name="realname", type="string", length=50, nullable=true,
     *     options={"comment"="The real name"})
     */
    protected $realname;

    /**
     * @var string
     *
     * @ORM\Column(name="email", type="string", length=50, nullable=false,
     *     options={"comment"="The email address"})
     */
    protected $email;

    /**
     * @var string
     *
     * @ORM\Column(name="comment", type="string", length=255, nullable=true,
     *     options={"comment"="A comment without functional meaning"})
     */
    protected $comment;

    /**
     * @var \Topalis\ConfigGroup\Entity\ConfigGroup
     *
     * @ORM\ManyToOne(targetEntity="Topalis\ConfigGroup\Entity\ConfigGroup", inversedBy="users", cascade={"persist"})
     * @ORM\JoinColumn(nullable=false)
     */
    protected $configGroup;

    /**
     * @var ArrayCollection
     *
     * @ORM\OneToMany(targetEntity="Topalis\Device\Entity\Device", mappedBy="user", cascade={"persist"})
     */
    protected $devices;

    /**
     * @var int userId assigned by vpp
     *
     * @ORM\Column(type="integer", nullable=true, unique=true)
     */
    protected $vppUserId;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true, options={"default" = "unregistered"})
     */
    protected $vppStatus = self::VPP_STATUS_UNREGISTERED;

    /**
     * @var string
     *
     * @ORM\Column(type="string", nullable=true)
     */
    protected $vppInviteLink;

    /**
     * @var ArrayCollection
     *
     * @ORM\OneToMany(targetEntity="Topalis\App\Entity\VppLicense", mappedBy="user")
     */
    protected $vppLicenses;

    public static function loadValidatorMetadata(ClassMetadata $metadata)
    {
        $metadata->addPropertyConstraint("login", new Assert\NotBlank());
        $metadata->addPropertyConstraint("realname", new Assert\NotBlank());
        $metadata->addPropertyConstraint("email", new Assert\NotBlank());
        $metadata->addPropertyConstraint("email", new Assert\Email());
    }

    public function __construct()
    {
        $this->devices = new ArrayCollection();
        $this->vppLicenses = new ArrayCollection();
    }

    /**
     * Get id
     *
     * @return integer
     */
    public function getId()
    {
        return $this->id;
    }

    /**
     * Set login
     *
     * @param string $login
     * @return User
     */
    public function setLogin($login)
    {
        $this->login = $login;

        return $this;
    }

    /**
     * Get login
     *
     * @return string
     */
    public function getLogin()
    {
        return $this->login;
    }

    /**
     * Set realname
     *
     * @param string $realname
     * @return User
     */
    public function setRealname($realname)
    {
        $this->realname = $realname;

        return $this;
    }

    /**
     * Get realname
     *
     * @return string
     */
    public function getRealname()
    {
        return $this->realname;
    }

    /**
     * Set email
     *
     * @param string $email
     * @return User
     */
    public function setEmail($email)
    {
        $this->email = $email;

        return $this;
    }

    /**
     * Get email
     *
     * @return string
     */
    public function getEmail()
    {
        return $this->email;
    }

    /**
     * Set comment
     *
     * @param string $comment
     * @return User
     */
    public function setComment($comment)
    {
        $this->comment = $comment;

        return $this;
    }

    /**
     * Get comment
     *
     * @return string
     */
    public function getComment()
    {
        return $this->comment;
    }

    /**
     * @param \Topalis\ConfigGroup\Entity\ConfigGroup $configGroup
     * @return \Topalis\User\Entity\User $this
     */
    public function setConfigGroup(\Topalis\ConfigGroup\Entity\ConfigGroup $configGroup)
    {
        if ($this->configGroup === $configGroup) {
            return $this;
        }

        if ($this->configGroup !== null) {
            $this->configGroup->removeUser($this);
        }

        $this->configGroup = $configGroup;
        $configGroup->addUser($this);

        return $this;
    }

    /**
     * @return \Topalis\ConfigGroup\Entity\ConfigGroup
     */
    public function getConfigGroup()
    {
        return $this->configGroup;
    }

    /**
     * @return Device[]
     */
    public function getDevices()
    {
        return $this->devices;
    }

    public function setDevices(array $devices)
    {
        $this->devices->clear();
        foreach ($devices as $device) {
            $this->addDevice($device);
        }

        return $this;
    }

    public function addDevice(Device $device)
    {
        if ($this->devices->contains($device)) {
            return $this;
        }

        $this->devices->add($device);
        $device->setUser($this);

        return $this;
    }

    public function removeDevice(Device $device)
    {
        $this->devices->removeElement($device);

        return $this;
    }

    /**
     * @return int
     */
    public function getVppUserId()
    {
        return $this->vppUserId;
    }

    /**
     * @param int $vppUserId
     *
     * @return $this
     */
    public function setVppUserId($vppUserId)
    {
        $this->vppUserId = $vppUserId;

        return $this;
    }

    /**
     * @return string
     */
    public function getVppStatus()
    {
        return $this->vppStatus;
    }

    /**
     * @param string $vppStatus
     *
     * @return $this
     */
    public function setVppStatus($vppStatus)
    {
        $this->vppStatus = $vppStatus;

        return $this;
    }

    /**
     * @return string
     */
    public function getVppInviteLink()
    {
        return $this->vppInviteLink;
    }

    /**
     * @param string $vppInviteLink
     *
     * @return $this
     */
    public function setVppInviteLink($vppInviteLink)
    {
        $this->vppInviteLink = $vppInviteLink;

        return $this;
    }
    /**
     * @param VppLicense[] $vppLicenses
     *
     * @return $this
     */
    public function setVppLicenses(array $vppLicenses)
    {
        $this->vppLicenses->clear();
        foreach ($vppLicenses as $vppLicense) {
            $this->addVppLicense($vppLicense);
        }

        return $this;
    }

    /**
     * @param VppLicense $vppLicense
     *
     * @return $this
     */
    public function addVppLicense(VppLicense $vppLicense)
    {
        if (!$this->vppLicenses->contains($vppLicense)) {
            $this->vppLicenses->add($vppLicense);
            $vppLicense->setUser($this);
        }

        return $this;
    }

    /**
     * @param VppLicense $vppLicense
     *
     * @return $this
     */
    public function removeVppLicense(VppLicense $vppLicense)
    {
        if ($this->vppLicenses->removeElement($vppLicense)) {
            $vppLicense->setUser(null);
        }

        return $this;
    }

    /**
     * @return VppLicense[]
     */
    public function getVppLicenses()
    {
        return $this->vppLicenses;
    }

    public function __toString()
    {
        return $this->getLogin();
    }
}
