<?php
namespace Topalis\User\Ldap;

use Topalis\Configuration\LdapConfig;
use Toyota\Component\Ldap\Core\Manager;
use Toyota\Component\Ldap\Core\Node;
use Toyota\Component\Ldap\Platform\Native\Driver;
use Topalis\Configuration\AuralisConfig;
use Topalis\User\Entity\UserEntityManager;

/**
 * Default LdapManager implementation.
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class LdapManager implements LdapManagerInterface
{
    /**
     * @var \Topalis\Configuration\LdapConfig
     */
    private $ldapConfig;

    /**
     * @var \Toyota\Component\Ldap\Core\Manager
     */
    private $manager;

    /**
     * @var callback
     */
    private $userFilterFunction;

    public function __construct()
    {
        $this->userFilterFunction = function ($user) {
            return true;
        };
    }

    /**
     * @param \Topalis\Configuration\AuralisConfig $config
     */
    public function setConfig(AuralisConfig $config)
    {
        $this->setLdapConfig($config->getLdapConfig());
    }

    public function setLdapConfig(LdapConfig $ldapConfig)
    {
        $this->ldapConfig = $ldapConfig;
    }

    /**
     * @param \Topalis\User\Entity\UserEntityManager $userEntityManager
     */
    public function setUserEntityManager(UserEntityManager $userEntityManager)
    {
        $this->setUserFilterFunction(function ($user) use ($userEntityManager) {
            return !$userEntityManager->exists("entity.login = :login", array("login" => $user['login']));
        });
    }

    public function setUserFilterFunction($userFilterFunction)
    {
        $this->userFilterFunction = $userFilterFunction;
    }

    /**
     * Connects and binds to the Ldap connection.
     *
     * @throws Toyota\Component\Ldap\Exception\ConnectionException if connection fails.
     * @throws Toyota\Component\Ldap\Exception\OptionException if connection configuration fails.
     * @throws Toyota\Component\Ldap\Exception\BindException if binding fails.
     */
    public function bind()
    {
        $config = $this->getLdapConfig();
        $params = array(
                "hostname" => $config->hostname,
                "base_dn"  => $config->baseDn,
                "options"  => array(
                    LDAP_OPT_NETWORK_TIMEOUT => 5,
                    LDAP_OPT_PROTOCOL_VERSION => 3,
                    LDAP_OPT_REFERRALS => 0, // avoid problems with ActiveDircectory
                )
        );
        if ($config->port !== null) {
            $params['port'] = $config->port;
        }
        if ($config->security !== null && $config->security != "None") {
            $params['security'] = $config->security;
        }
        $manager = new Manager($params, new Driver());
        $manager->connect();
        if (!empty($config->bindName)) {
            $manager->bind($config->bindName, $config->bindPassword);
        } else {
            $manager->bind();
        }

        $this->manager = $manager;
    }

    protected function filterUser($user)
    {
        return call_user_func($this->userFilterFunction, $user);
    }

    public function getUsers($baseDn = null, $inDepth = false, $limit = 0)
    {
        if (null === $this->manager) {
            $this->bind();
        }
        $filter = $this->getLdapConfig()->filter ?:
            "(&(|(objectClass=organizationalPerson)(objectclass=inetOrgPerson))(!(objectClass=computer)))";
        $result = $this->manager->search($baseDn, $filter, $inDepth,
            array("uid", "userPrincipalName", "cn", "mail", "ou"));
        $userCount = 0;
        $users = array();
        foreach ($result as $node) {
            /** @var $node Node */
            if (!$node->has("mail")) {
                continue;
            }

            $mail = $node->get("mail")->current();
            if (!$mail) {
                continue;
            }

            $login = null;
            if ($node->has("uid")) {
                $login = $node->get("uid")->current();
            } else if ($node->has("userPrincipalName")) {
                $login = $node->get("userPrincipalName")->current();
                $loginClean = strstr($login, "@", true);
                if ($loginClean) {
                    $login = $loginClean;
                }
            }
            if (!$login) {
                $login = strstr($mail, "@", true);
            }
            $login = preg_replace("/\s/", "", $login);

            $ou = null;
            if ($node->has("ou")) {
                $ou = $node->get("ou")->current();
            }
            if (!$ou) {
                $dn = $this->parseDn($node->getDn());
                $ou = isset($dn['ou']) ? $dn['ou'][0] : "";
            }
            $data = array(
                "login" => $login,
                "name" => $node->get("cn")->current(),
                "email" => $mail,
                "ou" => $ou
            );

            if (!$this->filterUser($data)) {
                continue;
            }

            $users[] = $data;
            $userCount++;
            if ($userCount == $limit) {
                break;
            }
        }

        return $users;
    }

    public function hasUsers($baseDn = null, $inDepth = false)
    {
        return (boolean) $this->getUsers($baseDn, $inDepth, 1);
    }

    public function isConfigured()
    {
        $config = $this->getLdapConfig();
        return !empty($config->hostname) && !empty($config->baseDn);
    }

    public function getLdapConfig()
    {
        return $this->ldapConfig;
    }

    public function getOuTree(Node $node = null)
    {
        if ($node === null) {
            $baseDn = $this->getLdapConfig()->baseDn;
            $name = $baseDn;
        } else {
            $baseDn = $node->getDn();
            if ($node->has("ou")) {
                $name = $node->get("ou")->current();
            } else if ($node->has("o")) {
                $name = $node->get("o")->current();
            }
        }

        $children = array();
        /** @var $result Node[] */
        $result = $this->manager->search($baseDn, "(|(objectclass=organization)(objectclass=organizationalUnit))",
            false, array("o", "ou"));
        foreach ($result as $node) {
            $subtree = $this->getOuTree($node);
            if (null === $subtree) {
                continue;
            }

            $children[] = $subtree;
        }

        return array(
            "id" => $baseDn,
            "data" => array(
                "title" => $name,
                "dn" => $baseDn,
            ),
            "attr" => array(
                "href" => $baseDn,
            ),
            "children" => $children
        );
    }

    /**
     * Parses the distinguished name and returns it as array.
     *
     * @param string $dn The distinguished name to parse.
     * @return array
     */
    public function parseDn($dn)
    {
        $parsed = array();
        $attributes = explode(",", $dn);
        foreach ($attributes as $attribute) {
            $attributeSplit = explode("=", $attribute);
            $parsed[strtolower($attributeSplit[0])][] = $attributeSplit[1];
        }

        return $parsed;
    }
}
