<?php
namespace Topalis\Util;

use Topalis\Kernel\Exception\Download\DownloadException;
use Topalis\Kernel\Exception\Curl\CurlException;

/**
 * Class to download data using curl.
 *
 * @author Andreas Hügel <andreas.huegel@topalis.com>
 * @author Holger Widmann <holger.widmann@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class Downloader
{
    /**
     * @var string
     */
    private $password;

    /**
     * @var string
     */
    private $proxy;

    /**
     * @var string
     */
    private $proxyPassword;

    /**
     * @var string
     */
    private $proxyUsername;

    /**
     * @var string
     */
    private $username;

    /**
     * @return string The password for authentication (basic auth).
     */
    public function getPassword()
    {
        return $this->password;
    }

    /**
     * Sets the password for authentication (basic auth).
     * @param string $password The password to set.
     */
    public function setPassword($password)
    {
        $this->password = $password;
    }

    /**
     * @return string The HTTP proxy to use.
     */
    public function getProxy()
    {
        return $this->proxy;
    }

    /**
     * Sets the HTTP proxy to use.
     *
     * @param string $proxy The HTTP proxy to set.
     */
    public function setProxy($proxy)
    {
        $this->proxy = $proxy;
    }

    /**
     * @return the string
     */
    public function getProxyPassword()
    {
        return $this->proxyPassword;
    }

    /**
     * @param string $proxyPassword
     */
    public function setProxyPassword($proxyPassword)
    {
        $this->proxyPassword = $proxyPassword;
    }

    /**
     * @return the string
     */
    public function getProxyUsername()
    {
        return $this->proxyUsername;
    }

    /**
     * @param string $proxyUsername
     */
    public function setProxyUsername($proxyUsername)
    {
        $this->proxyUsername = $proxyUsername;
    }

    /**
     *
     * @return string The username for authentication (basic auth).
     */
    public function getUsername()
    {
        return $this->username;
    }

    /**
     * Sets the username for authentication (basic auth).
     * @param string $username The username to set.
     */
    public function setUsername($username)
    {
        $this->username = $username;
    }

    /**
     * Prepare curl with default options.
     * @param int $timeout the maximal amount of time a cURL operation may take.
     * @return mixed Curl object.
     */
    private function prepareCurl($timeout = 60)
    {
        $curl = \curl_init();
        // disable IPv6, see #AUR-484
        curl_setopt($curl, CURLOPT_IPRESOLVE, CURL_IPRESOLVE_V4);
        if (strlen($this->getProxy()) > 0) {
            curl_setopt($curl, CURLOPT_PROXY, $this->getProxy());
            if (strlen($this->getProxyUsername()) > 0 && strlen($this->getProxyPassword()) > 0) {
                curl_setopt($curl, CURLOPT_PROXYAUTH, CURLAUTH_BASIC);
                curl_setopt($curl, CURLOPT_PROXYUSERPWD,
                    $this->getProxyUsername() . ":" . $this->getProxyPassword());
            }
        }
        curl_setopt($curl, CURLOPT_HEADER, 0);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_CONNECTTIMEOUT, 10);
        if ($timeout !== null) {
            curl_setopt($curl, CURLOPT_TIMEOUT, $timeout);
        } else {
            curl_setopt($curl, CURLOPT_TIMEOUT, 60);
        }

        if (strlen($this->getUsername()) > 0 && strlen($this->getPassword()) > 0) {
            curl_setopt($curl, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
            curl_setopt($curl, CURLOPT_USERPWD, $this->getUsername() . ":" . $this->getPassword());
        }

        return $curl;
    }

    /**
     * Download a url and save content in a file.
     * @param string $url The URL to download.
     * @param string $path The path of the file to save.
     * @param boolean $retry Retry on timeout.
     * @param int $timeout the maximal amount of time a cURL operation may take.
     * @return int HTTP status code.
     * @throws CurlException On CURL error.
     * @throws DownloadException If HTTP status code is greater or equal 400.
     */
    public function downloadFile($url, $path, $retry = false, $timeout = null)
    {
        $errorMessage = "";
        $errorCode = 0;
        // Get file.
        $curl = $this->prepareCurl($timeout);
        curl_setopt($curl, CURLOPT_URL, $url);
        $fp = fopen($path, "wb");
        curl_setopt($curl, CURLOPT_FILE, $fp);
        curl_setopt($curl, CURLOPT_BINARYTRANSFER, true);
        try {
            $ok = $this->curlExecFollow($curl, $timeout);
            fclose($fp);
            if (!$ok) {
                $errorCode = curl_errno($curl);
                $errorMessage = curl_error($curl);
            }
        } catch (CurlException $e) {
            $ok = false;
            $errorCode = $e->getCurlErrorCode();
            $errorMessage = $e->getCurlErrorMessage();
        }

        $httpStatus = intval(curl_getinfo($curl, CURLINFO_HTTP_CODE));
        curl_close($curl);
        if (!$ok) {
            if ($retry && $errorCode == CURLE_OPERATION_TIMEOUTED) {
                return $this->downloadFile($url, $path, false, $timeout);
            } else {
                throw new CurlException($url, $errorMessage, $errorCode);
            }
        }

        if ($httpStatus >= 400) {
            throw new DownloadException($url, $httpStatus);
        }
        return $httpStatus;
    }

    /**
     * Download the specified URL and return the recieved data.
     *
     * @param string $url The URL to download.
     * @param int $httpStatus The resulting HTTP code.
     * @param boolean $retry Retry on timeout.
     * @param int $timeout the maximal amount of time a cURL operation may take.
     * @return string The recieved data.
     * @throws CurlException On CURL error.
     * @throws DownloadException If HTTP status code is greater or equal 400.
     */
    public function loadFile($url, &$httpStatus = null, $retry = false, $timeout = null)
    {
        $curl = $this->prepareCurl($timeout);
        curl_setopt($curl, CURLOPT_URL, $url);
        try {
            $out = $this->curlExecFollow($curl, $timeout);
            if ($out === false) {
                $errorCode = curl_errno($curl);
                $errorMessage = curl_error($curl);
            }
        } catch (CurlException $e) {
            $out = false;
            $errorCode = $e->getCurlErrorCode();
            $errorMessage = $e->getCurlErrorMessage();
        }
        $tmpHttpStatus = intval(curl_getinfo($curl, CURLINFO_HTTP_CODE));

        curl_close($curl);

        if ($out === false) {
            if ($retry && $errorCode == CURLE_OPERATION_TIMEOUTED) {
                return $this->loadFile($url, $httpStatus, false, $timeout);
            } else {
                throw new CurlException($url, $errorMessage, $errorCode);
            }
        }

        if ($tmpHttpStatus >= 400) {
            throw new DownloadException($url, $tmpHttpStatus);
        }
        if ($httpStatus !== null) {
            $httpStatus = $tmpHttpStatus;
        }
        return $out;
    }

    /**
     * Execute curl handle and allow redirects even though
     * CURLOPT_FOLLOWLOCATION cannot be set beceause an open_basedir is set or
     * safe_mode is enabled.
     *
     * @param resource $ch The curl handle to execute.
     * @param int $timeout Timeout for the URL following.
     * @param int $maxredirect The maximum number of redirects. If null the
     *     maximum number of redirects is 5.
     * @return the result of curl_exec($ch)
     * @throws CurlException On error when following redirects.
     */
    private function curlExecFollow($ch, $timeout = 10, $maxredirect = 5)
    {
        $timeout = $timeout === null ? 10 : intval($timeout);
        $mr = $maxredirect === null ? 5 : intval($maxredirect);
        if (!ini_get('open_basedir') && !ini_get('safe_mode')) {
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, $mr > 0);
            curl_setopt($ch, CURLOPT_MAXREDIRS, $mr);
        } else {
            $url = curl_getinfo($ch, CURLINFO_EFFECTIVE_URL);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, false);
            if ($mr > 0) {
                $newurl = $url;
                $rch = $this->prepareCurl($timeout);
                curl_setopt($rch, CURLOPT_HEADER, true);
                curl_setopt($rch, CURLOPT_NOBODY, true);
                curl_setopt($rch, CURLOPT_FORBID_REUSE, false);
                curl_setopt($rch, CURLOPT_RETURNTRANSFER, true);
                do {
                    curl_setopt($rch, CURLOPT_URL, $newurl);
                    $header = curl_exec($rch);
                    $errorCode = curl_errno($rch);
                    if ($errorCode != 0) {
                        $errorMessage = curl_error($rch);
                        $code = 0;
                    } else {
                        $code = curl_getinfo($rch, CURLINFO_HTTP_CODE);
                        if ($code == 301 || $code == 302) {
                            preg_match('/Location:(.*?)\n/', $header, $matches);
                            $newurl = trim(array_pop($matches));
                        } else {
                            $code = 0;
                        }
                    }
                } while ($code && --$mr);
                curl_close($rch);
                if (!$mr) {
                    $errorCode = CURLE_TOO_MANY_REDIRECTS;
                    $errorMessage = 'Too many redirects. When following redirects, libcurl hit the maximum amount.';
                }
                if ($errorCode != 0) {
                    throw new CurlException($newurl, $errorMessage, $errorCode);
                }
                curl_setopt($ch, CURLOPT_URL, $newurl);
            }
        }
        return curl_exec($ch);
    }
}
