<?php
namespace Topalis\WinPhone\Command;

use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Helper\DialogHelper;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Topalis\Device\Entity\Device;
use Topalis\WinPhone\OMA\SyncML\Entity\Item;
use Topalis\WinPhone\OMA\SyncML\Entity\Target;
use Topalis\WinPhone\OMA\SyncML\Entity\Meta;

/**
 * This command allows to send a custom SyncML command for testing.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class SendSyncMLCommand extends ContainerAwareCommand
{
    protected function configure()
    {
        $this
            ->setName("syncml:send")
            ->setDefinition(array(
                new InputArgument("syncMLCommand", InputArgument::REQUIRED, "Command Add/Delete/Execute/Get/Replace"),
                new InputOption("device", "", InputOption::VALUE_OPTIONAL, "Device id"),
                new InputArgument("locURI", InputArgument::REQUIRED, "The locURI to query"),
                new InputArgument("data", InputArgument::OPTIONAL, "The data to use"),
                new InputOption("format", "", InputOption::VALUE_OPTIONAL,
                    "The format of the locURI"),
                new InputOption("output", "", InputOption::VALUE_OPTIONAL,
                    "output response in this file."),
            ))
            ->setDescription("Send SyncML command to a device and wait for response");
    }

    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $deviceEntityManager = $this->getContainer()->get("device.entity.manager.device");
        if (null === $input->getOption("device")) {
            /** @var DialogHelper $dialog */
            $dialog = $this->getHelper("dialog");
            /** @var Device[] $devices */
            $devices = $deviceEntityManager->createQueryBuilder("d")->select("d")
                ->where("d.os = :os")
                ->andWhere("d.deleted = 0")
                ->andWhere("d.mdmCert IS NOT NULL")
                ->setParameter("os", $this->getDeviceOs())
                ->getQuery()
                ->getResult()
            ;
            $deviceList = array();
            foreach ($devices as $device) {
                $deviceList[$device->getId()] = $device->getName();
            }
            $device = $dialog->select($output, "Choose a device", $deviceList);
        } else {
            $device = $input->getOption("device");
        }

        /** @var Device $device */
        $device = $deviceEntityManager->find($device);
        $commandEntityManager = $this->getContainer()->get("winphone.entity.manager.syncml_command");

        $command = $input->getArgument("syncMLCommand");
        $commandClass = "Topalis\WinPhone\OMA\SyncML\Entity\\" . $command;
        if (!class_exists($commandClass)) {
            throw new \InvalidArgumentException($command . " is no valid command");
        }

        $command = new $commandClass();
        $locURI = $input->getArgument("locURI");
        $data = $input->getArgument("data");
        $format = $input->getOption("format");
        $item = $this->createCommandItem($locURI, $data, $format ? array("Format" => $format) : null);
        $command->addItem($item);

        $syncMLAdmin = $this->getContainer()->get("winphone.oma.syncml.admin");

        $className = get_class($this);
        $pos = strrpos($className, "\\");
        $info = substr($className, $pos + 1);
        $syncMLCommand = $syncMLAdmin->addCommand($device, $command, $info);

        $output->writeln("<info>waiting for response</info>");
        while (true) {
            $commandEntityManager->refresh($syncMLCommand);
            if (null !== $syncMLCommand->getResponseTime()) {
                var_dump($syncMLCommand->getStatusCodes() ?: $syncMLCommand->getStatusCode());
                if (null !== $outputFile = $input->getOption("output")) {
                    file_put_contents($outputFile, "<?php return "
                        . var_export($syncMLCommand->getStatusCodes() ?: $syncMLCommand->getStatusCode(), true) . ";");
                }
                break;
            }
            sleep(3);
        }

    }

    private function createCommandItem($locURI, $data = null, array $meta = null)
    {
        $item = new Item();
        $target = new Target();
        $target->setLocURI($locURI);
        $item->setTarget($target);
        $item->setData($data);
        if ($meta !== null) {
            $item->setMeta(new Meta($meta));
        }

        return $item;
    }

    /**
     *
     * @return integer
     */
    private function getDeviceOs()
    {
        static $deviceOs = null;
        if ($deviceOs === null) {
            $deviceTypeFactory = $this->getContainer()->get("device.type.factory");
            $deviceOs = $deviceTypeFactory->getType("winphone")->getId();
        }

        return $deviceOs;
    }
}
