<?php
namespace Topalis\WinPhone\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Topalis\WinPhone\Entity\MSWlanConfig;
use Topalis\Device\Entity\Device;
use Doctrine\ORM\Event\PostFlushEventArgs;

/**
 * Listener to update devices if a WLAN configurations is added, changed, or
 * removed.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class WinPhoneWlanListener
{
    private $removeWlans = array();
    private $installWlans = array();

    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    private $container;

    public function __construct(\Symfony\Component\DependencyInjection\ContainerInterface $container)
    {
        $this->container = $container;
    }

    /**
     * @return \Topalis\WinPhone\OMA\SyncML\SyncMLAdmin
     */
    private function getSyncMLAdmin()
    {
        return $this->container->get("winphone.oma.syncml.admin");
    }

    public function postPersist(LifecycleEventArgs $eventArgs)
    {
        $wlanConfig = $eventArgs->getEntity();
        if (!$wlanConfig instanceof MSWlanConfig) {
            return;
        }

        $this->installWlans[$wlanConfig->getSsid()] = $wlanConfig;
    }

    public function postRemove(LifecycleEventArgs $eventArgs)
    {
        $wlanConfig = $eventArgs->getEntity();
        if (!$wlanConfig instanceof MSWlanConfig) {
            return;
        }

        $this->removeWlans[$wlanConfig->getSsid()] = $wlanConfig;
    }

    public function postUpdate(LifecycleEventArgs $eventArgs)
    {
        $wlanConfig = $eventArgs->getEntity();
        if (!$wlanConfig instanceof MSWlanConfig) {
            return;
        }

        $uow = $eventArgs->getEntityManager()->getUnitOfWork();
        $changeSet = $uow->getEntityChangeSet($wlanConfig);
        if (isset($changeSet['ssid'])) {
            $this->removeWlans[$changeSet[0]->getSsid()] = $changeSet[0];
        }
        $this->installWlans[$wlanConfig->getSsid()] = $wlanConfig;
    }

    public function postFlush(PostFlushEventArgs $eventArgs)
    {
        $eventArgs; // unused

        foreach ($this->removeWlans as $ssid => $wlanConfig) {
            unset($this->removeWlans[$ssid]); // required for the next flush
            if (isset($this->installWlans[$ssid])) {
                continue;
            }

            $this->removeWlanConfig($wlanConfig);
        }

        foreach ($this->installWlans as $ssid => $wlanConfig) {
            unset($this->installWlans[$ssid]); // required for the next flush
            $this->installWlanConfig($wlanConfig);
        }
    }

    /**
     *
     * @return integer
     */
    private function getDeviceOs()
    {
        static $deviceOs = null;
        if ($deviceOs === null) {
            $deviceTypeFactory = $this->container->get("device.type.factory");
            $deviceOs = $deviceTypeFactory->getType("winphone")->getId();
        }

        return $deviceOs;
    }

    private function installWlanConfig(MSWlanConfig $wlanConfig)
    {
        $configGroup = $wlanConfig->getConfigGroup();
        $ssid = $wlanConfig->getSsid();
        foreach ($configGroup->getDevices() as $device) {
            if (!$device->isActive() || !$device->isManaged() || $device->getOs() != $this->getDeviceOs()) {
                continue;
            }

            $this->getSyncMLAdmin()->installWiFiCSP($device, $ssid);
        }
    }

    private function removeWlanConfig(MSWlanConfig $wlanConfig)
    {
        $configGroup = $wlanConfig->getConfigGroup();
        $ssid = $wlanConfig->getSsid();
        foreach ($configGroup->getDevices() as $device) {
            if (!$device->isActive() || !$device->isManaged() || $device->getOs() != $this->getDeviceOs()) {
                continue;
            }

            $this->getSyncMLAdmin()->removeWiFiCSP($device, $ssid);
        }
    }
}
