<?php
namespace Topalis\WinPhone\MS;

/**
 * This class represents an app policy for use on Windows Phones.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class MSAppPolicy
{
    private $allowedApps = array();
    private $allowedPublishers = array();
    private $deniedApps = array();
    private $deniedPublishers = array();

    /**
     * @param \DOMElement $root
     * @return MSAppPolicy
     */
    public static function fromDOMElement(\DOMElement $root)
    {
        $appPolicy = new MSAppPolicy();

        $list = $root->firstChild;

        if ($list->nodeName == "Allow") {
            foreach ($list->childNodes as $child) {
                if ($child->nodeName == "App") {
                    $productId = trim($child->getAttribute("ProductId"), "{}");
                    $publisher = $child->getAttribute("PublisherName");
                    $appPolicy->addAllowedApp($productId, $publisher ?: null);
                }

                if ($child->nodeName == "Publisher") {
                    $publisher = $child->getAttribute("PublisherName");
                    $deniedApps = array();
                    foreach ($child->getElementsByTagName("DenyApp") as $denyAppElem) {
                        $deniedApps[] = trim($denyAppElem->getAttribute("ProductId"), "{}");
                    }
                    $appPolicy->addAllowedPublisher($publisher, $deniedApps);
                }
            }
        } else if ($list->nodeName == "Deny") {
            foreach ($list->childNodes as $child) {
                if ($child->nodeName == "App") {
                    $productId = trim($child->getAttribute("ProductId"), "{}");
                    $appPolicy->addDeniedApp($productId);
                }

                if ($child->nodeName == "Publisher") {
                    $publisher = $child->getAttribute("PublisherName");
                    $allowedApps = array();
                    foreach ($child->getElementsByTagName("AllowApp") as $allowAppElem) {
                        $allowedApps[] = trim($allowAppElem->getAttribute("ProductId"), "{}");
                    }
                    $appPolicy->addDeniedPublisher($publisher, $allowedApps);
                }
            }
        }

        return $appPolicy;
    }

    /**
     * @param string $xml
     * @return MSAppPolicy
     */
    public static function fromXml($xml)
    {
        if (!$xml) {
            return new MSAppPolicy();
        }

        $doc = new \DOMDocument();
        $doc->loadXML($xml);
        return self::fromDOMElement($doc->documentElement);
    }

    /**
     * Appends the app policy to the specified node.
     *
     * @param \DOMNode $node
     */
    public function appendToDOM(\DOMNode $node)
    {
        if ($node->nodeType == XML_DOCUMENT_NODE) {
            $doc = $node;
        } else {
            $doc = $node->ownerDocument;
        }
        $root = $doc->createElement("AppPolicy");
        $root->setAttribute("Version", "1");
        $root->setAttribute("xmlns", "http://schemas.microsoft.com/phone/2013/policy");
        $node->appendChild($root);

        if (!empty($this->deniedApps) || !empty($this->deniedPublishers)) {
            $denyElem = $doc->createElement("Deny");
            $root->appendChild($denyElem);

            foreach ($this->deniedApps as $productId) {
                $appElem = $doc->createElement("App");
                $appElem->setAttribute("ProductId", "{" . $productId . "}");
                $denyElem->appendChild($appElem);
            }

            foreach ($this->deniedPublishers as $publisher => $productIds) {
                $publisherElem = $doc->createElement("Publisher");
                $publisherElem->setAttribute("PublisherName", $publisher);
                foreach ($productIds as $productId) {
                    $allowAppElem = $doc->createElement("AllowApp");
                    $allowAppElem->setAttribute("ProductId", "{" . $productId . "}");
                    $publisherElem->appendChild($allowAppElem);
                }
                $denyElem->appendChild($publisherElem);
            }
        } else {
            $allowElem = $doc->createElement("Allow");
            $root->appendChild($allowElem);

            foreach ($this->allowedApps as $productId => $publisher) {
                $appElem = $doc->createElement("App");
                $appElem->setAttribute("ProductId", "{" . $productId . "}");
                if ($publisher) {
                    $appElem->setAttribute("PublisherName", $publisher);
                }
                $allowElem->appendChild($appElem);
            }

            foreach ($this->allowedPublishers as $publisher => $productIds) {
                $publisherElem = $doc->createElement("Publisher");
                $publisherElem->setAttribute("PublisherName", $publisher);
                foreach ($productIds as $productId) {
                    $denyAppElem = $doc->createElement("DenyApp");
                    $denyAppElem->setAttribute("ProductId", "{" . $productId . "}");
                    $publisherElem->appendChild($denyAppElem);
                }
                $allowElem->appendChild($publisherElem);
            }
        }
    }

    /**
     * @param string $productId
     * @param string|NULL $publisher
     */
    public function addAllowedApp($productId, $publisher = null)
    {
        $this->allowedApps[$productId] = $publisher;
    }

    /**
     * @return array Key value pairs with product id as key and an optional
     *     publisher name.
     */
    public function getAllowedApps()
    {
        return $this->allowedApps;
    }

    /**
     * @param string $productId
     */
    public function removeAllowedApp($productId)
    {
        unset($this->allowedApps[$productId]);
    }

    /**
     * @param string $publisher Publisher name.
     * @param array $deniedApps List of product ids denied for the specified
     *     publisher.
     */
    public function addAllowedPublisher($publisher, array $deniedApps = array())
    {
        $this->allowedPublishers[$publisher] = $deniedApps;
    }

    /**
     * @return array Array with publisher name as key and an array of allowed
     *     product ids denied for the publisher as value.
     */
    public function getAllowedPublishers()
    {
        return $this->allowedPublishers;
    }

    /**
     * @param string $publisher
     */
    public function removeAllowedPublisher($publisher)
    {
        unset($this->allowedPublishers[$publisher]);
    }

    /**
     * @param string $productId
     */
    public function addDeniedApp($productId)
    {
        if (!in_array($productId, $this->deniedApps)) {
            $this->deniedApps[] = $productId;
        }
    }

    /**
     * @return array Array of denied product ids.
     */
    public function getDeniedApps()
    {
        return $this->deniedApps;
    }

    /**
     * @param string $productId
     */
    public function removeDeniedApp($productId)
    {
        $key = array_search($productId, $this->deniedApps);
        if ($key !== false) {
            unset($this->deniedApps[$key]);
        }
    }

    /**
     * @param string $publisher Publisher name.
     * @param array $allowedApps List of product ids allowed for the specified
     *     publisher.
     */
    public function addDeniedPublisher($publisher, array $allowedApps = array())
    {
        $this->deniedPublishers[$publisher] = $allowedApps;
    }

    /**
     * @return array Array with publisher name as key and an array of product
     *     ids allowed for the publisher as value.
     */
    public function getDeniedPublishers()
    {
        return $this->deniedPublishers;
    }

    /**
     * @param string $publisher
     */
    public function removeDeniedPublisher($publisher)
    {
        unset($this->deniedPublishers[$publisher]);
    }

    /**
     * @return boolean true if this policy is a blacklisting, false otherwise.
     */
    public function isBlacklist()
    {
        return $this->getDeniedApps() || $this->getDeniedPublishers();
    }

    /**
     * @return boolean true if all apps are denied, i.e. neither allowed nor
     *     disallowed apps or publishers are set; false otherwise.
     */
    public function isDeniedAll()
    {
        return empty($this->allowedApps) && empty($this->allowedPublishers)
            && empty($this->deniedApps) && empty($this->deniedPublishers);
    }

    /**
     * Clears the app policy. Denies all apps.
     */
    public function clear()
    {
        $this->clearAllowed();
        $this->clearDenied();
    }

    /**
     * Clears all allowed apps and publishers. If neither allowed nor disallowed
     *     apps or publishers are set, all apps are denied.
     */
    public function clearAllowed()
    {
        $this->allowedApps = array();
        $this->allowedPublishers = array();
    }

    /**
     * Clears all denied apps and publishers. If neither allowed nor disallowed
     *     apps or publishers are set, all apps are denied.
     */
    public function clearDenied()
    {
        $this->deniedApps = array();
        $this->deniedPublishers = array();
    }

    /**
     * @param boolean $formatted Print plist formatted (i.e. with newlines and
     *     whitespace indention) if true; defaults to false.
     * @return string The app policy as XML.
     */
    public function toXml($formatted = false)
    {
        $doc = new \DOMDocument();
        $this->appendToDOM($doc);

        if ($formatted) {
            $doc->formatOutput = true;
        } else {
            $doc->preserveWhiteSpace = false;
        }

        return $doc->saveXML($doc->documentElement);
    }
}
