<?php
namespace Topalis\WinPhone\MS;

use Topalis\Util\StringUtil;

/**
 * PHP represantation of an Microsoft WLAN profile.
 * Not implemented: IHV
 *
 * @see https://msdn.microsoft.com/en-us/library/windows/desktop/ms707341%28v=vs.85%29.aspx
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class MSWlanProfile
{

    /**
     *
     * @var string
     */
    private $name;

    /**
     *
     * @var string
     */
    private $ssid;

    /**
     *
     * @var boolean
     */
    private $nonBroadcast;

    /**
     *
     * @var string
     */
    private $connectionType = "ESS";

    /**
     *
     * @var string
     */
    private $connectionMode;

    /**
     *
     * @var boolean
     */
    private $autoSwitch;

    /**
     *
     * @var array
     */
    private $phyTypes;

    /**
     *
     * @var string
     */
    private $authentication;

    /**
     *
     * @var string
     */
    private $encryption;

    /**
     *
     * @var boolean
     */
    private $useOneX;

    /**
     *
     * @var boolean
     */
    private $fipsMode;

    /**
     *
     * @var string
     */
    private $keyType;

    /**
     *
     * @var boolean
     */
    private $protected;

    /**
     *
     * @var string
     */
    private $keyMaterial;

    /**
     *
     * @var string
     */
    private $pmkCacheMode;

    /**
     *
     * @var integer
     */
    private $pmkCacheTTL;

    /**
     *
     * @var integer
     */
    private $pmkCacheSize;

    /**
     *
     * @var boolean
     */
    private $preAuthMode;

    /**
     *
     * @var integer
     */
    private $preAuthThrottle;

    /**
     *
     * @return string The name of the profile.
     */
    public function getName()
    {
        return $this->name;
    }

    /**
     *
     * @param string $name The name of the profile.
     */
    public function setName($name)
    {
        $this->name = $name;
        return $this;
    }

    /**
     *
     * @return string The SSID of the WLAN.
     */
    public function getSsid()
    {
        return $this->ssid;
    }

    /**
     * Sets the SSID. If the name is empty it will be set to the specified SSID.
     *
     * @param string $ssid The SSID of the WLAN.
     */
    public function setSsid($ssid)
    {
        $this->ssid = $ssid;

        if (strlen($this->getName()) == 0) {
            $this->setName($ssid);
        }

        return $this;
    }

    /**
     *
     * @return boolean|NULL true to connect to a network that does not
     *    broadcast its SSID (hidden network).
     */
    public function isNonBroadcast()
    {
        return $this->nonBroadcast;
    }

    /**
     *
     * @param boolean|NULL $nonBroadcast true to connect to a network that does
     *    not broadcast its SSID (hidden network).
     */
    public function setNonBroadcast($nonBroadcast)
    {
        $this->nonBroadcast = $nonBroadcast;
        return $this;
    }

    /**
     *
     * @return string "ESS" for infrastructure networks, or "IBSS" for ad-hoc
     *    networks.
     */
    public function getConnectionType()
    {
        return $this->connectionType;
    }

    /**
     *
     * @param string $connectionType "ESS" for infrastructure networks, or
     *    "IBSS" for ad-hoc networks.
     */
    public function setConnectionType($connectionType)
    {
        $this->connectionType = $connectionType;
        return $this;
    }

    /**
     *
     * @return string|NULL "auto" if the connection to the WLAN is automatic,
     *    or "manual" if it is initiated by user. Must be set to "manual" for
     *    ad-hoc networks. (Default if not defined: "auto" for infrastructure
     *    networks, and "manual" for ad-hoc networks.)
     */
    public function getConnectionMode()
    {
        return $this->connectionMode;
    }

    /**
     *
     * @param string $connectionMode "auto" if the connection to the WLAN is
     *    automatic, or "manual" if it is initiated by user. Must be set to
     *    "manual" for ad-hoc networks. (Default if not defined: "auto" for
     *    infrastructure networks, and "manual" for ad-hoc networks.)
     */
    public function setConnectionMode($connectionMode)
    {
        $this->connectionMode = $connectionMode;
        return $this;
    }

    /**
     * If autoSwitch is true and connection mode is set to "auto", a connection
     * to a more preferred network must be attempted whenever a more preferred
     * network comes into range. A more preferred network is one that is ordered
     * higher in the list of preferred wireless networks. This connection
     * attempt occurs when connected to another network.
     *
     * @return boolean
     */
    public function isAutoSwitch()
    {
        return $this->autoSwitch;
    }

    /**
     * If autoSwitch is true and connection mode is set to "auto", a connection
     * to a more preferred network must be attempted whenever a more preferred
     * network comes into range. A more preferred network is one that is ordered
     * higher in the list of preferred wireless networks. This connection
     * attempt occurs when connected to another network.
     *
     * @param boolean $autoSwitch
     */
    public function setAutoSwitch($autoSwitch)
    {
        $this->autoSwitch = $autoSwitch;
        return $this;
    }

    /**
     *
     * @return array|NULL The array contains the 802.11 standards allowed to
     *    use. Valid entries are a, b, g, and n. If not defined all standards
     *    are allowed.
     */
    public function getPhyTypes()
    {
        return $this->phyTypes;
    }

    /**
     *
     * @param array $phyTypes The array contains the 802.11 standards allowed
     *    to use. Valid entries are a, b, g, and n. If not defined all standards
     *    are allowed.
     */
    public function setPhyTypes(array $phyTypes)
    {
        $this->phyTypes = $phyTypes;
        return $this;
    }

    /**
     *
     * @return string|NULL open, shared, WPA, WPAPSK, WPA2, WPA2PSK, or null.
     */
    public function getAuthentication()
    {
        return $this->authentication;
    }

    /**
     *
     * @param string $authentication open, shared, WPA, WPAPSK, WPA2, WPA2PSK,
     *     or null.
     */
    public function setAuthentication($authentication)
    {
        $this->authentication = $authentication;
        return $this;
    }

    /**
     *
     * @return string|NULL none, WEP, TKIP, or AES.
     */
    public function getEncryption()
    {
        return $this->encryption;
    }

    /**
     *
     * @param string $encryption none, WEP, TKIP, or AES.
     */
    public function setEncryption($encryption)
    {
        $this->encryption = $encryption;
        return $this;
    }

    /**
     *
     * @return boolean|NULL true if 802.11x authentication is enabled.
     */
    public function isUseOneX()
    {
        return $this->useOneX;
    }

    /**
     *
     * @param boolean $useOneX true to enable 802.11x authentication.
     */
    public function setUseOneX($useOneX)
    {
        $this->useOneX = $useOneX;
        return $this;
    }

    /**
     *
     * @return boolean If true the security level of the connection
     *    must comply with the FIPS 140-2 standard.
     */
    public function isFipsMode()
    {
        return $this->fipsMode;
    }

    /**
     *
     * @param boolean $fipsMode If true the security level of the connection
     *    must comply with the FIPS 140-2 standard.
     */
    public function setFipsMode($fipsMode)
    {
        $this->fipsMode = $fipsMode;
        return $this;
    }

    /**
     *
     * @return string|NULL networkKey, passPhrase, or NULL.
     */
    public function getKeyType()
    {
        return $this->keyType;
    }

    /**
     *
     * @param string $keyType networkKey, or passPhrase. If encryption is set
     *     to WEP, keyType must be set to networkKey.
     */
    public function setKeyType($keyType)
    {
        $this->keyType = $keyType;
        return $this;
    }

    /**
     *
     * @return boolean|NULL true if the shared key is encrypted.
     */
    public function isProtected()
    {
        return $this->protected;
    }

    /**
     *
     * @param boolean $protected true if the shared key is encrypted.
     */
    public function setProtected($protected)
    {
        $this->protected = $protected;
        return $this;
    }

    /**
     *
     * @return string The WEP/WPA key.
     */
    public function getKeyMaterial()
    {
        return $this->keyMaterial;
    }

    /**
     *
     * @param string $keyMaterial The WEP/WPA key.
     */
    public function setKeyMaterial($keyMaterial)
    {
        $this->keyMaterial = $keyMaterial;
        return $this;
    }

    /**
     * Indicates whether PMK caching will be used. Only valid for WPA2 networks.
     * Disabled if not defined.
     *
     * @see 802.11i specification
     *
     * @return string|NULL "enabled" if PMK caching is enabled, or "disabled".
     */
    public function getPmkCacheMode()
    {
        return $this->pmkCacheMode;
    }

    /**
     * Indicates whether PMK caching will be used. Only valid for WPA2 networks.
     * Disabled if not defined.
     *
     * @param string $pmkCacheMode "enabled" to enable PMK caching, or
     *    "disabled".
     */
    public function setPmkCacheMode($pmkCacheMode)
    {
        $this->pmkCacheMode = $pmkCacheMode;
        return $this;
    }

    /**
     *
     * @return int|NULL Length of time, in minutes, that a PMK cache will be
     *    kept.
     */
    public function getPmkCacheTTL()
    {
        return $this->pmkCacheTTL;
    }

    /**
     *
     * @param int $pmkCacheTTL Length of time, in minutes, that a PMK cache
     *    will be kept. Valid values are 5 to 1440.
     */
    public function setPmkCacheTTL($pmkCacheTTL)
    {
        $this->pmkCacheTTL = $pmkCacheTTL;
        return $this;
    }

    /**
     *
     * @return int|NULL The number of PMK cache entries on the client. Defaults
     *    to 128 if not defined.
     */
    public function getPmkCacheSize()
    {
        return $this->pmkCacheSize;
    }

    /**
     *
     * @param int $pmkCacheSize The number of PMK cache entries on the client.
     *    Defaults to 128 if not defined. Valid values are 1 to 255.
     *
     */
    public function setPmkCacheSize($pmkCacheSize)
    {
        $this->pmkCacheSize = $pmkCacheSize;
        return $this;
    }

    /**
     * Indicates whether pre-authentication will be used by the client.
     * Necessary for WPA2 secure fast roaming. Only valid if PMK cache is
     * enabled. Defaults to false if not defined.
     *
     * @return boolean|NULL true if enabled.
     */
    public function isPreAuthMode()
    {
        return $this->preAuthMode;
    }

    /**
     * Indicates whether pre-authentication will be used by the client.
     * Necessary for WPA2 secure fast roaming. Only valid if PMK cache is
     * enabled. Defaults to false if not defined.
     *
     * @param boolean $preAuthMode true to enable.
     */
    public function setPreAuthMode($preAuthMode)
    {
        $this->preAuthMode = $preAuthMode;
        return $this;
    }

    /**
     * @return int  The number of pre-authentication attempts to try on
     *    neighboring APs. Only valid if PMK cache is enabled. Defaults to 3 if
     *    not defined.
     */
    public function getPreAuthThrottle()
    {
        return $this->preAuthThrottle;
    }

    /**
     *
     * @param int $preAuthThrottle The number of pre-authentication attempts to
     *    try on neighboring APs. Only valid if PMK cache is enabled. Defaults
     *    to 3 if not defined.
     */
    public function setPreAuthThrottle($preAuthThrottle)
    {
        $this->preAuthThrottle = $preAuthThrottle;
        return $this;
    }

    public function toXml($formatted = false)
    {
        $doc = new \DOMDocument();

        $root = $doc->createElement("WLANProfile");
        $root->setAttribute("xmlns", "http://www.microsoft.com/networking/WLAN/profile/v1");
        $doc->appendChild($root);

        $name = $doc->createElement("name", $this->getName());
        $root->appendChild($name);

        $ssidConfig = $doc->createElement("SSIDConfig");
        $root->appendChild($ssidConfig);
        $ssid = $doc->createElement("SSID");
        $ssidConfig->appendChild($ssid);
        $ssidName = $doc->createElement("name", $this->getSsid());
        $ssid->appendChild($ssidName);
        if ($this->isNonBroadcast() !== null) {
            $nonBroadcast = $doc->createElement("nonBroadcast", StringUtil::booltostr($this->isNonBroadcast()));
            $ssidConfig->appendChild($nonBroadcast);
        }

        $connectionType = $doc->createElement("connectionType", $this->getConnectionType());
        $root->appendChild($connectionType);

        if ($this->getConnectionMode() !== null) {
            $connectionMode = $doc->createElement("connectionMode", $this->getConnectionMode());
            $root->appendChild($connectionMode);
        }

        if ($this->isAutoSwitch() !== null) {
            $autoSwitch = $doc->createElement("autoSwitch", StringUtil::booltostr($this->isAutoSwitch()));
            $root->appendChild($autoSwitch);
        }

        $msm = $doc->createElement("MSM");

        if (count($this->getPhyTypes()) > 0) {
            $connectivity =$doc->createElement("connectivity");
            foreach ($this->getPhyTypes() as $type) {
                $phyType = $doc->createElement("phyType", $type);
                $connectivity->appendChild($phyType);
            }
            $msm->appendChild($connectivity);
        }

        $this->addSecurityNodeToDOM($msm);

        if ($msm->hasChildNodes()) {
            $root->appendChild($msm);
        }

        if ($formatted) {
            $doc->formatOutput = true;
        } else {
            $doc->preserveWhiteSpace = false;
        }

        return $doc->saveXML();
    }

    private function addSecurityNodeToDOM(\DOMNode $msm)
    {
        $doc = $msm->ownerDocument;
        $security = $doc->createElement("security");

        $authEncryption = $doc->createElement("authEncryption");
        if ($this->getAuthentication() !== null) {
            $authentication = $doc->createElement("authentication", $this->getAuthentication());
            $authEncryption->appendChild($authentication);
        }
        if ($this->getEncryption() !== null) {
            $encryption = $doc->createElement("encryption", $this->getEncryption());
            $authEncryption->appendChild($encryption);
        }
        if ($this->isUseOneX() !== null) {
            $useOneX = $doc->createElement("useOneX", StringUtil::booltostr($this->isUseOneX()));
            $authEncryption->appendChild($useOneX);
        }
        if ($this->isFipsMode() !== null) {
            $fipsMode = $doc->createElement("FIPSMode", StringUtil::booltostr($this->isFipsMode()));
            $authEncryption->appendChild($fipsMode);
        }
        if ($authEncryption->hasChildNodes()) {
            $security->appendChild($authEncryption);
        }

        $sharedKey = $doc->createElement("sharedKey");
        if ($this->getKeyType() !== null) {
            $keyType = $doc->createElement("keyType", $this->getKeyType());
            $sharedKey->appendChild($keyType);
        }
        if ($this->isProtected() !== null) {
            $protected = $doc->createElement("protected", StringUtil::booltostr($this->isProtected()));
            $sharedKey->appendChild($protected);
        }
        if ($this->getKeyMaterial() !== null) {
            $keyMaterial = $doc->createElement("keyMaterial", $this->getKeyMaterial());
            $sharedKey->appendChild($keyMaterial);
        }
        if ($sharedKey->hasChildNodes()) {
            $security->appendChild($sharedKey);
        }

        if ($this->getPmkCacheMode() !== null) {
            $pmkCacheMode = $doc->createElement("PMKCacheMode", $this->getPmkCacheMode());
            $security->appendChild($pmkCacheMode);
        }

        if ($this->getPmkCacheTTL() !== null) {
            $pmkCacheTTL = $doc->createElement("PMKCacheTTL", $this->getPmkCacheTTL());
            $security->appendChild($pmkCacheTTL);
        }

        if ($this->getPmkCacheSize() !== null) {
            $pmkCacheSize = $doc->createElement("PMKCacheSize", $this->getPmkCacheSize());
            $security->appendChild($pmkCacheSize);
        }

        if ($this->isPreAuthMode() !== null) {
            $preAuthMode = $doc->createElement("preAuthMode", StringUtil::booltostr($this->isPreAuthMode()));
            $security->appendChild($preAuthMode);
        }

        if ($this->getPreAuthThrottle() !== null) {
            $preAuthThrottle = $doc->createElement("preAuthThrottle", $this->getPreAuthThrottle());
            $security->appendChild($preAuthThrottle);
        }

        if ($security->hasChildNodes()) {
            $msm->appendChild($security);
        }
    }

    public static function fromXml($xml)
    {
        $profile = new MSWlanProfile();

        $doc = new \DOMDocument();
        $doc->loadXML($xml);
        $root = $doc->documentElement;

        foreach ($root->childNodes as $child) {
            if ($child->nodeType != XML_ELEMENT_NODE) {
                continue;
            }

            switch ($child->tagName) {
                case "name":
                    $profile->setName($child->nodeValue);
                    break;
                case "SSIDConfig":
                    $profile->setSsidConfigFromDOM($child);
                    break;
                case "connectionType":
                    $profile->setConnectionType($child->nodeValue);
                    break;
                case "connectionMode":
                    $profile->setConnectionMode($child->nodeValue);
                    break;
                case "autoSwitch":
                    $profile->setAutoSwitch(StringUtil::strtobool($child->nodeValue));
                    break;
                case "MSM":
                    foreach ($child->childNodes as $msmChild) {
                        if ($child->nodeType != XML_ELEMENT_NODE) {
                            continue;
                        }

                        switch ($msmChild->tagName) {
                            case "connectivity":
                                $phyTypes = array();
                                foreach ($msmChild->getElementsByTagName("phyType") as $phyType) {
                                    $phyTypes[] = $phyType->nodeValue;
                                }
                                $profile->setPhyTypes($phyTypes);
                                break;
                            case "security":
                                $profile->setSecurityFromDOM($msmChild);
                                break;
                        }
                    }
                    break;
            }
        }

        return $profile;
    }

    private function setSsidConfigFromDOM(\DOMElement $elem)
    {
        foreach ($elem->childNodes as $child) {
            if ($child->nodeType != XML_ELEMENT_NODE) {
                continue;
            }

            switch ($child->tagName) {
                case "SSID":
                    // element "hex" is not handled, yet
                    $name = $child->firstChild;
                    if ($name->tagName == "name") {
                        $this->setSsid($name->nodeValue);
                    }
                    break;
                case "nonBroadcast":
                    $this->setNonBroadcast(StringUtil::strtobool($child->nodeValue));
                    break;
            }
        }
    }

    private function setSecurityFromDOM(\DOMElement $elem)
    {
        foreach ($elem->childNodes as $child) {
            if ($child->nodeType != XML_ELEMENT_NODE) {
                continue;
            }

            switch ($child->tagName) {
                case "authEncryption":
                    foreach ($child->childNodes as $authChild) {
                        if ($child->nodeType != XML_ELEMENT_NODE) {
                            continue;
                        }

                        switch ($authChild->tagName) {
                            case "authentication":
                                $this->setAuthentication($authChild->nodeValue);
                                break;
                            case "encryption":
                                $this->setEncryption($authChild->nodeValue);
                                break;
                            case "useOneX":
                                $this->setUseOneX(StringUtil::strtobool($authChild->nodeValue));
                                break;
                            case "FIPSMode":
                                $this->setFipsMode(StringUtil::strtobool($authChild->nodeValue));
                                break;
                        }
                    }
                    break;
                case "sharedKey":
                    foreach ($child->childNodes as $sharedKeyChild) {
                        if ($child->nodeType != XML_ELEMENT_NODE) {
                            continue;
                        }

                        switch ($sharedKeyChild->tagName) {
                            case "keyType":
                                $this->setKeyType($sharedKeyChild->nodeValue);
                                break;
                            case "protected":
                                $this->setProtected(StringUtil::strtobool($sharedKeyChild->nodeValue));
                                break;
                            case "keyMaterial":
                                $this->setKeyMaterial($sharedKeyChild->nodeValue);
                                break;
                        }
                    }
                    break;
                case "PMKCacheMode":
                    $this->setPmkCacheMode($child->nodeValue);
                    break;
                case "PMKCacheTTL":
                    $this->setPmkCacheTTL(intval($child->nodeValue));
                    break;
                case "PMKCacheSize":
                    $this->setPmkCacheSize(intval($child->nodeValue));
                    break;
                case "preAuthMode":
                    $this->setPreAuthMode(StringUtil::strtobool($child->nodeValue));
                    break;
                case "preAuthThrottle":
                    $this->setPreAuthThrottle(intval($child->nodeValue));
                    break;
            }
        }
    }
}
