<?php
namespace Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider;

use Topalis\WinPhone\OMA\SyncML\Entity\Atomic;
use Topalis\WinPhone\OMA\SyncML\Entity\Item;
use Topalis\WinPhone\OMA\SyncML\Entity\Target;
use Topalis\WinPhone\OMA\SyncML\Entity\XmlConverter;
use Topalis\WinPhone\OMA\SyncML\Entity\Meta;
use Topalis\Util\StringUtil;

/**
 * Abstract class for configuration service providers.
 *
 * @see Windows Phone 8 Enterprise Device Management Protocol
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 */
abstract class AbstractConfigurationServiceProvider extends Atomic
{
    private $defaultCommand = "Add";

    private $forceDefaultCommand = false;

    private $values = array();

    private $locURIPrefix;

    protected static function getParentNamespace()
    {
        return XmlConverter::getNamespace();
    }

    public function getChildren()
    {
        $reflection = new \ReflectionClass($this);
        $children = $this->getChildrenForClass($reflection->getParentClass());

        $i = 0;
        foreach ($this->values as $key => $val) {
            $commandName = $val['command'];
            if ($commandName === null || $this->forceDefaultCommand) {
                $commandName = $this->getDefaultCommand();
            }

            if (!$this->isAllowed($key, $val['data'], $commandName)) {
                continue; // don't add command if it's not allowed
            }

            $className = static::getParentNamespace() . "\\" . $commandName;
            $command = new $className;
            $command->setCmdID($this->getCmdID() . "_" . ($i++));
            $item = new Item();
            $item->setTarget(new Target($this->getLocURIPrefix() . $key));
            if ($commandName != "Get" && $commandName != "Delete") {
                $item->setData($val['data']);
            }
            $meta = null;
            if (isset($val['format'])) {
                $meta['Format'] = $val['format'];
            }
            if (isset($val['type'])) {
                $meta['Type'] = $val['type'];
            }

            if ($meta) {
                $item->setMeta(new Meta($meta));
            }

            $command->addItem($item);
            $children[] = $command;
        }

        return $children;
    }

    public function getCommands()
    {
        return $this->getChildren();
    }

    public function setDefaultCommand($defaultCommand, $force = false)
    {
        if (!class_exists(static::getParentNamespace() . "\\" . $defaultCommand)) {
            throw new \InvalidArgumentException("Unknown command " . $defaultCommand);
        }

        $this->forceDefaultCommand = $force;
        $this->defaultCommand = $defaultCommand;
    }

    public function getDefaultCommand()
    {
        return $this->defaultCommand;
    }

    protected function getCommand($locURI)
    {
        return isset($this->values[$locURI]) ? $this->values[$locURI]["command"] : null;
    }

    /**
     *
     * @param string $locURI
     * @return string|number|NULL
     */
    protected function getData($locURI)
    {
        return isset($this->values[$locURI]) ? $this->values[$locURI]["data"] : null;
    }

    /**
     *
     * @param string $locURI
     * @return boolean|NULL true if the value converts to true, false otherwise,
     *     or null if it is not defined.
     */
    protected function isData($locURI)
    {
        $data = $this->getData($locURI);
        return $data === null ? null : StringUtil::strtobool($data);
    }

    protected function removeData($locURI)
    {
        unset($this->values[$locURI]);
    }

    protected function setData($locURI, $data, $command = null)
    {
        if ($command !== null && !class_exists(static::getParentNamespace() . "\\" . $command)) {
            throw new \InvalidArgumentException("Unknown command " . $command);
        }

        if (null === $data && "Exec" != $command) {
            $this->removeData($locURI);
        } else if (is_array($data)) {
            if (null === $data['data'] && "Exec" != $command) {
                $this->removeData($locURI);
            } else {
                $this->values[$locURI] = $data + array("command" => $command);
            }
        } else {
            $this->values[$locURI] = array("data" => $data, "command" => $command);
        }
    }

    public function getLocURIPrefix()
    {
        return $this->locURIPrefix;
    }

    protected function setLocURIPrefix($locURIPrefix)
    {
        $this->locURIPrefix = $locURIPrefix;
    }

    protected function isAllowed($locURI, $data, $command)
    {
        $locURI; // unused
        $data; // unused
        $command; // unused
        return true;
    }
}
