<?php
namespace Topalis\WinPhone\OMA\SyncML\Entity\ConfigurationServiceProvider;

/**
 * DeviceLock configuration service provider.
 *
 * @see Windows Phone 8 Enterprise Device Management Protocol
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 */
class DeviceLockCSP extends AbstractConfigurationServiceProvider
{
    /**
     *
     * @var string
     */
    private $providerId;

    /**
     *
     * @param string $cmdID
     * @param string $providerId
     */
    public function __construct($cmdID = null, $providerId = null)
    {
        parent::__construct($cmdID);
        if ($providerId !== null) {
            $this->setProviderId($providerId);
        }
        foreach ($this->getDefaults() as $locURI => $data) {
            $this->setData($locURI, array("data" => $data, "format" => "int"));
        }
    }

    /**
     * @return string
     */
    public function getProviderId()
    {
        return $this->providerId;
    }

    /**
     *
     * @param string $providerId
     */
    public function setProviderId($providerId)
    {
        $this->providerId = $providerId;
        $this->setLocURIPrefix("./Vendor/MSFT/DeviceLock/Provider/" . $providerId . "/");
    }

    /**
     * @return boolean true if device lock is enabled, false otherwise.
     *     (Default: true)
     */
    public function isDevicePasswordEnabled()
    {
        return $this->isData("DevicePasswordEnabled") ? false : true;
    }

    /**
     *
     * @param boolean $devicePasswordEnabled true to enabled device lock, false
     *     to disable. (Default: true)
     */
    public function setDevicePasswordEnabled($devicePasswordEnabled)
    {
        $data = $devicePasswordEnabled ? 0 : 1;
        $this->setData("DevicePasswordEnabled", array("data" => $data, "format" => "int"));
    }

    /**
     * @return boolean true if simple passwords, such as "1111" or "1234" are
     *    allowed, false otherwise. (Default: true)
     */
    public function isAllowSimplePassword()
    {
        return $this->isData("AllowSimpleDevicePassword");
    }

    /**
     *
     * @param boolean $allowSimplePassword true to allow simple passwords,
     *     such as "1111" or "1234", false to disallow. (Default: true)
     */
    public function setAllowSimplePassword($allowSimplePassword)
    {
        $data = $allowSimplePassword ? 1 : 0;
        $this->setData("AllowSimpleDevicePassword", array("data" => $data, "format" => "int"));
    }

    /**
     *
     * @return int The minimum number of characters required in the PIN. Valid
     *     values are 4 to 18. (Default: 4)
     */
    public function getMinDevicePasswordLength()
    {
        return $this->getData("MinDevicePasswordLength");
    }

    /**
     *
     * @param int $minDevicePasswordLength The minimum number of characters
     *     required in the PIN. Valid values are 4 to 18. (Default: 4)
     */
    public function setMinDevicePasswordLength($minDevicePasswordLength)
    {
        if ($minDevicePasswordLength < 4 || $minDevicePasswordLength > 18) {
            throw new \InvalidArgumentException("MinDevicePasswordLength must be between 4 and 18");
        }
        $this->setData("MinDevicePasswordLength", array("data" => $minDevicePasswordLength, "format" => "int"));
    }

    /**
     * @return int 0 if alphanumeric password required, 1 if numeric password
     *     required, 2 if user can choose a numeric password or alphanumeric
     *     password. (Default: 2)
     */
    public function getAlphanumericDevicePasswordRequired()
    {
        return $this->getData("AlphanumericDevicePasswordRequired");
    }

    /**
     *
     * @param int $alphanumericDevicePasswordRequired 0 if alphanumeric
     *     password required, 1 if numeric password required, 2 if user can
     *     choose a numeric password or alphanumeric password. (Default: 2)
     */
    public function setAlphanumericDevicePasswordRequired($alphanumericDevicePasswordRequired)
    {
        if ($alphanumericDevicePasswordRequired < 0 || $alphanumericDevicePasswordRequired > 2) {
            throw new \InvalidArgumentException("AlphanumericDevicePasswordRequired must be between 0 and 2");
        }
        $this->setData("AlphanumericDevicePasswordRequired",
            array("data" => $alphanumericDevicePasswordRequired, "format" => "int"));
    }

    /**
     * @return int The number of days before password expiration. Valid values
     *    are 0 to 730. 0 indicates that the password does not expire. (Default: 0)
     */
    public function getDevicePasswordExpiration()
    {
        return $this->getData("DevicePasswordExpiration");
    }

    /**
     *
     * @param int $devicePasswordExpiration The number of days before password
     *    expiration. Valid values are 0 to 730. 0 indicates that the password
     *    does not expire. (Default: 0)
     */
    public function setDevicePasswordExpiration($devicePasswordExpiration)
    {
        if ($devicePasswordExpiration < 0 || $devicePasswordExpiration > 730) {
            throw new \InvalidArgumentException("DevicePasswordExpiration must be between 0 and 730");
        }
        $this->setData("DevicePasswordExpiration", array("data" => $devicePasswordExpiration, "format" => "int"));
    }

    /**
     * @return int The number of passwords that can be stored in the history
     *    (can't be reused). Valid values are 0 to 50. (Default: 0)
     */
    public function getDevicePasswordHistory()
    {
        return $this->getData("DevicePasswordHistory");
    }

    /**
     *
     * @param int $devicePasswordHistory The number of passwords that can be
     *    stored in the history (can't be reused). Valid values are 0 to 50.
     *    (Default: 0)
     */
    public function setDevicePasswordHistory($devicePasswordHistory)
    {
        if ($devicePasswordHistory < 0 || $devicePasswordHistory > 50) {
            throw new \InvalidArgumentException("DevicePasswordHistory must be between 0 and 50");
        }
        $this->setData("DevicePasswordHistory", array("data" => $devicePasswordHistory, "format" => "int"));
    }

    /**
     * @return int The number of authentication failures allowed before the
     *     phone will be wiped. Valid values are 0 to 999. 0 indicates the
     *     phone will not be wiped regrdless of the number of authentication
     *     failures. (Default: 0)
     */
    public function getMaxDevicePasswordFailedAttempts()
    {
        return $this->getData("MaxDevicePasswordFailedAttempts");
    }

    /**
     *
     * @param int $maxDevicePasswordFailedAttempts The number of authentication
     *     failures allowed before the phone will be wiped. Valid values are 0
     *     to 999. 0 indicates the phone will not be wiped regardless of the
     *     number of authentication failures. (Default: 0)
     */
    public function setMaxDevicePasswordFailedAttempts($maxDevicePasswordFailedAttempts)
    {
        if ($maxDevicePasswordFailedAttempts < 0 || $maxDevicePasswordFailedAttempts > 999) {
            throw new \InvalidArgumentException("MaxDevicePasswordFailedAttempts must be between 0 and 999");
        }
        $this->setData("MaxDevicePasswordFailedAttempts",
            array("data" => $maxDevicePasswordFailedAttempts, "format" => "int"));
    }

    /**
     * @return int The amount of times in minutes that the phone can remain
     *    idle before it is password locked. Valid values are 0 to 999.
     *    0 indicates no time-out is specified. In this case, the maximum
     *    screen time-out allowed by the UI applies. (Default: 0)
     */
    public function getMaxInactivityTimeDeviceLock()
    {
        return $this->getData("MaxInactivityTimeDeviceLock");
    }

    /**
     *
     * @param int $maxInactivityTimeDeviceLock The amount of times in minutes
     *    that the phone can remain idle before it is password locked. Valid
     *    values are 0 to 999. 0 indicates no time-out is specified. In this
     *    case, the maximum screen time-out allowed by the UI applies.
     *    (Default: 0)
     */
    public function setMaxInactivityTimeDeviceLock($maxInactivityTimeDeviceLock)
    {
        if ($maxInactivityTimeDeviceLock < 0 || $maxInactivityTimeDeviceLock > 999) {
            throw new \InvalidArgumentException("MaxInactivityTimeDeviceLock must be between 0 and 999");
        }
        $this->setData("MaxInactivityTimeDeviceLock",
            array("data" => $maxInactivityTimeDeviceLock, "format" => "int"));
    }

    /**
     * @return int The number of complex element types (uppercase and lowercase
     *     letters, numbers, and punctuation) required for a string password.
     *     Valid values are 1 to 4. (Default: 1)
     */
    public function getMinDevicePasswordComplexCharacters()
    {
        return $this->getData("MinDevicePasswordComplexCharacters");
    }

    /**
     *
     * @param int $minDevicePasswordComplexCharacters The number of complex
     *     element types (uppercase and lowercase letters, numbers, and
     *     punctuation) required for a string password. Valid values are 1 to 4.
     *     (Default: 1)
     * @throws \InvalidArgumentException
     */
    public function setMinDevicePasswordComplexCharacters($minDevicePasswordComplexCharacters)
    {
        if ($minDevicePasswordComplexCharacters < 1 || $minDevicePasswordComplexCharacters > 4) {
            throw new \InvalidArgumentException("MinDevicePasswordComplexCharacters must be between 1 and 4");
        }
        $this->setData("MinDevicePasswordComplexCharacters",
            array("data" => $minDevicePasswordComplexCharacters, "format" => "int"));
    }

    protected function getDefaults()
    {
        return array(
            "DevicePasswordEnabled" => 1,
            "AllowSimpleDevicePassword" => 1,
            "MinDevicePasswordLength" => 4,
            "AlphanumericDevicePasswordRequired" => 2,
            "DevicePasswordExpiration" => 0,
            "DevicePasswordHistory" => 0,
            "MaxDevicePasswordFailedAttempts" => 0,
            "MaxInactivityTimeDeviceLock" => 0,
            "MinDevicePasswordComplexCharacters" => 1,
        );
    }
}
