<?php
namespace Topalis\WinPhone\OMA\SyncML\Handler;

use Topalis\Kernel\AbstractAuralisService;
use Topalis\WinPhone\OMA\SyncML\Entity\SyncML;
use Topalis\WinPhone\OMA\SyncML\Entity\SyncBody;
use Topalis\WinPhone\OMA\SyncML\Entity\SyncHdr;
use Topalis\WinPhone\OMA\SyncML\Entity\Status;
use Topalis\WinPhone\OMA\SyncML\Entity\Cred;
use Topalis\WinPhone\OMA\SyncML\Entity\Meta;
use Topalis\WinPhone\OMA\SyncML\Entity\Target;
use Topalis\WinPhone\OMA\SyncML\Entity\Source;
use Topalis\WinPhone\OMA\SyncML\Entity\Alert;
use Topalis\WinPhone\OMA\SyncML\SyncMLSession;
use Topalis\WinPhone\OMA\SyncML\Entity\XmlConverter;
use Topalis\WinPhone\Entity\SyncMLCommand;
use Topalis\Device\Entity\Device;
use Topalis\Util\StringUtil;
use Topalis\WinPhone\Event\OMA\SyncML\SyncMLEvents;
use Topalis\WinPhone\Event\OMA\SyncML\SyncMLEvent;

/**
 * This class handles OMA DM SyncML requests from Windows Phone.
 * CmdIDs that don't have are not related to a command id in the database are
 * prefixed with "x".
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 *
 */
class SyncMLHandler extends AbstractAuralisService
{
    /**
     * @return \Topalis\WinPhone\Entity\SyncMLCommandEntityManager
     */
    private function getCommandEntityManager()
    {
        return $this->container->get("winphone.entity.manager.syncml_command");
    }

    /**
     * @return \Topalis\Device\Entity\DeviceEntityManager
     */
    private function getDeviceEntityManager()
    {
        return $this->container->get("device.entity.manager.device");
    }

    /**
     * @return \Symfony\Component\EventDispatcher\EventDispatcherInterface
     */
    private function getEventDispatcher()
    {
        return $this->container->get("event_dispatcher");
    }

    /**
     *
     * @param \Topalis\WinPhone\OMA\SyncML\Entity\SyncML $request The request to
     *     handle.
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\SyncML The SyncML response.
     */
    public function handleSyncML(SyncML $request)
    {
        $session = new SyncMLSession($request);
        return $this->handleSyncMLSession($session);
    }

    /**
     * @param \Topalis\WinPhone\OMA\SyncML\SyncMLSession $session The session to
     *     handle. Must contain the SyncML request.
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\SyncML The SyncML response.
     */
    public function handleSyncMLSession(SyncMLSession $session)
    {
        $syncMLResponse = new SyncML();
        $syncMLResponse->setSyncHdr(new SyncHdr());
        $syncMLResponse->setSyncBody(new SyncBody());
        $syncMLResponse->getSyncBody()->setFinal(true);
        $session->setResponse($syncMLResponse);

        $syncML = $session->getRequest();
        $syncBody = $syncML->getSyncBody();
        $commands = $syncBody->getCommands();

        if (isset($commands[0]) && $commands[0]->getElementName() == "Alert"
            && ($commands[0]->getData() == Alert::CLIENT_INITIATED_MGMT
            || $commands[0]->getData() == Alert::SERVER_INITIATED_MGMT)) {
            $session->setInit(true);
        }

        if (!$this->handleHdr($syncML->getSyncHdr(), $session)) {
            return $syncMLResponse;
        }

        foreach ($syncBody->getStatusList() as $status) {
            $this->handleStatus($status, $session);
        }

        foreach ($syncBody->getResultsList() as $results) {
            $this->handleResults($results, $session);
        }

        $this->initHdr($syncMLResponse->getSyncHdr(), $session);

        foreach ($commands as $command) {
            $this->handleCommand($command, $session);
        }

        if ($session->isAbort()) {
            return $syncMLResponse;
        }

        // return next command. if we received no status for the last sent
        // command, we return it again.
        $syncMLCommand = $this->getCommandEntityManager()->findOneBy(
            array(
                "device" => $session->getDevice(),
                "status" => array(SyncMLCommand::STATUS_NEW, SyncMLCommand::STATUS_WAITING),
            ), array("id" => "ASC")
        );
        if ($syncMLCommand !== null) {
            $syncBodyResponse = $syncMLResponse->getSyncBody();
            $syncBodyResponse->addCommand(XmlConverter::fromXml($syncMLCommand->getCommand()));
            $syncMLCommand->setStatus(SyncMLCommand::STATUS_WAITING);
            $syncMLCommand->setSentCount($syncMLCommand->getSentCount() + 1);
            $this->getCommandEntityManager()->save($syncMLCommand);
        }

        return $syncMLResponse;
    }

    protected function handleHdr(SyncHdr $syncHdr, SyncMLSession $session)
    {
        $session->setId($syncHdr->getSessionID());
        $status = new Status();
        $status->setCmdID("x" . $session->popNextCmdID());
        $status->setMsgRef($syncHdr->getMsgID());
        $status->setCmdRef(0);
        $status->setCmd("SyncHdr");
        $status->addTargetRef($syncHdr->getTarget()->getLocURI());
        $status->addSourceRef($syncHdr->getSource()->getLocURI());
        $syncMLResponse = $session->getResponse();
        $syncMLResponse->getSyncBody()->addStatus($status);

        $cred = $syncHdr->getCred();
        $credData = $cred === null ?: $syncHdr->getCred()->getData();
        if ($cred->getMeta()->Format == "b64") {
            $credData = base64_decode($credData);
        }

        if (!$this->authenticate($credData, $cred->getMeta()->Type, $session)) {
            // remove credentials
            $syncMLResponse->getSyncHdr()->clearCred();
            $status->setData(401);
            return false;
        }

        $status->setData(200); // Authenticated

        return true;
    }

    protected function handleAlert(Alert $alert, SyncMLSession $session)
    {
        $status = new Status();
        $status->setCmdID("x" . $session->popNextCmdID());
        $status->setMsgRef($session->getRequest()->getSyncHdr()->getMsgID());
        $status->setCmdRef($alert->getCmdID());
        $status->setCmd($alert->getElementName());

        switch ($alert->getData()) {
            case Alert::CLIENT_INITIATED_MGMT:
                return;
            case Alert::SESSION_ABORT:
                $session->setAbort(true);
                $status->setData(200);
                break;
            case Alert::GENERIC_ALERT:
                if ($this->isUserUnenroll($alert)) {
                    $this->getEventDispatcher()->dispatch(SyncMLEvents::USER_UNENROLL,
                        new SyncMLEvent($session->getDevice()));
                    $status->setData(200);
                    break;
                }
                // not supported by Windows Phone MDM, yet
                /* fall-through */
            case Alert::CLIENT_EVENT:
                // not supported by Windows Phone MDM, yet
                /* fall-through */
            default:
                $status->setData(406); // not supported
                break;
        }

        $session->getResponse()->getSyncBody()->addStatus($status);
    }

    protected function handleCommand(\Topalis\WinPhone\OMA\SyncML\Entity\AbstractCommand $command,
        SyncMLSession $session)
    {
        switch ($command->getElementName()) {
            case "Alert":
                $this->handleAlert($command, $session);
                break;
            case "Replace":
                if ($session->isInit()) {
                    $this->handleReplace($command, $session);
                    return;
                } // else fall through
            default:
                $status = new Status();
                $status->setCmdID("x" . $session->popNextCmdID());
                $status->setMsgRef($session->getRequest()->getSyncHdr()->getMsgID());
                $status->setCmdRef($command->getCmdID());
                $status->setCmd($command->getElementName());
                $status->setData(405); // Command not allowed
                $session->getResponse()->getSyncBody()->addStatus($status);
                break;
        }
    }

    protected function handleReplace(\Topalis\WinPhone\OMA\SyncML\Entity\Replace $replace, SyncMLSession $session)
    {
        $device = $session->getDevice();
        $deviceInfo = $device->getDeviceinformation();
        $deviceInfoReceived = false;

        foreach ($replace->getItems() as $item) {
            $locURI = $item->getSource()->getLocURI();
            if (!StringUtil::startsWith($locURI, "./DevInfo/")) {
                continue;
            }

            $deviceInfoReceived = true;
            $key = substr(strrchr($locURI, "/"), 1);
            $deviceInfo[$key] = $item->getData();
            if ($key == "DevId") {
                $value = $item->getData();
                if (StringUtil::startsWith($value, "urn:uuid:")) {
                    $value = substr($value, strlen("urn:uuid:"));
                }
                $device->setDeviceid($value);
            }
        }

        if ($deviceInfoReceived) {
            $device->setDeviceInformation($deviceInfo);
            $device->setLastDeviceInformationUpdate($this->getTimeProvider()->getTime());
            $this->getDeviceEntityManager()->save($device);
        }
    }

    protected function handleResults(\Topalis\WinPhone\OMA\SyncML\Entity\Results $results, SyncMLSession $session)
    {
        $device = $session->getDevice();
        $deviceInfo = $device->getDeviceinformation();
        $deviceInfoReceived = false;

        foreach ($results->getItems() as $item) {
            if ($item->getSource() === null) {
                continue;
            }

            $locURI = $item->getSource()->getLocURI();
            $this->getLogger()->debug("Received the following value for ". $locURI . ": " . $item->getData(), array(
                "command" => $results->getCmdRef(),
                "device" => $device->getCombinedId()
            ));

            if ($locURI == "./Vendor/MSFT/Registry/HKLM/Software/Microsoft/Provisioning/DeviceEncryption/Enabled"
                || $locURI == "./Vendor/MSFT/PolicyManager/Device/Security/RequireDeviceEncryption") {
                $deviceInfoReceived = true;
                $key = "Encrypted";
                $deviceInfo[$key] = $item->getData();
                continue;
            }

            if (!StringUtil::startsWith($locURI, "./DevDetail/")
                && !StringUtil::startsWith($locURI, "./Vendor/MSFT/DeviceInstanceService/")) {
                continue;
            }

            $deviceInfoReceived = true;
            $key = substr(strrchr($locURI, "/"), 1);
            $deviceInfo[$key] = $item->getData();
            switch ($key) {
                case "DeviceName":
                    $device->setName($item->getData());
                    break;
                case "Lang":
                    $locale = str_replace("-", "_", $item->getData());
                    if (preg_match('/^[a-z0-9@_\\.\\-]*$/i', $locale)) {
                        $device->setLocale($locale);
                    }
                    break;
                case "SwV":
                    $device->setVersion($item->getData());
                    break;
            }
        }

        if ($deviceInfoReceived) {
            $device->setDeviceInformation($deviceInfo);
            $device->setLastDeviceInformationUpdate($this->getTimeProvider()->getTime());
            $this->getDeviceEntityManager()->save($device);
            $this->getEventDispatcher()->dispatch(SyncMLEvents::DEVICE_INFORMATION,
                new SyncMLEvent($session->getDevice()));
        }
    }

    protected function handleStatus(Status $status, SyncMLSession $session)
    {
        $cmdRef = $status->getCmdRef();
        $code = $status->getData();
        $this->getLogger()->debug("Handle status for command " . $cmdRef . " (result code: " . $code . ")");

        $device = $session->getDevice();

        if ($status->getCmdRef() == 0 && $status->getData() == 401) {
            $session->setServerAuthFailed(true);
        }

        $chal = $status->getChal();
        if ($chal !== null) {
            $nextNonce = $chal->getMeta()->NextNonce;
            if ($nextNonce) {
                $this->getLogger()->debug("Set new nonce for device " . $device->getId());
                $session->setNewNonce($nextNonce);
                $device->setOmaAuthNonce($nextNonce);
                $this->getDeviceEntityManager()->save($device);
            }
        }

        $cmdRefSplit = explode("_", $cmdRef);
        $cmdRefBase = $cmdRefSplit[0];
        $command = $this->getCommandEntityManager()->find($cmdRefBase);
        if ($command === null) {
            return;
        }

        if ($command->getDevice()->getId() != $device->getId()) {
            $this->getLogger()->warning("The device " . $device->getId()
                . " returned the command id " . $cmdRefBase
                . " that belongs to device " . $command->getDevice()->getId());
            return;
        }

        if (count($cmdRefSplit) == 1) {
            $command->setResponseTime($this->getTimeProvider()->getTime());
            $command->setStatusCode($code);
            // 202 => accepted for processing
            if ($code == 200 || ($command->getCommandType() == "Exec" && $code == 202)) {
                $command->setStatus(SyncMLCommand::STATUS_OK);
            } else if ($status->getData() != 401 || !$session->isServerAuthFailed() || $command->getSentCount() > 2) {
                // stay in waiting status if all of the above conditions don't match,
                // so the command will be sent again
                $command->setStatus(SyncMLCommand::STATUS_ERROR);
                $this->informAdmin($command, $status);
            }

            $cspStatus = $command->getCSPStatus();
            if ($cspStatus !== null) {
                $lastCommand = $cspStatus->getCommands()->last();
                if ($lastCommand == $command) {
                    if ($command->getStatus() == SyncMLCommand::STATUS_OK) {
                        if ($cspStatus->getPendingOperation() == "Delete") {
                            $cspStatus->setInstalled(false);
                        } else {
                            $cspStatus->setInstalled(true);
                        }
                    }
                    $cspStatus->setPendingOperation(null);
                }
            } else if ($command->getCommandType() == "Replace") {
                if ($command->getStatus() == SyncMLCommand::STATUS_OK) {
                    $replace = XmlConverter::fromXml($command->getCommand());
                    $replaceLocURI = $replace->getItem(0)->getTarget()->getLocURI();
                    if ($replaceLocURI ==
                        "./Vendor/MSFT/Registry/HKLM/Software/Microsoft/Provisioning/DeviceEncryption/Enabled") {
                        $deviceInfo = $device->getDeviceinformation();
                        $deviceInfo['Encrypted'] = 1;
                        $device->setDeviceInformation($deviceInfo);
                        $this->getDeviceEntityManager()->save($device);
                    }
                }
            } else if ($session->getDevice()->getWipestatus() == Device::WIPE_PENDING) {
                // check if this is the result of the wipe
                if ($command->getCommandType() == "Exec") {
                    $exec = XmlConverter::fromXml($command->getCommand());
                    $execLocUri = $exec->getItem(0)->getTarget()->getLocURI();
                    if ($execLocUri == "./Vendor/MSFT/DMClient/Unenroll"
                        || $execLocUri == "./Vendor/MSFT/RemoteWipe/doWipe") {
                        if ($command->getStatus() == SyncMLCommand::STATUS_OK) {
                            $session->getDevice()->setWipestatus(Device::WIPE_EXECUTED);
                        } else {
                            $session->getDevice()->setWipestatus(Device::WIPE_ERROR);
                        }
                    }
                }
            }
        } else {
            $statusCodes = $command->getStatusCodes();
            $statusCodes[$cmdRef] = $status->getData();
            $command->setStatusCodes($statusCodes);
        }

        $this->getCommandEntityManager()->save($command);
    }

    /**
     * @param Alert $alert A generic alert.
     * @return true if the specified alert indicates a user initiated
     *     unenrollment, false otherwise.
     * @since Windows Phone 8.1
     */
    protected function isUserUnenroll(Alert $alert)
    {
        if ($alert->getItems()) {
            $item = $alert->getItem(0);
            $meta = $item->getMeta();
            if ($meta->Type == "com.microsoft:mdm.unenrollment.userrequest" && $item->getData() == "1") {
                return true;
            }
        }

        return false;
    }

    protected function initHdr(SyncHdr $syncHdr, SyncMLSession $session)
    {
        $reqSyncHdr = $session->getRequest()->getSyncHdr();

        $syncHdr->setSessionID($session->getId());
        $syncHdr->setMsgID($reqSyncHdr->getMsgID());
        $syncHdr->setTarget(new Target($reqSyncHdr->getSource()->getLocURI()));
        $syncHdr->setSource($this->getSource());
        if ($session->getDevice() && ($session->isInit() || $session->getNewNonce())) {
            $usrPwdNonce = $session->getDevice()->getOmaAuthClient()
                . ":" . base64_decode($session->getDevice()->getOmaAuthNonce());
            $digest = base64_encode(md5($usrPwdNonce, true));
            $cred = new Cred();
            $cred->setMeta(new Meta(
                array("Format" => "b64", "Type" => "syncml:auth-md5")));
            $cred->setData($digest);
            $syncHdr->setCred($cred);
        }
    }

    protected function authenticate($credData, $type, SyncMLSession $session)
    {
        if ($type != "syncml:auth-basic") {
            $this->getLogger()->info(
                "Authentication failed: expected authentication type 'syncml:auth-basic', got '$type'");
        }

        list($id, $password) = explode(":", $credData, 2);
        $device = $this->getDeviceEntityManager()->find($id);
        if ($device !== null && $device->getOmaAuthAppSrv() == sha1($password)
            && $device->getWipestatus() != Device::WIPE_EXECUTED) {
            $session->setDevice($device);
            return true;
        }

        $this->getLogger()->info("Authentication failed: invalid credentials");
        return false;
    }

    /**
     *
     * @return \Topalis\WinPhone\OMA\SyncML\Entity\Source
     */
    protected function getSource()
    {
        static $source = null;
        if ($source === null) {
            $config = $this->getConfig();
            $source = new Source("https://" . $config->getHostname() . ":" . $config->getSecondaryPort()
                . "/omadm/winphone");
        }
        return $source;
    }

    private function informAdmin(SyncMLCommand $command, Status $status)
    {
        if ($status->getData() == 418 && $command->getCommandType() === "Add") {
            // item already exists. if just a node should be added we
            // don't need to inform the admin because the intended goal was
            // already achieved
            $add = XmlConverter::fromXml($command->getCommand());
            if (count($add->getItems()) === 1 && null !== $add->getItem(0)->getMeta()
                && $add->getItem(0)->getMeta()->Format === "node") {
                return;
            }
        }

        if ($command->getCommandType() === "Get" && ($status->getData() == 404 || $status->getData() == 500)) {
            /*
             * don't inform admin if the request for a device info attribute
             * failed with status 404 or 500. E.g. if the device has no SIM
             * inserted the request for ./DevDetail/Ext/Microsoft/MobileID fails
             * with status 500.
             */
            $deviceInfo = true;
            foreach ($status->getTargetRefs() as $locURI) {
                if (!StringUtil::startsWith($locURI, "./DevDetail/")
                    && !StringUtil::startsWith($locURI, "./Vendor/MSFT/DeviceInstanceService/")) {
                    $deviceInfo = false;
                    break;
                }
            }
            if ($deviceInfo) {
                return;
            }
        }

        $this->sendCommandFailedMessage($command, $status);
    }

    private function sendCommandFailedMessage(SyncMLCommand $command, Status $status)
    {
        try {
            $subject = $this->tr("admin.winphone.syncml.commandFailedSubject");
            $commandText = $command->getCommandType();
            if ($command->getInfo()) {
                $commandText .= " (" . $command->getInfo() . ")";
            }
            $device = $command->getDevice();
            // user might be deleted
            $user = $device->getUser() ? $device->getUser()->getLogin() : null;
            $deviceName = $device->getName();
            if ($device->getComment()) {
                $deviceName .= " (" . $device->getComment() . ")";
            }
            $text = $this->tr("admin.winphone.syncml.commandFailedText", array(
                "%command%" => $commandText,
                "%status%" => $command->getStatus(),
                "%user%" => $user,
                "%device%" => $deviceName,
                "%response%" => $status->toXml(true)
            ));

            $mailer = $this->container->get("mailer");
            if ($mailer->isConfigured()) {
                $mailer->sendAdminMessage($subject, $text);
            } else {
                $this->getLogger()->info($text);
            }
        } catch (\Exception $e) {
            $this->getLogger()->error("sending SyncML result error message failed", array("exception" => $e));
            if (isset($text)) {
                $this->getLogger()->info($text);
            }
        }
    }
}
