<?php
namespace Topalis\WinPhone\Soap;

use Symfony\Component\HttpFoundation\Response;

/**
 * This Response class is used to avoid setting the header Content-Type multiple
 * times to avoid aborting mod_fastcgi. PHP's SoapServer always sends a Content-
 * Type header. This class replaces the previously set header instead of adding
 * another one.
 *
 * @author Dominic Tubach <dominic.tubach@to.com>
 */
class SoapResponse extends Response
{
    /**
     * Constructor.
     *
     * @param string $content The response content
     * @param int    $status  The response status code
     * @param array  $headers An array of response headers
     *
     * @throws \InvalidArgumentException When the HTTP status code is not valid
     */
    public function __construct($content = '', $status = 200, array $headers = array())
    {
        if (!isset($headers['Content-Type'])) {
            $headers['Content-Type'] = "application/soap+xml; charset=utf-8";
        }
        parent::__construct($content, $status, $headers);
    }

    /**
     * Sends HTTP headers.
     *
     * @return SoapResponse
     */
    public function sendHeaders()
    {
        // headers have already been sent by the developer
        if (headers_sent()) {
            return $this;
        }

        // status
        header(sprintf('HTTP/%s %s %s', $this->version, $this->statusCode, $this->statusText));

        // headers
        foreach ($this->headers->allPreserveCase() as $name => $values) {
            foreach ($values as $value) {
                // avoid sending header Content-Type multiple times
                // it's already done by PHP's SoapServer.
                // mod_fastcgi aborts if this header is set multiple times
                header($name.': '.$value, $name == "Content-Type");
            }
        }

        // cookies
        foreach ($this->headers->getCookies() as $cookie) {
            setcookie($cookie->getName(), $cookie->getValue(), $cookie->getExpiresTime(),
                $cookie->getPath(), $cookie->getDomain(), $cookie->isSecure(), $cookie->isHttpOnly());
        }

        return $this;
    }
}
