<?php
namespace Topalis\WinPhone\Soap;

use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\DependencyInjection\ContainerAwareInterface;

/**
 * Base class for SOAP servers.
 *
 * @see SoapService
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
class SoapServer extends \SoapServer implements ContainerAwareInterface
{
    /**
     * @var \Symfony\Component\DependencyInjection\ContainerInterface
     */
    protected $container;

    /**
     * @var SoapService
     */
    private $soapService;

    /**
     * @var string
     */
    private $soapFault;

    /**
     * Constructor.
     *
     * @param string $wsdl The path to the WSDL file.
     * @param SoapService $soapService The SOAP service to use in this server.
     * @param array $options
     */
    public function __construct($wsdl, SoapService $soapService, array $options = null)
    {
        parent::__construct($wsdl, $options);
        $this->setSoapService($soapService);
    }

    /**
     * Handles the specified SOAP request and returns the result.
     *
     * @param string $soapRequest The SOAP request to handle.
     * @return string The result.
     */
    public function handle($soapRequest = null)
    {
        ob_start();
        parent::handle($soapRequest);
        $content = ob_get_clean();
        if ($this->soapFault) {
            $content = preg_replace("~<env:Body>.*</env:Body>~",
                "<env:Body><env:Fault>" . $this->soapFault . "</env:Fault></env:Body>", $content);
        }
        return $content;
    }

    /**
     * This method is for the SOAP service to use if an error occurs. If this
     * method is called the SOAP body will be replaced with a corresponding
     * SOAP fault. After calling this method the SOAP service shall return
     * null.
     *
     * @param string $code
     * @param string $subcode
     * @param string $reason
     * @param string $lang The language of the reason (IETF language tag).
     */
    public function createFault($code, $subcode, $reason, $lang = "en-US")
    {
        $this->soapFault = "<env:Code><env:Value>"
            . $code . "</env:Value><env:Subcode><env:Value>"
            . $subcode . "</env:Value></env:Subcode></env:Code><env:Reason><env:Text xml:lang=\"$lang\">"
            . $reason . "</env:Text></env:Reason>";
    }

    /**
     * Sets the Container associated with this Controller.
     *
     * @param ContainerInterface $container A ContainerInterface instance
     */
    public function setContainer(ContainerInterface $container = null)
    {
        $this->container = $container;
        $this->soapService->setContainer($container);
    }

    /**
     * @return \Topalis\WinPhone\Soap\SoapService
     */
    public function getSoapService()
    {
        return $this->soapService;
    }

    /**
     * @param SoapService $soapService
     */
    protected function setSoapService(SoapService $soapService)
    {
        $this->soapService = $soapService;
        $this->setObject($this->soapService);
        $this->soapService->setServer($this);
        if ($this->container) {
            $this->soapService->setContainer($this->container);
        }
    }
}
