<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFDictionary;
use CFPropertyList\CFNumber;
use CFPropertyList\CFString;
use Topalis\Profile\Plist\CFDictionaryWrapper;

// @codingStandardsIgnoreStart
/**
 * Abstract class representing payloads.
 * @see https://developer.apple.com/library/ios/#featuredarticles/iPhoneConfigurationProfileRef/Introduction/Introduction.html#//apple_ref/doc/uid/TP40010206-CH1-SW1
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
abstract class AbstractPayload extends CFDictionaryWrapper
{
    /**
     *
     * @var integer
     */
    protected static $defaultVersion = 1;

    /**
     * Constructor.
     * If "PayloadVersion" is not defined in the specified dictionary, it is set
     * to static::$defaultVersion if not null.
     * If "PayloadType" is not defined in the specified dictionary, it is set to
     * static::TYPE if not null.
     *
     * @param CFDictionary $dictionary Dictionary containing the initial payload
     *     data. If null a new dictionary will be created.
     */
    public function __construct(CFDictionary $dictionary = null)
    {
        parent::__construct($dictionary);

        if ($this->getType() === null && static::TYPE !== null) {
            $this->setType(static::TYPE);
        }
        if ($this->getVersion() === null && static::$defaultVersion !== null) {
            $this->setVersion(static::$defaultVersion);
        }
    }

    /**
     * @return String The payload's UUID, null if not defined.
     */
    public function getUUID()
    {
        return $this->getValue("PayloadUUID");
    }

    /**
     * Sets the payload's UUID.
     *
     * @param string $uuid The UUID to set.
     */
    public function setUUID($uuid)
    {
        $this->set("PayloadUUID", new CFString($uuid));
    }

    /**
     * @return int The payload's version, null if not defined.
     */
    public function getVersion()
    {
        return $this->getValue("PayloadVersion");
    }

    /**
     * Sets the payload's version.
     *
     * @param int $version The version to set.
     */
    public function setVersion($version)
    {
        $this->set("PayloadVersion", new CFNumber($version));
    }

    /**
     * @return string The paylod's type, null if not defined.
     */
    public function getType()
    {
        return $this->getValue("PayloadType");
    }

    /**
     * Sets the payload's type.
     *
     * @param string $type The type to set.
     */
    public function setType($type)
    {
        $this->set("PayloadType", new CFString($type));
    }

    /**
     * @return string The payload's identifier, null if not defined.
     */
    public function getIdentifier()
    {
        return $this->getValue("PayloadIdentifier");
    }

    /**
     * Sets the payload's identifier.
     *
     * @param string $identifier The identifier to set.
     */
    public function setIdentifier($identifier)
    {
        $this->set("PayloadIdentifier", new CFString($identifier));
    }

    /**
     * @return string The payload's organization, null if not defined.
     */
    public function getOrganization()
    {
        return $this->getValue("PayloadOrganization");
    }

    /**
     * Sets the payload's organization.
     *
     * @param string $organization The organization to set. If null the
     *     corresponding entry will be removed.
     */
    public function setOrganization($organization)
    {
        if ($organization === null) {
            $this->del("PayloadOrganization");
        } else {
            $this->set("PayloadOrganization", new CFString($organization));
        }
    }

    /**
     * @return string The payload's display name, null if not defined.
     */
    public function getDisplayName()
    {
        return $this->getValue("PayloadDisplayName");
    }

    /**
     * Sets the payload's display name.
     *
     * @param string $name The display name to set. If null the
     *     corresponding entry will be removed.
     */
    public function setDisplayName($name)
    {
        if ($name === null) {
            $this->del("PayloadDisplayName");
        } else {
            $this->set("PayloadDisplayName", new CFString($name));
        }
    }

    /**
     * @return string The payload's description, null if not defined.
     */
    public function getDescription()
    {
        return $this->getValue("PayloadDescription");
    }

    /**
     * Sets the payload's description.
     *
     * @param string $description The description to set. If null the
     *     corresponding entry will be removed.
     */
    public function setDescription($description)
    {
        if ($description === null) {
            $this->del("PayloadDescription");
        } else {
            $this->set("PayloadDescription", new CFString($description));
        }
    }
}
