<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFData;
use CFPropertyList\CFNumber;
use CFPropertyList\CFString;
use CFPropertyList\CFBoolean;

// @codingStandardsIgnoreStart
/**
 * Represents an exchange payload.
 * This payload configures an Exchange Active Sync account on the device.
 * @see https://developer.apple.com/library/ios/#featuredarticles/iPhoneConfigurationProfileRef/Introduction/Introduction.html#//apple_ref/doc/uid/TP40010206-CH1-SW25
 *
 * @author Florian Pfitzer <florian.pfitzer@topalis.com>
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class ExchangePayload extends AbstractPayload
{
    const TYPE = "com.apple.eas.account";

    public function isPreventMove()
    {
        return $this->getValue("PreventMove", false);
    }

    public function setPreventMove($preventMove)
    {
        if ($preventMove === null) {
            $this->del("PreventMove");
        } else {
            $this->set("PreventMove", new CFBoolean($preventMove));
        }
        return $this;
    }

    public function isPreventAppSheet()
    {
        return $this->getValue("PreventAppSheet", false);
    }

    public function setPreventAppSheet($preventAppSheet)
    {
        if ($preventAppSheet === null) {
            $this->del("PreventAppSheet");
        } else {
            $this->set("PreventAppSheet", new CFBoolean($preventAppSheet));
        }
        return $this;
    }

    public function getCertificate()
    {
        return $this->getValue("Certificate");
    }

    public function setCertificate($certificate)
    {
        if ($certificate === null) {
            $this->del("Certificate");
        } else {
            $this->set("Certificate", new CFData($certificate));
        }
    }

    public function getCertificateName()
    {
        return $this->getValue("CertificateName");
    }

    public function setCertificateName($certificateName)
    {
        if ($certificateName === null) {
            $this->del("CertificateName");
        } else {
            $this->set("CertificateName", new CFString($certificateName));
        }
    }

    public function getCertificatePassword()
    {
        return $this->get("CertificatePassword");
    }

    public function setCertificatePassword($certificatePassword)
    {
        if ($certificatePassword === null) {
            $this->del("CertificatePassword");
        } else {
            $this->set("CertificatePassword", new CFData($certificatePassword));
        }
    }

    public function getCertificateUUID()
    {
        return $this->getValue("PayloadCertificateUUID");
    }

    public function setCertificateUUID($certificateUUID)
    {
        if ($certificateUUID === null) {
            $this->del("PayloadCertificateUUID");
        } else {
            $this->set("PayloadCertificateUUID", new CFString($certificateUUID));
        }
    }

    public function isSMIMEEnabled()
    {
        return $this->getValue("SMIMEEnabled", false);
    }

    public function setSMIMEEnabled($smimeEnabled)
    {
        if ($smimeEnabled === null) {
            $this->del("SMIMEEnabled");
        } else {
            $this->set("SMIMEEnabled", new CFBoolean($smimeEnabled));
        }
        return $this;
    }

    public function getSMIMESigningCertificateUUID()
    {
        return $this->getValue("SMIMESigningCertificateUUID");
    }

    public function setSMIMESigningCertificateUUID($smimeSigningCertificateUUID)
    {
        if ($smimeSigningCertificateUUID === null) {
            $this->del("SMIMESigningCertificateUUID");
        } else {
            $this->set("SMIMESigningCertificateUUID", new CFString($smimeSigningCertificateUUID));
        }
    }

    public function getSMIMEEncryptionCertificateUUID()
    {
        return $this->getValue("SMIMEEncryptionCertificateUUID");
    }

    public function setSMIMEEncryptionCertificateUUID($smimeEncryptionCertificateUUID)
    {
        if ($smimeEncryptionCertificateUUID === null) {
            $this->del("SMIMEEncryptionCertificateUUID");
        } else {
            $this->set("SMIMEEncryptionCertificateUUID", new CFString($smimeEncryptionCertificateUUID));
        }
    }

    public function isDisableMailRecentsSyncing()
    {
        return $this->getValue("disableMailRecentsSyncing", false);
    }

    public function setDisableMailRecentsSyncing($disableMailRecentsSyncing)
    {
        $this->setValue("disableMailRecentsSyncing", $disableMailRecentsSyncing, self::TYPE_BOOLEAN);
        return $this;
    }

    /**
     * @param string $emailAddress
     *
     * @return $this
     */
    public function setEmailAddress($emailAddress)
    {
        $this->set("EmailAddress", new CFString($emailAddress));
        return $this;
    }

    /**
     * @return string
     */
    public function getEmailAddress()
    {
        return $this->getValue("EmailAddress");
    }

    /**
     * @param string $host
     *
     * @return $this
     */
    public function setHost($host)
    {
        $this->set("Host", new CFString($host));
        return $this;
    }

    /**
     * @return string
     */
    public function getHost()
    {
        return $this->getValue("Host");
    }

    public function isSSL()
    {
        return $this->getValue("SSL", true);
    }

    public function setSSL($sslEnabled)
    {
        if ($sslEnabled === null) {
            $this->del("SSL");
        } else {
            $this->set("SSL", new CFBoolean($sslEnabled));
        }
        return $this;
    }

    /**
     * @param string $password
     *
     * @return $this
     */
    public function setPassword($password)
    {
        if ($password === null) {
            $this->del("Password");
        } else {
            $this->set("Password", new CFString($password));
        }
        return $this;
    }

    /**
     * @return string
     */
    public function getPassword()
    {
        return $this->getValue("Password");
    }

    /**
     * @param string $userName
     *
     * @return $this
     */
    public function setUserName($userName)
    {
        $this->set("UserName", new CFString($userName));
        return $this;
    }

    /**
     * @return string
     */
    public function getUserName()
    {
        return $this->getValue("UserName");
    }

    public function getMailNumberOfPastDaysToSync()
    {
        return $this->getValue("MailNumberOfPastDaysToSync");
    }

    public function setMailNumberOfPastDaysToSync($mailNumberOfPastDaysToSync)
    {
        if ($mailNumberOfPastDaysToSync === null) {
            $this->del("MailNumberOfPastDaysToSync");
        } else {
            $this->set("MailNumberOfPastDaysToSync", new CFNumber($mailNumberOfPastDaysToSync));
        }
        return $this;
    }

}
