<?php
namespace Topalis\Profile\Payload;

use CFPropertyList\CFDictionary;
use CFPropertyList\CFNumber;
use CFPropertyList\CFString;

// @codingStandardsIgnoreStart
/**
 * This class represents a SCEP payload.
 * An SCEP payload automates the request of a client certificate from an SCEP
 * server, as described in Over-the-Air Profile Delivery and Configuration.
 * Note: the keys defined in Apple's reference must be set in a sub dictionary
 * in "PayloadContent".
 * @see https://developer.apple.com/library/ios/#featuredarticles/iPhoneConfigurationProfileRef/Introduction/Introduction.html#//apple_ref/doc/uid/TP40010206-CH1-SW18
 *
 * @author Dominic Tubach <dominic.tubach@topalis.com>
 */
// @codingStandardsIgnoreEnd
class ScepPayload extends AbstractPayload
{
    const TYPE = "com.apple.security.scep";

    /**
     * @var \Topalis\Profile\Payload\MinimalPayload
     */
    private $content;

    public function __construct(CFDictionary $dictionary = null)
    {
        parent::__construct($dictionary);
        $contentDict = $this->get("PayloadContent");
        if ($contentDict === null) {
            $contentDict = new CFDictionary();
            $this->set("PayloadContent", $contentDict);
        }
        $this->content = new MinimalPayload($contentDict);
    }

    public function getUrl()
    {
        return $this->getContent()->getValue("URL");
    }

    public function setUrl($url)
    {
        $this->getContent()->set("URL", new CFString($url));
    }

    public function getName()
    {
        return $this->getContent()->getValue("Name");
    }

    public function setName($name)
    {
        if ($name === null) {
            $this->getContent()->del("Name");
        } else {
            $this->getContent()->set("Name", new CFString($name));
        }
    }

    public function getSubject()
    {
        return $this->getContent()->getValue("Subject");
    }

    public function setSubject(array $subject = null)
    {
        $this->getContent()->setValue("Subject", $subject, self::TYPE_ARRAY);
    }

    public function getChallenge()
    {
        return $this->getContent()->getValue("Challenge");
    }

    public function setChallenge($challenge)
    {
        if ($challenge === null) {
            $this->getContent()->del("Challenge");
        } else {
            $this->getContent()->set("Challenge", new CFString($challenge));
        }
    }

    public function getKeysize()
    {
        return $this->getContent()->getValue("Keysize");
    }

    public function setKeysize($keysize)
    {
        if ($keysize === null) {
            $this->getContent()->del("Keysize");
        } else {
            $this->getContent()->set("Keysize", new CFNumber($keysize));
        }
    }

    public function getKeyType()
    {
        return $this->getContent()->getValue("KeyType");
    }

    public function setKeyType($keyType)
    {
        if ($keyType === null) {
            $this->getContent()->del("KeyType");
        } else {
            $this->getContent()->set("KeyType", new CFString($keyType));
        }
    }

    public function getKeyUsage()
    {
        return $this->getContent()->getValue("KeyUsage");
    }

    public function setKeyUsage($keyUsage)
    {
        if ($keyUsage === null) {
            $this->getContent()->del("KeyUsage");
        } else {
            $this->getContent()->set("KeyUsage", new CFNumber($keyUsage));
        }
    }

    public function getSubjectAltName()
    {
        return $this->getContent()->getValue("SubjectAltName");
    }

    public function setSubjectAltName(array $subjectAltName = null)
    {
        $this->getContent()->setValue("SubjectAltName", $subjectAltName, self::TYPE_DICTIONARY);
    }

    public function getCACaps()
    {
        return $this->getContent()->getValue("GetCACaps");
    }

    public function setCACaps(array $caCaps = null)
    {
        $this->getContent()->setValue("GetCACaps", $caCaps, self::TYPE_DICTIONARY);
    }

    /**
     * @return \Topalis\Profile\Payload\MinimalPayload
     */
    protected function getContent()
    {
        return $this->content;
    }
}
